#include "attachpoint.h"
#include <stdlib.h>
#include "spot.h"
#include "component.h"
#include "subfunction.h"

int NewAttachPointID(GCD *CircuitData, int SetAttachPointID)
{
  
  CircuitData->Info.AttachPointID += 10;
  if (SetAttachPointID != NoID)
    CircuitData->Info.AttachPointID = SetAttachPointID-10;
  /* SetAttachPointID est le prochain ID utilisable, il doit etre renvoye lors du prochain appel */
  return CircuitData->Info.AttachPointID;
}

ErrorCheck AddAttachPoint(GCD *CircuitData, int Nb, int ComponentID)
{
  AttachPoint  *NewAttach, *AttachCursor, *PrevAttach;
  
  NewAttach = (AttachPoint*)malloc(sizeof(AttachPoint));
  
  if(NewAttach==NULL)
    return MemoryAllocationError;

  NewAttach->IDNumber=Nb;
  NewAttach->ComponentID = ComponentID;
  NewAttach->SpotLocation=NULL;
  NewAttach->SpotSize = 0;
  NewAttach->CoreSpotSize = 0;
  NewAttach->InvisibleSpotSize = 0;
  InitState(NewAttach->State);
  UnSetAttachSelected(NewAttach->State);
  NewAttach->pNext = NULL;
 
  PrevAttach = NULL;
  AttachCursor = CircuitData->ListAttachPoint;
  while (AttachCursor != NULL)
    {
      PrevAttach = AttachCursor;
      AttachCursor = AttachCursor->pNext;
    }
  if (PrevAttach == NULL)
    CircuitData->ListAttachPoint = NewAttach;
  else 
    PrevAttach->pNext = NewAttach;
  
  return AllOk;
}

ErrorCheck EditAttachPoint(GCD *CircuitData, int AttachID, AttachPoint AttachProp)
{
  AttachPoint *AttachCursor;
  ComponentLibrary *CompLib;
  int Counter,  X11=0, X12=0, X21=0, X22=0, Xin, Yin;
  Boolean  IDFound;
  SpotDef SpotProp;
  ErrorCheck Check;

  printf("Edit Attach Prop : %s %s \n", AttachProp.SpotName, AttachProp.AttachPointName);
  CompLib = CircuitData->CompLib;
  
  AttachCursor=CircuitData->ListAttachPoint;
  IDFound = False;
  while ((AttachCursor!=NULL) && (IDFound==False))
    {
      if(AttachCursor->IDNumber== AttachID)
	{
	  IDFound = True;
	  
	  AttachCursor->State = AttachProp.State;
	  if (IsAttachToRight(AttachProp.State))
	    {
	      X11 =  0; X12 =  1;
	      X21 = -1; X22 =  0;
	    }
	  if (IsAttachToBottom(AttachProp.State))
	    {
	      X11 =  1; X12 =  0;
	      X21 =  0; X22 =  1;
	    }
	  if (IsAttachToLeft(AttachProp.State))
	    {
	      X11 =  0; X12 = -1;
	      X21 =  1; X22 =  0;
	    }
	  if (IsAttachToTop(AttachProp.State))
	    {
	      X11 = -1; X12 =  0;
	      X21 =  0; X22 = -1;
	    }
	  
	  AttachCursor->Reference.x = AttachProp.Reference.x;
	  AttachCursor->Reference.y = AttachProp.Reference.y;
	  strncpy(AttachCursor->SpotName, AttachProp.SpotName, SpotNameLength);
	  AttachCursor->SpotName[SpotNameLength-1] = '\0';

	  strncpy(AttachCursor->AttachPointName, AttachProp.AttachPointName, PinNameLength);
	  AttachCursor->AttachPointName[PinNameLength-1] = '\0';

	  printf("%s -> %s\n",  AttachProp.SpotName, AttachCursor->SpotName);
	  Check = GetSpotProperties(CompLib, AttachCursor->SpotName, &SpotProp, False);
	  if (Check == NotFound)
	    {
	      printf("Error trying to find Attach %s\n", AttachCursor->SpotName);
	      printf("ERROR: Spot Not Found while editing AttachPoint!!\n");
	      return LibraryError;
	    }
	  
	  if (SpotProp.SpotSize != 0)
	    SpotProp.SpotLocation = (Point *) malloc (sizeof(Point) * SpotProp.SpotSize);
	  if (SpotProp.CoreSpotSize != 0)
	    SpotProp.CoreSpotLocation = (Point *) malloc (sizeof(Point) * SpotProp.CoreSpotSize);
	  if (SpotProp.InvisibleSpotSize != 0)
	    SpotProp.InvisibleSpotLocation = (Point *) malloc (sizeof(Point) * SpotProp.InvisibleSpotSize);
	  GetSpotProperties(CompLib, AttachCursor->SpotName, &SpotProp, True);
	  
	  if (AttachCursor->SpotSize != 0)
	    free (AttachCursor->SpotLocation);
	  if (AttachCursor->CoreSpotSize != 0)
	    free (AttachCursor->CoreSpotLocation);
	  if (AttachCursor->InvisibleSpotSize != 0)
	    free (AttachCursor->InvisibleSpotLocation);
	  
	  AttachCursor->SpotSize = SpotProp.SpotSize;
	  AttachCursor->CoreSpotSize = SpotProp.CoreSpotSize;
	  AttachCursor->InvisibleSpotSize = SpotProp.InvisibleSpotSize;

	  if (SpotProp.SpotSize != 0)
	    AttachCursor->SpotLocation = (Point *) malloc(sizeof(Point) * AttachCursor->SpotSize);
	  if (SpotProp.CoreSpotSize != 0)
	    AttachCursor->CoreSpotLocation = (Point *) malloc(sizeof(Point) * AttachCursor->CoreSpotSize);
	  if (SpotProp.InvisibleSpotSize != 0)
	    AttachCursor->InvisibleSpotLocation = (Point *) malloc(sizeof(Point) * AttachCursor->InvisibleSpotSize);
	  
	  for (Counter = 0 ; Counter <  AttachCursor->SpotSize ; Counter++)
	    {
	      Xin = SpotProp.SpotLocation[Counter].x;
	      Yin = SpotProp.SpotLocation[Counter].y;
	      AttachCursor->SpotLocation[Counter].x =  X11 * Xin + X12 * Yin;
	      AttachCursor->SpotLocation[Counter].y =  X21 * Xin + X22 * Yin;
	    }
	  for (Counter = 0 ; Counter <  AttachCursor->CoreSpotSize ; Counter++)
	    {
	      Xin = SpotProp.CoreSpotLocation[Counter].x;
	      Yin = SpotProp.CoreSpotLocation[Counter].y;
	      AttachCursor->CoreSpotLocation[Counter].x =  X11 * Xin + X12 * Yin;
	      AttachCursor->CoreSpotLocation[Counter].y =  X21 * Xin + X22 * Yin;
	    }
	  for (Counter = 0 ; Counter <  AttachCursor->InvisibleSpotSize ; Counter++)
	    {
	      Xin = SpotProp.InvisibleSpotLocation[Counter].x;
	      Yin = SpotProp.InvisibleSpotLocation[Counter].y;
	      AttachCursor->InvisibleSpotLocation[Counter].x =  X11 * Xin + X12 * Yin;
	      AttachCursor->InvisibleSpotLocation[Counter].y =  X21 * Xin + X22 * Yin;
	    }
	  if (SpotProp.SpotSize != 0)
	    free(SpotProp.SpotLocation);
	  if (SpotProp.CoreSpotSize != 0)
	    free(SpotProp.CoreSpotLocation);
	  if (SpotProp.InvisibleSpotSize != 0)
	    free(SpotProp.InvisibleSpotLocation);
	  
	}
      AttachCursor=AttachCursor->pNext;
    }

   if (IDFound == True)
    return Found;
   else return NotFound;
}

ErrorCheck  GetAttachPointNumber(GCD *CircuitData, int *Size )
{
  int Counter;
  AttachPoint *AttachCursor;
  
  AttachCursor =  CircuitData->ListAttachPoint;
  Counter = 0;
  while (AttachCursor != NULL)
    {
      Counter++;
      AttachCursor = AttachCursor->pNext;
    }
  *Size = Counter;
  return AllOk;
}

ErrorCheck DisplayAttachPoint(GCD *CircuitData)
{
  AttachPoint *AttachCursor;
  int Counter;
 
  printf("\t-------Liste des points d'attache:-------\n"); 
  AttachCursor=CircuitData->ListAttachPoint;
  while(AttachCursor!=NULL)
    {
      printf("Number %d. Reference: %d %d\n",
	     AttachCursor->IDNumber,AttachCursor->Reference.x,AttachCursor->Reference.y);
      for( Counter = 0 ; Counter < AttachCursor->SpotSize ; Counter++)
	printf("\t%d %d \n", AttachCursor->SpotLocation[Counter].x , AttachCursor->SpotLocation[Counter].y);
      AttachCursor=AttachCursor->pNext;
    }
  printf("\n");
  return AllOk;
}


ErrorCheck DeleteAttachPoint(GCD *CircuitData, int AttachID)
{
  AttachPoint *AttachCursor,  **AttachNext;
  Boolean IDFound;
   
  AttachCursor = CircuitData->ListAttachPoint; 
  IDFound = False;
  AttachNext = &(CircuitData->ListAttachPoint);

  while ((AttachCursor!= NULL) && (IDFound ==False))
    {      
      if (AttachCursor->IDNumber == AttachID)
	{
	  IDFound = True;
	  *AttachNext = AttachCursor->pNext;
	  if (AttachCursor->SpotLocation != NULL)
	    free(AttachCursor->SpotLocation);
	  free(AttachCursor);
	}
      else 
	AttachNext = &(AttachCursor->pNext);
      AttachCursor = *AttachNext;
    }

  if (IDFound == True)
    return Found;
  else return NotFound;
}

ErrorCheck GetAttachPointProperties(GCD *CircuitData, int AttachID, AttachPoint *AttachProp)
{
  AttachPoint *AttachCursor;
  Boolean IDFound;

  AttachCursor=CircuitData->ListAttachPoint;
  IDFound = False;
  while ((AttachCursor!=NULL) && (IDFound == False))
    {
      if(AttachCursor->IDNumber== AttachID)
	{
	  IDFound = True;
	  strncpy(AttachProp->AttachPointName, AttachCursor->AttachPointName,PinNameLength);
	  AttachProp->AttachPointName[PinNameLength-1] = '\0';

	  AttachProp->IDNumber =  AttachCursor->IDNumber;
	  AttachProp->State =  AttachCursor->State;
	  AttachProp->ComponentID = AttachCursor->ComponentID;
	  AttachProp->InterfaceID = AttachCursor->InterfaceID;
	  AttachProp->RoutingAttachID = AttachCursor->RoutingAttachID;
	  strncpy(AttachProp->SpotName, AttachCursor->SpotName, SpotNameLength);
	  AttachProp->SpotName[SpotNameLength-1] = '\0';
	  AttachProp->Reference.x = AttachCursor->Reference.x;
	  AttachProp->Reference.y = AttachCursor->Reference.y;
	  AttachProp->SpotLocation = NULL;
	  AttachProp->SpotSize = 0;
	  AttachProp->CoreSpotSize = 0;
	  AttachProp->InvisibleSpotSize = 0;
	}
      AttachCursor=AttachCursor->pNext;
    }

  if (IDFound == True)
    return Found;
  else return NotFound;
}


ErrorCheck SetIIDForAttachPoint(GCD *CircuitData, int AttachID, void *InterfaceID)
{
  AttachPoint *AttachCursor;
  Boolean IDFound;

  AttachCursor = CircuitData->ListAttachPoint;
  IDFound = False;
  while ((AttachCursor != NULL) && (IDFound == False))
    {
      if ( AttachCursor->IDNumber == AttachID)
	{
	  IDFound = True;
	  AttachCursor->InterfaceID = InterfaceID;
	}
      AttachCursor = AttachCursor->pNext;
    }
  if (IDFound == True)
    return Found;
  else return NotFound;
}

ErrorCheck GetIIDForAttachPoint(GCD *CircuitData, int AttachID, void **InterfaceID)
{
  AttachPoint *AttachCursor;
  Boolean IDFound;

  AttachCursor = CircuitData->ListAttachPoint;
  IDFound = False;
  while ((AttachCursor != NULL) && (IDFound == False))
    {
      if ( AttachCursor->IDNumber == AttachID)
	{
	  IDFound = True;
	  *InterfaceID = AttachCursor->InterfaceID;
	}
      AttachCursor = AttachCursor->pNext;
    }
  if (IDFound == True)
    return Found;
  else return NotFound;
}
ErrorCheck CheckAttachUsed(GCD *CircuitData, int AttachID)
{
  Link *LinkCursor;
  Boolean Used;

  Used = False;
  LinkCursor  = CircuitData->ListLink;
  while ((LinkCursor != NULL) && (Used == False))
    {
      if ((LinkCursor->StartAttachID == AttachID) || (LinkCursor->EndAttachID == AttachID))
	Used = True;
      LinkCursor = LinkCursor->pNext;
    }
  return Used;
}

ErrorCheck AttachRoutingIDToID(GCD *CircuitData, int RoutingID, int *ID)
{
  AttachPoint *AttachCursor;
  Boolean IDFound;

  AttachCursor = CircuitData->ListAttachPoint;
  IDFound = False;
  while ((AttachCursor != NULL) && (IDFound == False))
    {
      if (AttachCursor->RoutingAttachID == RoutingID)
	{
	  *ID = AttachCursor->IDNumber;
	  IDFound = True;
	}
      AttachCursor = AttachCursor->pNext;
    }
  if (IDFound == True)
    return Found;
  else return NotFound;
}

ErrorCheck AttachIDToRoutingID(GCD *CircuitData, int ID, int *RoutingID)
{
  AttachPoint *AttachCursor;
  Boolean IDFound;

  AttachCursor = CircuitData->ListAttachPoint;
  IDFound = False;
  while ((AttachCursor != NULL) && (IDFound == False))
    {
      if (AttachCursor->IDNumber == ID)
	{
	  *RoutingID = AttachCursor->RoutingAttachID;
	  IDFound = True;
	}
      AttachCursor = AttachCursor->pNext;
    }
  if (IDFound == True)
    return Found;
  else return NotFound;
}


ErrorCheck GetAttachPointIDFromCoordinates(GCD *CircuitData, int x, int y, int *AttachID)
{
  AttachPoint *AttachCursor;
  Component CompProp;
  SubFunction SubFProp;
  Boolean FirstAttachFound;
  int Deltax, Deltay, MinLength=0, MinID, Length, SpotLocationCounter;
  
  AttachCursor = CircuitData->ListAttachPoint;
  FirstAttachFound = False;
  MinID = NoID;
  while (AttachCursor != NULL)
    {
      GetComponentProperties(CircuitData, AttachCursor->ComponentID, &CompProp, False);
      GetSubFunctionProperties(CircuitData, CompProp.SubFunctionID, &SubFProp);
      
      for (SpotLocationCounter = 0 ; SpotLocationCounter < AttachCursor->SpotSize ; SpotLocationCounter++)
	{
	  Deltax = SubFProp.Reference.x + CompProp.Reference.x + AttachCursor->Reference.x 
	    + AttachCursor->SpotLocation[SpotLocationCounter].x - x;

	  Deltay = SubFProp.Reference.y + CompProp.Reference.y + AttachCursor->Reference.y 
	    + AttachCursor->SpotLocation[SpotLocationCounter].y - y;

	  Length = Deltax * Deltax + Deltay * Deltay;
	  if ((FirstAttachFound == False) || (Length < MinLength))
	    {
	      FirstAttachFound = True;
	      MinLength = Length;
	      MinID = AttachCursor->IDNumber;
	    }
	}
      for (SpotLocationCounter = 0 ; SpotLocationCounter < AttachCursor->CoreSpotSize ; SpotLocationCounter++)
	{
	  Deltax = SubFProp.Reference.x + CompProp.Reference.x + AttachCursor->Reference.x 
	    + AttachCursor->CoreSpotLocation[SpotLocationCounter].x - x;

	  Deltay = SubFProp.Reference.y + CompProp.Reference.y + AttachCursor->Reference.y 
	    + AttachCursor->CoreSpotLocation[SpotLocationCounter].y - y;

	  Length = Deltax * Deltax + Deltay * Deltay;
	  if ((FirstAttachFound == False) || (Length < MinLength))
	    {
	      FirstAttachFound = True;
	      MinLength = Length;
	      MinID = AttachCursor->IDNumber;
	    }
	}
      AttachCursor = AttachCursor->pNext;
    }
  
  *AttachID = MinID;
  
  if (FirstAttachFound == True)
    return Found;
  else 
    return NotFound;
}

 

ErrorCheck SelectAttachPointFromID( GCD *CircuitData, int SelectedID)
{
  AttachPoint *AttachCursor;
  Component CompProp;
  int AttachNb, *AttachIDTab=NULL, Counter;
  Boolean ToBeSelected, IDTypeFound, ToBeFree = False;
  
  IDTypeFound = False;
  if ( IsSubFunctionID(SelectedID))
    {
      IDTypeFound = True;
      GetSubFunctionAttachNumber(CircuitData, SelectedID, &AttachNb);
      if (AttachNb != 0)
	{
	  AttachIDTab = (int *) malloc (sizeof(int) * AttachNb);
	  ToBeFree = True;
	  GetSubFunctionAttachID(CircuitData, SelectedID, AttachIDTab);
	}
    }
  if ( IsComponentID(SelectedID))
    {
      IDTypeFound = True;
      GetComponentProperties(CircuitData, SelectedID, &CompProp, True);
      AttachNb = CompProp.AttachPointNb;
      AttachIDTab = CompProp.AttachPointIDTab;
    }
  if (IsAttachID(SelectedID))
    {
      IDTypeFound = True;
      AttachNb = 1;
      AttachIDTab = (int *) malloc (sizeof(int));
      ToBeFree = True;
      AttachIDTab[0] = SelectedID;
    }
  if (IDTypeFound == True)
    {
      AttachCursor = CircuitData->ListAttachPoint;
      while (AttachCursor != NULL)
	{
	  ToBeSelected = False;
	  for (Counter = 0 ; (Counter < AttachNb) && (ToBeSelected == False) ; Counter++)
	    if (AttachIDTab[Counter] == AttachCursor->IDNumber)
	      ToBeSelected = True;
	  
	  if (ToBeSelected == True)
	    SetAttachSelected(AttachCursor->State);
	  else 
	    UnSetAttachSelected(AttachCursor->State);
	  AttachCursor = AttachCursor->pNext;
	}
      if (ToBeFree == True)
	free(AttachIDTab);
    }
  return AllOk;
}
  
ErrorCheck GetSelectedAttachPointID(GCD *CircuitData, int *SelectedAttachID)
{
  AttachPoint *AttachCursor;
  int ID = NoID;
  Boolean AttachFound;
  
  AttachFound = False;
  AttachCursor = CircuitData->ListAttachPoint;
  while ((AttachCursor != NULL) && (AttachFound == False))
    {
      if (IsAttachSelected(AttachCursor->State))
	{
	  AttachFound = True;
	  ID = AttachCursor->IDNumber;
	}
      AttachCursor = AttachCursor->pNext;
    }
  if (AttachFound == True)
    {
      *SelectedAttachID = ID;
      return Found;
    }
  else
    return NotFound;
}


