/* autosort.c */

#include "autosort.h"
#include <stdlib.h>
#include "dlink.h"   
#include "mainwindow.h"

ErrorCheck AutoSort(GCD *CircuitData) 
{ 
  DrawLink *DLinkCursor, *DL1, *DL2;
  Boolean NotSortedFound;
  
  DLinkCursor = CircuitData->TopLink;
  while (DLinkCursor != NULL)
    {
      if (IsDLinkSelected(DLinkCursor->State))
	UnSetDLinkSortedOk(DLinkCursor->State);
      else
	SetDLinkSortedOk(DLinkCursor->State);
	
      DLinkCursor = DLinkCursor->LinkDown;
    }

  do 
    {
      NotSortedFound  = False;
      DL1 = CircuitData->TopLink;
      while ( (DL1 !=NULL) && (NotSortedFound == False))
	{
	  if (!IsDLinkSortedOk(DL1->State))
	    NotSortedFound = True;
	  else 
	    DL1 = DL1->LinkDown;
	}
      
      if (NotSortedFound == True)
	{
	  GetMiniDLink(CircuitData, &DL2);	
	  printf("Length %d Width %d\n", DL2->LinkLength, DL2->LinkWidth);

	  if (DL1 != DL2)
	    ExchangeDrawLink(CircuitData, DL1, DL2);
	  
	  SetDLinkSortedOk(DL2->State); /* because DL2 is now sorted */
	}
    } while (NotSortedFound == True);

  return AllOk;
}

ErrorCheck GetMiniDLink(GCD *CircuitData, DrawLink **DLink)
{
  DrawLink *DLinkCursor, *MiniLink;
  int CurrentLength, CurrentWidth, LengthDirection, WidthDirection; /* *Direction : +1 Ascendant -1 Descendant */
                                                                    /* 0 Indifferent */
  Boolean OkForWidth, OkForLength, IndifferentForWidth, IndifferentForLength, NewMiniLinkFound;
  
  WidthDirection = 0;
  LengthDirection = 0;
  if (CircuitData->Info.Opt.WidthOrder == AscendantOrder)
    WidthDirection = +1;
  if (CircuitData->Info.Opt.WidthOrder == DescendantOrder)
    WidthDirection = -1;

  if (CircuitData->Info.Opt.LengthOrder == AscendantOrder)
    LengthDirection = +1;
  if (CircuitData->Info.Opt.LengthOrder == DescendantOrder)
    LengthDirection = -1;

  CurrentLength = 0;
  CurrentWidth = 0;
  MiniLink = NULL; 
  DLinkCursor = CircuitData->TopLink; 
  while (DLinkCursor!=NULL)
    {
      OkForWidth = False;
      OkForLength = False;
      IndifferentForWidth = False;
      IndifferentForLength = False;
      NewMiniLinkFound = False;
      
      if (!IsDLinkSortedOk(DLinkCursor->State))
	{
	 
	  if ((LengthDirection != 0) 
	      && ((LengthDirection * CurrentLength) > (LengthDirection * DLinkCursor->LinkLength)))
	    OkForLength = True;
	  if ((LengthDirection == 0) || (CurrentLength == DLinkCursor->LinkLength))
	    IndifferentForLength = True;
	  
	  if ((WidthDirection != 0) 
	      && ((WidthDirection * CurrentWidth) > (WidthDirection * DLinkCursor->LinkWidth)))
	    OkForWidth = True;
	  if ((WidthDirection == 0) || (CurrentWidth == DLinkCursor->LinkWidth))
	    IndifferentForWidth = True;
	  
	  if ((CircuitData->Info.Opt.SortWidthFirst == True) 
	      && ((OkForWidth == True) || ((IndifferentForWidth == True) && (OkForLength == True))))
	    NewMiniLinkFound = True;
	  
	  if ((CircuitData->Info.Opt.SortWidthFirst == False) 
	      && ((OkForLength == True) || ((IndifferentForLength == True) && (OkForWidth == True))))
	    NewMiniLinkFound = True;
	  
	  if ((MiniLink == NULL) || (NewMiniLinkFound == True))
	      {
		MiniLink = DLinkCursor;
		CurrentLength = DLinkCursor->LinkLength;
		CurrentWidth = DLinkCursor->LinkWidth;
	      }
	}
      DLinkCursor = DLinkCursor->LinkDown;
    }

  *DLink = MiniLink;
  return AllOk;
}


  
