
#include <stdlib.h>
#include <math.h>
#include <gnome.h>
#include <gdk/gdkrgb.h>

#include "xrmtypes.h"
#include "circuitviewwindow.h"

#include "interface.h"
#include "liste.h"
#include "mainwindow.h"
#include "popupmenu.h"
#include "attachpoint.h"
#include "component.h"
#include "subfunction.h"
#include "link.h" 
#include "dlink.h"
#include "layout.h"

GtkWidget *create_FinalCircuitViewWindow(void)
{
  GtkWidget *Window, *CircuitDraw, *HRuler, *VRuler, *Table;
  GCD *CircuitData;
  Header  DataHeader;
  int HSize, VSize;

  Window = create_CircuitViewWindow();
  CircuitData = GetCircuitData();
 
  HRuler = gtk_object_get_data(GTK_OBJECT(Window),"HRuler");
  VRuler = gtk_object_get_data(GTK_OBJECT(Window),"VRuler");
  Table = gtk_object_get_data(GTK_OBJECT(Window),"Table");

  CircuitDraw = gtk_drawing_area_new ();
  gtk_widget_set_events (CircuitDraw, 
			 GDK_EXPOSURE_MASK
			 | GDK_LEAVE_NOTIFY_MASK
			 | GDK_BUTTON_PRESS_MASK
			 | GDK_BUTTON_RELEASE_MASK
			 | GDK_POINTER_MOTION_MASK
			 | GDK_POINTER_MOTION_HINT_MASK);

  gtk_widget_set_name (CircuitDraw, "CircuitDraw");
  gtk_object_set_data (GTK_OBJECT (Window), "CircuitDraw", CircuitDraw);

  gtk_table_attach(GTK_TABLE (Table), CircuitDraw, 1, 2, 1, 2, (GtkAttachOptions)  (GTK_EXPAND | GTK_FILL),
		   (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
  
  gtk_widget_show (CircuitDraw);

  CircuitData->Info.CVPopupMenu = create_CVPopupMenu();
  
  gtk_signal_connect (GTK_OBJECT (CircuitDraw), "configure_event",
                      GTK_SIGNAL_FUNC (on_CircuitDraw_configure_event),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (CircuitDraw), "expose_event",
                      GTK_SIGNAL_FUNC (on_CircuitDraw_expose_event),
                      NULL);
  gtk_signal_connect_object (GTK_OBJECT (CircuitDraw), "button_press_event",
			     GTK_SIGNAL_FUNC (ButtonAction),
			     NULL);
  gtk_signal_connect (GTK_OBJECT (CircuitDraw), "button_release_event",
                      GTK_SIGNAL_FUNC (on_CircuitDraw_button_release_notify_event),
                      NULL);

   gtk_signal_connect (GTK_OBJECT (CircuitDraw), "leave_notify_event",
                      GTK_SIGNAL_FUNC (on_CircuitDraw_leave_notify_event),
                      NULL);
   gtk_signal_connect (GTK_OBJECT (CircuitDraw), "motion_notify_event",
                      GTK_SIGNAL_FUNC (on_CircuitDraw_motion_notify_event),
                      NULL);
  CircuitData->Info.BufferWidth = CircuitDraw->allocation.width;
  CircuitData->Info.BufferHeight = CircuitDraw->allocation.height;
  HSize = CircuitData->Info.BufferWidth / CircuitData->Info.Opt.ScaleFactor;
  VSize = CircuitData->Info.BufferHeight / CircuitData->Info.Opt.ScaleFactor;
  gtk_ruler_set_range( GTK_RULER(HRuler), 0, DataHeader.Width, 0, HSize);

  gtk_signal_connect_object(GTK_OBJECT(CircuitDraw), "motion_notify_event",
			    (GtkSignalFunc) EVENT_METHOD(HRuler, motion_notify_event),
			    GTK_OBJECT(HRuler));
  gtk_ruler_set_range( GTK_RULER(VRuler), 0, DataHeader.Height, 0, VSize);
  gtk_signal_connect_object(GTK_OBJECT(CircuitDraw), "motion_notify_event",
			    (GtkSignalFunc) EVENT_METHOD(VRuler, motion_notify_event),
			    GTK_OBJECT(VRuler));

  return  Window;
}
void InitCircuitViewMask(GCD *CircuitData, GtkWidget *Window)
{
  GtkWidget  *BetweenCheck, *InteriorCheck, *ExteriorCheck, *SubFunctionCheck, *ComponentCheck, *AttachCheck, 
    *InvisibleCheck, *LinkCheck;
  BetweenCheck = gtk_object_get_data(GTK_OBJECT(Window),"BetweenCheck");
  InteriorCheck = gtk_object_get_data(GTK_OBJECT(Window),"InteriorCheck");
  ExteriorCheck  = gtk_object_get_data(GTK_OBJECT(Window),"ExteriorCheck");
  SubFunctionCheck = gtk_object_get_data(GTK_OBJECT(Window),"SubFunctionCheck");
  ComponentCheck = gtk_object_get_data(GTK_OBJECT(Window),"ComponentCheck");
  AttachCheck = gtk_object_get_data(GTK_OBJECT(Window),"AttachCheck");
  InvisibleCheck = gtk_object_get_data(GTK_OBJECT(Window),"InvisibleCheck");
  LinkCheck = gtk_object_get_data(GTK_OBJECT(Window),"LinkCheck");
  
  if (IsOnBetweenPins(CircuitData->Info.Opt.CircuitMask))
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(BetweenCheck), TRUE);
  if (IsOnCompoIn(CircuitData->Info.Opt.CircuitMask))
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(InteriorCheck), TRUE);
  if (IsOnCompoOut(CircuitData->Info.Opt.CircuitMask))
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ExteriorCheck), TRUE);
  if (IsOnSubFunction(CircuitData->Info.Opt.CircuitMask))
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(SubFunctionCheck), TRUE);
  if (IsOnComponent(CircuitData->Info.Opt.CircuitMask))
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ComponentCheck), TRUE);
  if (IsOnAttachPoint(CircuitData->Info.Opt.CircuitMask))
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(AttachCheck), TRUE);
  if (IsOnInvisible(CircuitData->Info.Opt.CircuitMask))
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(InvisibleCheck), TRUE);
  if (IsOnLink(CircuitData->Info.Opt.CircuitMask))
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(LinkCheck), TRUE);

}
gboolean on_CircuitDraw_configure_event (GtkWidget *CircuitDraw, GdkEventConfigure *Event)
{
  int Counter, HStart, VStart, HSize, VSize;
  GtkWidget *HRuler, *VRuler;
  guchar  *CVBackupBuffer;
  GCD *CircuitData;
  
  if ((CircuitDraw == NULL) || (CircuitDraw->window == NULL))
    {
      printf("CircuitDraw == NULL\n");
      return TRUE;
    }
  CircuitData = GetCircuitData();
  
  CircuitData->Info.BufferWidth = CircuitDraw->allocation.width;
  CircuitData->Info.BufferHeight = CircuitDraw->allocation.height;

  CVBackupBuffer = CircuitData->Info.CVBackupBuffer;
  if (CVBackupBuffer != NULL)
    free(CVBackupBuffer);
  CVBackupBuffer = (guchar *) malloc(CircuitData->Info.BufferWidth*CircuitData->Info.BufferHeight*3);
  
  CircuitData->Info.CVBackupBuffer = CVBackupBuffer;
  for (Counter = 0 ; Counter < CircuitData->Info.BufferWidth * CircuitData->Info.BufferHeight * 3 ; Counter ++)
    CVBackupBuffer[Counter] = (guchar) BlackRGB;

  HSize = CircuitData->Info.BufferWidth / CircuitData->Info.Opt.ScaleFactor;
  VSize = CircuitData->Info.BufferHeight / CircuitData->Info.Opt.ScaleFactor;
  HStart = CircuitData->Info.CVOrigin.x;
  VStart = CircuitData->Info.CVOrigin.y;
  HRuler = gtk_object_get_data(GTK_OBJECT(CircuitData->Info.CircuitViewWindow),"HRuler");
  VRuler = gtk_object_get_data(GTK_OBJECT(CircuitData->Info.CircuitViewWindow),"VRuler");
  gtk_ruler_set_range( GTK_RULER(HRuler), HStart, HStart + HSize, 0, CircuitData->Info.BufferWidth);
  gtk_ruler_set_range( GTK_RULER(VRuler), VStart, VStart + VSize, 0, CircuitData->Info.BufferHeight);
  UpdateCircuitView(UCVDemand); 

  return TRUE;
}
gboolean
on_CircuitDraw_expose_event            (GtkWidget       *CircuitDraw,
                                        GdkEventExpose  *event,
                                        gpointer         user_data)
{
  GCD *CircuitData;
  guchar *CVBackupBuffer;

  CircuitData = GetCircuitData();
  CVBackupBuffer = CircuitData->Info.CVBackupBuffer;
  
  if ((CircuitDraw->window != NULL) && (CVBackupBuffer != NULL))
    gdk_draw_rgb_image (CircuitDraw->window,
			CircuitDraw->style->white_gc,
			event->area.x, event->area.y,
			event->area.width, event->area.height,
			GDK_RGB_DITHER_NONE,
			CVBackupBuffer + event->area.x  * 3 +  CircuitData->Info.BufferWidth * 3 * event->area.y,
			CircuitData->Info.BufferWidth * 3);

  return FALSE;
}

gboolean on_CircuitDraw_motion_notify_event     (GtkWidget       *CircuitDraw,
						 GdkEventMotion  *Event,
						 gpointer         user_data)
{
  int XPos, YPos, ID, HSize, VSize, HStart, VStart, xlocal, ylocal;
  GtkWidget *HRuler, *VRuler; 
  GdkModifierType State;
  ErrorCheck Check = NotFound;
  GCD *CircuitData;
 
  CircuitData = GetCircuitData();
  if (Event->is_hint)
    {
      gdk_window_get_pointer (Event->window, &XPos, &YPos, &State);
      XPos /= CircuitData->Info.Opt.ScaleFactor;
      YPos /=  CircuitData->Info.Opt.ScaleFactor;
      xlocal = XPos + CircuitData->Info.CVOrigin.x;
      ylocal = YPos + CircuitData->Info.CVOrigin.y;
      CircuitData->Info.PointerLocation.x = xlocal;
      CircuitData->Info.PointerLocation.y = ylocal;
      
      if (CircuitData->Info.ButtonAction == TranslateStart)
	{
	  
	  if ( (CircuitData->Info.TranslateStart.x != XPos) 
	       ||(CircuitData->Info.TranslateStart.y != YPos))
	    {
	      
	      CircuitData->Info.CVOrigin.x = CircuitData->Info.CVOriginStart.x - XPos 
		+ CircuitData->Info.TranslateStart.x;
	      CircuitData->Info.CVOrigin.y = CircuitData->Info.CVOriginStart.y - YPos
		+ CircuitData->Info.TranslateStart.y;
	      
	      CircuitData->Info.CVOriginStart.x = CircuitData->Info.CVOrigin.x;
	      CircuitData->Info.CVOriginStart.y = CircuitData->Info.CVOrigin.y;
	      CircuitData->Info.TranslateStart.x =  XPos;
	      CircuitData->Info.TranslateStart.y =  YPos;

	      HRuler = gtk_object_get_data(GTK_OBJECT(CircuitData->Info.CircuitViewWindow), "HRuler"); 
	      VRuler = gtk_object_get_data(GTK_OBJECT(CircuitData->Info.CircuitViewWindow), "VRuler"); 
	      
	      HSize = CircuitData->Info.BufferWidth / CircuitData->Info.Opt.ScaleFactor;
	      VSize = CircuitData->Info.BufferHeight / CircuitData->Info.Opt.ScaleFactor;
	      HStart = CircuitData->Info.CVOrigin.x;
	      VStart = CircuitData->Info.CVOrigin.y;
	      gtk_ruler_set_range( GTK_RULER(HRuler), HStart, HStart + HSize, 0, CircuitData->Info.BufferWidth);
	      gtk_ruler_set_range( GTK_RULER(VRuler), VStart, VStart + VSize, 0, CircuitData->Info.BufferHeight);

	      UpdateCircuitView(UCVModification);
	    }
	  return TRUE;
	}
      else
	{
	  if (CircuitData->Info.Opt.SelectionFollowMouse == True)
	    {
	      Check = GetIDFromCoordinates(xlocal, ylocal, AttachIDRef, &ID, True);
	      UpdateCircuitView(UCVSelection);
	    }
	  else
	    Check = GetIDFromCoordinates(xlocal, ylocal, AttachIDRef, &ID, False);
	  if (Check == Found)
	    UpdateCVAppBarFromID(ID, xlocal, ylocal);
	  else
	    UpdateCVAppBarFromID(NoID, xlocal, ylocal);
	}
    }
  return TRUE;
}

gboolean on_CircuitDraw_leave_notify_event (GtkWidget *CircuitDraw, GdkEventMotion *Event, gpointer user_data)
{
  GCD *CircuitData;

  CircuitData = GetCircuitData();
  if (CircuitData->Info.ButtonAction == TranslateStart)
    CircuitData->Info.ButtonAction = TranslateAction;
  return TRUE;
}
gboolean on_CircuitDraw_button_release_notify_event(GtkWidget *CircuitDraw, GdkEventButton *Event, gpointer user_data) 
{
  GCD *CircuitData;

  CircuitData = GetCircuitData();
  if ((Event->type == GDK_BUTTON_RELEASE) && (Event->button == 1) && 
      (CircuitData->Info.ButtonAction == TranslateStart))
    CircuitData->Info.ButtonAction = TranslateAction;

  return TRUE;
}


void UpdateCVAppBarFromID(int ID, int x, int y)
{
  GtkWidget *CircuitViewWindow, *AppBar;
  GCD *CircuitData;
  AttachPoint AttachProp;
  Component CompProp;
  SubFunction SubFProp;
  char Buffer[BufferLength*3];
  double XCoord, YCoord;
  
  CircuitData = GetCircuitData();
  if (ID != NoID)
    {
      GetAttachPointProperties(CircuitData, ID, &AttachProp);
      GetComponentProperties(CircuitData, AttachProp.ComponentID, &CompProp, False);
      GetSubFunctionProperties(CircuitData, CompProp.SubFunctionID, &SubFProp);
      if (CircuitData->Info.Opt.Millimeter == True)
	{
	  XCoord = SquareToMillimeter(x);
	  YCoord = SquareToMillimeter(y);
	  sprintf(Buffer, "%4d %4d : %6.2f mm %6.2f mm : %s - %s - %s", x, y, XCoord, YCoord, SubFProp.Name, CompProp.ComponentName, AttachProp.AttachPointName);
	}
      else
	{
	  XCoord = SquareToInch(x);
	  YCoord = SquareToInch(y);
	  sprintf(Buffer, "%4d %4d : %6.2f in %6.2f in : %s - %s - %s", x, y, XCoord, YCoord, SubFProp.Name, CompProp.ComponentName, AttachProp.AttachPointName);
	}
    }
  else
    {
      if (CircuitData->Info.Opt.Millimeter == True)
	{
	  XCoord = SquareToMillimeter(x);
	  YCoord = SquareToMillimeter(y);
	  sprintf(Buffer, "%4d %4d : %6.2f mm %6.2f mm :", x, y, XCoord, YCoord);
	}
      else
	{
	  XCoord = SquareToInch(x);
	  YCoord = SquareToInch(y);
	  sprintf(Buffer, "%4d %4d : %6.2f in %6.2f in :", x, y, XCoord, YCoord);
	}
    }

  CircuitViewWindow = CircuitData->Info.CircuitViewWindow;
  AppBar = gtk_object_get_data(GTK_OBJECT(CircuitViewWindow), "AppBar");
  
  gnome_appbar_set_status(GNOME_APPBAR(AppBar), Buffer);
}

ErrorCheck InitCircuitView(GCD *CircuitData)
{
  int HStart, VStart, HSize, VSize;
  GtkWidget *CircuitView, *HRuler, *VRuler, *CircuitDraw;
  
  CircuitView = CircuitData->Info.CircuitViewWindow;
  CircuitDraw = gtk_object_get_data(GTK_OBJECT(CircuitView), "CircuitDraw"); 
  if ((CircuitData->Info.CVBackupBuffer == NULL) || (CircuitDraw->window == NULL))
    {
      if (CircuitData->Info.CVBackupBuffer == NULL)
	printf("Buffer == NULL\n");
      if (CircuitDraw->window == NULL)
	printf("Window == NULL\n");
      return Exception; /* The buffer is not created yet ! */
    }
  
  HRuler = gtk_object_get_data(GTK_OBJECT(CircuitView), "HRuler"); 
  VRuler = gtk_object_get_data(GTK_OBJECT(CircuitView), "VRuler"); 
  CircuitDraw = gtk_object_get_data(GTK_OBJECT(CircuitView), "CircuitDraw"); 

  HSize = CircuitData->Info.BufferWidth / CircuitData->Info.Opt.ScaleFactor;
  VSize = CircuitData->Info.BufferHeight / CircuitData->Info.Opt.ScaleFactor;
  HStart = CircuitData->Info.CVOrigin.x;
  VStart = CircuitData->Info.CVOrigin.y;
  gtk_ruler_set_range( GTK_RULER(HRuler), HStart, HStart + HSize, 0, CircuitData->Info.BufferWidth);
  gtk_ruler_set_range( GTK_RULER(VRuler), VStart, VStart + VSize, 0, CircuitData->Info.BufferHeight);

  ClearCVBuffer(CircuitData, CircuitDraw);
  return AllOk;
}


void UpdateCircuitView(UCVEvent Event)
{
  GdkGC *GC = NULL;
  GtkWidget *CircuitDraw;
  guchar *CVBuffer, *CVBackupBuffer;
  AttachPoint *AttachCursor;
  Component CompProp, *CompCursor;
  SubFunction SubFProp, *SubFCursor;
  Boolean AttachFound, AlternateSelected;
  DrawLink *DLinkCursor;
  Link LinkProp, *LinkCursor;
  int Counter, x, y, Width, Height, Deltax, Deltay, *AlternateAttach = NULL, AlternateSize, AttachCounter,
    xCounter, yCounter, Pix_W, Pix_H, Pix_x, Pix_y, BufferWidth, OriginX, OriginY, Newx, Newy;
  DrawLink DLinkProp;
  PaletteIndex Index = SpotPalette, InteriorIndex, ExteriorIndex, BetweenIndex;
  ErrorCheck Check;
  LayoutDef LayoutProp;
  GCD *CircuitData;
  
  CircuitData = GetCircuitData();
  if ( ! ((CircuitData->Info.Opt.UpdateContinually == True) || (Event == UCVDemand) 
	  || ((CircuitData->Info.Opt.UpdateClicked == True) && (Event == UCVClick))))
    return;
  Check = InitCircuitView(CircuitData);
  if (Check == Exception)
    return;
  
  printf("Updating\n");
  if (CircuitData->Info.Opt.ScaleFactor > 1)
    {
      Pix_W = CircuitData->Info.Opt.ScaleFactor;
      Pix_H = CircuitData->Info.Opt.ScaleFactor;
    }
  else
    {
      Pix_W = 1;
      Pix_H = 1;
    }
  BufferWidth = CircuitData->Info.BufferWidth;

  CircuitDraw = gtk_object_get_data(GTK_OBJECT(CircuitData->Info.CircuitViewWindow), "CircuitDraw"); 
  if (GC == NULL)
    GC = gdk_gc_new(CircuitDraw->window);

  Width = CircuitData->Info.BufferWidth / CircuitData->Info.Opt.ScaleFactor;
  Height = CircuitData->Info.BufferHeight / CircuitData->Info.Opt.ScaleFactor;
  OriginX = CircuitData->Info.CVOrigin.x;
  OriginY = CircuitData->Info.CVOrigin.y;
  CVBuffer = CircuitData->Info.CVBuffer;

  Index = AlternatePalette; /* Draw circuit border */
  for (Counter = 0 ; Counter <= CircuitData->DataHeader.Width ; Counter++)
    {
      x = Counter - OriginX;
      y = CircuitData->DataHeader.Height - OriginY;
      if ( (x >= 0) && (y >= 0) && (x < Width) && (y < Height))
	{
	  Pix_x = (int) floor(x * CircuitData->Info.Opt.ScaleFactor);
	  Pix_y = (int) floor(y * CircuitData->Info.Opt.ScaleFactor);
	  for (xCounter = Pix_x ; xCounter < Pix_x + Pix_W ; xCounter++)
	    for (yCounter = Pix_y ; yCounter < Pix_y + Pix_H; yCounter++)
	      {
		BufferTabR(CVBuffer, xCounter, yCounter, BufferWidth) =  
		  CircuitData->Info.Opt.ColorPalette[Index].Red;
		BufferTabG(CVBuffer, xCounter, yCounter, BufferWidth) =  
		  CircuitData->Info.Opt.ColorPalette[Index].Green;
		BufferTabB(CVBuffer, xCounter, yCounter, BufferWidth) = 
		  CircuitData->Info.Opt.ColorPalette[Index].Blue;
	      }
	}
      y = 0 - OriginY;
      if ( (x >= 0) && (y >= 0) && (x < Width) && (y < Height))
	{
	  Pix_x = (int) floor(x * CircuitData->Info.Opt.ScaleFactor);
	  Pix_y = (int) floor(y * CircuitData->Info.Opt.ScaleFactor);
	  for (xCounter = Pix_x ; xCounter < Pix_x + Pix_W ; xCounter++)
	    for (yCounter = Pix_y ; yCounter < Pix_y + Pix_H; yCounter++)
	      {
		BufferTabR(CVBuffer, xCounter, yCounter, BufferWidth) =  
		  CircuitData->Info.Opt.ColorPalette[Index].Red;
		BufferTabG(CVBuffer, xCounter, yCounter, BufferWidth) =  
		  CircuitData->Info.Opt.ColorPalette[Index].Green;
		BufferTabB(CVBuffer, xCounter, yCounter, BufferWidth) = 
		  CircuitData->Info.Opt.ColorPalette[Index].Blue;
	      }
	}
    }
  for (Counter = 0 ; Counter <= CircuitData->DataHeader.Height ; Counter++)
    {
      x = CircuitData->DataHeader.Width - OriginX;
      y = Counter - OriginY;
      if ( (x >= 0) && (y >= 0) && (x < Width) && (y < Height))
	{
	  Pix_x = (int) floor(x * CircuitData->Info.Opt.ScaleFactor);
	  Pix_y = (int) floor(y * CircuitData->Info.Opt.ScaleFactor);
	  for (xCounter = Pix_x ; xCounter < Pix_x + Pix_W ; xCounter++)
	    for (yCounter = Pix_y ; yCounter < Pix_y + Pix_H; yCounter++)
	      {
		BufferTabR(CVBuffer, xCounter, yCounter, BufferWidth) =  
		  CircuitData->Info.Opt.ColorPalette[Index].Red;
		BufferTabG(CVBuffer, xCounter, yCounter, BufferWidth) =  
		  CircuitData->Info.Opt.ColorPalette[Index].Green;
		BufferTabB(CVBuffer, xCounter, yCounter, BufferWidth) = 
		  CircuitData->Info.Opt.ColorPalette[Index].Blue;
	      }
	}
      x = 0 - OriginX;
      if ( (x >= 0) && (y >= 0) && (x < Width) && (y < Height))
	{
	  Pix_x = (int) floor(x * CircuitData->Info.Opt.ScaleFactor);
	  Pix_y = (int) floor(y * CircuitData->Info.Opt.ScaleFactor);
	  for (xCounter = Pix_x ; xCounter < Pix_x + Pix_W ; xCounter++)
	    for (yCounter = Pix_y ; yCounter < Pix_y + Pix_H; yCounter++)
	      {
		BufferTabR(CVBuffer, xCounter, yCounter, BufferWidth) =  
		  CircuitData->Info.Opt.ColorPalette[Index].Red;
		BufferTabG(CVBuffer, xCounter, yCounter, BufferWidth) =  
		  CircuitData->Info.Opt.ColorPalette[Index].Green;
		BufferTabB(CVBuffer, xCounter, yCounter, BufferWidth) = 
		  CircuitData->Info.Opt.ColorPalette[Index].Blue;
	      }
	}
    }
  

  /* SubFunction */
  SubFCursor = CircuitData->ListSubFunction;
  while ((SubFCursor != NULL) && (IsOnSubFunction(CircuitData->Info.Opt.CircuitMask)))
    {
      x = SubFCursor->Reference.x - OriginX;
      y = SubFCursor->Reference.y - OriginY;
	  
      if ( (x >= 0) && (y >= 0) && (x < Width) && (y < Height))
	{
	  if (IsSubFSelected(SubFCursor->State))
	    Index = SubFunctionPalette;
	  else
	    Index = SubFunctionUnSelectedPalette;
	  
	  Pix_x = (int) floor(x * CircuitData->Info.Opt.ScaleFactor);
	  Pix_y = (int) floor(y * CircuitData->Info.Opt.ScaleFactor);
	  for (xCounter = Pix_x ; xCounter < Pix_x + Pix_W ; xCounter++)
	    for (yCounter = Pix_y ; yCounter < Pix_y + Pix_H; yCounter++)
	      {
		BufferTabR(CVBuffer, xCounter, yCounter, BufferWidth) =  
		  CircuitData->Info.Opt.ColorPalette[Index].Red;
		BufferTabG(CVBuffer, xCounter, yCounter, BufferWidth) =  
		  CircuitData->Info.Opt.ColorPalette[Index].Green;
		BufferTabB(CVBuffer, xCounter, yCounter, BufferWidth) = 
		  CircuitData->Info.Opt.ColorPalette[Index].Blue;
	      }
	}
      SubFCursor = SubFCursor->pNext;
    }
  /* Component */
  CompCursor = CircuitData->ListComponent;
  while ((CompCursor != NULL) && (IsOnComponent(CircuitData->Info.Opt.CircuitMask)))
    {
      GetSubFunctionProperties(CircuitData, CompCursor->SubFunctionID, &SubFProp);
      x = SubFProp.Reference.x + CompCursor->Reference.x - OriginX;
      y = SubFProp.Reference.y + CompCursor->Reference.y - OriginY;
	  
      if ( (x >= 0) && (y >= 0) && (x < Width) && (y < Height))
	{
	  if (IsComponentSelected(CompCursor->State))
	    Index = ComponentPalette;
	  else
	    Index = ComponentUnSelectedPalette;

	  Pix_x = (int) floor(x * CircuitData->Info.Opt.ScaleFactor);
	  Pix_y = (int) floor(y * CircuitData->Info.Opt.ScaleFactor);
	  for (xCounter = Pix_x ; xCounter < Pix_x + Pix_W ; xCounter++)
	    for (yCounter = Pix_y ; yCounter < Pix_y + Pix_H; yCounter++)
	      {
		BufferTabR(CVBuffer, xCounter, yCounter, BufferWidth) = 
		  CircuitData->Info.Opt.ColorPalette[Index].Red;
		BufferTabG(CVBuffer, xCounter, yCounter, BufferWidth) = 
		  CircuitData->Info.Opt.ColorPalette[Index].Green;
		BufferTabB(CVBuffer, xCounter, yCounter, BufferWidth) = 
		  CircuitData->Info.Opt.ColorPalette[Index].Blue;
	      }
	}
      
      GetLayoutProperties(CircuitData->CompLib, CompCursor->LayoutName, CompCursor->LayoutClass, &LayoutProp,
			  False, False);
      if (LayoutProp.BetweenPinsSize != 0)
	LayoutProp.BetweenPinsPosition = (Point *) malloc(sizeof(Point) * LayoutProp.BetweenPinsSize);
      if (LayoutProp.CompoInSize != 0)
	    LayoutProp.CompoInPosition = (Point *) malloc(sizeof(Point) * LayoutProp.CompoInSize);
      if (LayoutProp.CompoOutSize != 0)
	LayoutProp.CompoOutPosition = (Point *) malloc(sizeof(Point) * LayoutProp.CompoOutSize);
      
      GetLayoutProperties(CircuitData->CompLib, CompCursor->LayoutName, CompCursor->LayoutClass, &LayoutProp,
			  False, True);
      if (IsComponentSelected(CompCursor->State))
	{
	  BetweenIndex = BetweenPinsPalette;
	  InteriorIndex = CompoInPalette;
	  ExteriorIndex = CompoOutPalette;
	}	  
      else
	{
	  BetweenIndex = BetweenPinsUnSelectedPalette;
	  InteriorIndex =  CompoInUnSelectedPalette;
	  ExteriorIndex = CompoOutUnSelectedPalette;
	}

      if (IsOnBetweenPins(CircuitData->Info.Opt.CircuitMask))
	for (Counter = 0 ; Counter < LayoutProp.BetweenPinsSize  ; Counter++)
	  {
	    Newx = x + LayoutProp.BetweenPinsPosition[Counter].x;
	    Newy = y + LayoutProp.BetweenPinsPosition[Counter].y;
	    
	    if ( (Newx >= 0) && (Newy >= 0) && (Newx < Width) && (Newy < Height))
	      {
	      Pix_x = (int) floor(Newx * CircuitData->Info.Opt.ScaleFactor);
	      Pix_y = (int) floor(Newy * CircuitData->Info.Opt.ScaleFactor);
	      for (xCounter = Pix_x ; xCounter < Pix_x + Pix_W ; xCounter++)
		for (yCounter = Pix_y ; yCounter < Pix_y + Pix_H; yCounter++)
		  {
		    BufferTabR(CVBuffer, xCounter, yCounter, BufferWidth) = 
		      CircuitData->Info.Opt.ColorPalette[BetweenIndex].Red;
		    BufferTabG(CVBuffer, xCounter, yCounter, BufferWidth) = 
		      CircuitData->Info.Opt.ColorPalette[BetweenIndex].Green;
		    BufferTabB(CVBuffer, xCounter, yCounter, BufferWidth) = 
		      CircuitData->Info.Opt.ColorPalette[BetweenIndex].Blue;
		  }
	      }
	  }
      if (IsOnCompoIn(CircuitData->Info.Opt.CircuitMask))
	for (Counter = 0 ; Counter < LayoutProp.CompoInSize  ; Counter++)
	  {
	    Newx = x + LayoutProp.CompoInPosition[Counter].x;
	    Newy = y + LayoutProp.CompoInPosition[Counter].y;
	    
	    if ( (Newx >= 0) && (Newy >= 0) && (Newx < Width) && (Newy < Height))
	      {
		Pix_x = (int) floor(Newx * CircuitData->Info.Opt.ScaleFactor);
		Pix_y = (int) floor(Newy * CircuitData->Info.Opt.ScaleFactor);
		for (xCounter = Pix_x ; xCounter < Pix_x + Pix_W ; xCounter++)
		  for (yCounter = Pix_y ; yCounter < Pix_y + Pix_H; yCounter++)
		    {
		      BufferTabR(CVBuffer, xCounter, yCounter, BufferWidth) = 
			CircuitData->Info.Opt.ColorPalette[InteriorIndex].Red;
		      BufferTabG(CVBuffer, xCounter, yCounter, BufferWidth) = 
			CircuitData->Info.Opt.ColorPalette[InteriorIndex].Green;
		      BufferTabB(CVBuffer, xCounter, yCounter, BufferWidth) = 
			CircuitData->Info.Opt.ColorPalette[InteriorIndex].Blue;
		    }
	      }
	  }
      if (IsOnCompoOut(CircuitData->Info.Opt.CircuitMask))
	for (Counter = 0 ; Counter < LayoutProp.CompoOutSize  ; Counter++)
	  {
	    Newx = x + LayoutProp.CompoOutPosition[Counter].x;
	    Newy = y + LayoutProp.CompoOutPosition[Counter].y;
	    
	    if ( (Newx >= 0) && (Newy >= 0) && (Newx < Width) && (Newy < Height))
	      {
		Pix_x = (int) floor(Newx * CircuitData->Info.Opt.ScaleFactor);
		Pix_y = (int) floor(Newy * CircuitData->Info.Opt.ScaleFactor);
		for (xCounter = Pix_x ; xCounter < Pix_x + Pix_W ; xCounter++)
		  for (yCounter = Pix_y ; yCounter < Pix_y + Pix_H; yCounter++)
		    {
		      BufferTabR(CVBuffer, xCounter, yCounter, BufferWidth) = 
			CircuitData->Info.Opt.ColorPalette[ExteriorIndex].Red;
		      BufferTabG(CVBuffer, xCounter, yCounter, BufferWidth) = 
			CircuitData->Info.Opt.ColorPalette[ExteriorIndex].Green;
		      BufferTabB(CVBuffer, xCounter, yCounter, BufferWidth) = 
			CircuitData->Info.Opt.ColorPalette[ExteriorIndex].Blue;
		    }
	      }
	  }
	    
      if (LayoutProp.BetweenPinsSize != 0)
	free(LayoutProp.BetweenPinsPosition);
      if (LayoutProp.CompoInSize != 0)
	free(LayoutProp.CompoInPosition);
      if (LayoutProp.CompoOutSize != 0)
	free(LayoutProp.CompoOutPosition);
      CompCursor = CompCursor->pNext;
    }
  
  /******** Attach points alternate or not */
  AlternateSize = 0;
  AttachCursor = CircuitData->ListAttachPoint;
  while (AttachCursor != NULL)
    {
      AlternateSize++;
      AttachCursor = AttachCursor->pNext;
    }
  if (AlternateSize != 0)
     AlternateAttach = (int *) malloc(sizeof(int) * AlternateSize);
  
  AlternateSize = 0;
  DLinkCursor = CircuitData->TopLink;
  while (DLinkCursor != NULL) 
    {
      if (IsDLinkSelected(DLinkCursor->State))
	{
	  GetLinkProperties(CircuitData, DLinkCursor->LinkID, &LinkProp);

	  AttachFound = False;
	  for (Counter = 0 ; (AttachFound == False) && (Counter < AlternateSize) ; Counter++)
	    if (AlternateAttach[Counter] == LinkProp.StartAttachID)
	      AttachFound = True;
	  if (AttachFound == False)
	    {
	      AlternateAttach[AlternateSize] = LinkProp.StartAttachID;
	      AlternateSize++;
	    }

	  AttachFound = False;
	  for (Counter = 0 ; (AttachFound == False) && (Counter < AlternateSize) ; Counter++)
	    if (AlternateAttach[Counter] == LinkProp.EndAttachID)
	      AttachFound = True;

	  if (AttachFound == False)
	    {
	      AlternateAttach[AlternateSize] = LinkProp.EndAttachID;
	      AlternateSize++;
	    }
	}
      DLinkCursor = DLinkCursor->LinkDown;
    }
  
  AttachCursor = CircuitData->ListAttachPoint;
  while ((AttachCursor != NULL) && (IsOnAttachPoint(CircuitData->Info.Opt.CircuitMask)))
  {
      GetComponentProperties(CircuitData, AttachCursor->ComponentID, &CompProp, False);
      GetSubFunctionProperties(CircuitData, CompProp.SubFunctionID, &SubFProp);
      Deltax = SubFProp.Reference.x + CompProp.Reference.x + AttachCursor->Reference.x - OriginX;
      Deltay = SubFProp.Reference.y + CompProp.Reference.y + AttachCursor->Reference.y - OriginY;
      for (Counter = 0 ; Counter < AttachCursor->SpotSize ; Counter++)
	{
	  x = AttachCursor->SpotLocation[Counter].x + Deltax;
	  y = AttachCursor->SpotLocation[Counter].y + Deltay;
	  
	  AlternateSelected = False;
	  for(AttachCounter = 0 ; (AlternateSelected == False) && (AttachCounter < AlternateSize) ; AttachCounter++)
	    if (AlternateAttach[AttachCounter] ==  AttachCursor->IDNumber)
	      AlternateSelected = True;

	  if ( (x >= 0) && (y >= 0) && (x < Width) && (y < Height))
	    {

	      if (AlternateSelected == True)
		Index = AlternatePalette;
	      else
		if (IsAttachSelected(AttachCursor->State))
		  Index = SpotPalette;
		else
		  Index = SpotUnSelectedPalette;

	      Pix_x = (int) floor(x * CircuitData->Info.Opt.ScaleFactor);
	      Pix_y = (int) floor(y * CircuitData->Info.Opt.ScaleFactor);
	      for (xCounter = Pix_x ; xCounter < Pix_x + Pix_W ; xCounter++)
		for (yCounter = Pix_y ; yCounter < Pix_y + Pix_H; yCounter++)
		  {
		    BufferTabR(CVBuffer, xCounter, yCounter, BufferWidth) = 
		      CircuitData->Info.Opt.ColorPalette[Index].Red;
		    BufferTabG(CVBuffer, xCounter, yCounter, BufferWidth) = 
		      CircuitData->Info.Opt.ColorPalette[Index].Green;
		    BufferTabB(CVBuffer, xCounter, yCounter, BufferWidth) = 
		      CircuitData->Info.Opt.ColorPalette[Index].Blue;
		  }
	    }
	}
      for (Counter = 0 ; Counter < AttachCursor->CoreSpotSize ; Counter++)
	{
	  x = AttachCursor->CoreSpotLocation[Counter].x + Deltax;
	  y = AttachCursor->CoreSpotLocation[Counter].y + Deltay;
	  
	  AlternateSelected = False;
	  for(AttachCounter = 0 ; (AlternateSelected == False) && (AttachCounter < AlternateSize) ; AttachCounter++)
	    if (AlternateAttach[AttachCounter] ==  AttachCursor->IDNumber)
	      AlternateSelected = True;

	  if ( (x >= 0) && (y >= 0) && (x < Width) && (y < Height))
	    {
	      if (AlternateSelected == True)
		Index = AlternatePalette;
	      else
		if (IsAttachSelected(AttachCursor->State))
		  Index = CoreSpotPalette;
		else
		  Index = CoreSpotUnSelectedPalette;
	      
	      Pix_x = (int) floor(x * CircuitData->Info.Opt.ScaleFactor);
	      Pix_y = (int) floor(y * CircuitData->Info.Opt.ScaleFactor);
	      for (xCounter = Pix_x ; xCounter < Pix_x + Pix_W ; xCounter++)
		for (yCounter = Pix_y ; yCounter < Pix_y + Pix_H; yCounter++)
		  {
		    BufferTabR(CVBuffer, xCounter, yCounter, BufferWidth) = 
		      CircuitData->Info.Opt.ColorPalette[Index].Red;
		    BufferTabG(CVBuffer, xCounter, yCounter, BufferWidth) = 
		      CircuitData->Info.Opt.ColorPalette[Index].Green;
		    BufferTabB(CVBuffer, xCounter, yCounter, BufferWidth) = 
		      CircuitData->Info.Opt.ColorPalette[Index].Blue;
		  }
	    }
	}
      if (IsOnInvisible(CircuitData->Info.Opt.CircuitMask))
	for (Counter = 0 ; Counter < AttachCursor->InvisibleSpotSize ; Counter++)
	  {
	    x = AttachCursor->CoreSpotLocation[Counter].x + Deltax;
	    y = AttachCursor->CoreSpotLocation[Counter].y + Deltay;
	    
	    AlternateSelected = False;
	    for(AttachCounter = 0 ; (AlternateSelected == False) && (AttachCounter < AlternateSize) ; AttachCounter++)
	      if (AlternateAttach[AttachCounter] ==  AttachCursor->IDNumber)
		AlternateSelected = True;
	    
	    if ( (x >= 0) && (y >= 0) && (x < Width) && (y < Height))
	      {
		if (AlternateSelected == True)
		Index = AlternatePalette;
		else
		  if (IsAttachSelected(AttachCursor->State))
		    Index = InvisibleSpotPalette;
		  else
		    Index = InvisibleSpotUnSelectedPalette;
		Pix_x = (int) floor(x * CircuitData->Info.Opt.ScaleFactor);
		Pix_y = (int) floor(y * CircuitData->Info.Opt.ScaleFactor);
		for (xCounter = Pix_x ; xCounter < Pix_x + Pix_W ; xCounter++)
		  for (yCounter = Pix_y ; yCounter < Pix_y + Pix_H; yCounter++)
		    {
		      BufferTabR(CVBuffer, xCounter, yCounter, BufferWidth) = 
			CircuitData->Info.Opt.ColorPalette[Index].Red;
		      BufferTabG(CVBuffer, xCounter, yCounter, BufferWidth) = 
			CircuitData->Info.Opt.ColorPalette[Index].Green;
		      BufferTabB(CVBuffer, xCounter, yCounter, BufferWidth) = 
			CircuitData->Info.Opt.ColorPalette[Index].Blue;
		    }
	      }
	  }
      AttachCursor = AttachCursor->pNext;
    }
  
  if (AlternateAttach != NULL)
    free(AlternateAttach);
  /********* Links */
  
  LinkCursor = CircuitData->ListLink;
  while ((LinkCursor != NULL) && (IsOnLink(CircuitData->Info.Opt.CircuitMask)))
    {
      if (IsLinkRouted(LinkCursor->State))
	{
	  GetDLinkProperties(CircuitData, LinkCursor->IDNumber, &DLinkProp);
	  
	  if (IsDLinkSelected(DLinkProp.State))
	    Index = LinkPalette;
	  else
	    Index = LinkUnSelectedPalette;
	  for (Counter = 0 ; Counter < LinkCursor->Road.PathSize ; Counter++)
	    {
	      x = LinkCursor->Road.PathTab[Counter].x - OriginX;
	      y = LinkCursor->Road.PathTab[Counter].y - OriginY;
	      
	      if ( (x >= 0) && (y >= 0) && (x < Width) && (y < Height))
		{
		  Pix_x = (int) floor(x * CircuitData->Info.Opt.ScaleFactor);
		  Pix_y = (int) floor(y * CircuitData->Info.Opt.ScaleFactor);
		  for (xCounter = Pix_x ; xCounter < Pix_x + Pix_W ; xCounter++)
		    for (yCounter = Pix_y ; yCounter < Pix_y + Pix_H; yCounter++)
		      {
			BufferTabR(CVBuffer, xCounter, yCounter, BufferWidth) = 
			  CircuitData->Info.Opt.ColorPalette[Index].Red;
			BufferTabG(CVBuffer, xCounter, yCounter, BufferWidth) = 
			  CircuitData->Info.Opt.ColorPalette[Index].Green;
			BufferTabB(CVBuffer, xCounter, yCounter, BufferWidth) = 
			  CircuitData->Info.Opt.ColorPalette[Index].Blue;
		      }
		}		  
	    }
	}
      LinkCursor = LinkCursor->pNext;
    }

  CVBackupBuffer = CircuitData->Info.CVBackupBuffer;
  memcpy(CVBackupBuffer, CVBuffer, CircuitData->Info.BufferWidth * CircuitData->Info.BufferHeight * 3);
  gdk_draw_rgb_image (CircuitDraw->window,
		      CircuitDraw->style->white_gc,
		      0, 0, CircuitData->Info.BufferWidth, CircuitData->Info.BufferHeight,
		      GDK_RGB_DITHER_NONE,
		      CVBackupBuffer, CircuitData->Info.BufferWidth * 3);
  
  return ;
}

void ClearCVBuffer(GCD *CircuitData, GtkWidget *DrawArea)
{
  int Counter;
  
  if (CircuitData->Info.CVBuffer != NULL)
    free(CircuitData->Info.CVBuffer);
  
  if (DrawArea->window == NULL)
    {
      printf("DrawArea == NULL!\n");
      return;
    }
  
  CircuitData->Info.CVBuffer = (guchar *) malloc(CircuitData->Info.BufferWidth * CircuitData->Info.BufferHeight * 3);
  for (Counter = 0 ; Counter < CircuitData->Info.BufferWidth * CircuitData->Info.BufferHeight * 3 ; Counter++)
    CircuitData->Info.CVBuffer[Counter] = (guchar) BlackRGB;
}


void on_TranslateButton_toggled (GtkToggleButton *TranslateButton, GtkWidget *CircuitViewWindow)
{
  GtkWidget *ZoomInButton, *ZoomOutButton, *CircuitDraw;
  GdkCursor *Cursor;
  GCD *CircuitData;

  CircuitData = GetCircuitData();
  CircuitDraw = gtk_object_get_data(GTK_OBJECT(CircuitData->Info.CircuitViewWindow), "CircuitDraw"); 

  ZoomInButton = gtk_object_get_data(GTK_OBJECT(CircuitViewWindow), "ZoomInButton");
  ZoomOutButton = gtk_object_get_data(GTK_OBJECT(CircuitViewWindow), "ZoomOutButton");
  if (GTK_TOGGLE_BUTTON(TranslateButton)->active)
    {
      Cursor = gdk_cursor_new (TranslateCursor);
      gdk_window_set_cursor (CircuitDraw->window, Cursor);
      gdk_cursor_destroy (Cursor); 

      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ZoomInButton), FALSE);
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ZoomOutButton), FALSE);
      CircuitData->Info.ButtonAction = TranslateAction;
    }
  if ((!(GTK_TOGGLE_BUTTON(TranslateButton)->active)) && (!(GTK_TOGGLE_BUTTON(ZoomInButton)->active)) 
      &&(!(GTK_TOGGLE_BUTTON(ZoomOutButton)->active)))
    {
      Cursor = gdk_cursor_new (StandardCursor);
      gdk_window_set_cursor (CircuitDraw->window, Cursor);
      gdk_cursor_destroy (Cursor); 
      CircuitData->Info.ButtonAction = NoAction;  
    }
}
void on_ZoomInButton_toggled (GtkToggleButton *ZoomInButton, GtkWidget *CircuitViewWindow)
{
  GtkWidget *TranslateButton, *ZoomOutButton, *CircuitDraw;
  GdkCursor *Cursor;
  GCD *CircuitData;

  CircuitData = GetCircuitData();
  CircuitDraw = gtk_object_get_data(GTK_OBJECT(CircuitData->Info.CircuitViewWindow), "CircuitDraw"); 

  TranslateButton = gtk_object_get_data(GTK_OBJECT(CircuitViewWindow), "TranslateButton");
  ZoomOutButton = gtk_object_get_data(GTK_OBJECT(CircuitViewWindow), "ZoomOutButton");
  if (GTK_TOGGLE_BUTTON(ZoomInButton)->active)
    {
      Cursor = gdk_cursor_new (ZoomInCursor);
      gdk_window_set_cursor (CircuitDraw->window, Cursor);
      gdk_cursor_destroy (Cursor); 
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(TranslateButton), FALSE);
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ZoomOutButton), FALSE);
      CircuitData->Info.ButtonAction = ZoomInAction;
    }
  if ((!(GTK_TOGGLE_BUTTON(TranslateButton)->active)) && (!(GTK_TOGGLE_BUTTON(ZoomInButton)->active)) 
      &&(!(GTK_TOGGLE_BUTTON(ZoomOutButton)->active)))
    {
      Cursor = gdk_cursor_new (StandardCursor);
      gdk_window_set_cursor (CircuitDraw->window, Cursor);
      gdk_cursor_destroy (Cursor); 
      CircuitData->Info.ButtonAction = NoAction;  
    }
}
void on_ZoomOutButton_toggled (GtkToggleButton *ZoomOutButton, GtkWidget *CircuitViewWindow)
{
  GtkWidget *TranslateButton, *ZoomInButton, *CircuitDraw;
  GdkCursor *Cursor;
  GCD *CircuitData;

  CircuitData = GetCircuitData();
  CircuitDraw = gtk_object_get_data(GTK_OBJECT(CircuitData->Info.CircuitViewWindow), "CircuitDraw"); 

  TranslateButton = gtk_object_get_data(GTK_OBJECT(CircuitViewWindow), "TranslateButton");
  ZoomInButton = gtk_object_get_data(GTK_OBJECT(CircuitViewWindow), "ZoomInButton");
  if (GTK_TOGGLE_BUTTON(ZoomOutButton)->active)
    {
      Cursor = gdk_cursor_new (ZoomOutCursor);
      gdk_window_set_cursor (CircuitDraw->window, Cursor);
      gdk_cursor_destroy (Cursor); 
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(TranslateButton), FALSE);
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ZoomInButton), FALSE);
      CircuitData->Info.ButtonAction = ZoomOutAction;
    }
  if ((!(GTK_TOGGLE_BUTTON(TranslateButton)->active)) && (!(GTK_TOGGLE_BUTTON(ZoomInButton)->active)) 
      &&(!(GTK_TOGGLE_BUTTON(ZoomOutButton)->active)))
    {
      Cursor = gdk_cursor_new (StandardCursor);
      gdk_window_set_cursor (CircuitDraw->window, Cursor);
      gdk_cursor_destroy (Cursor); 
      CircuitData->Info.ButtonAction = NoAction;  
    }
}

gboolean ButtonAction(GtkWidget *CircuitDraw, GdkEventButton  *Event, gpointer user_data)
{
  GtkWidget *CircuitViewWindow, *TranslateButton, *ZoomInButton, *ZoomOutButton;
  int x, y, xlocal, ylocal, Width, Height, ID;
  GdkModifierType State;
  GCD *CircuitData;
  ErrorCheck Check;

  if (Event->type == GDK_BUTTON_PRESS) 
    {
      GdkEventButton *ButtonEvent = (GdkEventButton *) Event; 
      
      CircuitData = GetCircuitData();
      CircuitViewWindow = CircuitData->Info.CircuitViewWindow;

      TranslateButton = gtk_object_get_data(GTK_OBJECT(CircuitViewWindow), "TranslateButton");
      ZoomInButton = gtk_object_get_data(GTK_OBJECT(CircuitViewWindow), "ZoomInButton");
      ZoomOutButton = gtk_object_get_data(GTK_OBJECT(CircuitViewWindow), "ZoomOutButton");

      if (ButtonEvent->button == 2) /* middle button */
	{
	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(TranslateButton), FALSE);
	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ZoomInButton), FALSE);
	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ZoomOutButton), FALSE);
	  CircuitData->Info.ButtonAction = NoAction;
	  UpdateCircuitView(UCVModification);
	  return TRUE;
	}
      if (ButtonEvent->button == 3)                                 /* right button */
	{
 	  if (CircuitData->Info.ButtonAction == TranslateStart)
	    CircuitData->Info.ButtonAction = TranslateAction;
	  
	  gtk_menu_popup (GTK_MENU (CircuitData->Info.CVPopupMenu), NULL, NULL, NULL, NULL,
			  Event->button, Event->time);
	  
	}
      if (ButtonEvent->button == 1)
	{
	  gdk_window_get_pointer (ButtonEvent->window, &x, &y, &State);
	   x /= CircuitData->Info.Opt.ScaleFactor;
	   y /= CircuitData->Info.Opt.ScaleFactor;
	   xlocal = x + CircuitData->Info.CVOrigin.x;
	   ylocal = y + CircuitData->Info.CVOrigin.y;

	  if (CircuitData->Info.ButtonAction == TranslateAction)
	    {
	      CircuitData->Info.TranslateStart.x = x;
	      CircuitData->Info.TranslateStart.y = y;
	      CircuitData->Info.CVOriginStart.x = CircuitData->Info.CVOrigin.x;
	      CircuitData->Info.CVOriginStart.y = CircuitData->Info.CVOrigin.y;
	      CircuitData->Info.PointerLocation.x = xlocal;
	      CircuitData->Info.PointerLocation.y = ylocal;
	      CircuitData->Info.ButtonAction = TranslateStart;
	      UpdateCircuitView(UCVModification);
	      return TRUE;
	    }

	  if (CircuitData->Info.ButtonAction == ZoomInAction)
	    {
	      CircuitData->Info.Opt.ScaleFactor *= 2;
	      
	      Width = CircuitData->Info.BufferWidth / CircuitData->Info.Opt.ScaleFactor;
	      Height = CircuitData->Info.BufferHeight / CircuitData->Info.Opt.ScaleFactor;
	      CircuitData->Info.CVOrigin.x = xlocal - Width / 2;
	      CircuitData->Info.CVOrigin.y = ylocal - Height / 2;

	      UpdateCircuitView(UCVModification);
	      return TRUE;
	    }
	  if (CircuitData->Info.ButtonAction == ZoomOutAction)
	    {
	      CircuitData->Info.Opt.ScaleFactor /= 2;
	      
	      Width = CircuitData->Info.BufferWidth / CircuitData->Info.Opt.ScaleFactor;
	      Height = CircuitData->Info.BufferHeight / CircuitData->Info.Opt.ScaleFactor;
	      CircuitData->Info.CVOrigin.x = xlocal - Width / 2;
	      CircuitData->Info.CVOrigin.y = ylocal - Height / 2;

	      UpdateCircuitView(UCVModification);
	      return TRUE;
	    }
	  Check = GetIDFromCoordinates(xlocal, ylocal, AttachIDRef, &ID, True);
	  if (Check == Found)
	    UpdateCVAppBarFromID(ID, xlocal, ylocal);

	  return TRUE;
	}
      return TRUE;
    }
  return FALSE;
}


void on_BetweenCheck_toggled (GtkToggleButton *CheckButton, GtkWidget *CircuitViewWindow)
{
  GCD *CircuitData;
  CircuitData = GetCircuitData();

  if (GTK_TOGGLE_BUTTON(CheckButton)->active)
    SetOnBetweenPins(CircuitData->Info.Opt.CircuitMask);
  else
    SetOffBetweenPins(CircuitData->Info.Opt.CircuitMask);
  UpdateCircuitView(UCVDemand);
}
void on_InteriorCheck_toggled   (GtkToggleButton *CheckButton, GtkWidget *CircuitViewWindow)
{
  GCD *CircuitData;
  CircuitData = GetCircuitData();

  if (GTK_TOGGLE_BUTTON(CheckButton)->active)
    SetOnCompoIn(CircuitData->Info.Opt.CircuitMask);
  else
    SetOffCompoIn(CircuitData->Info.Opt.CircuitMask);
  UpdateCircuitView(UCVDemand);
}
void on_ExteriorCheck_toggled(GtkToggleButton *CheckButton, GtkWidget *CircuitViewWindow)   
{
  GCD *CircuitData;
  CircuitData = GetCircuitData();

  if (GTK_TOGGLE_BUTTON(CheckButton)->active)
    SetOnCompoOut(CircuitData->Info.Opt.CircuitMask);
  else
    SetOffCompoOut(CircuitData->Info.Opt.CircuitMask);
  UpdateCircuitView(UCVDemand);
}
void on_SubFunctionCheck_toggled(GtkToggleButton *CheckButton, GtkWidget *CircuitViewWindow)
{
  GCD *CircuitData;
  CircuitData = GetCircuitData();

  if (GTK_TOGGLE_BUTTON(CheckButton)->active)
    SetOnSubFunction(CircuitData->Info.Opt.CircuitMask);
  else
    SetOffSubFunction(CircuitData->Info.Opt.CircuitMask);
  UpdateCircuitView(UCVDemand);
}
void on_ComponentCheck_toggled(GtkToggleButton *CheckButton, GtkWidget *CircuitViewWindow)
{
  GCD *CircuitData;
  CircuitData = GetCircuitData();

  if (GTK_TOGGLE_BUTTON(CheckButton)->active)
    SetOnComponent(CircuitData->Info.Opt.CircuitMask);
  else
    SetOffComponent(CircuitData->Info.Opt.CircuitMask);
  UpdateCircuitView(UCVDemand);
}
void on_AttachCheck_toggled(GtkToggleButton *CheckButton, GtkWidget *CircuitViewWindow)   
{
  GCD *CircuitData;
  CircuitData = GetCircuitData();

  if (GTK_TOGGLE_BUTTON(CheckButton)->active)
    SetOnAttachPoint(CircuitData->Info.Opt.CircuitMask);
  else
    SetOffAttachPoint(CircuitData->Info.Opt.CircuitMask);
  UpdateCircuitView(UCVDemand);
}
void on_InvisibleCheck_toggled(GtkToggleButton *CheckButton, GtkWidget *CircuitViewWindow)
{
  GCD *CircuitData;
  CircuitData = GetCircuitData();

  if (GTK_TOGGLE_BUTTON(CheckButton)->active)
    SetOnInvisible(CircuitData->Info.Opt.CircuitMask);
  else
    SetOffInvisible(CircuitData->Info.Opt.CircuitMask);
  UpdateCircuitView(UCVDemand);
}
void on_LinkCheck_toggled(GtkToggleButton *CheckButton, GtkWidget *CircuitViewWindow)
{
  GCD *CircuitData;
  CircuitData = GetCircuitData();

  if (GTK_TOGGLE_BUTTON(CheckButton)->active)
    SetOnLink(CircuitData->Info.Opt.CircuitMask);
  else
    SetOffLink(CircuitData->Info.Opt.CircuitMask);
  UpdateCircuitView(UCVDemand);
}
