
#include "compdesc.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "layout.h"

ErrorCheck LoadCompDescLib(ComponentLibrary *CompLib, char *CompDescFileName)
{
  int Counter, CompDescNb, CompDescCounter; 
  FILE *CompDescFile;
  CompDescDef CompDescBuffer;
  ErrorCheck Check;
  
  CompDescFile = fopen(CompDescFileName,"r");
  if (CompDescFile == NULL)
    {
      printf(" Error Opening CompDescFile %s\n", CompDescFileName);
      return FileAccessError;
    }
 
  fscanf(CompDescFile, "%d\n", &CompDescNb);
  for (CompDescCounter = 0 ; CompDescCounter < CompDescNb ; CompDescCounter++)
    {
      fscanf(CompDescFile, "%[^\n]\n",  CompDescBuffer.CompDescName);
      fscanf(CompDescFile, "%[^\n]\n",  CompDescBuffer.CompDescClass);
      fscanf(CompDescFile, "%[^\n]\n",  CompDescBuffer.LayoutName);
      fscanf(CompDescFile, "%[^\n]\n",  CompDescBuffer.LayoutClass);
      fscanf(CompDescFile,"%d\n", &(CompDescBuffer.PinNb));

      if ( CompDescBuffer.PinNb != 0)	
	CompDescBuffer.PinNameTab = (char **) malloc ( CompDescBuffer.PinNb * sizeof(char *));

      for (Counter = 0 ; Counter < CompDescBuffer.PinNb ; Counter++)
	{
	  CompDescBuffer.PinNameTab[Counter] = (char *) malloc(sizeof(char) * PinNameLength);
	  fscanf(CompDescFile,"%[^\n]\n", CompDescBuffer.PinNameTab[Counter]);
	}
      
      Check = CheckLayout(CompLib, CompDescBuffer.LayoutName, CompDescBuffer.LayoutClass);
      if (Check == Found)
	AddCompDescDef(CompLib, CompDescBuffer);
      
      if (CompDescBuffer.PinNb != 0)
	{
	  for (Counter  = 0 ; Counter  < CompDescBuffer.PinNb ; Counter++)
	    free(CompDescBuffer.PinNameTab[Counter]);
	  free(CompDescBuffer.PinNameTab);	 
	}
    }
  fclose(CompDescFile);
  return AllOk;
}

ErrorCheck AddCompDescDef( ComponentLibrary *CompLib, CompDescDef CompDescDefProp)
{
  CompDescDef *CompDescDefCursor, *CompDescDefCursorPrevious, *NewCompDescDef;
  Boolean CompDescFound;
  int Counter;
  
  CompDescFound = False;
  CompDescDefCursor = CompLib->CompDescDefList;
  CompDescDefCursorPrevious  = NULL;
  while ( (CompDescDefCursor != NULL ) && (CompDescFound == False))
    {
      if ( strcmp(CompDescDefCursor->CompDescName, CompDescDefProp.CompDescName) >= 0 )
	CompDescFound = True;
      else
	{
	  CompDescDefCursorPrevious = CompDescDefCursor;
	  CompDescDefCursor = CompDescDefCursor->pNext;
	}
    }
  
  NewCompDescDef = (CompDescDef *) malloc( sizeof(CompDescDef));
  NewCompDescDef->pNext = CompDescDefCursor;

  if (CompDescDefCursorPrevious != NULL)
    CompDescDefCursorPrevious->pNext = NewCompDescDef;
  else 
    CompLib->CompDescDefList =  NewCompDescDef;
  
  strncpy(NewCompDescDef->CompDescName, CompDescDefProp.CompDescName, CompDescNameLength);
  NewCompDescDef->CompDescName[CompDescNameLength-1] = '\0';
  strncpy(NewCompDescDef->CompDescClass, CompDescDefProp.CompDescClass, CompDescClassLength);
  NewCompDescDef->CompDescClass[CompDescClassLength-1] = '\0';
  
  strncpy(NewCompDescDef->LayoutName, CompDescDefProp.LayoutName, LayoutNameLength);
  NewCompDescDef->LayoutName[LayoutNameLength-1] = '\0';
  strncpy(NewCompDescDef->LayoutClass, CompDescDefProp.LayoutClass, LayoutClassLength);
  NewCompDescDef->LayoutClass[LayoutClassLength-1] = '\0';
  
  NewCompDescDef->PinNb =  CompDescDefProp.PinNb;

  if ( NewCompDescDef->PinNb != 0)	
    NewCompDescDef->PinNameTab = (char **) malloc ( NewCompDescDef->PinNb * sizeof(char *));

  for (Counter =  0 ; Counter <  NewCompDescDef->PinNb ; Counter++ )
    {
      NewCompDescDef->PinNameTab[Counter] = (char *) malloc (sizeof(char) * PinNameLength);
      
      strncpy( NewCompDescDef->PinNameTab[Counter], CompDescDefProp.PinNameTab[Counter], PinNameLength);
      NewCompDescDef->PinNameTab[Counter][PinNameLength-1] = '\0';
    }
  return AllOk;
}

ErrorCheck AddCompDescLibToLoad(ComponentLibrary *CompLib, LibToLoad CompDescLib)
{
  LibToLoad *NewCompDesc, *CompDescCursor;
  
  NewCompDesc = NULL;
  CompDescCursor = CompLib->CompDescLibList;
  while (CompDescCursor != NULL)
    {
      NewCompDesc = CompDescCursor;
      CompDescCursor = CompDescCursor->pNext;
    }
  
  if (NewCompDesc != NULL)
    {
      NewCompDesc->pNext = (LibToLoad *) malloc (sizeof(LibToLoad));
      CompDescCursor = NewCompDesc->pNext;
    }
  else 
    {
      CompLib->CompDescLibList = (LibToLoad *) malloc (sizeof(LibToLoad));
      CompDescCursor = CompLib->CompDescLibList;
    }

  strncpy(CompDescCursor->LibName, CompDescLib.LibName, LibNameLength);
  CompDescCursor->pNext = NULL;
  return AllOk;
}
  
ErrorCheck GetCompDescPinNumber(ComponentLibrary *CompLib, char *CompDescName, int *CompPinNb)
{
  CompDescDef *CompDescCursor;
  Boolean CompDescFound;

  CompDescCursor  = CompLib->CompDescDefList;
  CompDescFound = False;

  while ((CompDescCursor != NULL) && (CompDescFound == False))
    {
      if (strcmp(CompDescCursor->CompDescName, CompDescName) == 0)
	{
	  CompDescFound = True;
	  *CompPinNb = CompDescCursor->PinNb;
	}
      CompDescCursor = CompDescCursor->pNext;
    }

  if (CompDescFound == True)
    return Found;
  else return NotFound; 
}

ErrorCheck GetCompDescProperties(ComponentLibrary *CompLib, char *CompDescName, CompDescDef *CompDescProp, Boolean WithPinName)
{
  CompDescDef *CompDescCursor;
  Boolean CompDescFound;
  int Counter;

  CompDescCursor  = CompLib->CompDescDefList;
  CompDescFound = False;
  
  while ((CompDescCursor != NULL) && (CompDescFound == False))
    {
      if (strcmp(CompDescCursor->CompDescName, CompDescName) == 0)
	{
	  CompDescFound = True;
	  strncpy(CompDescProp->CompDescName, CompDescCursor->CompDescName , CompDescNameLength);
	  CompDescProp->CompDescName[CompDescNameLength-1] = '\0';
	  strncpy(CompDescProp->CompDescClass, CompDescCursor->CompDescClass , CompDescClassLength);
	  CompDescProp->CompDescClass[CompDescClassLength-1] = '\0';

	  strncpy(CompDescProp->LayoutName, CompDescCursor->LayoutName, LayoutNameLength);
	  CompDescProp->LayoutName[LayoutNameLength-1] = '\0';
	  strncpy(CompDescProp->LayoutClass, CompDescCursor->LayoutClass, LayoutClassLength);
	  CompDescProp->LayoutClass[LayoutClassLength-1] = '\0';
	  if (WithPinName == True)
	    for (Counter = 0 ; Counter < CompDescCursor->PinNb ; Counter++)
	      {
		strncpy(CompDescProp->PinNameTab[Counter], CompDescCursor->PinNameTab[Counter], PinNameLength);
		CompDescProp->PinNameTab[Counter][PinNameLength-1] = '\0';
	      }
	}
      CompDescCursor = CompDescCursor->pNext;
    }

  if (CompDescFound == True)
    return Found;
  else return NotFound;   
}

ErrorCheck GetCompDescNumber(ComponentLibrary *CompLib, char *ClassName, int *CompDescNumber)
{
  CompDescDef *CompDescCursor;
  int Counter;

  CompDescCursor = CompLib->CompDescDefList;
  Counter = 0;
  while (CompDescCursor != NULL)
    {
      if ( strcmp(ClassName, CompDescCursor->CompDescClass) == 0)
	Counter++;
      CompDescCursor = CompDescCursor->pNext;
    }
  *CompDescNumber = Counter;
  
  return AllOk;
}

ErrorCheck GetCompDescTab(ComponentLibrary *CompLib, char *ClassName, char **CompDescTab)
{
  CompDescDef *CompDescCursor;
  int Counter;

  CompDescCursor = CompLib->CompDescDefList;
  Counter = 0;
  while (CompDescCursor != NULL)
    {
      if ( strcmp(ClassName, CompDescCursor->CompDescClass) == 0)
	{
	  strncpy(CompDescTab[Counter], CompDescCursor->CompDescName, CompDescNameLength);
	  CompDescTab[Counter][CompDescNameLength-1] = '\0';
	  Counter++;
	}
      CompDescCursor = CompDescCursor->pNext;
    }
  return AllOk;
}

ErrorCheck GetCompDescClassNumber(ComponentLibrary *CompLib, int *ClassNumber)
{
  CompDescDef *CompDescCursor;
  int Counter, CompDescNb, CompDescClassNb;
  char **CompDescClassTab;
  Boolean NewClass;

  CompDescCursor = CompLib->CompDescDefList;
  Counter = 0;
  while (CompDescCursor != NULL)
    {
      Counter++;
      CompDescCursor = CompDescCursor->pNext;
    }

  CompDescNb = Counter;
  
  CompDescClassTab = (char **) malloc(sizeof(char *) * CompDescNb);

  CompDescCursor = CompLib->CompDescDefList;
  CompDescClassNb = 0;
  
  while (CompDescCursor != NULL)
    {
      NewClass = True;
      for (Counter = 0 ; (Counter < CompDescClassNb) && (NewClass == True) ; Counter++)
	{
	  if (strcmp(CompDescCursor->CompDescClass, CompDescClassTab[Counter]) ==0) 
	    {
	      NewClass = False; /* la classe a deja ete comptee */
	    }
	}
      if (NewClass == True) /* C est une nouvelle classe */
	{
	  CompDescClassTab[CompDescClassNb] = (char *)malloc(sizeof (char) * CompDescClassLength);

	  strncpy (CompDescClassTab[CompDescClassNb], CompDescCursor->CompDescClass , CompDescClassLength);
	  CompDescClassTab[CompDescClassNb][CompDescClassLength-1] = '\0';
	  CompDescClassNb++;
	}
      CompDescCursor = CompDescCursor->pNext;
    }
  *ClassNumber  = CompDescClassNb;

  for (Counter = 0 ; Counter < CompDescClassNb ; Counter++)
    free(CompDescClassTab[Counter]);
  free(CompDescClassTab);

  return AllOk;
}	    

ErrorCheck GetCompDescClassTab(ComponentLibrary *CompLib, char **CompDescClassTab)
{
  CompDescDef *CompDescCursor;
  int Counter, CompDescClassCounter;
  Boolean NewClass;

  CompDescCursor = CompLib->CompDescDefList;

  CompDescClassCounter = 0;
  while (CompDescCursor != NULL)
    {
      NewClass = True;
      for (Counter = 0 ; (Counter < CompDescClassCounter) && (NewClass == True) ; Counter++)
	  if (strcmp(CompDescCursor->CompDescClass, CompDescClassTab[Counter]) ==0) 
	      NewClass = False; /* la classe a deja ete comptee */

      if (NewClass == True) /* C est une nouvelle classe */
	{
	  strncpy (CompDescClassTab[CompDescClassCounter], CompDescCursor->CompDescClass , CompDescClassLength);
	  CompDescClassTab[CompDescClassCounter][CompDescClassLength-1] = '\0';
	  CompDescClassCounter++;
	}
      CompDescCursor = CompDescCursor->pNext;
    }

  return AllOk;
}	      

ErrorCheck CheckCompDesc(ComponentLibrary *CompLib, char *CompDescName)
{
  CompDescDef *CompDescCursor;
  Boolean CompDescFound;
  
  CompDescCursor = CompLib->CompDescDefList;
  CompDescFound = False;

  while ((CompDescCursor != NULL) && (CompDescFound == False))
    {
      if (strcmp(CompDescCursor->CompDescName, CompDescName) == 0)
	CompDescFound = True;
      CompDescCursor = CompDescCursor->pNext;
    }
  if (CompDescFound == True)
    return Found;
  else return NotFound;
}

ErrorCheck DisplayCompDescLib(ComponentLibrary *CompLib)
{
  CompDescDef *CompDescCursor;
  int Counter;
  CompDescCursor = CompLib->CompDescDefList;
  printf("CompDesc : \n");
  while (CompDescCursor != NULL)
    {
      printf("----------------------------\n");
      printf("CompDesc: %s %s\nLayout: %s %s\n",  CompDescCursor->CompDescName, CompDescCursor->CompDescClass, 
	     CompDescCursor->LayoutName,CompDescCursor->LayoutClass);
      
      for (Counter = 0 ; Counter < CompDescCursor->PinNb ; Counter++)
	printf("%d: %s\n", Counter, CompDescCursor->PinNameTab[Counter]);
      CompDescCursor = CompDescCursor->pNext;
    }
  return AllOk;
}
