
#include "component.h"
#include <stdlib.h>
#include <stdio.h>
#include "mainwindow.h"
#include "attachpoint.h"
#include "compdesc.h"
#include "link.h"
#include "layout.h"
#include "subfunction.h"

int NewComponentID(GCD *CircuitData, int SetComponentID)
{
  CircuitData->Info.ComponentID += 10;
  if (SetComponentID != NoID)
    CircuitData->Info.ComponentID = SetComponentID-10; 
  /* SetComponentID est le prochain ID utilisable, il doit etre renvoye lors du prochain appel */
  return  CircuitData->Info.ComponentID;
}

ErrorCheck AddComponent(GCD *CircuitData, int CompID, int SubFID)
{
  Component *NewComp, *CompCursor, *PrevComp;
  int AttachID;
  AttachPoint AttachProp;

  NewComp = (Component*)malloc(sizeof(Component));
  if( NewComp==NULL)
    return MemoryAllocationError;
  
  strncpy( NewComp->ComponentName, CircuitData->Info.Opt.DefaultComponentName, ComponentNameLength);
  NewComp->ComponentName[ComponentNameLength-1] = '\0';
  
  strncpy(NewComp->CompDescName, "Default", CompDescNameLength);
  NewComp->CompDescName[CompDescNameLength-1] = '\0';

  strncpy(NewComp->LayoutName, "Default", LayoutNameLength);
  NewComp->LayoutName[LayoutNameLength-1] = '\0';
  strncpy(NewComp->LayoutClass, "Default", LayoutClassLength);
  NewComp->LayoutClass[LayoutClassLength-1] = '\0';

  AttachID = NewAttachPointID(CircuitData, NoID);
  NewComp->AttachPointIDTab = (int *) malloc (sizeof(int));
  NewComp->AttachPointIDTab[0] = AttachID;
  NewComp->AttachPointNb = 1;
  NewComp->IDNumber= CompID;
  NewComp->SubFunctionID = SubFID;
  NewComp->Reference.x=2;
  NewComp->Reference.y=2;
  InitState(NewComp->State);
  UnSetComponentSelected(NewComp->State);
  SetComponentToBottom(NewComp->State);
  NewComp->pNext = NULL;
  
  strncpy(AttachProp.AttachPointName ,"Default", PinNameLength);
  AttachProp.AttachPointName[PinNameLength-1]  = '\0';
  strncpy(AttachProp.SpotName ,"Default", SpotNameLength);
  AttachProp.SpotName[SpotNameLength-1]  = '\0';

  AttachProp.Reference.x = 0;
  AttachProp.Reference.y = 0;
  AddAttachPoint(CircuitData,AttachID, CompID);
  EditAttachPoint (CircuitData, AttachID, AttachProp);
  
  PrevComp = NULL;
  CompCursor = CircuitData->ListComponent;
  while (CompCursor != NULL)
    {
      PrevComp = CompCursor;
      CompCursor = CompCursor->pNext;
    }
  if (PrevComp == NULL)
    CircuitData->ListComponent = NewComp;
  else 
    PrevComp->pNext = NewComp;

  return AllOk;
}

ErrorCheck EditComponent(GCD *CircuitData, int CompID, Component CompProp,  Boolean AddAttachPointUnder)
{
  Component *CompCursor;
  int CompPinNb, CompOccupation, Counter, LinkNb, *LinkIDTab;
  Boolean IDFound, Propagate, ToLeftOld, ToRightOld, ToTopOld, ToBottomOld, ToLeft, ToRight, ToTop, ToBottom;
  ComponentLibrary *CompLib;

  CompLib = CircuitData->CompLib;

  ToLeft = False;
  ToRight = False;
  ToTop = False;
  ToBottom = False;
  
  if (IsComponentToLeft(CompProp.State))
    ToLeft = True;
  if (IsComponentToRight(CompProp.State))
    ToRight = True;
  if (IsComponentToTop(CompProp.State))
    ToTop = True;
  if (IsComponentToBottom(CompProp.State))
    ToBottom = True;
  
  CompCursor=CircuitData->ListComponent;
  Counter=0;
  IDFound = False;
  Propagate = False;
  while ((CompCursor!=NULL) && (IDFound == False))
    {
      if(CompCursor->IDNumber == CompID)
	{
	  IDFound = True;
	  Propagate = False;
	  
	  ToLeftOld = False;
	  ToRightOld = False;
	  ToTopOld = False;
	  ToBottomOld = False;
	  
	  if (IsComponentToLeft(CompCursor->State))
	    ToLeftOld = True;
	  if (IsComponentToRight(CompCursor->State))
	    ToRightOld = True;
	  if (IsComponentToTop(CompCursor->State))
	    ToTopOld = True;
	  if (IsComponentToBottom(CompCursor->State))
	    ToBottomOld = True;
	  
	  strncpy(CompCursor->ComponentName,CompProp.ComponentName, ComponentNameLength);
	  CompCursor->ComponentName[ComponentNameLength-1] = '\0';
	  
	  if ((CompCursor->Reference.x != CompProp.Reference.x)||(CompCursor->Reference.y != CompProp.Reference.y))
	    {
	      Propagate = True;
	      CompCursor->Reference.x = CompProp.Reference.x;  
	      CompCursor->Reference.y = CompProp.Reference.y;
	    }
	  
	  if ((ToLeftOld != ToLeft) || (ToRightOld != ToRight) || (ToTopOld != ToTop) || (ToBottomOld != ToBottom))
	    {
	      Propagate = True;
	      if (ToLeft == True)
		SetComponentToLeft(CompCursor->State);
	      else
		UnSetComponentToLeft(CompCursor->State);
	      if (ToRight == True)
		SetComponentToRight(CompCursor->State);
	      else
		UnSetComponentToRight(CompCursor->State);
	      if (ToTop == True)
		SetComponentToTop(CompCursor->State);
	      else
		UnSetComponentToTop(CompCursor->State);
	      if (ToBottom == True)
		SetComponentToBottom(CompCursor->State);
	      else
		UnSetComponentToBottom(CompCursor->State);
	    }

	  GetCompDescPinNumber(CompLib, CompProp.CompDescName, &CompPinNb);
	  GetCompOccupation(CircuitData, CompID, &CompOccupation);
	  if ((CompOccupation != 0) && (CircuitData->Info.Opt.AllowEditComponent == False))
	    return Forbidden;
	  if (CompOccupation > CompPinNb)
	    return ArgumentError;

	  if ((strcmp(CompCursor->LayoutName, CompProp.LayoutName) != 0) 
	      ||(strcmp(CompCursor->LayoutClass, CompProp.LayoutClass) != 0))
	    Propagate = True;
	  
	  strncpy(CompCursor->CompDescName, CompProp.CompDescName, CompDescNameLength);
	  CompCursor->CompDescName[CompDescNameLength-1] = '\0';
	  
	  strncpy(CompCursor->LayoutName, CompProp.LayoutName, LayoutNameLength);
	  CompCursor->LayoutName[LayoutNameLength-1] = '\0';
	  strncpy(CompCursor->LayoutClass, CompProp.LayoutClass, LayoutClassLength);
	  CompCursor->LayoutClass[LayoutClassLength-1] = '\0';
	}
      else CompCursor=CompCursor->pNext;
    }
  
  if (AddAttachPointUnder==True)
    ReplaceAttachForComp(CircuitData, CompID);

  if (Propagate == True)
    {
      UnDrawAllLinks(CircuitData);
      GetLinkNumber(CircuitData, &LinkNb);
      if (LinkNb != 0)
	{
	  LinkIDTab = (int *) malloc (sizeof(int) * LinkNb);
	  GetLinkUsingComponent(CircuitData, CompID, LinkIDTab, &LinkNb);
	  for (Counter = 0 ; Counter < LinkNb ; Counter++)
	    UpdateLink(CircuitData, LinkIDTab[Counter]);
      
	  free(LinkIDTab);
	}
    }
  return AllOk;
}

ErrorCheck ReplaceAttachForComp( GCD *CircuitData, int CompID)
{
  Component *CompCursor;
  Boolean CompIDFound;
  CompDescDef CompDescProp;
  LayoutDef  LayoutProp;
  int CompPinNb, Counter, *NewAttachIDTab= NULL, X11=0, X12=0, X21=0, X22=0, Xin, Yin;
  ComponentLibrary *CompLib;
  AttachPoint AttachProp;

  CompLib = CircuitData->CompLib;
  CompCursor = CircuitData->ListComponent;
  CompIDFound = False;
  while ((CompCursor != NULL) && (CompIDFound == False))
    {
      if (CompCursor->IDNumber == CompID)
	{
	  CompIDFound = True;
	  
	  GetCompDescPinNumber(CompLib, CompCursor->CompDescName, &CompPinNb);
	  CompDescProp.PinNameTab = (char **) malloc (CompPinNb * sizeof (char *)); /* on recup les infos */
	  LayoutProp.PinPosition = (Point *) malloc (CompPinNb* sizeof (Point));
	  LayoutProp.SpotNameTab = (char **) malloc (CompPinNb * sizeof (char *));	  
	  for(Counter=0 ; Counter < CompPinNb ; Counter++)  /* on alloue le tableau de nom */
	    {
	      CompDescProp.PinNameTab[Counter] = (char *) malloc (sizeof(char) * PinNameLength);
	      LayoutProp.SpotNameTab[Counter] = (char *) malloc (sizeof(char) * SpotNameLength);
	    }
	  GetCompDescProperties(CompLib, CompCursor->CompDescName, &CompDescProp, True);
	  GetLayoutProperties(CompLib, CompCursor->LayoutName, CompCursor->LayoutClass, &LayoutProp, True, False);
	  
   	  NewAttachIDTab = (int *) malloc (sizeof(int) * CompPinNb);
	  for (Counter = 0 ; (Counter < CompPinNb) || (Counter < CompCursor->AttachPointNb) ; Counter ++)
	    {
	      if ( (Counter < CompCursor->AttachPointNb) && (Counter < CompPinNb))
		/* partie commune au nouveau et a l ancien compo */
		NewAttachIDTab[Counter] = CompCursor->AttachPointIDTab[Counter];
	      
	      if ( (Counter < CompCursor->AttachPointNb) && (Counter >= CompPinNb))
		/* partie a effacer les attachs ne sont plus utilisees */
		DeleteAttachPoint(CircuitData, CompCursor->AttachPointIDTab[Counter]);

	      if ( (Counter >= CompCursor->AttachPointNb) && (Counter < CompPinNb))
		{/* partie nouvelle les attachs doivent etres crees */
		  NewAttachIDTab[Counter] = NewAttachPointID(CircuitData, NoID);
		  AddAttachPoint(CircuitData, NewAttachIDTab[Counter], CompID);
		}
	    }

	  free(CompCursor->AttachPointIDTab); 
	  CompCursor->AttachPointNb = CompPinNb;
	  for(Counter = 0 ; Counter < CompPinNb ; Counter++) 
	    { /* on edite les points d attache avec les nouvelles valeurs */
	      strncpy(AttachProp.AttachPointName, CompDescProp.PinNameTab[Counter],  PinNameLength);
	      AttachProp.AttachPointName[PinNameLength-1] = '\0';
	      
	      strncpy(AttachProp.SpotName, LayoutProp.SpotNameTab[Counter], SpotNameLength);;
	      AttachProp.SpotName[SpotNameLength-1] = '\0';
	      InitState(AttachProp.State);
	      if (IsComponentToRight(CompCursor->State))
		{
		  SetAttachToRight(AttachProp.State);
		  X11 =  0; X12 =  1;
		  X21 = -1; X22 =  0;
		}
	      if (IsComponentToBottom(CompCursor->State))
		{
		  SetAttachToBottom(AttachProp.State);
		  X11 =  1; X12 =  0;
		  X21 =  0; X22 =  1;
		}
	      if (IsComponentToLeft(CompCursor->State))
		{
		  SetAttachToLeft(AttachProp.State);
		  X11 =  0; X12 = -1;
		  X21 =  1; X22 =  0;
		}
	      if (IsComponentToTop(CompCursor->State))
		{
		  SetAttachToTop(AttachProp.State);
		  X11 = -1; X12 =  0;
		  X21 =  0; X22 = -1;
		}
	      Xin = LayoutProp.PinPosition[Counter].x;
	      Yin = LayoutProp.PinPosition[Counter].y;
	      AttachProp.Reference.x = X11 * Xin + X12 * Yin;
	      AttachProp.Reference.y = X21 * Xin + X22 * Yin;
	      EditAttachPoint(CircuitData,NewAttachIDTab[Counter],AttachProp);
	    }
	  CompCursor->AttachPointNb = CompPinNb;
	  CompCursor->AttachPointIDTab = NewAttachIDTab;
	  
	  if (CompPinNb != 0)
	    {
	      for (Counter=0 ; Counter < CompPinNb  ; Counter++)  /* on supprime le tableau de nom */
		{
		  free(CompDescProp.PinNameTab[Counter]);
		  free(LayoutProp.SpotNameTab[Counter]);
		}
	      free(CompDescProp.PinNameTab);
	      free(LayoutProp.PinPosition);
	      free(LayoutProp.SpotNameTab);
	    }
	}
      CompCursor = CompCursor->pNext;
    }
  if (CompIDFound == True)
    return Found;
  else
    return NotFound;
}
ErrorCheck GetLinkUsingComponent(GCD *CircuitData, int CompID, int *LinkIDTab, int *LinkNb)
{
  Link *LinkCursor;
  Component CompProp;
  int Counter, LinkCounter, AttachNb, Start, End;
  Boolean Used;
  
  GetComponentProperties(CircuitData, CompID, &CompProp, True);
  AttachNb = CompProp.AttachPointNb; 
  
  LinkCursor = CircuitData->ListLink;
  LinkCounter = 0;
  while (LinkCursor != NULL)
    {
      Start = LinkCursor->StartAttachID;
      End = LinkCursor->EndAttachID;

      Used = False;
      for (Counter = 0 ; (Counter < AttachNb) && (Used == False) ; Counter++)
	if ((CompProp.AttachPointIDTab[Counter] == Start) || (CompProp.AttachPointIDTab[Counter] == End))
	  Used = True;
      
      if (Used == True)
	{
	  LinkIDTab[LinkCounter] = LinkCursor->IDNumber;
	  LinkCounter++;
	}
      LinkCursor = LinkCursor->pNext;
    }

  *LinkNb = LinkCounter;
  free(CompProp.AttachPointIDTab);
  return AllOk;
}
ErrorCheck DeleteComponentAttachPoint(GCD *CircuitData, int CompID)
{
  Component *CompCursor;
  int Counter;
  Boolean IDFound;
  
  CompCursor = CircuitData->ListComponent;
  IDFound = False;
  while ((CompCursor!=NULL) && (IDFound == False))
    {
      if (CompCursor->IDNumber == CompID)
	{
	  IDFound = True;
	  if (CompCursor->AttachPointIDTab != NULL)
	    {
	      for (Counter =0 ; Counter < CompCursor->AttachPointNb ; Counter++)
		DeleteAttachPoint(CircuitData, CompCursor->AttachPointIDTab[Counter]);
	      
	      free(CompCursor->AttachPointIDTab);
	      CompCursor->AttachPointIDTab = NULL;
	    }	      
	}
      CompCursor = CompCursor->pNext;
    }
  if (IDFound == True)
   return Found;
  else return NotFound;
}

ErrorCheck DisplayComponent(GCD *CircuitData)
{
  Component *CompCursor;
  
  printf("\nListe des composants:\n"); 
  CompCursor=CircuitData->ListComponent;
  while(CompCursor!=NULL)
    {
      printf("%s Number %d From SF %d\n",CompCursor->ComponentName,CompCursor->IDNumber, CompCursor->SubFunctionID);
      printf("\tCompType %s LayoutType %s\n",CompCursor->CompDescName, CompCursor->LayoutName);
      CompCursor=CompCursor->pNext;
    }
  printf("\n");
  return AllOk;
}


ErrorCheck DisplayComponentBuffer(GCD *CircuitData)
{
  printf("\nListe des composants: dans le buffer\n"); 
  printf("nom %s Number %d \n", CircuitData->ComponentBuffer.ComponentName, CircuitData->ComponentBuffer.IDNumber);
  printf("\tCompType %s LayoutType %s\n",CircuitData->ComponentBuffer.CompDescName, CircuitData->ComponentBuffer.LayoutName);

  printf("\n");
  return AllOk;
}

ErrorCheck DeleteComponent(GCD *CircuitData, int CompID)
{
  Component *CompCursor, **CompNext;
  Boolean IDFound;
  int LinkNb, *LinkIDTab, Counter;

  CompNext = &( CircuitData->ListComponent);
  CompCursor= CircuitData->ListComponent;
  IDFound = False;
  while ((CompCursor!=NULL) && (IDFound == False))
    {
     if (CompCursor->IDNumber == CompID)
       {
	 IDFound = True;
	 GetLinkNumber(CircuitData, &LinkNb);
	 if (LinkNb != 0)
	   {
	     LinkIDTab = (int *) malloc (sizeof(int) * LinkNb);
	     GetLinkUsingComponent(CircuitData, CompID, LinkIDTab, &LinkNb);
	     
	     for (Counter = 0 ; Counter < LinkNb ; Counter++)
	       DeleteLink(CircuitData, LinkIDTab[Counter]);
	     
	     free(LinkIDTab);
	   }

	 DeleteComponentAttachPoint(CircuitData, CompID);
	 *CompNext = CompCursor->pNext;
	 free(CompCursor);
       }
     else
       CompNext = &(CompCursor->pNext);
     CompCursor = *CompNext;
   }

  if (IDFound == True)
    return Found;
  else return NotFound;
}

ErrorCheck GetComponentProperties(GCD *CircuitData, int CompID, Component *Comp, Boolean WithAttachIDTab)
{
  Component *CompCursor;
  int Counter;
  Boolean IDFound;

  CompCursor=CircuitData->ListComponent;
  IDFound =  False; 
  while ((CompCursor!=NULL) && (IDFound== False))
    {
      if(CompCursor->IDNumber==CompID)						       
	{										
	  IDFound = True;								
	  strncpy(Comp->ComponentName,CompCursor->ComponentName, ComponentNameLength);
	  Comp->ComponentName[ComponentNameLength-1] = '\0';					
	  
	  strncpy(Comp->CompDescName,CompCursor->CompDescName, CompDescNameLength); 
	  Comp->CompDescName[CompDescNameLength-1] = '\0';				       

	  strncpy(Comp->LayoutName,CompCursor->LayoutName, LayoutNameLength); 
	  Comp->LayoutName[LayoutNameLength-1] = '\0';				       

	  strncpy(Comp->LayoutClass, CompCursor->LayoutClass, LayoutClassLength); 
	  Comp->LayoutClass[LayoutClassLength-1] = '\0';				       
	  
	  Comp->SubFunctionID=CompCursor->SubFunctionID;				
	  Comp->IDNumber = CompCursor->IDNumber;
	  Comp->State = CompCursor->State;
	  Comp->Reference.x = CompCursor->Reference.x;
	  Comp->Reference.y = CompCursor->Reference.y;
	  Comp->AttachPointNb = CompCursor->AttachPointNb; 
	  if (WithAttachIDTab ==True)
	    {
	      Comp->AttachPointIDTab = (int *) malloc (sizeof(int) *  CompCursor->AttachPointNb); 
	      for (Counter = 0 ; Counter < CompCursor->AttachPointNb; Counter++)
		Comp->AttachPointIDTab[Counter] = CompCursor->AttachPointIDTab[Counter];
	    }
	}
      CompCursor=CompCursor->pNext;
    }
  if (IDFound == True)
    return Found;
  else return NotFound;
}
ErrorCheck GetComponentAttachNb(GCD *CircuitData, int CompID, int *AttachNb)
{
  Component *CompCursor;
  Boolean IDFound;

  CompCursor = CircuitData->ListComponent;
  IDFound = False;
  while ((CompCursor != NULL) && (IDFound==False))
    {
      if (CompCursor->IDNumber == CompID)
	*AttachNb  = CompCursor->AttachPointNb;
      CompCursor = CompCursor->pNext;
    }
  if (IDFound == True)
    return Found;
  else return NotFound;
}

ErrorCheck CutComponent(GCD *CircuitData, int CompID)
{
  int LinkNb;
  
  AddComponentToBuffer(CircuitData,CompID);
  GetComponentLinkNumber(CircuitData, CompID, &LinkNb, False);
  if (LinkNb != 0)
    {
      if (CircuitData->Info.Opt.AllowDeleteComponent == True)
	DeleteComponent(CircuitData,CompID);
      else
	return Forbidden;
    }
  else
    DeleteComponent(CircuitData,CompID);
  return AllOk;
}


ErrorCheck CopyComponent(GCD *CircuitData, int CompID)
{
  AddComponentToBuffer(CircuitData, CompID);
  return AllOk;
}

ErrorCheck AddComponentToBuffer(GCD *CircuitData, int CompID)
{
  Component *CompCursor;
  Boolean IDFound;
  
  CompCursor= CircuitData->ListComponent; 
  IDFound = False;
  while ((CompCursor!=NULL) && (IDFound ==False))
    {
      if (CompCursor->IDNumber==CompID)
	{
	  IDFound = True;
	  
	  strncpy(CircuitData->ComponentBuffer.ComponentName,CompCursor->ComponentName,ComponentNameLength);
	  CircuitData->ComponentBuffer.ComponentName[ComponentNameLength-1] = '\0';
	  strncpy(CircuitData->ComponentBuffer.CompDescName, CompCursor->CompDescName, CompDescNameLength );
	  CircuitData->ComponentBuffer.CompDescName[CompDescNameLength-1] = '\0';
	  
	  strncpy(CircuitData->ComponentBuffer.LayoutName, CompCursor->LayoutName, LayoutNameLength );
	  CircuitData->ComponentBuffer.LayoutName[LayoutNameLength-1] = '\0';
	  strncpy(CircuitData->ComponentBuffer.LayoutClass, CompCursor->LayoutClass, LayoutClassLength );
	  CircuitData->ComponentBuffer.LayoutClass[LayoutClassLength-1] = '\0';
	  
	  CircuitData->ComponentBuffer.AttachPointNb = CompCursor->AttachPointNb;
	  UnSetComponentSelected(CircuitData->ComponentBuffer.State);
	  CircuitData->ComponentBuffer.State = CompCursor->State;
	  CircuitData->ComponentBuffer.Reference.x=CompCursor->Reference.x;
	  CircuitData->ComponentBuffer.Reference.y=CompCursor->Reference.y;
	  CircuitData->ComponentBuffer.SubFunctionID = CompCursor->SubFunctionID;
	  
	  CircuitData->ComponentBufferEmpty = False;
	}
      CompCursor = CompCursor->pNext;
    }
  return AllOk;
}

ErrorCheck PasteComponent(GCD *CircuitData, int SubFunctionID, int *PasteID, int x, int y)
{
  Component Comp;
  int CompID;
  
  if (CircuitData->ComponentBufferEmpty == True)
    return BufferEmpty;

  strncpy(Comp.ComponentName,CircuitData->ComponentBuffer.ComponentName,ComponentNameLength);
  Comp.ComponentName[ComponentNameLength-1] = '\0';
  
  strncpy(Comp.CompDescName,CircuitData->ComponentBuffer.CompDescName, CompDescNameLength); 
  Comp.CompDescName[CompDescNameLength-1] = '\0';				       
  
  strncpy(Comp.LayoutName,CircuitData->ComponentBuffer.LayoutName, LayoutNameLength); 
  Comp.LayoutName[LayoutNameLength-1] = '\0';				       
  strncpy(Comp.LayoutClass,CircuitData->ComponentBuffer.LayoutClass, LayoutClassLength); 
  Comp.LayoutClass[LayoutClassLength-1] = '\0';				       
  
  Comp.AttachPointNb = CircuitData->ComponentBuffer.AttachPointNb;
  Comp.State = CircuitData->ComponentBuffer.State;
  
  if ((x == NoX) || (y == NoY))
    {
      Comp.Reference.x=CircuitData->ComponentBuffer.Reference.x;
      Comp.Reference.y=CircuitData->ComponentBuffer.Reference.y;
    }
  else
    {
      Comp.Reference.x = x;
      Comp.Reference.y = y;
    }
  CompID=NewComponentID(CircuitData, NoID);
  *PasteID = CompID;
  AddComponent(CircuitData, CompID, SubFunctionID); 
  EditComponent(CircuitData, CompID, Comp, True);
    
  return AllOk; 
}

/*********************************************************************************/
ErrorCheck SetIIDForComponent(GCD *CircuitData, int CompID, void *InterfaceID)
{
  Component *CompCursor;
  Boolean IDFound;
  CompCursor = CircuitData->ListComponent;
  IDFound = False;
  while ((CompCursor != NULL) && (IDFound == False)) 
    {
      if ( CompCursor->IDNumber == CompID)
	{
	  IDFound  = True;
	  CompCursor->InterfaceID = InterfaceID;
	}
      CompCursor = CompCursor->pNext;
    }
  if (IDFound == True)
    return Found;
  else return NotFound;
}
ErrorCheck GetIIDForComponent(GCD *CircuitData, int CompID, void  **InterfaceID)
{
  Component *CompCursor;
  Boolean IDFound;
  CompCursor = CircuitData->ListComponent;
  IDFound =  False;
  while ((CompCursor != NULL) && (IDFound == False))
    {
      if ( CompCursor->IDNumber == CompID)
	{
	  IDFound = True;
	  *InterfaceID = CompCursor->InterfaceID;
	}
      CompCursor = CompCursor->pNext;
    }
  if (IDFound == True)
    return Found;
  else return NotFound;
}

ErrorCheck GetCompOccupation( GCD  *CircuitData, int CompID, int  *Occupation)
{
  Component *CompCursor;
  int MaxAttachNb;
  Boolean IDFound;
  int Counter;
  
  MaxAttachNb = 0;
  CompCursor = CircuitData->ListComponent;
  IDFound = False;
  while ((CompCursor != NULL) && (IDFound == False ))
    {
      if (CompCursor->IDNumber == CompID)
	{
	  IDFound = True;
	  for (Counter = 0 ; Counter < CompCursor->AttachPointNb ; Counter++)
	    if (CheckAttachUsed(CircuitData, CompCursor->AttachPointIDTab[Counter]) == True)
	      MaxAttachNb = Counter + 1; 
	}
      CompCursor = CompCursor->pNext;
    }
  *Occupation = MaxAttachNb;
  if (IDFound == True)
    return Found;
  else return NotFound;
}

ErrorCheck GetComponentIDFromCoordinates(GCD *CircuitData, int x, int y, int *CompID)
{
  Component *CompCursor;
  SubFunction SubFProp;
  Boolean FirstCompFound;
  int Deltax, Deltay, MinLength=0, MinID, Length;
  
  CompCursor = CircuitData->ListComponent;
  FirstCompFound = False;
  MinID = NoID;
  while (CompCursor != NULL)
    {
      GetSubFunctionProperties(CircuitData, CompCursor->SubFunctionID, &SubFProp);
      Deltax = SubFProp.Reference.x + CompCursor->Reference.x - x;
      Deltay = SubFProp.Reference.y + CompCursor->Reference.y - y;
      Length = Deltax * Deltax + Deltay * Deltay;
	  
      if ((FirstCompFound == False) || (Length < MinLength))
	{
	  FirstCompFound = True;
	  MinLength = Length;
	  MinID = CompCursor->IDNumber;
	}
      CompCursor = CompCursor->pNext;
    }
  *CompID = MinID;
  if (FirstCompFound == True)
    return Found;
  else 
    return NotFound;
}

ErrorCheck SelectComponentFromID(GCD *CircuitData, int ComponentID)
{
  Component *CompCursor;
  Boolean IDFound;

  IDFound = False;
  CompCursor = CircuitData->ListComponent;
  while ((CompCursor != NULL ) && (IDFound == False))
    {
      if (CompCursor->IDNumber == ComponentID)
	{
	  SetComponentSelected(CompCursor->State);
	  IDFound = True;
	}
      else
	UnSetComponentSelected(CompCursor->State);
      CompCursor = CompCursor->pNext;
    }

  if (IDFound == True)
    return Found;
  else 
    return NotFound;
}
  
ErrorCheck GetComponentLinkNumber(GCD *CircuitData, int CompID, int *LinkNb, Boolean Internal)
{
  Link *LinkCursor;
  int Counter, StartAttachID, EndAttachID;
  AttachPoint AttachStartProp, AttachEndProp;

  Counter = 0; 
  LinkCursor = CircuitData->ListLink;
  while (LinkCursor != NULL)
    {
      StartAttachID = LinkCursor->StartAttachID;
      EndAttachID = LinkCursor->EndAttachID;

      GetAttachPointProperties(CircuitData, StartAttachID, &AttachStartProp);
      GetAttachPointProperties(CircuitData, EndAttachID, &AttachEndProp);
	 
      if ((Internal == True) && ((AttachStartProp.ComponentID == CompID) && (AttachEndProp.ComponentID == CompID)))
	Counter++;

      if ((Internal == False) && ((AttachStartProp.ComponentID == CompID) || (AttachEndProp.ComponentID == CompID)))
	Counter++;
      
      LinkCursor = LinkCursor->pNext;
    }
  *LinkNb = Counter;
  
  return AllOk;
}
