
#include "componentlibrary.h"
#include <stdlib.h>
#include "libset.h"
#include "spot.h"
#include "layout.h"
#include "compdesc.h"

ErrorCheck CreateComponentLibrary( ComponentLibrary **CompLib)
{
  *CompLib = (ComponentLibrary *) malloc (sizeof(ComponentLibrary));
  (*CompLib)->SpotDefList = NULL;
  (*CompLib)->LayoutDefList = NULL;
  (*CompLib)->CompDescDefList = NULL;
  (*CompLib)->LocationList = NULL;
  (*CompLib)->SpotLibList = NULL;
  (*CompLib)->LayoutLibList = NULL;
  (*CompLib)->CompDescLibList = NULL;
  return AllOk;
}

ErrorCheck DestroyComponentLibrary( ComponentLibrary *CompLib)
{
 SpotDef *SpotCursor;
 LayoutDef *LayoutCursor;
 CompDescDef *CompDescCursor;
 SearchLocation *SearchLocationCursor;
 LibToLoad *SpotLibCursor;
 LibToLoad *LayoutLibCursor;
 LibToLoad *CompDescLibCursor;
 int Counter;
 
 while ( CompLib->SpotDefList != NULL)
    {
      SpotCursor = CompLib->SpotDefList ;
      if (SpotCursor->SpotSize != 0)
	free(SpotCursor->SpotLocation);
      if (SpotCursor->CoreSpotSize != 0)
	free(SpotCursor->CoreSpotLocation);
      if (SpotCursor->InvisibleSpotSize != 0)
	free(SpotCursor->InvisibleSpotLocation);
      
      CompLib->SpotDefList  = CompLib->SpotDefList->pNext;
      free(SpotCursor);
    }

  while ( CompLib->LayoutDefList != NULL)
    {
      LayoutCursor = CompLib->LayoutDefList ;
      if (LayoutCursor->PinNb != 0)
	{
	  free( LayoutCursor->PinPosition);
	  for (Counter = 0 ; Counter < LayoutCursor->PinNb ; Counter++)
	    free(LayoutCursor->SpotNameTab[Counter]);
	  free(LayoutCursor->SpotNameTab);
	}
      if (LayoutCursor->BetweenPinsSize != 0)
	free(LayoutCursor->BetweenPinsPosition);
      if (LayoutCursor->CompoInSize != 0)
	free(LayoutCursor->CompoInPosition);
      if (LayoutCursor->CompoOutSize != 0)
	free(LayoutCursor->CompoOutPosition);

      CompLib->LayoutDefList  = CompLib->LayoutDefList->pNext;
      free(LayoutCursor);
    }

  while ( CompLib->CompDescDefList != NULL)
    {
      CompDescCursor = CompLib->CompDescDefList ;
      if (CompDescCursor->PinNb != 0)
	{
	  for (Counter = 0 ; Counter < CompDescCursor->PinNb ; Counter++)
	    free(CompDescCursor->PinNameTab[Counter]);
	  free(CompDescCursor->PinNameTab);
	}
      CompLib->CompDescDefList  = CompLib->CompDescDefList->pNext;
      free(CompDescCursor);
    }
  while ( CompLib->LocationList != NULL)
    {
      SearchLocationCursor = CompLib->LocationList ;
      CompLib->LocationList  = CompLib->LocationList->pNext;
      free(SearchLocationCursor);
    }
  while ( CompLib->SpotLibList != NULL)
    {
      SpotLibCursor = CompLib->SpotLibList ;
      CompLib->SpotLibList  = CompLib->SpotLibList->pNext;
      free(SpotLibCursor);
    }
  while ( CompLib->LayoutLibList != NULL)
    {
      LayoutLibCursor = CompLib->LayoutLibList ;
      CompLib->LayoutLibList  = CompLib->LayoutLibList->pNext;
      free(LayoutLibCursor);
    }
  while ( CompLib->CompDescLibList != NULL)
    {
      CompDescLibCursor = CompLib->CompDescLibList ;
      CompLib->CompDescLibList  = CompLib->CompDescLibList->pNext;
      free(CompDescLibCursor);
    }
  free(CompLib);
  return AllOk; 
}


ErrorCheck LoadComponentLibrary(ComponentLibrary **CompLib, char *LibSetFileName, GtkWidget *Text)
{
  char FullLibName[DirectoryNameLength+LibNameLength];
  Boolean FileFound;
  SearchLocation *LocationCursor;
  LibToLoad *LibCursor;
  FILE *TestFile;
  char Buffer[BufferLength];
  GdkColor OkColor={0, 0x0000, 0xFFFF, 0x0000}, FailedColor={0, 0xFFFF, 0x0000, 0x0000};
  
  CreateComponentLibrary(CompLib);
  LoadDefaultLib(*CompLib);
  
  LoadLibSet(*CompLib, LibSetFileName, Text);
  /* Spot Section */ 
  sprintf(Buffer, "\n\tLoading Spot Definition...\n");
  gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
  LibCursor = (*CompLib)->SpotLibList;

  while (LibCursor != NULL)
    {
      LocationCursor = (*CompLib)->LocationList;
      FileFound  = False;
      while( (LocationCursor != NULL) && (FileFound == False))
	{	  
	  sprintf(FullLibName,"%s/%s",LocationCursor->DirectoryName,LibCursor->LibName);
	  sprintf(Buffer,"Opening %s...",FullLibName);
 	  gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);

	  TestFile = fopen(FullLibName,"r");
	  
	  if (TestFile !=NULL)
	    {
	      sprintf(Buffer,"\tOk\n\n");
	      gtk_text_insert(GTK_TEXT(Text), NULL, &OkColor, NULL, Buffer, -1);
	      fclose(TestFile);
	      FileFound = True;
	      LoadSpotLib(*CompLib,FullLibName); 
	    }
	  else
	    {
	      sprintf(Buffer,"\tFailed!\n");
	      gtk_text_insert(GTK_TEXT(Text), NULL, &FailedColor, NULL, Buffer, -1);
	      LocationCursor = LocationCursor->pNext;
	    }
	}
      LibCursor = LibCursor->pNext;
    }
 
  /* Layout Section */ 
  sprintf(Buffer, "\n\tLoading Layout Definition...\n");
  gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
  LibCursor = (*CompLib)->LayoutLibList;
  while (LibCursor != NULL)
    {
      LocationCursor = (*CompLib)->LocationList;
      FileFound  = False;
      while( (LocationCursor != NULL) && (FileFound == False))
	{	  
	  sprintf(FullLibName,"%s/%s",LocationCursor->DirectoryName,LibCursor->LibName);
	  sprintf(Buffer,"Opening %s...",FullLibName);
	  gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
	  TestFile = fopen(FullLibName,"r");
	  
	  if (TestFile !=NULL)
	    {
	      sprintf(Buffer,"\tOk\n\n");
	      gtk_text_insert(GTK_TEXT(Text), NULL, &OkColor, NULL, Buffer, -1);
	      fclose(TestFile);
	      FileFound = True;
	      LoadLayoutLib(*CompLib,FullLibName);
	    }
	  else
	    {
	      sprintf(Buffer,"\tFailed!\n");
	      gtk_text_insert(GTK_TEXT(Text), NULL, &FailedColor, NULL, Buffer, -1);
	      LocationCursor = LocationCursor->pNext;
	    }
	}
      LibCursor = LibCursor->pNext;
    }

  /* CompDesc Section */ 
  sprintf(Buffer, "\n\tLoading CompDesc Definition...\n");
  gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
  LibCursor = (*CompLib)->CompDescLibList;
  
  while (LibCursor != NULL)
    {
      LocationCursor = (*CompLib)->LocationList;
      FileFound  = False;
      while( (LocationCursor != NULL) && (FileFound == False))
	{	  
	  sprintf(FullLibName,"%s/%s",LocationCursor->DirectoryName,LibCursor->LibName);
	  sprintf(Buffer,"Opening %s...",FullLibName);
	  gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
	  TestFile = fopen(FullLibName,"r");
	  
	  if (TestFile !=NULL)
	    {
	      sprintf(Buffer,"\tOk\n\n");
	      gtk_text_insert(GTK_TEXT(Text), NULL, &OkColor, NULL, Buffer, -1);
	      fclose(TestFile);
	      FileFound = True;
	      LoadCompDescLib(*CompLib,FullLibName);
	    }
	  else
	    {
	      sprintf(Buffer,"\tFailed!\n");
	      gtk_text_insert(GTK_TEXT(Text), NULL, &FailedColor, NULL, Buffer, -1);
	      LocationCursor = LocationCursor->pNext;
	    }
	}
      LibCursor = LibCursor->pNext;
    }
  DisplayLib(*CompLib);
  return AllOk;
}
ErrorCheck LoadDefaultSpotLib(ComponentLibrary *CompLib)
{
  SpotDef DefaultSpot;

  DefaultSpot.SpotSize = 0;
  DefaultSpot.CoreSpotSize = 1;
  DefaultSpot.InvisibleSpotSize = 0;
  DefaultSpot.CoreSpotLocation = (Point *) malloc(sizeof(Point) * DefaultSpot.CoreSpotSize);
  DefaultSpot.CoreSpotLocation[0].x = 2;
  DefaultSpot.CoreSpotLocation[0].y = 2;
  
  strncpy(DefaultSpot.SpotName,"Default", SpotNameLength);
  DefaultSpot.SpotName[SpotNameLength-1] = '\0';

  AddSpotDef(CompLib, DefaultSpot);
  if (DefaultSpot.SpotSize != 0)
    free(DefaultSpot.SpotLocation);
  if (DefaultSpot.CoreSpotSize != 0)
    free(DefaultSpot.CoreSpotLocation);
  if (DefaultSpot.InvisibleSpotSize != 0)
    free(DefaultSpot.InvisibleSpotLocation);
  return AllOk;
}
ErrorCheck LoadDefaultLayoutLib(ComponentLibrary *CompLib)
{
  LayoutDef DefaultLayout;
  int Counter;

  strncpy(DefaultLayout.LayoutName ,"Default", LayoutNameLength);
  DefaultLayout.LayoutName[LayoutNameLength-1] = '\0';
  strncpy(DefaultLayout.LayoutClass, "Default", LayoutClassLength);
  DefaultLayout.LayoutClass[LayoutClassLength-1] = '\0';
  
  DefaultLayout.PinNb = 1;
  DefaultLayout.PinPosition = (Point *) malloc (sizeof(Point) *  DefaultLayout.PinNb);
  DefaultLayout.PinPosition[0].x = 2; 
  DefaultLayout.PinPosition[0].y = 2;
  DefaultLayout.BetweenPinsSize = 0;
  DefaultLayout.CompoInSize = 0;
  DefaultLayout.CompoOutSize = 0;
  
  DefaultLayout.SpotNameTab = (char **) malloc (sizeof(char *) * DefaultLayout.PinNb);
  
  for (Counter = 0 ; Counter <  DefaultLayout.PinNb ; Counter++) 
    {
      DefaultLayout.SpotNameTab[Counter] =  (char *) malloc (sizeof(char) * SpotNameLength);
      strncpy(DefaultLayout.SpotNameTab[Counter],"Default", SpotNameLength);
      DefaultLayout.SpotNameTab[Counter][SpotNameLength-1] = '\0';
    }

  AddLayoutDef(CompLib, DefaultLayout);
  if (DefaultLayout.PinNb != 0) 
    { 
      for (Counter = 0 ; Counter < DefaultLayout.PinNb ; Counter++) 
	free(DefaultLayout.SpotNameTab[Counter]);
      free(DefaultLayout.SpotNameTab);
      free(DefaultLayout.PinPosition);
    }
  return AllOk;
}

ErrorCheck LoadDefaultCompDescLib(ComponentLibrary *CompLib)
{
  CompDescDef DefaultCompDesc;
  int Counter;
  
  strncpy(DefaultCompDesc.CompDescName, "Default", CompDescNameLength);
  DefaultCompDesc.CompDescName[CompDescNameLength-1] = '\0';
  strncpy(DefaultCompDesc.CompDescClass, "Default", CompDescClassLength);
  DefaultCompDesc.CompDescClass[CompDescClassLength-1] = '\0';
  strncpy(DefaultCompDesc.LayoutName, "Default", LayoutNameLength);
  DefaultCompDesc.LayoutName[LayoutNameLength-1] = '\0';
  strncpy(DefaultCompDesc.LayoutClass, "Default", LayoutClassLength);
  DefaultCompDesc.LayoutClass[LayoutClassLength-1] = '\0';
  
  DefaultCompDesc.PinNb = 1;
  DefaultCompDesc.PinNameTab = (char **) malloc (sizeof(char *) * DefaultCompDesc.PinNb);
  for (Counter = 0 ; Counter < DefaultCompDesc.PinNb ; Counter++)
    {
      DefaultCompDesc.PinNameTab[Counter] = (char *) malloc( sizeof(char )  * PinNameLength);
      strncpy(DefaultCompDesc.PinNameTab[Counter], "Default", PinNameLength);
      DefaultCompDesc.PinNameTab[Counter][PinNameLength-1] = '\0';
    }
  AddCompDescDef(CompLib, DefaultCompDesc);
  
  if (DefaultCompDesc.PinNb != 0)
    {
      for (Counter = 0 ; Counter < DefaultCompDesc.PinNb ; Counter++)
	free( DefaultCompDesc.PinNameTab[Counter]);
      free(DefaultCompDesc.PinNameTab);
    }
  
  return AllOk;
}
  
ErrorCheck LoadDefaultLib(ComponentLibrary *CompLib)
{
  LoadDefaultCompDescLib(CompLib);
  LoadDefaultLayoutLib(CompLib); 
  LoadDefaultSpotLib(CompLib);
  return AllOk;
}

ErrorCheck DisplayLib(ComponentLibrary *CompLib)
{
  DisplaySpotLib(CompLib);
  DisplayLayoutLib(CompLib);
  DisplayCompDescLib(CompLib);
  return AllOk;
}
  
