
#include "componentwindow.h"
#include "liste.h"
#include "interface.h"
#include "subfunction.h"
#include "circuitviewwindow.h"
#include "mainwindow.h"
#include "component.h"
#include "linkwindow.h"
#include "editwindows.h"
#include "link.h"
#include "dlink.h"
#include "attachpoint.h"
#include "support_custom.h"

void cb_AddSubFunction(GtkWidget *Button, GtkWidget *Dummy)
{
  GtkWidget *SubTree, *NewTreeItem , *RootItem, *ComponentTreeWindow;
  int FatherID, NewID;
  GCD *CircuitData;
  
  if (IRQ(True) == False)
    return;
  CircuitData = GetCircuitData();
  RootItem = CircuitData->Info.CTRootItem;
  
  FatherID = GPOINTER_TO_INT( gtk_object_get_data( GTK_OBJECT(RootItem), ObjID));
  if (FatherID != 0)
    {
      IRQ(False);
      return;
    }

  SubTree = GTK_TREE_ITEM_SUBTREE( GTK_TREE_ITEM(RootItem));
  if (SubTree == NULL )
    {
      SubTree = gtk_tree_new();
      gtk_tree_set_view_mode (GTK_TREE(SubTree), GTK_TREE_VIEW_ITEM);
      gtk_tree_item_set_subtree (GTK_TREE_ITEM(RootItem), SubTree);
    }
  
  ComponentTreeWindow = CircuitData->Info.ComponentWindow;
    
  NewID = NewSubFunctionID(CircuitData, NoID);
  NewTreeItem = CreateNewTreeItem(NULL, NewID, SubTree, ComponentTreeWindow);

  AddSubFunction(CircuitData, NewID);
  SetIIDForSubFunction(CircuitData, NewID, (void *)NewTreeItem);

  gtk_tree_item_expand(GTK_TREE_ITEM( RootItem));
  UpdateCircuitView(UCVModification);
  CircuitData->Info.NewModification = True;
  IRQ(False);
}

void cb_AddComponent(GtkWidget *Button, GtkWidget *RootTree)
{
  GtkWidget *SubTree, *NewTreeItem, *ComponentTreeWindow, *TreeItem;
  int FatherID, NewID;
  GCD *CircuitData;
  GList *List;

  if (IRQ(True) == False)
    return;
  CircuitData = GetCircuitData();

  List = GTK_TREE_SELECTION( GTK_TREE(RootTree));
  if (List == NULL)
    {
      IRQ(False);
      return;
    }
  TreeItem = GTK_WIDGET(List->data);
  FatherID = GPOINTER_TO_INT (gtk_object_get_data( GTK_OBJECT(TreeItem), ObjID));

  if (!IsSubFunctionID(FatherID))
    {
      IRQ(False);
      return ;
    }
  
  SubTree = GTK_TREE_ITEM_SUBTREE( GTK_TREE_ITEM(TreeItem));
  if (SubTree == NULL )
    {
      SubTree = gtk_tree_new();
      gtk_tree_set_view_mode (GTK_TREE(SubTree), GTK_TREE_VIEW_ITEM);
      gtk_tree_item_set_subtree (GTK_TREE_ITEM(TreeItem), SubTree);
    }

  ComponentTreeWindow = CircuitData->Info.ComponentWindow;

  NewID = NewComponentID(CircuitData, NoID);

  NewTreeItem = CreateNewTreeItem(NULL, NewID, SubTree, ComponentTreeWindow);

  AddComponent(CircuitData, NewID, FatherID);
  SetIIDForComponent(CircuitData, NewID, (void *)NewTreeItem);
  gtk_tree_item_expand(GTK_TREE_ITEM( TreeItem));
  
  AddAttachPointUnderComponent(NewID);
  UpdateCircuitView(UCVModification);
  CircuitData->Info.NewModification = True;
  IRQ(False);
}

void cb_CTCut(GtkWidget *Button, GtkWidget* RootTree)
{
  gpointer NewID;
  GList *List;
  GtkWidget *TreeItem;
  GCD *CircuitData;

  if (IRQ(True) == False)
    return;
  CircuitData  = GetCircuitData();

  List = GTK_TREE_SELECTION( GTK_TREE(RootTree));
  if (List == NULL)
    {
      IRQ(False);
      return;
    }

  TreeItem = GTK_WIDGET(List->data);
  NewID = gtk_object_get_data( GTK_OBJECT(TreeItem), ObjID);
  CutWithID( GPOINTER_TO_INT(NewID));
  IRQ(False);
}
void CutWithID(int CutID)
{
  GCD *CircuitData;
  GtkWidget *TreeItem, *RootTree, *ErrorWindow;
  GList *List;
  ErrorCheck Check;

  CircuitData = GetCircuitData();

  RootTree = CircuitData->Info.CTComponentTree;
  List = GTK_TREE_SELECTION( GTK_TREE(RootTree));
  if ( IsComponentID(CutID)) 
    {
      GetIIDForComponent(CircuitData, CutID, (void **) &TreeItem);
      Check = CutComponent( CircuitData, CutID);
      if (Check == Forbidden)
	{
	  ErrorWindow = create_ComponentSuppressionForbidden();
	  gtk_widget_show(ErrorWindow);
	  return;
	}
      if(GTK_TREE_ITEM_SUBTREE(GTK_TREE_ITEM(TreeItem)) != NULL)
	gtk_tree_item_remove_subtree(GTK_TREE_ITEM(TreeItem));
      gtk_tree_remove_items(GTK_TREE(RootTree), List);
      UpdateCircuitView(UCVModification);
      CircuitData->Info.NewModification = True;
      UpdateLLClist();
      UpdateDLClist();
    }

  if (IsSubFunctionID(CutID))
    {
      GetIIDForSubFunction(CircuitData, CutID, (void **) &TreeItem);
      Check = CutSubFunction (CircuitData, GPOINTER_TO_INT(CutID));
      if (Check == Forbidden)
	{
	  ErrorWindow = create_SubFunctionSuppressionForbidden();
	  gtk_widget_show(ErrorWindow);
	  return;
	}
      if(GTK_TREE_ITEM_SUBTREE(GTK_TREE_ITEM(TreeItem)))
	gtk_tree_item_remove_subtree(GTK_TREE_ITEM(TreeItem));
      gtk_tree_remove_items(GTK_TREE(RootTree), List);
      UpdateCircuitView(UCVModification);
      CircuitData->Info.NewModification = True;
      UpdateLLClist();
      UpdateDLClist();
    }
}

void cb_CTCopy(GtkWidget *Button, GtkWidget* RootTree)
{
  gpointer NewID;
  GList *List;
  GtkWidget *TreeItem;

  if (IRQ(True) == False)
    return;
  List = GTK_TREE_SELECTION( GTK_TREE(RootTree));
  if (List == NULL)
    {
      IRQ(False);
      return;
    }

  TreeItem = GTK_WIDGET(List->data);
  NewID = gtk_object_get_data( GTK_OBJECT(TreeItem), ObjID);
  CopyWithID(GPOINTER_TO_INT(NewID));
  IRQ(False);
}
void CopyWithID(int CopyID)
{
  GCD *CircuitData;

  CircuitData  = GetCircuitData();
 
  if (IsComponentID(CopyID))
    {
      CopyComponent( CircuitData, CopyID);
      CircuitData->Info.NewModification = True;
    }
  if (IsSubFunctionID(CopyID))
    {
      CopySubFunction (CircuitData, CopyID);
      CircuitData->Info.NewModification = True;
    }
}

void cb_CTPaste(GtkWidget *Button, GtkWidget* RootTree)
{
  gpointer NewID;
  GList *List;
  GtkWidget *TreeItem;
  GCD *CircuitData;
  
  if (IRQ(True) == False)
    return;
  CircuitData  = GetCircuitData();

  List = GTK_TREE_SELECTION( GTK_TREE(RootTree));
  if (List == NULL)
    {
      IRQ(False);
      return;
    }

  TreeItem = GTK_WIDGET(List->data);
  NewID = gtk_object_get_data( GTK_OBJECT(TreeItem), ObjID);
  PasteWithID( GPOINTER_TO_INT(NewID), NoX, NoY);
  IRQ(False);
}    
void PasteWithID(int FatherID, int x, int y)
{
  GCD *CircuitData;
  GtkWidget *TreeItem, *ErrorWindow;
  int PasteID;
  ErrorCheck Check;
  
  CircuitData = GetCircuitData();
  if ( IsCircuitID(FatherID))
    {
      TreeItem = CircuitData->Info.CTRootItem;
      Check = PasteSubFunction(CircuitData, &PasteID, x, y);
      if (Check == BufferEmpty)
	{
	  ErrorWindow = create_SubFunctionBufferEmpty();
	  gtk_widget_show(ErrorWindow);
	  return;
	}
      AddWithIDUnder(PasteID, TreeItem);
      UpdateCircuitView(UCVModification);
      UpdateLLClist();
      CircuitData->Info.NewModification = True;
      gtk_tree_item_expand(GTK_TREE_ITEM( TreeItem));
    }

  if (IsSubFunctionID(FatherID))
    {
      GetIIDForSubFunction(CircuitData, FatherID, (void **) &TreeItem);
      Check = PasteComponent(CircuitData, FatherID, &PasteID, x, y);
      if (Check == BufferEmpty)
	{
	  ErrorWindow = create_ComponentBufferEmpty();
	  gtk_widget_show(ErrorWindow);
	  return;
	}
      AddWithIDUnder(PasteID, TreeItem);
      UpdateCircuitView(UCVModification);
      CircuitData->Info.NewModification = True;
      gtk_tree_item_expand(GTK_TREE_ITEM( TreeItem));
    }
}

void cb_CTSetStartPoint(GtkWidget *Button, GtkWidget *RootTree)
{
  gpointer NewID;
  GList *List;
  GCD *CircuitData;
  GtkWidget *TreeItem;

  if (IRQ(True) == False)
    return;
  CircuitData = GetCircuitData();

  List = GTK_TREE_SELECTION( GTK_TREE(RootTree));
  if (List == NULL)
    {
      IRQ(False);
      return;
    }

  TreeItem = GTK_WIDGET(List->data);
  NewID = gtk_object_get_data(GTK_OBJECT(TreeItem), ObjID);
  
  SetExtremityWithID(GPOINTER_TO_INT(NewID), True);
  IRQ(False);
}

void cb_CTSetEndPoint(GtkWidget *Button, GtkWidget* RootTree)
{
  gpointer NewID;
  GList *List;
  GCD *CircuitData;
  GtkWidget *TreeItem;

  if (IRQ(True) == False)
    return;
  CircuitData = GetCircuitData();

  List = GTK_TREE_SELECTION( GTK_TREE(RootTree));
  if (List == NULL)
    {
      IRQ(False);
      return;
    }

  TreeItem = GTK_WIDGET(List->data);
  NewID = gtk_object_get_data(GTK_OBJECT(TreeItem), ObjID);
  SetExtremityWithID(GPOINTER_TO_INT(NewID), False);
  IRQ(False);
}
void SetExtremityWithID(int AttachID, Boolean Start)
{
  GCD *CircuitData;
  
  CircuitData = GetCircuitData();
  if ( IsAttachID(AttachID))
    {
      if (Start == True)
	EditLink(CircuitData, True, 0, AttachID, ExceptionAttach);
      else
	EditLink(CircuitData, True, 0, ExceptionAttach, AttachID);
      
      CircuitData->Info.NewModification = True;
    }
  UpdateLLClist();
  UpdateDLClist();
  UpdateCircuitView(UCVModification);
}

void cb_CTEdit(GtkWidget *Button, GtkWidget* RootTree)
{
  gpointer NewID;
  GList *List;
  GCD *CircuitData;
  GtkWidget *TreeItem;

  if (IRQ(True) == False)
    return;
  CircuitData = GetCircuitData();

  List = GTK_TREE_SELECTION( GTK_TREE(RootTree));
  if (List == NULL)
    {
      IRQ(False);
      return;
    }

  TreeItem = GTK_WIDGET(List->data);
  NewID = gtk_object_get_data(GTK_OBJECT(TreeItem), ObjID);
  
  EditWithID(GPOINTER_TO_INT(NewID), NoX, NoY);
  IRQ(False);
}

void AddWithIDUnder(int ID, GtkWidget *RootItem)
{
  GtkWidget *SubTree, *NewTreeItem, *Window;
  int FatherID, Counter;
  GCD *CircuitData;

  CircuitData = GetCircuitData();

  FatherID = GPOINTER_TO_INT (gtk_object_get_data( GTK_OBJECT(RootItem), ObjID));

  SubTree = GTK_TREE_ITEM_SUBTREE( GTK_TREE_ITEM(RootItem));
  if (SubTree == NULL )
    {
      SubTree = gtk_tree_new();
      gtk_tree_item_set_subtree (GTK_TREE_ITEM(RootItem), SubTree);
    }

  Window = CircuitData->Info.ComponentWindow;
  if (IsComponentID(ID))
    {
      Component ComponentProp;

      GetComponentProperties(CircuitData, ID, &ComponentProp, False);
      NewTreeItem = CreateNewTreeItem(ComponentProp.ComponentName, ID, SubTree, Window);
      
      SetIIDForComponent(CircuitData, ID, (void *)NewTreeItem);
      AddAttachPointUnderComponent(ID);
      UpdateCircuitView(UCVModification);
   }
  if (IsSubFunctionID(ID))
    {
      SubFunction SFProp;
      int *CompIDTab,  CompNb;
      
      GetSubFunctionProperties(CircuitData, ID, &SFProp);
      NewTreeItem = CreateNewTreeItem(SFProp.Name, ID, SubTree, Window);

      SetIIDForSubFunction(CircuitData, ID, (void *)NewTreeItem);
      UpdateCircuitView(UCVModification);

      GetSubFunctionCompNumber(CircuitData, ID, &CompNb);
      if (CompNb != 0)
	{
	  CompIDTab = (int *) malloc( CompNb * sizeof(int));
	  GetSubFunctionCompIDTab(CircuitData, ID, CompIDTab);
	  
	  for (Counter = 0 ; Counter < CompNb; Counter++)
	    AddWithIDUnder(CompIDTab[Counter], NewTreeItem);
	  free(CompIDTab);
	}
    }

  CircuitData->Info.NewModification = True;
  gtk_tree_item_expand(GTK_TREE_ITEM(RootItem));
}

void AddAttachPointUnderComponent(int ID)
{
  GtkWidget *SubTree, *NewTreeItem , *Window, *TreeItem;
  int NbAttachPoint, Counter;
  GCD *CircuitData;
  Component ComponentProperties;
  AttachPoint AttachPointProperties;

  CircuitData = GetCircuitData();
  GetIIDForComponent(CircuitData, ID, (void **) &TreeItem);

  GetComponentAttachNb(CircuitData, ID, &NbAttachPoint);
  if ( NbAttachPoint== 0)
    return;
  GetComponentProperties(CircuitData, ID, &ComponentProperties, True);
  
  Window = CircuitData->Info.ComponentWindow;
  SubTree = GTK_TREE_ITEM_SUBTREE( GTK_TREE_ITEM(TreeItem));

  if (SubTree !=NULL)
    gtk_tree_item_remove_subtree(GTK_TREE_ITEM(TreeItem));
  
  SubTree = gtk_tree_new();
  gtk_tree_item_set_subtree (GTK_TREE_ITEM(TreeItem), SubTree);

  
  for (Counter = 0 ; Counter < NbAttachPoint ; Counter++)
    {
      GetAttachPointProperties(CircuitData, ComponentProperties.AttachPointIDTab[Counter], &AttachPointProperties);
      NewTreeItem = CreateNewTreeItem(AttachPointProperties.AttachPointName, 
				      ComponentProperties.AttachPointIDTab[Counter], SubTree, Window);
      SetIIDForAttachPoint(CircuitData, ComponentProperties.AttachPointIDTab[Counter], NewTreeItem);
    }
  gtk_tree_item_expand(GTK_TREE_ITEM( TreeItem));
  free (ComponentProperties.AttachPointIDTab);
  CircuitData->Info.NewModification = True;
}

ErrorCheck UpdateCTWindow(void)
{
  GCD *CircuitData;
  GtkWidget *CTRootItem, *RootTree, *ComponentWindow;
  SubFunction *SubFCursor;
  GList *List;
  CircuitData = GetCircuitData();
  
  CTRootItem = CircuitData->Info.CTRootItem;
  RootTree = CircuitData->Info.CTComponentTree;

  ComponentWindow = CircuitData->Info.ComponentWindow;
  List = NULL;
  List = g_list_append (List, CTRootItem);
  gtk_tree_remove_items(GTK_TREE(RootTree), List);
  
  CTRootItem = CreateNewTreeItem(CircuitData->DataHeader.Name, 0, RootTree, ComponentWindow);
  
  CircuitData->Info.CTRootItem = CTRootItem;
  gtk_tree_select_child(GTK_TREE(RootTree), GTK_WIDGET(CTRootItem ));  

  SubFCursor = CircuitData->ListSubFunction;
  while (SubFCursor != NULL)
    {
      AddWithIDUnder( SubFCursor->IDNumber, CTRootItem);
      SubFCursor = SubFCursor->pNext;
    }

  CircuitData->Info.NewModification = True;
  return AllOk;
}

ErrorCheck SelectFromID(int ID)
{
  GtkWidget *RootItem, *TreeItem=NULL, *DLClist, *RootTree;
  GCD *CircuitData;
  int RowData = NoID;

  CircuitData = GetCircuitData();

  if (IsLinkID(ID))
    { /* pour les liens */
      DLClist = CircuitData->Info.DLClist;
      cb_SelectNoneDLink(NULL, DLClist);
      RowData = gtk_clist_find_row_from_data( GTK_CLIST( DLClist), GINT_TO_POINTER(ID));
      if (RowData != NoID)
	gtk_clist_select_row( GTK_CLIST( DLClist), RowData, 0);

      if (RowData != NoID)
	return Found;
      else return NotFound;
    }
      
  
  switch (ID %10)
    {
    case ComponentIDRef : 
      GetIIDForComponent(CircuitData, ID, (void **) &TreeItem);
      break;
    case AttachIDRef : 
      GetIIDForAttachPoint(CircuitData, ID, (void **) &TreeItem);
      break;
    case SubFunctionIDRef : 
      GetIIDForSubFunction(CircuitData, ID, (void **) &TreeItem);
      break;
    default : printf("SelectID called with wrong type !!!\n");
    }
  
  RootItem = CircuitData->Info.CTRootItem;
  
  RootTree = gtk_object_get_data(GTK_OBJECT(RootItem), "Tree"); 
  gtk_tree_select_child(GTK_TREE(RootTree), GTK_WIDGET(TreeItem));
  return Found;
}


void cb_ComponentTree_selectionchanged (GtkTree *RootTree, GtkWidget *ComponentWindow)
{
  GtkWidget *TreeItem;
  int SelectedID;
  GCD *CircuitData;
  GList *List;

  CircuitData = GetCircuitData();

  List = GTK_TREE_SELECTION( GTK_TREE(RootTree));
  if (List == NULL)
      return;

  TreeItem = GTK_WIDGET(List->data);
  SelectedID = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(TreeItem), ObjID));
  if ( IsSubFunctionID(SelectedID))
    SelectSubFunctionFromID(CircuitData, SelectedID);
  if (IsComponentID(SelectedID)) 
    SelectComponentFromID(CircuitData, SelectedID); 

  SelectAttachPointFromID(CircuitData, SelectedID);
  UpdateCircuitView(UCVSelection);
}

GtkWidget *CreateNewTreeItem(char *Label, int ID, GtkWidget *SubTree, GtkWidget *Window)
{
  GtkWidget *HBox, *ItemLabel, *PixmapWidget = NULL, *TreeItem;
  GCD *CircuitData;

  CircuitData = GetCircuitData();
  TreeItem = gtk_tree_item_new ();

  HBox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (HBox);
  switch ( (ID %10))
    {
    case CircuitIDRef :   PixmapWidget = create_pixmap(Window,"xroadmaker/circuit.xpm", FALSE);
      if (Label == NULL)
	Label = DefaultCircuitName;
      break;
    case SubFunctionIDRef :   PixmapWidget = create_pixmap(Window,"xroadmaker/subfunction.xpm", FALSE);
      if (Label == NULL)
	Label = CircuitData->Info.Opt.DefaultSubFunctionName;
      break;
    case ComponentIDRef :   PixmapWidget = create_pixmap(Window,"xroadmaker/component.xpm", FALSE);
      if (Label == NULL)
	Label = CircuitData->Info.Opt.DefaultComponentName;
      break;
    case AttachIDRef :   PixmapWidget = create_pixmap(Window,"xroadmaker/attachpoint.xpm", FALSE);
      if (Label == NULL)
	Label = DefaultAttachName;
      break;
    default : PixmapWidget = create_pixmap(Window,"xroadmaker/circuit.xpm", FALSE);
      printf("\n*************\n Error In Tree Item ID \n**************\n");
    }
  gtk_widget_show( PixmapWidget );
  gtk_box_pack_start (GTK_BOX (HBox), PixmapWidget, FALSE, FALSE, 5);
  ItemLabel = gtk_label_new (Label);
  gtk_object_set_data( GTK_OBJECT(TreeItem),"Label", ItemLabel);
  gtk_object_set_data( GTK_OBJECT(TreeItem),"Window", Window);
  gtk_widget_show(ItemLabel);
  gtk_box_pack_start (GTK_BOX (HBox), ItemLabel, FALSE, FALSE, 5);
  gtk_container_add(GTK_CONTAINER(TreeItem), HBox);
  gtk_object_set_data(GTK_OBJECT(TreeItem), ObjID, GINT_TO_POINTER(ID));
  gtk_object_set_data(GTK_OBJECT(TreeItem), "Tree", SubTree);
  gtk_tree_append (GTK_TREE(SubTree), TreeItem);
  gtk_widget_show (TreeItem);
  return TreeItem;
}

