#include <stdlib.h>
#include <stdio.h>
#include "dlink.h"
#include "link.h"
#include "drawtracks.h"

ErrorCheck DeleteDLink(GCD *CircuitData, int LinkID)
{
  DrawLink *DLinkCursor;
  Boolean IDFound; 
  
  UnDrawLink(CircuitData, LinkID);
  
  DLinkCursor= CircuitData->TopLink;
  IDFound = False;
  while ((DLinkCursor!=NULL) && (IDFound == False))
   {
     if (DLinkCursor->LinkID== LinkID)
       {
	 IDFound = True;
	 
	 if (DLinkCursor->LinkUp != NULL)
	   DLinkCursor->LinkUp->LinkDown = DLinkCursor->LinkDown;
	 else
	   CircuitData->TopLink = DLinkCursor->LinkDown;

	 if (DLinkCursor->LinkDown!= NULL)
	   DLinkCursor->LinkDown->LinkUp = DLinkCursor->LinkUp;
	 else
	   CircuitData->BottomLink = DLinkCursor->LinkUp;
	 
	 free(DLinkCursor);
       }
     
     DLinkCursor = DLinkCursor->LinkDown;
   }    
  SetLinkNotToDraw(CircuitData, LinkID);

  if (IDFound == True)
    return Found;
  else return NotFound;
}

ErrorCheck SelectDLink(GCD *CircuitData, int LinkID)
{
  DrawLink *DLinkCursor;
  Boolean IDFound;

  IDFound  = False;

  DLinkCursor=CircuitData->TopLink;
  while ((DLinkCursor!=NULL) && (IDFound == False))
    {
      if(DLinkCursor->LinkID == LinkID)
	{
	  IDFound = True;
	  SetDLinkSelected(DLinkCursor->State);
	}
      DLinkCursor  = DLinkCursor->LinkDown;
    }
  if (IDFound == True)
    return Found;
  else return NotFound;
}
ErrorCheck UnSelectDLink(GCD *CircuitData, int LinkID)
{
  DrawLink *DLinkCursor;
  Boolean IDFound;

  IDFound  = False;

  DLinkCursor=CircuitData->TopLink;
  while ((DLinkCursor!=NULL) && (IDFound == False))
    {
      if(DLinkCursor->LinkID == LinkID)
	{
	  IDFound = True;
	  UnSetDLinkSelected(DLinkCursor->State);
	}
      DLinkCursor  = DLinkCursor->LinkDown;
    }

  if (IDFound == True)
    return Found;
  else return NotFound;
}
ErrorCheck GetSelectDLink(GCD *CircuitData, int* TabSelectedDLink, int *TabSize)
{
  int Counter;
  DrawLink *DLinkCursor;

  Counter=0; 
  DLinkCursor=CircuitData->TopLink;
  while (DLinkCursor!=NULL) 
    {
      if (IsDLinkSelected(DLinkCursor->State))
	{
	  TabSelectedDLink[Counter]=DLinkCursor->LinkID;
	  Counter++;
	}
     DLinkCursor=DLinkCursor->LinkDown;
    }

  *TabSize=Counter;
  return AllOk;
}
ErrorCheck GetUnSelectDLink(GCD *CircuitData, int* TabUnSelectedDLink, int *TabSize)
{
  int Counter;
  DrawLink *DLinkCursor;

  Counter=0; 
  DLinkCursor=CircuitData->TopLink;
  while (DLinkCursor!=NULL) 
    {
      if( !IsDLinkSelected(DLinkCursor->State))
	{
	  TabUnSelectedDLink[Counter]=DLinkCursor->LinkID;
	  Counter++;
	}
     DLinkCursor=DLinkCursor->LinkDown;
    }

  *TabSize=Counter;
  return AllOk;
}


ErrorCheck GetAllDLink(GCD *CircuitData, int* TabDLink, int *DLinkNb, Boolean Routed)
{
  int Counter;
  DrawLink *DLinkCursor;
  Boolean DLinkCursorRouted;

  Counter=0; 
  DLinkCursor=CircuitData->TopLink;
  while (DLinkCursor!=NULL) 
    {
      DLinkCursorRouted = (IsDLinkRouted(DLinkCursor->State)) ? True : False;
      if ((Routed == Maybe) || (Routed == DLinkCursorRouted))
	{
	  TabDLink[Counter]=DLinkCursor->LinkID;
	  Counter++;
	}
      DLinkCursor=DLinkCursor->LinkDown;
    }
  *DLinkNb = Counter;
  return AllOk;
}

ErrorCheck ExchangeDrawLink(GCD *CircuitData,DrawLink *DLink1, DrawLink *DLink2)
{
  DrawLink *Temp;
  
  UnDrawLink(CircuitData, DLink1->LinkID);
  UnDrawLink(CircuitData, DLink2->LinkID);

  if ((DLink1->LinkUp != DLink2) && (DLink1->LinkDown != DLink2))
    {/* Deux maillons non adjacents */
      Temp = DLink1->LinkUp;
      DLink1->LinkUp = DLink2->LinkUp;
      DLink2->LinkUp = Temp;
      
      Temp = DLink1->LinkDown;
      DLink1->LinkDown = DLink2->LinkDown;
      DLink2->LinkDown = Temp;
    }
  if (DLink1->LinkUp == DLink2) 
    {
      Temp = DLink1->LinkDown;
      DLink1->LinkDown = DLink2;
      DLink2->LinkDown = Temp; 
      
      Temp = DLink2->LinkUp;
      DLink2->LinkUp = DLink1;
      DLink1->LinkUp = Temp;
    }
  
  else
    if (DLink1->LinkDown == DLink2)
      {
	Temp = DLink1->LinkUp;
	DLink1->LinkUp = DLink2;
	DLink2->LinkUp = Temp;
	
	Temp = DLink2->LinkDown;
	DLink2->LinkDown = DLink1;
	DLink1->LinkDown = Temp;
      }
  
  if (DLink1->LinkUp == NULL)
      CircuitData->TopLink = DLink1;
  else 
    DLink1->LinkUp->LinkDown = DLink1;

  if (DLink2->LinkUp == NULL)
      CircuitData->TopLink = DLink2;
  else 
    DLink2->LinkUp->LinkDown = DLink2;
  
 if (DLink1->LinkDown == NULL)
     CircuitData->BottomLink = DLink1;
 else
   DLink1->LinkDown->LinkUp = DLink1;
  
  if (DLink2->LinkDown == NULL)
      CircuitData->BottomLink = DLink2;
  else 
    DLink2->LinkDown->LinkUp = DLink2;

  return AllOk;
}
     
ErrorCheck InsertDrawLink(GCD *CircuitData, DrawLink *Source, DrawLink *Dest, Boolean InsertUnder)
{
  UnDrawLink(CircuitData, Source->LinkID);
  UnDrawLink(CircuitData, Dest->LinkID);
  
  if (Source == Dest)
    printf("Trying to insert Source and Dest are same\n");

  if (Source->LinkDown == NULL)
    CircuitData->BottomLink = Source->LinkUp;
  else 
    Source->LinkDown->LinkUp = Source->LinkUp;
  
  if (Source->LinkUp == NULL)
    CircuitData->TopLink = Source->LinkDown;
  else 
    Source->LinkUp->LinkDown = Source->LinkDown;

  if (InsertUnder == True)
    {
      Source->LinkUp = Dest;
      Source->LinkDown = Dest->LinkDown;
      Dest->LinkDown = Source;
    }
  else 
    {
      Source->LinkDown = Dest;
      Source->LinkUp = Dest->LinkUp;
      Dest->LinkUp = Source;
    }
  if (Source->LinkUp == NULL)
    CircuitData->TopLink = Source;
  else 
    Source->LinkUp->LinkDown = Source;

  if (Source->LinkDown == NULL)
    CircuitData->BottomLink = Source;
  else
    Source->LinkDown->LinkUp = Source;

  if (Dest->LinkUp == NULL)
    CircuitData->TopLink = Dest;
  else 
    Dest->LinkUp->LinkDown = Dest;

  if (Dest->LinkDown == NULL)
    CircuitData->BottomLink = Dest;
  else
    Dest->LinkDown->LinkUp = Dest;

  return AllOk; 
}


ErrorCheck GetDLinkNumber( GCD *CircuitData, int *DLinkNumber)
{
  int Counter;
  
  DrawLink *DLinkCursor;

  DLinkCursor = CircuitData->TopLink;
  Counter = 0;
  while (DLinkCursor != NULL)
    {
      Counter++;
      DLinkCursor = DLinkCursor->LinkDown;
    }
  *DLinkNumber  = Counter;
  return AllOk;
}
ErrorCheck GetDLinkProperties( GCD *CircuitData, int DLinkID, DrawLink *DLinkProp)
{
  Boolean IDFound;
  DrawLink *DLinkCursor;

  DLinkCursor = CircuitData->TopLink;
  IDFound = False;
  while ((DLinkCursor != NULL) && (IDFound== False))
    {
      if (DLinkCursor->LinkID == DLinkID)
	{
	  IDFound = True;
	  DLinkProp->LinkID = DLinkCursor->LinkID;
	  DLinkProp->LinkLength = DLinkCursor->LinkLength;
	  DLinkProp->LinkWidth = DLinkCursor->LinkWidth;
	  DLinkProp->State = DLinkCursor->State;
	  DLinkProp->RoutedOrder =  DLinkCursor->RoutedOrder;
	}
      DLinkCursor = DLinkCursor->LinkDown;
    }
  if (IDFound == True)
    return Found;
  else return NotFound;

}

ErrorCheck DisplayDrawList(GCD *CircuitData)
{
  DrawLink *DLinkCursor;

  printf(" ------------------------ Draw List\n");

  DLinkCursor = CircuitData->TopLink;
  
  while (DLinkCursor != NULL)
    {
      printf("LinkID %d Length %d State %d \n", DLinkCursor->LinkID, DLinkCursor->LinkLength, DLinkCursor->State);

      DLinkCursor = DLinkCursor->LinkDown;
    }
  return AllOk;
}

ErrorCheck AddLinkToDrawList(GCD *CircuitData, int LinkID)
{
  DrawLink *NewDLink;
  Link LinkProp;

  NewDLink = (DrawLink *) malloc( sizeof(DrawLink) );
  
  if (NewDLink == NULL)
    {
      printf("Error\n");
      return MemoryAllocationError;
    }
  
  NewDLink->LinkID = LinkID;
  SetLinkToDraw(CircuitData, LinkID);
  GetLinkProperties(CircuitData, LinkID, &LinkProp);
  
  NewDLink->LinkLength = LinkProp.LinkLength;
  NewDLink->LinkWidth = LinkProp.LinkWidth;
  printf("Add Width %d\n", NewDLink->LinkWidth); 
  InitState(NewDLink->State);
  UnSetDLinkSelected(NewDLink->State);
  SetDLinkSortedOk(NewDLink->State);
  UnSetDLinkRouted(NewDLink->State);
  NewDLink->RoutedOrder = NoOrder;
  NewDLink->LinkUp = CircuitData->BottomLink;
  NewDLink->LinkDown = NULL;

  CircuitData->BottomLink = NewDLink;

  if (NewDLink->LinkUp != NULL)
    {
      NewDLink->LinkUp->LinkDown = NewDLink;
    }
  else 
    CircuitData->TopLink = NewDLink;
  return AllOk;
}
ErrorCheck UpdateDLink(GCD *CircuitData, int LinkID)
{
  DrawLink *DLinkCursor;
  Link LinkProp;
  Boolean IDFound;

  GetLinkProperties(CircuitData, LinkID, &LinkProp);
  
  DLinkCursor = CircuitData->TopLink; 
  IDFound = False;
  while ((DLinkCursor!=NULL) && (IDFound == False))
    {
      if (DLinkCursor->LinkID == LinkID)
	{
	  IDFound = True;
	  DLinkCursor->LinkLength = LinkProp.LinkLength;
	  DLinkCursor->LinkWidth = LinkProp.LinkWidth;
	  printf("UpdateDL %d",DLinkCursor->LinkWidth);
	}
      DLinkCursor = DLinkCursor->LinkDown;
    }
  if (IDFound == True)
    return Found;
  else return NotFound;
}


ErrorCheck ScrollUp(GCD *CircuitData)
{
  DrawLink *DLinkCursor;
  
  DLinkCursor = CircuitData->TopLink;
  while (DLinkCursor!= NULL)
    {
      if (IsDLinkSelected(DLinkCursor->State))
	UnSetDLinkSortedOk(DLinkCursor->State);
      else 
	SetDLinkSortedOk(DLinkCursor->State);

      DLinkCursor = DLinkCursor->LinkDown;
    }
  
  DLinkCursor = CircuitData->TopLink;
  while (DLinkCursor!= NULL)
    {     
      if ((IsDLinkSortedOk(DLinkCursor->State)) && ( DLinkCursor->LinkDown != NULL) 
	  && (!IsDLinkSortedOk(DLinkCursor->LinkDown->State)))
	{
	  SetDLinkSortedOk(DLinkCursor->LinkDown->State);
	  ExchangeDrawLink(CircuitData,DLinkCursor, DLinkCursor->LinkDown);
	}
      else
	DLinkCursor = DLinkCursor->LinkDown;
    }
  return AllOk;
}

ErrorCheck ScrollDown(GCD *CircuitData)
{
  DrawLink *DLinkCursor;
  
  DLinkCursor = CircuitData->BottomLink;
  while (DLinkCursor!= NULL)
    {
      if (IsDLinkSelected(DLinkCursor->State))
	UnSetDLinkSortedOk(DLinkCursor->State);
      else 
	SetDLinkSortedOk(DLinkCursor->State);
      DLinkCursor = DLinkCursor->LinkUp;
    }
  
  DLinkCursor = CircuitData->BottomLink;
  while (DLinkCursor!= NULL)
    {     
      if ((IsDLinkSortedOk(DLinkCursor->State)) && ( DLinkCursor->LinkUp != NULL) 
	  && (!IsDLinkSortedOk(DLinkCursor->LinkUp->State)))
	{
	  SetDLinkSortedOk(DLinkCursor->LinkUp->State);
	  ExchangeDrawLink(CircuitData, DLinkCursor, DLinkCursor->LinkUp);
	}
      else
	DLinkCursor = DLinkCursor->LinkUp;
    }
  return AllOk;
}
ErrorCheck ScrollTop(GCD *CircuitData)
{
  DrawLink *DLinkFreePos, *DLinkToSort, *DLinkCursor, *NextSort;
  Boolean FreePosNotFound;

  DLinkCursor = CircuitData->TopLink;
  while (DLinkCursor!= NULL)
    {
      if (IsDLinkSelected(DLinkCursor->State))
	UnSetDLinkSortedOk(DLinkCursor->State);
      else 
	SetDLinkSortedOk(DLinkCursor->State);
      DLinkCursor = DLinkCursor->LinkDown;
    }
  
  DLinkFreePos = CircuitData->TopLink;
  while ((DLinkFreePos != NULL) && (!IsDLinkSortedOk(DLinkFreePos->State)))
    DLinkFreePos = DLinkFreePos->LinkDown; 
  
  FreePosNotFound = False;
  if (DLinkFreePos == NULL) 
    FreePosNotFound = True;

  DLinkToSort = CircuitData->TopLink;
  while ((DLinkToSort != NULL) && (FreePosNotFound == False))
    {
      NextSort = DLinkToSort->LinkDown;
      if (!IsDLinkSortedOk(DLinkToSort->State))
	{
	  SetDLinkSortedOk(DLinkToSort->State);
	  InsertDrawLink(CircuitData, DLinkToSort, DLinkFreePos, False);
	}
	 
      DLinkToSort = NextSort;
    }
  return AllOk;
}

ErrorCheck ScrollBottom(GCD *CircuitData)
{
  DrawLink *DLinkFreePos, *DLinkToSort, *DLinkCursor, *NextSort;
  Boolean FreePosNotFound;

  DLinkCursor = CircuitData->BottomLink;
  while (DLinkCursor!= NULL)
    {
      if (IsDLinkSelected(DLinkCursor->State))
	UnSetDLinkSortedOk(DLinkCursor->State);
      else 
	SetDLinkSortedOk(DLinkCursor->State);
      DLinkCursor = DLinkCursor->LinkUp;
    }
  
  DLinkFreePos = CircuitData->BottomLink;
  while ((DLinkFreePos != NULL) && (!IsDLinkSortedOk(DLinkFreePos->State)))
    DLinkFreePos = DLinkFreePos->LinkUp; 

  FreePosNotFound = False;
  if (DLinkFreePos == NULL) 
    FreePosNotFound = True;

  DLinkToSort = CircuitData->BottomLink;
  while ((DLinkToSort != NULL) && (FreePosNotFound == False))
    {
      NextSort = DLinkToSort->LinkUp;
      if (!IsDLinkSortedOk(DLinkToSort->State))
	{
	  SetDLinkSortedOk(DLinkToSort->State);
	  InsertDrawLink(CircuitData, DLinkToSort, DLinkFreePos, True);
	}
      DLinkToSort = NextSort;
    }

  return AllOk;
}

ErrorCheck CheckDrawListConsistency(GCD *CircuitData)
{
  int OrderCounter;
  DrawLink *DLinkCursor;
  Link *LinkCursor;
  Boolean UnDrawStart, IDFound, LinkNotRoutedFound;
  
  DLinkCursor = CircuitData->TopLink;
  UnDrawStart = False;
  OrderCounter = 0;
  while (DLinkCursor != NULL)
    {
      if ( (!IsDLinkRouted(DLinkCursor->State) || (DLinkCursor->RoutedOrder != OrderCounter)))
	UnDrawStart = True;
      if ((UnDrawStart == True) && (IsDLinkRouted(DLinkCursor->State)))
	{
	  UnSetDLinkRouted(DLinkCursor->State);
	  DLinkCursor->RoutedOrder = NoOrder;
	  
	  LinkCursor = CircuitData->ListLink;
	  IDFound = False;
	  while ((LinkCursor != NULL) && (IDFound == False))
	    {
	      if (LinkCursor->IDNumber == DLinkCursor->LinkID)
		{
		  IDFound = True;
		  UnSetLinkRouted(LinkCursor->State);
		  LinkCursor->RoutedOrder = NoOrder;
		  LinkCursor->Road.JobDone = False;
		  if (LinkCursor->Road.SegSize != 0)
		    free(LinkCursor->Road.SegTab);
		  if (LinkCursor->Road.PathSize != 0)
		    free(LinkCursor->Road.PathTab);
		  if (LinkCursor->Road.DiagonalSize != 0)
		    free(LinkCursor->Road.DiagonalTab);
		  if (LinkCursor->Road.DepSize != 0)
		    free(LinkCursor->Road.DepTab);
		  LinkCursor->Road.SegSize = 0;
		  LinkCursor->Road.PathSize = 0;
		  LinkCursor->Road.DiagonalSize = 0;
		  LinkCursor->Road.DepSize = 0;
		}
	      LinkCursor = LinkCursor->pNext;
	    }
	  if (IDFound == False)
	    printf("Error Link to be undrawn not found\n");
	}
      OrderCounter++;
      DLinkCursor = DLinkCursor->LinkDown;
    }

  OrderCounter = 0;
  DLinkCursor = CircuitData->TopLink;
  LinkNotRoutedFound = False;
  while ((DLinkCursor != NULL) && (LinkNotRoutedFound == False))
    {
      if (IsDLinkRouted(DLinkCursor->State))
	OrderCounter++;
      else 
	LinkNotRoutedFound = True;
      DLinkCursor = DLinkCursor->LinkDown;
    }
  GetRoutedOrder(CircuitData, OrderCounter); /* il n y a pas de trous dans les RoutedOrder et on continue a numeroter a la suite */
  return AllOk;
}

ErrorCheck ExtendByColorDLink(GCD *CircuitData)
{
  DrawLink *DLinkCursor;
  Link LinkProp;
  int *RedTab, *GreenTab, *BlueTab, LinkNb, ColorCounter, Counter;
  Boolean NewColor, InColor;

  GetDLinkNumber(CircuitData, &LinkNb);
  if (LinkNb < 2) /* si il n y a qu un lien on ne peut etendre la selection */
    return AllOk;

  RedTab = (int *) malloc(sizeof(int) * LinkNb);
  GreenTab = (int *) malloc(sizeof(int) * LinkNb);
  BlueTab = (int *) malloc(sizeof(int) * LinkNb);
 
  DLinkCursor = CircuitData->TopLink;
  ColorCounter = 0;
  while (DLinkCursor != NULL)
    {
      if (IsDLinkSelected(DLinkCursor->State))
	{
	  GetLinkProperties(CircuitData, DLinkCursor->LinkID, &LinkProp);
	  NewColor = True;
	  for (Counter = 0 ; (NewColor == True) && (Counter < ColorCounter); Counter++)
	    if ( ( RedTab[Counter] == LinkProp.LinkColor.Red) 
		 && (GreenTab[Counter] == LinkProp.LinkColor.Green) 
		 && (BlueTab[Counter] == LinkProp.LinkColor.Blue)) 
	      NewColor = False;

	  if (NewColor == True)
	    {
	      RedTab[ColorCounter] = LinkProp.LinkColor.Red;
	      GreenTab[ColorCounter] = LinkProp.LinkColor.Green;
	      BlueTab[ColorCounter] = LinkProp.LinkColor.Blue;
	      ColorCounter++;
	    }
	}
      DLinkCursor = DLinkCursor->LinkDown;
    }

  DLinkCursor = CircuitData->TopLink;
  while (DLinkCursor != NULL)
    {
      if (!IsDLinkSelected(DLinkCursor->State))
	{
	  GetLinkProperties(CircuitData, DLinkCursor->LinkID, &LinkProp);
	  InColor = False;
	  for (Counter = 0 ; (InColor == False) && (Counter < ColorCounter) ; Counter++)
	    if ( (LinkProp.LinkColor.Red == RedTab[Counter]) 
		 && ( LinkProp.LinkColor.Green == GreenTab[Counter])
		 && ( LinkProp.LinkColor.Blue == BlueTab[Counter]))
	      InColor = True;
	  
	  if (InColor == True)
	    SetDLinkSelected(DLinkCursor->State);
	}
      DLinkCursor = DLinkCursor->LinkDown;
    }
  free(RedTab);
  free(GreenTab);
  free(BlueTab);

  return AllOk;
}





