
#include <stdlib.h>
#include <stdio.h>
#include <gnome.h>

#include "drawtracks.h"
#include "subfunction.h"
#include "component.h"
#include "attachpoint.h"
#include "dlink.h"
#include "link.h"
#include "roadncircuit.h"
#include "computecircuit.h"
#include "liste.h"
#include "filltab.h"
#include "findpath.h"
#include "misc.h"
#include "mainwindow.h"
#include "miscwindows.h"
#include "layout.h"

ErrorCheck GetRoutedOrder(GCD *CircuitData, int SetOrder)
{
  CircuitData->Info.Order++;
  if (SetOrder != NoOrder)
    CircuitData->Info.Order = SetOrder - 1;

  return CircuitData->Info.Order;
} 

ErrorCheck DrawOneTrack(GCD *CircuitData, int RoutingID)
{
  int Counter, AttachPointNb, DLinkNb, Deltax, Deltay, *PreviousTrackToDraw, PreviousTrackNb, AttachID, LinkID;
  AttachPoint *AttachCursor, AttachProp;
  Circuit Board;
  RoadMap Map;
  DrawLink  *DLinkCursor;
  Link LinkProp;
  Point Meet;
  ErrorCheck Match;
  SubFunction SubFProp;
  Component CompProp;
  
  GetAttachPointNumber(CircuitData, &AttachPointNb);
  GetDLinkNumber(CircuitData, &DLinkNb);

  /* remplissage du tableau des preferences de directions */
  Board.DirectionPrefTab[0] = CircuitData->Info.Opt.DirectionPrefTab[0];
  Board.DirectionPrefTab[1] = CircuitData->Info.Opt.DirectionPrefTab[1];
  Board.DirectionPrefTab[2] = CircuitData->Info.Opt.DirectionPrefTab[2];

  /* Remplissage de AttachTab */
  Board.AttachNb = AttachPointNb;
  if ( Board.AttachNb != 0)
    Board.AttachTab = (AttachCarac *) malloc(sizeof(AttachCarac) * Board.AttachNb);

  AttachCursor  = CircuitData->ListAttachPoint;
  while (AttachCursor!=NULL)
    {
      
      Board.AttachTab[AttachCursor->RoutingAttachID].Size = NoAttach;
      Board.AttachTab[AttachCursor->RoutingAttachID].Attach2Link = NULL;
      
      Board.AttachTab[AttachCursor->RoutingAttachID].AttachLocation = (Point *) malloc(sizeof(Point) * AttachCursor->SpotSize);
  
      GetComponentProperties(CircuitData, AttachCursor->ComponentID, &CompProp, False);
      GetSubFunctionProperties(CircuitData, CompProp.SubFunctionID, &SubFProp);
      Deltax = SubFProp.Reference.x + CompProp.Reference.x + AttachCursor->Reference.x;
      Deltay = SubFProp.Reference.y + CompProp.Reference.y + AttachCursor->Reference.y;
      AttachID = AttachCursor->RoutingAttachID;
      Board.AttachTab[AttachCursor->RoutingAttachID].AttachLocationNb = AttachCursor->SpotSize;
      Board.AttachTab[AttachCursor->RoutingAttachID].CoreAttachLocationNb = AttachCursor->CoreSpotSize;
      Board.AttachTab[AttachCursor->RoutingAttachID].InvisibleAttachLocationNb = AttachCursor->InvisibleSpotSize;
      if (AttachCursor->SpotSize !=0)
	Board.AttachTab[AttachCursor->RoutingAttachID].AttachLocation = 
	  (Point *) malloc(sizeof(Point) * AttachCursor->SpotSize);
      if (AttachCursor->CoreSpotSize !=0)
	Board.AttachTab[AttachCursor->RoutingAttachID].CoreAttachLocation = 
	  (Point *) malloc(sizeof(Point) * AttachCursor->CoreSpotSize);
      if (AttachCursor->InvisibleSpotSize !=0)
	Board.AttachTab[AttachCursor->RoutingAttachID].InvisibleAttachLocation = 
	  (Point *) malloc(sizeof(Point) * AttachCursor->InvisibleSpotSize);  

      for (Counter = 0 ; Counter < Board.AttachTab[AttachID].AttachLocationNb ; Counter++)
	{
	  Board.AttachTab[AttachID].AttachLocation[Counter].x = AttachCursor->SpotLocation[Counter].x + Deltax;
	  Board.AttachTab[AttachID].AttachLocation[Counter].y = AttachCursor->SpotLocation[Counter].y + Deltay;
	}
      for (Counter = 0 ; Counter < Board.AttachTab[AttachID].CoreAttachLocationNb ; Counter++)
	{
	  Board.AttachTab[AttachID].CoreAttachLocation[Counter].x = AttachCursor->CoreSpotLocation[Counter].x + Deltax;
	  Board.AttachTab[AttachID].CoreAttachLocation[Counter].y = AttachCursor->CoreSpotLocation[Counter].y + Deltay;
	}
       for (Counter = 0 ; Counter < Board.AttachTab[AttachID].InvisibleAttachLocationNb ; Counter++)
	{
	  Board.AttachTab[AttachID].InvisibleAttachLocation[Counter].x = AttachCursor->InvisibleSpotLocation[Counter].x + Deltax;
	  Board.AttachTab[AttachID].InvisibleAttachLocation[Counter].y = AttachCursor->InvisibleSpotLocation[Counter].y + Deltay;
	}
      AttachCursor = AttachCursor->pNext;
    }
  
  /* Remplissage de LinkTab et Map */
  Board.LinkNb = DLinkNb;
  if ( Board.LinkNb != 0)
    Board.LinkTab = (LinkCarac *) malloc(sizeof(LinkCarac) * Board.LinkNb );  
  
  InitRoad(&Map, DLinkNb);
  PreviousTrackToDraw = (int *) malloc(sizeof(int) * DLinkNb);
  PreviousTrackNb = 0;

  DLinkCursor = CircuitData->TopLink;
  while (DLinkCursor != NULL)
    {
      GetLinkProperties(CircuitData, DLinkCursor->LinkID, &LinkProp);
      LinkID = LinkProp.RoutingLinkID;
      GetAttachPointProperties(CircuitData, LinkProp.StartAttachID, &AttachProp);
      Board.LinkTab[LinkID].Start = AttachProp.RoutingAttachID;
      
      GetAttachPointProperties(CircuitData, LinkProp.EndAttachID, &AttachProp);      
      Board.LinkTab[LinkID].End = AttachProp.RoutingAttachID;

      Board.LinkTab[LinkID].LinkWidth = LinkProp.LinkWidth;
      Board.LinkTab[LinkID].LinkSize = 0;
      Board.LinkTab[LinkID].Link2Link = NULL;
      
      if (IsLinkRouted(LinkProp.State))
	{
	  LoadTrack(CircuitData, &(Map.Line[LinkID]), LinkProp.IDNumber);
	  PreviousTrackToDraw[PreviousTrackNb] = LinkID;
	  PreviousTrackNb++;
	}
      DLinkCursor =  DLinkCursor->LinkDown;
    }

  Board.Width =  CircuitData->DataHeader.Width;
  Board.Height = CircuitData->DataHeader.Height;
  Board.AreaNb = 0;
  Board.AreaTab = NULL;
  Board.VortexTab = NULL;
  Board.VortexNb = 0;
  ComputeCircuit(&Board);
  
  InitSquareTab(&Board, RoutingID);
  MapArea(CircuitData, &Board); 
  TracePreviousRoad(&Board, Map.Line, PreviousTrackNb, PreviousTrackToDraw, RoutingID); 
  /* trace les pistes dont les routingID sont dans le tableau */
  free(PreviousTrackToDraw);
  FeatherCircuit(&Board, RoutingID);
  Match = FillTab(RoutingID, &Board, Map, &Meet);
  /*   DisplayCircuit(Board); */
 
  if (Match == Found)
    {
      Match = FindPath(&Board,  RoutingID,  &Map, Meet);
      /*       DisplayCircuit(Board); */
      if (Match == Found)
	{
	  FeatherPath(Board, &Map, RoutingID);
	  SaveTrack(CircuitData, Map.Line[RoutingID], RoutingID);
	}
      else 
	printf("\b\bBUG!!! : le chemin inverse ne peut etre trouve alors que le chemin direct a ete trouve!!!\n");
    }
  
  free(Board.VortexTab);
  DestroyCircuit(&Board);
  DestroyRoad(&Map); 

  if (Match == Found)
    return  Found;
  else return NotFound;
}

ErrorCheck MapArea(GCD *CircuitData, Circuit *pBoard)
{
  Component *CompCursor;
  SubFunction SubFProp;
  LayoutDef LayoutProp;
  int Counter, x, y;

  CompCursor = CircuitData->ListComponent;
  while(CompCursor !=  NULL)
    {
      if ((IsComponentBetweenPins(CompCursor->State)) || (IsComponentCompoIn(CompCursor->State)) 
	  || (IsComponentCompoOut(CompCursor->State)))
	{
	  GetSubFunctionProperties(CircuitData, CompCursor->SubFunctionID, &SubFProp);
	  GetLayoutProperties(CircuitData->CompLib, CompCursor->LayoutName, CompCursor->LayoutClass, &LayoutProp, False, False);
	  if (CompCursor->AttachPointNb != 0)
	    {
	      LayoutProp.PinPosition = (Point *) malloc (sizeof(Point) * CompCursor->AttachPointNb);
	      LayoutProp.SpotNameTab = (char **) malloc (sizeof(char *) * CompCursor->AttachPointNb);
	      for (Counter = 0 ; Counter < CompCursor->AttachPointNb ; Counter++)
		LayoutProp.SpotNameTab[Counter] = (char *) malloc (sizeof(char) * SpotNameLength);
	    }
	  
	  if (LayoutProp.BetweenPinsSize != 0)
	    LayoutProp.BetweenPinsPosition = (Point *) malloc(sizeof(Point) * LayoutProp.BetweenPinsSize);
	  if (LayoutProp.CompoInSize != 0)
	    LayoutProp.CompoInPosition = (Point *) malloc(sizeof(Point) * LayoutProp.CompoInSize);
	  if (LayoutProp.CompoOutSize != 0)
	    LayoutProp.CompoOutPosition = (Point *) malloc(sizeof(Point) * LayoutProp.CompoOutSize);
	  
	  GetLayoutProperties(CircuitData->CompLib, CompCursor->LayoutName, CompCursor->LayoutClass, 
			 &LayoutProp, False, True);
	  if (IsComponentBetweenPins(CompCursor->State))
	    {
	      for (Counter = 0 ; Counter < LayoutProp.BetweenPinsSize ; Counter++)
		{
		  x = LayoutProp.BetweenPinsPosition[Counter].x + SubFProp.Reference.x;
		  y = LayoutProp.BetweenPinsPosition[Counter].y + SubFProp.Reference.y;
		  SetSquareUsed(pBoard->SquareTab[x][y].SquareState);
		}
	    }
	  if (IsComponentCompoIn(CompCursor->State))
	    {
	      for (Counter = 0 ; Counter < LayoutProp.CompoInSize ; Counter++)
		{
		  x = LayoutProp.CompoInPosition[Counter].x + SubFProp.Reference.x;
		  y = LayoutProp.CompoInPosition[Counter].y + SubFProp.Reference.y;
		  SetSquareUsed(pBoard->SquareTab[x][y].SquareState);
		}
	    }
	  if (IsComponentCompoOut(CompCursor->State))
	    {
	      for (Counter = 0 ; Counter < LayoutProp.CompoOutSize ; Counter++)
		{
		  x = LayoutProp.CompoOutPosition[Counter].x + SubFProp.Reference.x;
		  y = LayoutProp.CompoOutPosition[Counter].y + SubFProp.Reference.y;
		  SetSquareUsed(pBoard->SquareTab[x][y].SquareState);
		}
	    }
	  if (CompCursor->AttachPointNb != 0)
	    {
	      for (Counter = 0 ; Counter < CompCursor->AttachPointNb ; Counter++)
		free(LayoutProp.SpotNameTab[Counter]);
	      free(LayoutProp.PinPosition);
	      free(LayoutProp.SpotNameTab);
	    }
	      
	  if (LayoutProp.BetweenPinsSize != 0)
	    free(LayoutProp.BetweenPinsPosition);
	  if (LayoutProp.CompoInSize != 0)
	    free(LayoutProp.CompoInPosition);
	  if (LayoutProp.CompoOutSize != 0)
	    free(LayoutProp.CompoOutPosition);
	}
      CompCursor = CompCursor->pNext;
    }
  return AllOk;
}
ErrorCheck SaveTrack(GCD *CircuitData, RoadCarac Map, int RoutingLinkID)
{
  int Counter, ID;
  Link *LinkCursor;
  DrawLink *DLinkCursor;
  Boolean LinkIDFound, DLinkIDFound;

  LinkCursor = CircuitData->ListLink;
  LinkIDFound = False;
  while ((LinkCursor!= NULL) && (LinkIDFound == False))
    {
      if (LinkCursor->RoutingLinkID == RoutingLinkID)
	{
	  LinkIDFound = True;
	  if (IsLinkRouted(LinkCursor->State))
	    printf("Bug le lien n a pas ete deroute \n");

	  LinkCursor->RoutedOrder = GetRoutedOrder(CircuitData, NoOrder);
	  SetLinkRouted(LinkCursor->State);
	  
	  DLinkCursor = CircuitData->TopLink;
	  DLinkIDFound = False;
	  while ((DLinkCursor != NULL) && (DLinkIDFound == False))
	    { /* on synchronise le DLink et le Link */
	      if (DLinkCursor->LinkID == LinkCursor->IDNumber)
		{
		  if (IsDLinkRouted(DLinkCursor->State))
		    printf("le dlink n a pas ete deroute\n");
		  DLinkIDFound = True;
		  SetDLinkRouted(DLinkCursor->State);
		  DLinkCursor->RoutedOrder = LinkCursor->RoutedOrder;
		}
	      DLinkCursor = DLinkCursor->LinkDown;
	    }

	  LinkCursor->Road.SegSize = Map.SegSize;
	  LinkCursor->Road.PathSize = Map.PathSize;
	  LinkCursor->Road.CorePathSize = Map.CorePathSize;
	  LinkCursor->Road.DiagonalSize = Map.DiagonalSize;
	  LinkCursor->Road.DepSize = Map.DepSize;
	  LinkCursor->Road.JobDone = True;
	  if (Map.SegSize != 0)
	    LinkCursor->Road.SegTab = (Segment *) malloc(sizeof(Segment) * Map.SegSize);
	  if (Map.PathSize != 0)
	    LinkCursor->Road.PathTab = (Point *) malloc(sizeof(Point) * Map.PathSize);
	  if (Map.DiagonalSize != 0)
	    LinkCursor->Road.DiagonalTab = (Point *) malloc(sizeof(Point) * Map.DiagonalSize);
	  if (Map.DepSize != 0)
	    LinkCursor->Road.DepTab = (DepCarac *) malloc(sizeof(DepCarac) * Map.DepSize);

	  for (Counter = 0 ; Counter < Map.SegSize ; Counter++)
	    {
	      LinkCursor->Road.SegTab[Counter].Start = Map.SegTab[Counter].Start;
	      LinkCursor->Road.SegTab[Counter].End = Map.SegTab[Counter].End;
	    }
	  
	  for (Counter = 0 ; Counter < Map.PathSize ; Counter++)
	    LinkCursor->Road.PathTab[Counter] = Map.PathTab[Counter];
	  for (Counter = 0 ; Counter < Map.DiagonalSize ; Counter++)
	    LinkCursor->Road.DiagonalTab[Counter] = Map.DiagonalTab[Counter];
	  for (Counter = 0 ; Counter < Map.DepSize ; Counter++)
	    {
	      LinkCursor->Road.DepTab[Counter].Type =  Map.DepTab[Counter].Type;
	      
	      switch (LinkCursor->Road.DepTab[Counter].Type)
		{
		case AttachDep : AttachRoutingIDToID(CircuitData, Map.DepTab[Counter].Nb, &ID);
		  break;
		case LinkDep : LinkRoutingIDToID(CircuitData, Map.DepTab[Counter].Nb, &ID);
		  break;
		default : printf("Error the dependance is neither Attach nor Link !!!!\nErrors may occur!!!\n\n");
		}
	      LinkCursor->Road.DepTab[Counter].Nb =  ID;
	    }
	}
      LinkCursor = LinkCursor->pNext;
    } 
  if (LinkIDFound == True)
    return Found;
  else return NotFound;
}
  
ErrorCheck LoadTrack(GCD *CircuitData, RoadCarac *Map, int LinkID)
{
  int Counter, ID;
  Link *LinkCursor;
  Boolean IDFound;

  LinkCursor = CircuitData->ListLink;
  IDFound = False;
  while ((LinkCursor!= NULL) && (IDFound == False))
    {
      if (LinkCursor->IDNumber == LinkID)
	{
	  IDFound = True;
	  Map->JobDone = True;
	  Map->SegSize = LinkCursor->Road.SegSize;
	  Map->PathSize = LinkCursor->Road.PathSize;
	  Map->CorePathSize = LinkCursor->Road.CorePathSize;
	  Map->DiagonalSize = LinkCursor->Road.DiagonalSize;
	  Map->DepSize = LinkCursor->Road.DepSize;
	  
	  if (Map->SegSize != 0)
	    Map->SegTab = (Segment *) malloc(sizeof(Segment) * Map->SegSize);
	  if ( Map->PathSize != 0)
	    Map->PathTab = (Point *) malloc(sizeof(Point) * Map->PathSize);
	  if ( Map->DiagonalSize != 0)
	    Map->DiagonalTab = (Point *) malloc(sizeof(Point) * Map->DiagonalSize);
	  if ( Map->DepSize != 0)
	   Map->DepTab = (DepCarac *) malloc(sizeof(DepCarac) * Map->DepSize);
	  
	  for (Counter = 0 ; Counter < Map->SegSize ; Counter++)
	    {
	      Map->SegTab[Counter].Start.x = LinkCursor->Road.SegTab[Counter].Start.x;
	      Map->SegTab[Counter].Start.y = LinkCursor->Road.SegTab[Counter].Start.y;
	      Map->SegTab[Counter].End.x = LinkCursor->Road.SegTab[Counter].End.x;
	      Map->SegTab[Counter].End.y = LinkCursor->Road.SegTab[Counter].End.y;
	    }
	  
	  for (Counter = 0 ; Counter < Map->PathSize ; Counter++)
	    {
	      Map->PathTab[Counter].x = LinkCursor->Road.PathTab[Counter].x;
	      Map->PathTab[Counter].y = LinkCursor->Road.PathTab[Counter].y;
	    }
	  for (Counter = 0 ; Counter < Map->DiagonalSize ; Counter++)
	    {
	      Map->DiagonalTab[Counter].x = LinkCursor->Road.DiagonalTab[Counter].x;
	      Map->DiagonalTab[Counter].y = LinkCursor->Road.DiagonalTab[Counter].y;
	    }
	  for (Counter = 0 ; Counter < Map->DepSize ; Counter++)
	    {
	      Map->DepTab[Counter].Type = LinkCursor->Road.DepTab[Counter].Type;
	      switch (Map->DepTab[Counter].Type)
		{
		case AttachDep : AttachIDToRoutingID(CircuitData, LinkCursor->Road.DepTab[Counter].Nb , &ID);
		  break;
		case LinkDep : LinkIDToRoutingID(CircuitData,  LinkCursor->Road.DepTab[Counter].Nb, &ID);
		  break;
		default : printf("Error the dependance is neither Attach nor Link !!!!\nErrors may occur!!!\n\n");
		}
	      Map->DepTab[Counter].Nb =  ID;
	    }
	}
      LinkCursor = LinkCursor->pNext;
    } 
  if (IDFound == True)
    return Found;
  else return NotFound;
}


ErrorCheck MediumRouting(GCD *CircuitData, Boolean UnderMagic)
{/* Trace toutes les pistes, s arrete si une piste ne peut etre tracee. */
  int IntegritySize, Counter, RoutingNb, *RoutingIDTab, TabSize;
  Integrity *IntegrityTab;
  DrawLink *DLinkCursor;
  Link LinkProp;
  ErrorCheck Check;
  Boolean LinkUnroutable, ContinueRouterOnError, Cancel;
  
  GetAttachPointNumber(CircuitData, &TabSize);
  IntegrityTab = (Integrity *)malloc(sizeof(Integrity) * TabSize);

  CheckCircuitIntegrity(CircuitData, IntegrityTab, &IntegritySize);
  if (IntegritySize != 0)
    ShowCircuitIntegrity(IntegrityTab, IntegritySize);
  
  free(IntegrityTab);
  
  if (IntegritySize != 0)
    return Exception;

  SetRoutingID(CircuitData);
  
  GetDLinkNumber(CircuitData, &RoutingNb);
  if (RoutingNb == 0)
    return Found; 
  
  RoutingIDTab = (int *) malloc (sizeof(int) * (RoutingNb));
  if (RoutingIDTab == NULL)
    printf("malloc return NULL Pointer\n");
  
  RoutingNb = 0;
  DLinkCursor = CircuitData->TopLink;
  while (DLinkCursor != NULL)
    {/* on recup les routing ID de tous les liens a tracer */
      if (!IsDLinkRouted(DLinkCursor->State))
	{
	  GetLinkProperties(CircuitData, DLinkCursor->LinkID, &LinkProp);
	  RoutingIDTab[RoutingNb] = LinkProp.RoutingLinkID;
	  RoutingNb++;
	}
      DLinkCursor = DLinkCursor->LinkDown;
    } 
  
  Check = Found;
  LinkUnroutable = False;
  Cancel = False;
  for (Counter = 0 ; (Counter < RoutingNb) && (Cancel == False) && (Check == Found) ; Counter++)
    {
      if ((UnderMagic == False) && (CircuitData->Info.RouterCancelRequest == True))
	Cancel = True;
      Check = DrawOneTrack(CircuitData, RoutingIDTab[Counter]);
      
      if (UnderMagic == False)
	UpdateRouterStatusWindow(CircuitData, 0, 1, Counter, RoutingNb);
      
      while (gtk_events_pending())
	gtk_main_iteration();    

      if (Check == NotFound)
	LinkUnroutable = True;
   }
  
  free(RoutingIDTab);
  if (LinkUnroutable == False)
    return Found;
  else
    return NotFound;
}

ErrorCheck SetRoutingID(GCD *CircuitData)
{
  DrawLink *DLinkCursor;
  AttachPoint *AttachCursor;
  int  RoutingLinkIDCounter, RoutingAttachIDCounter;

  DLinkCursor = CircuitData->TopLink;
  RoutingLinkIDCounter = 0;
  while(DLinkCursor != NULL)
    {
      SetLinkRoutingID(CircuitData, DLinkCursor->LinkID, RoutingLinkIDCounter);
      RoutingLinkIDCounter++;
      DLinkCursor = DLinkCursor->LinkDown;
    }

  AttachCursor = CircuitData->ListAttachPoint;
  RoutingAttachIDCounter = 0;
  while (AttachCursor != NULL)
    {
      AttachCursor->RoutingAttachID = RoutingAttachIDCounter;
      RoutingAttachIDCounter++;
      AttachCursor = AttachCursor->pNext;
    }
  return AllOk;
}
