
#include <stdlib.h>
#include <math.h>
#include "fig.h"
#include "spot.h"
#include "component.h"
#include "attachpoint.h"
#include "subfunction.h"
#include "link.h"
ErrorCheck PrintFigHeader(FILE *FigFile)
{
  fprintf(FigFile,"#FIG 3.2\n");
  fprintf(FigFile,"Landscape\n");
  fprintf(FigFile,"Center\n");
  fprintf(FigFile,"Metric\n");
  fprintf(FigFile,"A4\n");
  fprintf(FigFile,"100.00\n");
  fprintf(FigFile,"Single\n");
  fprintf(FigFile,"-2\n");
  fprintf(FigFile,"1200 2\n");
  return AllOk;
}

ErrorCheck ExportToFig(GCD *CircuitData, char *FigFileName, ComponentLibrary *CompLib)
{
  FILE *FigFile;
  AttachPoint *AttachCursor;
  Component CompProp;
  SubFunction SubFProp;
  Link *LinkCursor;
  FigSpot FSData;
  FigPath FPData;
  ErrorCheck NonEmpty;
  FigFile = fopen(FigFileName, "w");
  if (FigFile == NULL)
    {
      printf("Error writing fig file\n");
      return FileAccessError;
    }

  PrintFigHeader(FigFile);
  PrintFigFrame(FigFile, CircuitData);
  AttachCursor = CircuitData->ListAttachPoint;
  while(AttachCursor != NULL)
    {
      GetSpotProperties(CompLib, AttachCursor->SpotName, &(FSData.SpotProp), False);
      if (FSData.SpotProp.SpotSize != 0)
	FSData.SpotProp.SpotLocation = (Point *) malloc(sizeof(Point) * FSData.SpotProp.SpotSize);
      if (FSData.SpotProp.CoreSpotSize != 0)
	FSData.SpotProp.CoreSpotLocation = (Point *) malloc(sizeof(Point) * FSData.SpotProp.CoreSpotSize);
      if (FSData.SpotProp.InvisibleSpotSize != 0)
	FSData.SpotProp.InvisibleSpotLocation = (Point *) malloc(sizeof(Point) * FSData.SpotProp.InvisibleSpotSize);
            
      GetSpotProperties(CompLib, AttachCursor->SpotName, &(FSData.SpotProp), True);
      GetComponentProperties(CircuitData, AttachCursor->ComponentID, &CompProp, False);
      GetSubFunctionProperties(CircuitData, CompProp.SubFunctionID, &SubFProp);
      FSData.Reference.x = SubFProp.Reference.x + CompProp.Reference.x + AttachCursor->Reference.x;
      FSData.Reference.y = SubFProp.Reference.y + CompProp.Reference.y + AttachCursor->Reference.y;
            
      PrintFigSpot(FigFile, FSData);
      if (FSData.SpotProp.SpotSize != 0)
	free(FSData.SpotProp.SpotLocation);
      if (FSData.SpotProp.CoreSpotSize != 0)
	free(FSData.SpotProp.CoreSpotLocation);
      if (FSData.SpotProp.InvisibleSpotSize != 0)
	free(FSData.SpotProp.InvisibleSpotLocation);
      AttachCursor = AttachCursor->pNext;
    }

  LinkCursor = CircuitData->ListLink;
  while (LinkCursor != NULL)
    {
      if (IsLinkRouted(LinkCursor->State))
	{
	  NonEmpty = GetLinkSegTab(CircuitData, LinkCursor->IDNumber, &FPData);
	  if (NonEmpty == Found)
	    {
	      PrintFigPath(FigFile, FPData);
	      if (FPData.SegSize != 0)
		free(FPData.SegTab);
	    }
	}
      LinkCursor = LinkCursor->pNext;
    }
  fclose(FigFile);
  return AllOk;
}

ErrorCheck PrintFigFrame(FILE *FigFile, GCD *CircuitData)
{
  int XMin, XMax, YMin, YMax, SquareDim;
  
  XMin = FigDeltax;
  YMin = FigDeltay;
  SquareDim = CircuitData->DataHeader.Width + XMin;
  XMax = SquareToFig(SquareDim);
  
  SquareDim = CircuitData->DataHeader.Height + YMin;
  YMax = SquareToFig(SquareDim);
  
  fprintf(FigFile,"2 2 0 %d 0 7 100 0 -1 0.000 0 0 -1 0 0 5 \n\t  %d %d %d %d %d %d %d %d %d %d \n",
	  FrameFigWidth, XMin, YMin, XMax, YMin, XMax, YMax, XMin, YMax, XMin, YMin);
  
  return AllOk;
}

ErrorCheck PrintFigSpot(FILE *FigFile, FigSpot FSData)
{
  int XMin, YMin, XMax, YMax, X1, Y1, X2, Y2, X3, Y3, X4, Y4, X5, Y5,Counter, XReference, YReference;
  double SquareDim; 
  
  AutoRangeSpot(FSData.SpotProp, &XMin ,&XMax, &YMax, &YMin);
  XReference = FSData.Reference.x;
  YReference = FSData.Reference.y;
  SquareDim = XReference  + XMin;
  XMin = SquareToFig(SquareDim);
  SquareDim = YReference + YMin;
  YMin = SquareToFig(SquareDim);

  SquareDim =  XReference + XMax;
  XMax = SquareToFig(SquareDim);
  SquareDim =  YReference + YMax;
  YMax = SquareToFig(SquareDim);
  
  fprintf(FigFile,"6 %d %d %d %d\n", XMin, YMin, XMax, YMax); /* Compound */
  
  for(Counter = 0 ; Counter < FSData.SpotProp.SpotSize ; Counter++)
    {
      fprintf(FigFile,"2 2 0 0 0 0 100 0 20 0.000 0 0 0 0 0 5 \n\t "); /* Rectangle */
      X1 = SquareToFig(XReference + FSData.SpotProp.SpotLocation[Counter].x);
      Y1 = SquareToFig(YReference + FSData.SpotProp.SpotLocation[Counter].y);
      X2 = SquareToFig(XReference + FSData.SpotProp.SpotLocation[Counter].x + 1);
      Y2 = SquareToFig(YReference + FSData.SpotProp.SpotLocation[Counter].y);
      X3 = SquareToFig(XReference + FSData.SpotProp.SpotLocation[Counter].x + 1);
      Y3 = SquareToFig(YReference + FSData.SpotProp.SpotLocation[Counter].y + 1);
      X4 = SquareToFig(XReference + FSData.SpotProp.SpotLocation[Counter].x);
      Y4 = SquareToFig(YReference + FSData.SpotProp.SpotLocation[Counter].y + 1);
      X5 = SquareToFig(XReference + FSData.SpotProp.SpotLocation[Counter].x);
      Y5 = SquareToFig(YReference + FSData.SpotProp.SpotLocation[Counter].y);
      
      fprintf(FigFile,"%d %d %d %d %d %d %d %d %d %d \n", X1, Y1, X2, Y2, X3, Y3, X4, Y4, X5, Y5);
    }
  for(Counter = 0 ; Counter < FSData.SpotProp.CoreSpotSize ; Counter++)
   {
      fprintf(FigFile,"2 2 0 0 0 0 100 0 20 0.000 0 0 0 0 0 5 \n\t "); /* Rectangle */
      X1 = SquareToFig(XReference + FSData.SpotProp.CoreSpotLocation[Counter].x);
      Y1 = SquareToFig(YReference + FSData.SpotProp.CoreSpotLocation[Counter].y);
      X2 = SquareToFig(XReference + FSData.SpotProp.CoreSpotLocation[Counter].x + 1);
      Y2 = SquareToFig(YReference + FSData.SpotProp.CoreSpotLocation[Counter].y);
      X3 = SquareToFig(XReference + FSData.SpotProp.CoreSpotLocation[Counter].x + 1);
      Y3 = SquareToFig(YReference + FSData.SpotProp.CoreSpotLocation[Counter].y + 1);
      X4 = SquareToFig(XReference + FSData.SpotProp.CoreSpotLocation[Counter].x);
      Y4 = SquareToFig(YReference + FSData.SpotProp.CoreSpotLocation[Counter].y + 1);
      X5 = SquareToFig(XReference + FSData.SpotProp.CoreSpotLocation[Counter].x);
      Y5 = SquareToFig(YReference + FSData.SpotProp.CoreSpotLocation[Counter].y);
      
      fprintf(FigFile,"%d %d %d %d %d %d %d %d %d %d \n", X1, Y1, X2, Y2, X3, Y3, X4, Y4, X5, Y5);
    }
  fprintf(FigFile,"-6\n");
  return AllOk;
}

ErrorCheck PrintFigPath(FILE *FigFile, FigPath FPData)
{
  int Counter, PointCounter, Width;
  Point *PointCoord;
  
  if (FPData.SegSize == 0)
    {
      printf("No Segment in Link\n");
      return AllOk; 
    }
  Width = FPData.LinkWidth;
  /* la liaison etait deja realisee par ailleurs */
  PointCoord = (Point *) malloc(sizeof(Point) * (FPData.SegSize+1));
  PointCounter = 0;
  for (Counter = 0 ; Counter < FPData.SegSize ; )
    {
      PointCoord[PointCounter].x = FPData.SegTab[Counter].Start.x;
      PointCoord[PointCounter].y = FPData.SegTab[Counter].Start.y;
      PointCounter++;
      PointCoord[PointCounter].x = FPData.SegTab[Counter].End.x;
      PointCoord[PointCounter].y = FPData.SegTab[Counter].End.y;
      Counter++;
      if ((Counter == FPData.SegSize) ||  /* on a fait le dernier segment*/ 
	  (FPData.SegTab[Counter].Start.x != PointCoord[PointCounter].x ) ||
	  (FPData.SegTab[Counter].Start.y != PointCoord[PointCounter].y ))
	{/* il n y a pas continuite en ce point... ou c est le dernier point */
	  PointCounter++;
	  PrintFigPolyline(FigFile, PointCoord, PointCounter, Width);
	  PointCounter = 0;
	}
    }
  free(PointCoord);
  return AllOk;
}

ErrorCheck PrintFigPolyline(FILE *FigFile, Point *PathCoord, int PointNb, int Width)
{
  int Counter, X, Y, FigWidth;
  double SquareWidth; 
  
  SquareWidth = Width * M_SQRT2 * PercentageIn;
  FigWidth = SquareToFigWidth(SquareWidth);
  fprintf(FigFile,"2 1 0 %d 0 0 105 0 -1 0.000 0 1 -1 0 0 %d \n\t ", FigWidth, PointNb);
  for (Counter = 0 ; Counter < PointNb ; Counter++)
    {
      X = SquareToFig(PathCoord[Counter].x);
      Y = SquareToFig(PathCoord[Counter].y);
      fprintf(FigFile,"%d %d ", X, Y);
    }
  fprintf(FigFile,"\n");
  return AllOk;
}
	
