
#include "filemng.h"
#include "spot.h"
#include "attachpoint.h"
#include "compdesc.h"
#include "layout.h"
#include "subfunction.h"
#include "link.h"
#include "liste.h"
#include "component.h"
#include "componentwindow.h"
#include "linkwindow.h"
#include "dlink.h"

/* recuperer le nombre d elements de chaque categories */
int GetNumber(GCD *CircuitData, ListType Type)
{
  Component *CompCursor;
  AttachPoint *AttachPointCursor;
  Link *LinkCursor;
  SubFunction *SubFCursor;
  DrawLink *DLinkCursor;
  LinkPermutation *PermCursor;
  int Counter;

  Counter=0;
  switch(Type)
    {
    case ComponentType :
      CompCursor = CircuitData->ListComponent;
      while(CompCursor != NULL)
	{
	  Counter++;
	  CompCursor=CompCursor->pNext;
	}
      break;
      
    case AttachPointType :
      AttachPointCursor = CircuitData->ListAttachPoint;
      while(AttachPointCursor != NULL)
	{
	  Counter++;
	  AttachPointCursor=AttachPointCursor->pNext;
	}
      break;
      
    case LinkType :
      LinkCursor = CircuitData->ListLink;
      while(LinkCursor != NULL)
	{
	  Counter++;
	  LinkCursor = LinkCursor->pNext;
	}
      break;
      
    case SubFunctionType :
      SubFCursor = CircuitData->ListSubFunction;
      while(SubFCursor != NULL)
	  {
	    Counter++;
	    SubFCursor = SubFCursor->pNext;
	  }
      break;
    case DrawLinkType :
      DLinkCursor = CircuitData->TopLink;
      while(DLinkCursor != NULL)
	  {
	    Counter++;
	    DLinkCursor = DLinkCursor->LinkDown;
	  }
      break;
    case LinkPermutationType :
      PermCursor = CircuitData->ListPermutation;
      while (PermCursor != NULL)
	{
	  Counter++;
	  PermCursor = PermCursor->pNext;
	}
      break;
    default:
      printf("GetNumber: appele avec mauvais paramtre.\n");
    }
  return Counter;
}

ErrorCheck WriteGCD(GCD *CircuitData, char *Name, int *NewID, GtkWidget *Text)
{
  FILE *fPrinter;
  Component *CompCursor;
  AttachPoint *AttachCursor;
  SubFunction *SubFCursor;
  SubFunctionBuffer SubFBuff;
  Link *LinkCursor;
  DrawLink *DLinkCursor;
  LinkPermutation *PermCursor;
  int Counter;
  char Buffer[BufferLength];
  
  fPrinter=fopen(Name,"w");
  if (fPrinter == NULL)
    return  FileAccessError;

  if (Text != NULL)
    {
      sprintf(Buffer,"Writing General Info\n");
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
    }
  fprintf(fPrinter,"XRoadMakerCircuit\n");
  fprintf(fPrinter,"Data File from XRoadMaker version 0.6\n");
  fprintf(fPrinter,"\n");
  fprintf(fPrinter,"*** General information ***\n");
  fprintf(fPrinter,"%s\n",CircuitData->DataHeader.Name);
  fprintf(fPrinter,"%s\n",CircuitData->DataHeader.Version);
  fprintf(fPrinter,"%s\n",CircuitData->DataHeader.Designer);
  fprintf(fPrinter,"%s\n",CircuitData->DataHeader.Society);
  fprintf(fPrinter,"%s\n",CircuitData->DataHeader.Description);
  fprintf(fPrinter,"%s\n",CircuitData->DataHeader.RevisionDate);
  fprintf(fPrinter,"%d ",CircuitData->DataHeader.Width);
  fprintf(fPrinter,"%d\n",CircuitData->DataHeader.Height);
  fprintf(fPrinter,"\n");

  if (Text != NULL)
    {
      sprintf(Buffer, "Project : %s\n",CircuitData->DataHeader.Name);
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
      sprintf(Buffer, "Version : %s\n",CircuitData->DataHeader.Version);
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
      sprintf(Buffer, "Designer : %s\n",CircuitData->DataHeader.Designer);
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
      sprintf(Buffer, "Society : %s\n",CircuitData->DataHeader.Society);
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
      sprintf(Buffer, "Description : %s\n",CircuitData->DataHeader.Description);
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
      sprintf(Buffer, "Revision Date : %s\n",CircuitData->DataHeader.RevisionDate);
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
      sprintf(Buffer, "Circuit Width : %d\n",CircuitData->DataHeader.Width);
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
      sprintf(Buffer, "Circuit Height : %d\n",CircuitData->DataHeader.Height);
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
      
      sprintf(Buffer, "\nWriting Flags\n");
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
    }
  fprintf(fPrinter,"Flags\n");
  fprintf(fPrinter,"%d ",CircuitData->ComponentBufferEmpty);
  fprintf(fPrinter,"%d\n",CircuitData->SubFunctionBufferEmpty);
  for (Counter = 0 ; Counter < IDTypeNumber ; Counter++)
    fprintf(fPrinter,"%d ",NewID[Counter]);
  fprintf(fPrinter, "\n");
  
  if (Text != NULL)
    {
      sprintf(Buffer,"Writing AttachPoint List\n");
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
    }

  fprintf(fPrinter,"*** Attach Point List ***\n");
  fprintf(fPrinter,"AttachPoint Number: ");
  fprintf(fPrinter,"%d\n",GetNumber(CircuitData, AttachPointType));

  AttachCursor = CircuitData->ListAttachPoint;
  while(AttachCursor != NULL)
    {
      fprintf(fPrinter,"AttachPoint:\n");
      fprintf(fPrinter,"%s\n",AttachCursor->AttachPointName);
      fprintf(fPrinter,"%d ",AttachCursor->IDNumber);
      fprintf(fPrinter,"%d\n",AttachCursor->ComponentID);
      fprintf(fPrinter,"%d\n",AttachCursor->State);
      fprintf(fPrinter,"%s\n",AttachCursor->SpotName);
      fprintf(fPrinter,"%d ",AttachCursor->Reference.x);
      fprintf(fPrinter,"%d\n",AttachCursor->Reference.y);
      fprintf(fPrinter,"%d %d %d\n",AttachCursor->SpotSize, AttachCursor->CoreSpotSize, AttachCursor->InvisibleSpotSize);

      for(Counter=0 ; Counter < AttachCursor->SpotSize ; Counter++)
	{
	  fprintf(fPrinter,"%d ",AttachCursor->SpotLocation[Counter].x);
	  fprintf(fPrinter,"%d\n",AttachCursor->SpotLocation[Counter].y);
	}
      for(Counter=0 ; Counter < AttachCursor->CoreSpotSize ; Counter++)
	{
	  fprintf(fPrinter,"%d ",AttachCursor->CoreSpotLocation[Counter].x);
	  fprintf(fPrinter,"%d\n",AttachCursor->CoreSpotLocation[Counter].y);
	}
      for(Counter=0 ; Counter < AttachCursor->InvisibleSpotSize ; Counter++)
	{
	  fprintf(fPrinter,"%d ",AttachCursor->InvisibleSpotLocation[Counter].x);
	  fprintf(fPrinter,"%d\n",AttachCursor->InvisibleSpotLocation[Counter].y);
	}
      AttachCursor=AttachCursor->pNext;
    }
  fprintf(fPrinter,"\n");

  if (Text != NULL)
    {
      sprintf(Buffer, "Writing Component List\n"); 
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
    }
  
  fprintf(fPrinter,"*** Component List ***\n");
  fprintf(fPrinter,"Component Number: ");
  fprintf(fPrinter,"%d\n",GetNumber(CircuitData, ComponentType));

  CompCursor = CircuitData->ListComponent;
  while(CompCursor != NULL)
    {
      fprintf(fPrinter,"Component:\n");
      fprintf(fPrinter,"%s\n",CompCursor->ComponentName);
      fprintf(fPrinter,"%s\n",CompCursor->CompDescName);
      fprintf(fPrinter,"%s\n",CompCursor->LayoutName);
      fprintf(fPrinter,"%s\n",CompCursor->LayoutClass);
      fprintf(fPrinter,"%d ",CompCursor->IDNumber);
      fprintf(fPrinter,"%d\n",CompCursor->SubFunctionID);
      fprintf(fPrinter,"%d\n",CompCursor->State);
      fprintf(fPrinter,"%d ",CompCursor->Reference.x);
      fprintf(fPrinter,"%d\n",CompCursor->Reference.y);
      fprintf(fPrinter,"AttachPointNb: %d\n",CompCursor->AttachPointNb);

      for(Counter = 0 ; Counter < CompCursor->AttachPointNb; Counter++)
	{
	  fprintf(fPrinter,"%d ",CompCursor->AttachPointIDTab[Counter]);
	}
      fprintf(fPrinter,"\n");
      CompCursor=CompCursor->pNext;
    }
  fprintf(fPrinter,"\n");

  if (CircuitData->ComponentBufferEmpty == False)
    {
      if (Text != NULL)
	{
	  sprintf(Buffer, "Writing Component Buffer\n"); 
	  gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
	}

      fprintf(fPrinter,"*** Component Buffer ***\n");
      fprintf(fPrinter,"%s\n",CircuitData->ComponentBuffer.ComponentName);
      fprintf(fPrinter,"%s\n",CircuitData->ComponentBuffer.CompDescName);
      fprintf(fPrinter,"%s\n",CircuitData->ComponentBuffer.LayoutName);
      fprintf(fPrinter,"%s\n",CircuitData->ComponentBuffer.LayoutClass);
      fprintf(fPrinter,"%d ",CircuitData->ComponentBuffer.Reference.x);
      fprintf(fPrinter,"%d\n",CircuitData->ComponentBuffer.Reference.y);
      fprintf(fPrinter,"%d\n",CircuitData->ComponentBuffer.State);
      fprintf(fPrinter,"\n");
    }

  if (Text != NULL)
    {
      sprintf(Buffer,"Writing SubFunction List\n");
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
    }

  SubFCursor = CircuitData->ListSubFunction;
  fprintf(fPrinter,"*** SubFunction List ***\n");
  fprintf(fPrinter,"SubFunction Number: ");
  fprintf(fPrinter,"%d\n",GetNumber(CircuitData, SubFunctionType));

  SubFCursor = CircuitData->ListSubFunction;
  while(SubFCursor != NULL)
    {
      fprintf(fPrinter,"SubFunction:\n");
      fprintf(fPrinter,"%s\n",SubFCursor->Name);
      fprintf(fPrinter,"%d\n",SubFCursor->IDNumber);
      fprintf(fPrinter,"%d\n",SubFCursor->State);
      fprintf(fPrinter,"%d ",SubFCursor->Reference.x);
      fprintf(fPrinter,"%d\n",SubFCursor->Reference.y);

      SubFCursor = SubFCursor->pNext;
    }
  fprintf(fPrinter,"\n");


  if (CircuitData->SubFunctionBufferEmpty == False)
    {
      SubFBuff = CircuitData->ListSubFunctionBuffer;
      if (Text != NULL)
	{
	  sprintf(Buffer, "Writing SubFunction Buffer\n");
	  gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
	}

      fprintf(fPrinter,"*** SubFunction Buffer ***\n");
      fprintf(fPrinter,"%s\n",SubFBuff.Name);
      fprintf(fPrinter,"Comp Number: %d\n",SubFBuff.NbComp);
      fprintf(fPrinter,"Link Number: %d\n",SubFBuff.NbLink);
      fprintf(fPrinter,"%d ",SubFBuff.Reference.x);
      fprintf(fPrinter,"%d\n",SubFBuff.Reference.y);
     
      for(Counter = 0 ; Counter < SubFBuff.NbComp ; Counter++)
	{
	  fprintf(fPrinter,"Component:\n");
	  fprintf(fPrinter,"%s\n",SubFBuff.TabCompSF[Counter].ComponentName);
	  fprintf(fPrinter,"%s\n",SubFBuff.TabCompSF[Counter].CompDescName);
	  fprintf(fPrinter,"%s\n",SubFBuff.TabCompSF[Counter].LayoutName);
	  fprintf(fPrinter,"%s\n",SubFBuff.TabCompSF[Counter].LayoutClass);
	  fprintf(fPrinter,"%d ",SubFBuff.TabCompSF[Counter].Reference.x);
	  fprintf(fPrinter,"%d\n",SubFBuff.TabCompSF[Counter].Reference.y);
	  fprintf(fPrinter,"%d\n",SubFBuff.TabCompSF[Counter].State);
	}
      for(Counter = 0 ; Counter < SubFBuff.NbLink ; Counter++)
	{
	  fprintf(fPrinter,"Link:\n");
	  fprintf(fPrinter,"%d ",SubFBuff.TabLinkSF[Counter].StartAttachID);
	  fprintf(fPrinter,"%d\n",SubFBuff.TabLinkSF[Counter].EndAttachID);
	  fprintf(fPrinter,"%d\n",SubFBuff.TabLinkSF[Counter].State);
	  fprintf(fPrinter,"%c ",SubFBuff.TabLinkSF[Counter].LinkColor.Red);
	  fprintf(fPrinter,"%c ",SubFBuff.TabLinkSF[Counter].LinkColor.Green);
	  fprintf(fPrinter,"%c\n",SubFBuff.TabLinkSF[Counter].LinkColor.Blue);
	  fprintf(fPrinter,"%d %d %d %d\n",SubFBuff.LinkInfo[Counter][0], SubFBuff.LinkInfo[Counter][1],
		  SubFBuff.LinkInfo[Counter][2], SubFBuff.LinkInfo[Counter][3]);
	}
      fprintf(fPrinter,"\n");
    }
  
  if (Text != NULL)
    {
      sprintf(Buffer,"Writing Link List\n");
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
    }

  fprintf(fPrinter,"*** Link List ***\n");
  fprintf(fPrinter,"Link Number: ");
  fprintf(fPrinter,"%d\n",GetNumber(CircuitData, LinkType));
 
  LinkCursor = CircuitData->ListLink;
  while(LinkCursor != NULL)
    {
      fprintf(fPrinter,"Link:\n");
      fprintf(fPrinter,"%d\n",LinkCursor->IDNumber);
      fprintf(fPrinter,"%d ",LinkCursor->StartAttachID);
      fprintf(fPrinter,"%d\n",LinkCursor->EndAttachID);
      fprintf(fPrinter,"%d\n",LinkCursor->State);
      fprintf(fPrinter,"%d ",LinkCursor->LinkLength);
      fprintf(fPrinter,"%d\n",LinkCursor->LinkWidth);
      fprintf(fPrinter,"%d ",LinkCursor->Start.x);
      fprintf(fPrinter,"%d\n",LinkCursor->Start.y);
      fprintf(fPrinter,"%d ",LinkCursor->End.x);
      fprintf(fPrinter,"%d\n",LinkCursor->End.y);
      fprintf(fPrinter,"%d\n",LinkCursor->RoutedOrder);
      fprintf(fPrinter,"%c ",LinkCursor->LinkColor.Red);
      fprintf(fPrinter,"%c ",LinkCursor->LinkColor.Green);
      fprintf(fPrinter,"%c\n",LinkCursor->LinkColor.Blue);
      if (IsLinkRouted(LinkCursor->State))
	{
	  fprintf(fPrinter,"%d ",LinkCursor->Road.SegSize);
	  fprintf(fPrinter,"%d ",LinkCursor->Road.PathSize);
	  fprintf(fPrinter,"%d ",LinkCursor->Road.CorePathSize);
	  fprintf(fPrinter,"%d ",LinkCursor->Road.DiagonalSize);
	  fprintf(fPrinter,"%d ",LinkCursor->Road.DepSize);
	  fprintf(fPrinter,"%d\n",LinkCursor->Road.JobDone);
	  for(Counter = 0 ; Counter < LinkCursor->Road.SegSize ; Counter++)
	    {
	      fprintf(fPrinter,"%d ",LinkCursor->Road.SegTab[Counter].Start.x);
	      fprintf(fPrinter,"%d\n",LinkCursor->Road.SegTab[Counter].Start.y);
	      fprintf(fPrinter,"%d ",LinkCursor->Road.SegTab[Counter].End.x);
	      fprintf(fPrinter,"%d\n",LinkCursor->Road.SegTab[Counter].End.y);
	    }
	  
	  for(Counter=0 ; Counter < LinkCursor->Road.PathSize ; Counter++)
	    {
	      fprintf(fPrinter,"%d ",LinkCursor->Road.PathTab[Counter].x);
	      fprintf(fPrinter,"%d ",LinkCursor->Road.PathTab[Counter].y);
	      fprintf(fPrinter,"\n");
	    }
	  for(Counter=0 ; Counter < LinkCursor->Road.DiagonalSize ; Counter++)
	    {
	      fprintf(fPrinter,"%d ",LinkCursor->Road.DiagonalTab[Counter].x);
	      fprintf(fPrinter,"%d ",LinkCursor->Road.DiagonalTab[Counter].y);
	      fprintf(fPrinter,"\n");
	    }
	  for(Counter = 0 ; Counter < LinkCursor->Road.DepSize ; Counter++)
	    {
	      fprintf(fPrinter,"%d\n",LinkCursor->Road.DepTab[Counter].Type);
	      fprintf(fPrinter,"%d\n",LinkCursor->Road.DepTab[Counter].Nb);
	    }
	}
      LinkCursor=LinkCursor->pNext;
    } 
  fprintf(fPrinter,"\n");
  
  if (Text != NULL)
    {
      sprintf(Buffer, "Writing LinkToDraw List\n");
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
    }

  fprintf(fPrinter,"*** DrawLink List ***\n");
  fprintf(fPrinter,"DrawLink Number: ");
  fprintf(fPrinter,"%d\n",GetNumber(CircuitData, DrawLinkType));
  
  DLinkCursor = CircuitData->TopLink;
  while (DLinkCursor != NULL)
    {
      fprintf(fPrinter,"%d ",DLinkCursor->LinkID);
      fprintf(fPrinter,"%d ",DLinkCursor->LinkLength);
      fprintf(fPrinter,"%d\n",DLinkCursor->LinkWidth);
      fprintf(fPrinter,"%d\n",DLinkCursor->State);
      fprintf(fPrinter,"%d\n",DLinkCursor->RoutedOrder);
    DLinkCursor = DLinkCursor->LinkDown;
    }
  fprintf(fPrinter,"\n");
  
  if (Text != NULL)
    {
      sprintf(Buffer, "Writing Permutation List\n");
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
    }

  fprintf(fPrinter,"*** Permutation List ***\n");
  fprintf(fPrinter,"Permutation Number: ");
  fprintf(fPrinter,"%d\n",GetNumber(CircuitData, LinkPermutationType));
  
  PermCursor = CircuitData->ListPermutation;
  while (PermCursor != NULL)
    {
      fprintf(fPrinter,"%d ", PermCursor->LinkID1);
      fprintf(fPrinter,"%d ", PermCursor->LinkID2);
      fprintf(fPrinter,"%d\n", PermCursor->Number);
      PermCursor = PermCursor->pNext;
    }
  
  fprintf(fPrinter,"*** End ***\n");
  if (Text != NULL)
    {
      sprintf(Buffer, "File Writing Successful\n");
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
    }

  fclose(fPrinter);
  return AllOk;
}

ErrorCheck ReadGCD(GCD *CircuitData, char *Name, int *NewID, GtkWidget *Text)
{
  FILE *fReader;
  Component *Comp=NULL,*PreviousComp;
  AttachPoint *AttachCursor=NULL, *PreviousAttachCursor;
  Link *LinkCursor=NULL, *PreviousLink;
  SubFunction *Sub=NULL, *PreviousSub;
  DrawLink *NewDLink;
  LinkPermutation *PrevPerm, *NewPerm;
  int Counter, NbElem, ElemCounter;
  char DummyChar[BufferLength], Buffer[BufferLength];
  ErrorCheck Check;
  Boolean ErrorFound;
  GdkColor ErrorColor = { 0, 0xFFFF, 0x0000, 0x0000};
  
  ErrorFound = False;
  fReader=fopen(Name,"r");
  if (fReader == NULL)
    return FileAccessError;
  
  fscanf(fReader,"%[^\n]\n",DummyChar);
  if (strcmp( DummyChar, "XRoadMakerCircuit") != 0)
    return FileAccessError;
  
  fscanf(fReader,"%[^\n]\n",DummyChar);
  fscanf(fReader,"\n");
  fscanf(fReader,"%[^\n]\n",DummyChar);
  fscanf(fReader,"%[^\n]\n",CircuitData->DataHeader.Name);
  fscanf(fReader,"%[^\n]\n",CircuitData->DataHeader.Version);
  fscanf(fReader,"%[^\n]\n",CircuitData->DataHeader.Designer);
  fscanf(fReader,"%[^\n]\n",CircuitData->DataHeader.Society);
  fscanf(fReader,"%[^\n]\n",CircuitData->DataHeader.Description);
  fscanf(fReader,"%[^\n]\n",CircuitData->DataHeader.RevisionDate);
  fscanf(fReader,"%d ",&(CircuitData->DataHeader.Width));
  fscanf(fReader,"%d\n",&(CircuitData->DataHeader.Height));
  fscanf(fReader,"\n");

  if (Text != NULL)
    {
      sprintf(Buffer, "Reading General Info\n");
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
      sprintf(Buffer, "Project : %s\n",CircuitData->DataHeader.Name);
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
      sprintf(Buffer, "Version : %s\n",CircuitData->DataHeader.Version);
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
      sprintf(Buffer, "Designer : %s\n",CircuitData->DataHeader.Designer);
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
      sprintf(Buffer, "Society : %s\n",CircuitData->DataHeader.Society);
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
      sprintf(Buffer, "Description : %s\n",CircuitData->DataHeader.Description);
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
      sprintf(Buffer, "Revision Date : %s\n",CircuitData->DataHeader.RevisionDate);
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
      sprintf(Buffer, "Circuit Width : %d\n",CircuitData->DataHeader.Width);
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
      sprintf(Buffer, "Circuit Height : %d\n",CircuitData->DataHeader.Height);
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
      
      sprintf(Buffer, "\nReading Flags\n");
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
    }
  fscanf(fReader,"%[^\n]\n",DummyChar);
  fscanf(fReader,"%d ",(int *) &( CircuitData->ComponentBufferEmpty));
  fscanf(fReader,"%d\n",(int *) &( CircuitData->SubFunctionBufferEmpty));
  for (Counter = 0 ; Counter < IDTypeNumber ; Counter++)
    fscanf(fReader,"%d ",&(NewID[Counter]));
  fscanf(fReader,"\n");
  
  if (Text != NULL)
    {
      sprintf(Buffer, "Reading AttachPoint List\n");
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
    }
  fscanf(fReader,"%[^\n]\n",DummyChar);
  fscanf(fReader,"AttachPoint Number: %d\n",&NbElem);
  PreviousAttachCursor = NULL;
  for(ElemCounter = 0 ; ElemCounter < NbElem ; ElemCounter++)
    {
      AttachCursor=(AttachPoint*)malloc(sizeof(AttachPoint));
      if (AttachCursor == NULL) 
	return MemoryAllocationError;
      
      if (PreviousAttachCursor==NULL)
	CircuitData->ListAttachPoint = AttachCursor;
      else
	PreviousAttachCursor->pNext = AttachCursor;
      
      fscanf(fReader,"%[^\n]\n",DummyChar);
      fscanf(fReader,"%[^\n]\n",AttachCursor->AttachPointName);
      fscanf(fReader,"%d ",&(AttachCursor->IDNumber));
      fscanf(fReader,"%d\n",&(AttachCursor->ComponentID));
      fscanf(fReader,"%d\n",&(AttachCursor->State));
      fscanf(fReader,"%[^\n]\n", AttachCursor->SpotName);
      fscanf(fReader,"%d ",&(AttachCursor->Reference.x));
      fscanf(fReader,"%d\n",&(AttachCursor->Reference.y));
      fscanf(fReader,"%d %d %d\n",&(AttachCursor->SpotSize),&(AttachCursor->CoreSpotSize), &(AttachCursor->InvisibleSpotSize));
     
      Check = CheckSpot(CircuitData->CompLib, AttachCursor->SpotName);
      if (Check == NotFound)
	{
	  if (Text != NULL)
	    {
	      sprintf(Buffer, "Error reading attachpoint %s : \nSpot %s does not exist in library\n",
		      AttachCursor->AttachPointName, AttachCursor->SpotName);
	      gtk_text_insert(GTK_TEXT(Text), NULL, &ErrorColor, NULL, Buffer, -1);
	    }
	  ErrorFound = True;
	}
      if (AttachCursor->SpotSize != 0)
	AttachCursor->SpotLocation = (Point*)malloc(AttachCursor->SpotSize * sizeof(Point));
      if (AttachCursor->CoreSpotSize != 0)
	AttachCursor->CoreSpotLocation = (Point*)malloc(AttachCursor->CoreSpotSize * sizeof(Point));
      if (AttachCursor->InvisibleSpotSize != 0)
	AttachCursor->InvisibleSpotLocation = (Point*)malloc(AttachCursor->InvisibleSpotSize * sizeof(Point));
      
      for(Counter = 0 ; Counter < AttachCursor->SpotSize ; Counter++)
	{
	  fscanf(fReader,"%d ",&(AttachCursor->SpotLocation[Counter].x));
	  fscanf(fReader,"%d\n",&(AttachCursor->SpotLocation[Counter].y));
	}
      for(Counter = 0 ; Counter < AttachCursor->CoreSpotSize ; Counter++)
	{
	  fscanf(fReader,"%d ",&(AttachCursor->CoreSpotLocation[Counter].x));
	  fscanf(fReader,"%d\n",&(AttachCursor->CoreSpotLocation[Counter].y));
	}
      for(Counter = 0 ; Counter < AttachCursor->InvisibleSpotSize ; Counter++)
	{
	  fscanf(fReader,"%d ",&(AttachCursor->InvisibleSpotLocation[Counter].x));
	  fscanf(fReader,"%d\n",&(AttachCursor->InvisibleSpotLocation[Counter].y));
	}
      AttachCursor->pNext = NULL;
      PreviousAttachCursor = AttachCursor;
      if (ErrorFound == True)
	return LibraryError;
    }
  fscanf(fReader,"\n");
  
  if (Text != NULL)
    {
      sprintf(Buffer, "Reading Component List\n");
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
    }
  fscanf(fReader,"%[^\n]\n",DummyChar);
  fscanf(fReader,"Component Number: %d\n",&NbElem);
  PreviousComp = NULL;
  for(ElemCounter = 0 ; ElemCounter < NbElem ; ElemCounter++)
    {
      Comp = (Component*)malloc(sizeof(Component));
      if (Comp == NULL) 
	return MemoryAllocationError;
      if (PreviousComp == NULL)
	CircuitData->ListComponent = Comp;
      else
	PreviousComp->pNext = Comp;

      fscanf(fReader,"%[^\n]\n",DummyChar);
      fscanf(fReader,"%[^\n]\n",Comp->ComponentName);
      fscanf(fReader,"%[^\n]\n",Comp->CompDescName);
      fscanf(fReader,"%[^\n]\n",Comp->LayoutName);
      fscanf(fReader,"%[^\n]\n",Comp->LayoutClass);
      fscanf(fReader,"%d ",&(Comp->IDNumber));
      fscanf(fReader,"%d\n",&(Comp->SubFunctionID));
      fscanf(fReader,"%d\n",&(Comp->State));
      fscanf(fReader,"%d ",&(Comp->Reference.x));
      fscanf(fReader,"%d\n",&(Comp->Reference.y));
      fscanf(fReader,"AttachPointNb: %d\n",&(Comp->AttachPointNb));

      if (Comp->AttachPointNb != 0)
	Comp->AttachPointIDTab = (int *)malloc(Comp->AttachPointNb * sizeof(int));

      for(Counter = 0 ; Counter < Comp->AttachPointNb ; Counter++)
	fscanf(fReader,"%d ",&(Comp->AttachPointIDTab[Counter]));
      fscanf(fReader,"\n");
      Check = CheckCompDesc(CircuitData->CompLib, Comp->CompDescName);
      if (Check == NotFound)
	{
	  if (Text != NULL)
	    {
	      sprintf(Buffer, "Error reading component %s : \nDescription %s does not exist in library\n",
		      Comp->ComponentName, Comp->CompDescName);
	      gtk_text_insert(GTK_TEXT(Text), NULL, &ErrorColor, NULL, Buffer, -1);
	    }
	  ErrorFound = True;
	}
      
      Check = CheckLayout(CircuitData->CompLib, Comp->LayoutName,Comp->LayoutClass);
      if (Check == NotFound)
	{
	  if (Text != NULL)
	    {
	      sprintf(Buffer, "Error reading component %s : \nLayout %s does not exist in library\n",
		      Comp->ComponentName, Comp->LayoutName);
	      gtk_text_insert(GTK_TEXT(Text), NULL, &ErrorColor, NULL, Buffer, -1);
	    }
	  ErrorFound = True;
	}
            
      PreviousComp = Comp;
      Comp->pNext=NULL;
      if (ErrorFound == True)
	return LibraryError;
    }
  fscanf(fReader,"\n");

  if (CircuitData->ComponentBufferEmpty == False)
    {
      if (Text != NULL)
	{
	  sprintf(Buffer, "Reading Component Buffer\n");
	  gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
	}
      fscanf(fReader,"%[^\n]\n",DummyChar);
      fscanf(fReader,"%[^\n]\n",CircuitData->ComponentBuffer.ComponentName);
      fscanf(fReader,"%[^\n]\n",CircuitData->ComponentBuffer.CompDescName);
      fscanf(fReader,"%[^\n]\n",CircuitData->ComponentBuffer.LayoutName);
      fscanf(fReader,"%[^\n]\n",CircuitData->ComponentBuffer.LayoutClass);
      fscanf(fReader,"%d ",&(CircuitData->ComponentBuffer.Reference.x));
      fscanf(fReader,"%d\n",&(CircuitData->ComponentBuffer.Reference.y));
      fscanf(fReader,"%d\n",&(CircuitData->ComponentBuffer.State));
     fscanf(fReader,"\n");
    }

  if (Text != NULL)
    {
      sprintf(Buffer, "Reading SubFunction List\n");
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
    }
  fscanf(fReader,"%[^\n]\n",DummyChar);
  fscanf(fReader,"SubFunction Number: %d\n",&NbElem);
  PreviousSub = NULL;
  for(ElemCounter = 0 ; ElemCounter < NbElem ; ElemCounter++)
    {
      Sub = (SubFunction*)malloc(sizeof(SubFunction));
      if (Sub == NULL) 
	return MemoryAllocationError;

      if (PreviousSub==NULL)
	CircuitData->ListSubFunction = Sub;
      else
	PreviousSub->pNext = Sub;

      fscanf(fReader,"%[^\n]\n",DummyChar);
      fscanf(fReader,"%[^\n]\n",Sub->Name);
      fscanf(fReader,"%d\n",&(Sub->IDNumber));
      fscanf(fReader,"%d\n",&(Sub->State));
      fscanf(fReader,"%d ",&(Sub->Reference.x));
      fscanf(fReader,"%d\n",&(Sub->Reference.y));
      PreviousSub=Sub;
      Sub->pNext=NULL;
    }

  fscanf(fReader,"\n");

  if (CircuitData->SubFunctionBufferEmpty == False)
    {
      if (Text != NULL)
	{
	  sprintf(Buffer, "Reading SubFunction Buffer\n");
	  gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
	}
      fscanf(fReader,"%[^\n]\n",DummyChar);
      fscanf(fReader,"%[^\n]\n",CircuitData->ListSubFunctionBuffer.Name);
      fscanf(fReader,"Comp Number: %d\n", &(CircuitData->ListSubFunctionBuffer.NbComp));
      fscanf(fReader,"Link Number: %d\n", &(CircuitData->ListSubFunctionBuffer.NbLink));

      fscanf(fReader,"%d ", &(CircuitData->ListSubFunctionBuffer.Reference.x));
      fscanf(fReader,"%d\n", &(CircuitData->ListSubFunctionBuffer.Reference.y));

      if (CircuitData->ListSubFunctionBuffer.NbComp != 0)
	CircuitData->ListSubFunctionBuffer.TabCompSF = 
	  (Component *) malloc( sizeof(Component) * CircuitData->ListSubFunctionBuffer.NbComp);

      if (CircuitData->ListSubFunctionBuffer.NbLink != 0)
	{
	  CircuitData->ListSubFunctionBuffer.TabLinkSF = 
	    (Link *) malloc( sizeof(Link) * CircuitData->ListSubFunctionBuffer.NbLink);
	  CircuitData->ListSubFunctionBuffer.LinkInfo = 
	    (int **) malloc( sizeof(int *) * CircuitData->ListSubFunctionBuffer.NbLink);
	  for (Counter  = 0 ; Counter < CircuitData->ListSubFunctionBuffer.NbLink ; Counter++)
	    CircuitData->ListSubFunctionBuffer.LinkInfo[Counter] = (int *) malloc(sizeof(int) * 4);
	}

      for(Counter = 0 ; Counter < CircuitData->ListSubFunctionBuffer.NbComp ; Counter++)
	{
	  fscanf(fReader,"%[^\n]\n",DummyChar); 
	  fscanf(fReader,"%[^\n]\n",CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].ComponentName);
	  fscanf(fReader,"%[^\n]\n",CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].CompDescName);
	  fscanf(fReader,"%[^\n]\n",CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].LayoutName);
	  fscanf(fReader,"%[^\n]\n",CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].LayoutClass);
	  fscanf(fReader,"%d ",&(CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].Reference.x));
	  fscanf(fReader,"%d\n",&(CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].Reference.y));
	  fscanf(fReader,"%d\n",&(CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].State));
	}

      for(Counter = 0 ; Counter < CircuitData->ListSubFunctionBuffer.NbLink ; Counter++)
	{
	  fscanf(fReader,"%[^\n]\n",DummyChar); 
	  fscanf(fReader,"%d ", &(CircuitData->ListSubFunctionBuffer.TabLinkSF[Counter].StartAttachID));
	  fscanf(fReader,"%d\n", &(CircuitData->ListSubFunctionBuffer.TabLinkSF[Counter].EndAttachID));
	  fscanf(fReader,"%d\n", &(CircuitData->ListSubFunctionBuffer.TabLinkSF[Counter].State));
	  fscanf(fReader,"%c ", &(CircuitData->ListSubFunctionBuffer.TabLinkSF[Counter].LinkColor.Red));
	  fscanf(fReader,"%c ", &(CircuitData->ListSubFunctionBuffer.TabLinkSF[Counter].LinkColor.Green));
	  fscanf(fReader,"%c\n", &(CircuitData->ListSubFunctionBuffer.TabLinkSF[Counter].LinkColor.Blue));
	  fscanf(fReader,"%d %d %d %d\n", &(CircuitData->ListSubFunctionBuffer.LinkInfo[Counter][0]), 
		 &(CircuitData->ListSubFunctionBuffer.LinkInfo[Counter][1]),
		 &(CircuitData->ListSubFunctionBuffer.LinkInfo[Counter][2]), 
		 &(CircuitData->ListSubFunctionBuffer.LinkInfo[Counter][3]));
	}
      fscanf(fReader,"\n");
    }
  
  if (Text != NULL)
    {
      sprintf(Buffer, "Reading Link List\n");
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
    }
  fscanf(fReader,"%[^\n]\n",DummyChar);
  fscanf(fReader,"Link Number: %d\n",&NbElem);
  PreviousLink=NULL;
  for(ElemCounter = 0 ; ElemCounter < NbElem ; ElemCounter++)
    {
      LinkCursor = (Link*)malloc(sizeof(Link));
      if (LinkCursor==NULL) 
	return MemoryAllocationError;
      if (PreviousLink==NULL)
	CircuitData->ListLink=LinkCursor;
      else
	PreviousLink->pNext=LinkCursor;
      fscanf(fReader,"%[^\n]\n",DummyChar);
      fscanf(fReader,"%d\n",&(LinkCursor->IDNumber));
      fscanf(fReader,"%d ",&(LinkCursor->StartAttachID));
      fscanf(fReader,"%d\n",&(LinkCursor->EndAttachID));
      fscanf(fReader,"%d\n", &(LinkCursor->State));
      fscanf(fReader,"%d ",&(LinkCursor->LinkLength));
      fscanf(fReader,"%d\n",&(LinkCursor->LinkWidth));
      fscanf(fReader,"%d ",&(LinkCursor->Start.x));
      fscanf(fReader,"%d\n",&(LinkCursor->Start.y));
      fscanf(fReader,"%d ",&(LinkCursor->End.x));
      fscanf(fReader,"%d\n", &( LinkCursor->End.y));
      fscanf(fReader,"%d\n",&(LinkCursor->RoutedOrder));
      fscanf(fReader,"%c ",&(LinkCursor->LinkColor.Red));
      fscanf(fReader,"%c ",&(LinkCursor->LinkColor.Green));
      fscanf(fReader,"%c\n",&(LinkCursor->LinkColor.Blue));
      if (IsLinkRouted(LinkCursor->State))
	{
	  fscanf(fReader,"%d ",&(LinkCursor->Road.SegSize));
	  fscanf(fReader,"%d ",&(LinkCursor->Road.PathSize));
	  fscanf(fReader,"%d ",&(LinkCursor->Road.CorePathSize));
	  fscanf(fReader,"%d ",&(LinkCursor->Road.DiagonalSize));
	  fscanf(fReader,"%d ",&(LinkCursor->Road.DepSize));
	  fscanf(fReader,"%d\n",(int *) &( LinkCursor->Road.JobDone));

	  if (LinkCursor->Road.SegSize != 0)
	    {
	      LinkCursor->Road.SegTab = (Segment*)malloc(LinkCursor->Road.SegSize*sizeof(Segment));
	      for(Counter = 0 ; Counter < LinkCursor->Road.SegSize ; Counter++)
		{
		  fscanf(fReader,"%d ",&(LinkCursor->Road.SegTab[Counter].Start.x));   
		  fscanf(fReader,"%d\n",&(LinkCursor->Road.SegTab[Counter].Start.y));
		  fscanf(fReader,"%d ",&(LinkCursor->Road.SegTab[Counter].End.x));
		  fscanf(fReader,"%d\n",&(LinkCursor->Road.SegTab[Counter].End.y));
		}
	    }
	  if (LinkCursor->Road.PathSize != 0)
	    {
	      LinkCursor->Road.PathTab = (Point*)malloc(LinkCursor->Road.PathSize*sizeof(Point));
	      for(Counter = 0 ; Counter < LinkCursor->Road.PathSize ; Counter++)
		{
		  fscanf(fReader,"%d ",&(LinkCursor->Road.PathTab[Counter].x));
		  fscanf(fReader,"%d ",&(LinkCursor->Road.PathTab[Counter].y));
		  fscanf(fReader,"\n");
		}
	    }
	  if (LinkCursor->Road.DiagonalSize != 0)
	    {
	      LinkCursor->Road.DiagonalTab = (Point*)malloc(LinkCursor->Road.DiagonalSize*sizeof(Point));
	      for(Counter = 0 ; Counter < LinkCursor->Road.DiagonalSize ; Counter++)
		{
		  fscanf(fReader,"%d ",&(LinkCursor->Road.DiagonalTab[Counter].x));
		  fscanf(fReader,"%d ",&(LinkCursor->Road.DiagonalTab[Counter].y));
		  fscanf(fReader,"\n");
		}
	    }
	  if (LinkCursor->Road.DepSize != 0)
	    {
	      LinkCursor->Road.DepTab = (DepCarac*)malloc(LinkCursor->Road.DepSize*sizeof(DepCarac));
	      for(Counter = 0 ; Counter < LinkCursor->Road.DepSize ; Counter++)
		{
		  fscanf(fReader,"%d\n",(int *) &( LinkCursor->Road.DepTab[Counter].Type)); 
		  fscanf(fReader,"%d\n",&(LinkCursor->Road.DepTab[Counter].Nb));
		}
	    }
	}
      PreviousLink=LinkCursor;
      LinkCursor->pNext=NULL;
    }
  fscanf(fReader,"\n");

  if (Text != NULL)
    {
      sprintf(Buffer, "Reading LinkToDraw List\n");
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
    }
  fscanf(fReader,"%[^\n]\n",DummyChar);
  fscanf(fReader,"DrawLink Number: %d\n",&NbElem);
  for(ElemCounter = 0 ; ElemCounter < NbElem ; ElemCounter++)
    {
      NewDLink = (DrawLink *) malloc( sizeof(DrawLink) );
      fscanf(fReader,"%d ",&(NewDLink->LinkID));
      fscanf(fReader,"%d ",&(NewDLink->LinkLength));
      fscanf(fReader,"%d\n",&(NewDLink->LinkWidth));
      fscanf(fReader,"%d\n", &(NewDLink->State));
      fscanf(fReader,"%d\n",(int *) &(NewDLink->RoutedOrder));
      
      NewDLink->LinkUp = CircuitData->BottomLink;
      NewDLink->LinkDown = NULL;
      CircuitData->BottomLink = NewDLink;

      if (NewDLink->LinkUp != NULL)
	NewDLink->LinkUp->LinkDown = NewDLink;
      else 
	CircuitData->TopLink = NewDLink;
    }
  fscanf(fReader,"\n");
  if (Text != NULL)
    {
      sprintf(Buffer, "Reading Permutation List\n");
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
    }
  fscanf(fReader,"%[^\n]\n",DummyChar);
  fscanf(fReader,"Permutation Number: %d\n",&NbElem);
  
  PrevPerm = NULL;
  for(ElemCounter = 0 ; ElemCounter < NbElem ; ElemCounter++)
    {
      NewPerm = (LinkPermutation *) malloc(sizeof(LinkPermutation));
      if (PrevPerm == NULL)
	CircuitData->ListPermutation = NewPerm;
      else
	PrevPerm->pNext = NewPerm;

      fscanf(fReader,"%d ",&(NewPerm->LinkID1));
      fscanf(fReader,"%d ",&(NewPerm->LinkID2));
      fscanf(fReader,"%d\n",&(NewPerm->Number));
      NewPerm->pNext = NULL;
      PrevPerm = NewPerm;
    }
  if (Text != NULL)
    {
      sprintf(Buffer, "File Reading Successful\n");
      gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, Buffer, -1);
    }
  fclose(fReader);
  return AllOk;
}

ErrorCheck OpenCircuitFile(GCD *CircuitData, char *FileName, GtkWidget *Text)
{
  int NewID[IDTypeNumber], FileNewID[IDTypeNumber];
  ErrorCheck Check;

  NewID[0] = CircuitData->Info.AttachPointID;
  NewID[1] = CircuitData->Info.ComponentID;
  NewID[2] = CircuitData->Info.SubFunctionID;
  NewID[3] = CircuitData->Info.LinkID;

  DeleteCircuitData(CircuitData, False); 
  InitCircuitData(CircuitData, False, False);
  
  if (Text != NULL)
    gtk_text_freeze(GTK_TEXT(Text));
  Check = ReadGCD(CircuitData, FileName, FileNewID, Text);
  
  if (Check != AllOk)
    {
      DeleteCircuitData(CircuitData, False); 
      InitCircuitData(CircuitData, True, False);
      CircuitData->Info.AttachPointID = NewID[0];
      CircuitData->Info.ComponentID = NewID[1];
      CircuitData->Info.SubFunctionID = NewID[2];
      CircuitData->Info.LinkID = NewID[3];
      CircuitData->Info.RoutedOrder = -1;  
    }

  Check = CheckCircuitLibrary(CircuitData);
  if (Check == LibraryError)
    {
      DeleteCircuitData(CircuitData, False); 
      InitCircuitData(CircuitData, True, False);
      CircuitData->Info.AttachPointID = NewID[0];
      CircuitData->Info.ComponentID = NewID[1];
      CircuitData->Info.SubFunctionID = NewID[2];
      CircuitData->Info.LinkID = NewID[3];
      CircuitData->Info.RoutedOrder = -1;  
   }
    
  if (Text != NULL)
    gtk_text_thaw(GTK_TEXT(Text));
  
  CircuitData->Info.AttachPointID = FileNewID[0];
  CircuitData->Info.ComponentID = FileNewID[1];
  CircuitData->Info.SubFunctionID = FileNewID[2];
  CircuitData->Info.LinkID = FileNewID[3];

  CheckDrawListConsistency(CircuitData);
  UpdateInterfaceFromData();
  
  if (Check == AllOk)
    return AllOk;
  else
    return FileAccessError;
}

ErrorCheck OpenEmptyFile(GCD *CircuitData)
{
  
  CircuitData = GetCircuitData();

  DeleteCircuitData(CircuitData, False); 
  InitCircuitData(CircuitData, True, False);

  UpdateInterfaceFromData();
  return AllOk;
}

ErrorCheck SaveCircuitFile(GCD *CircuitData, char *FileName, GtkWidget *Text)
{
  int NewID[IDTypeNumber];
  ErrorCheck Check;
  CircuitData = GetCircuitData();
  
  NewID[0] = CircuitData->Info.AttachPointID;
  NewID[1] = CircuitData->Info.ComponentID;
  NewID[2] = CircuitData->Info.SubFunctionID;
  NewID[3] = CircuitData->Info.LinkID;

  if (Text != NULL)
    gtk_text_freeze(GTK_TEXT(Text));
  Check = WriteGCD(CircuitData, FileName, NewID, Text);
  if (Text != NULL)
    gtk_text_thaw(GTK_TEXT(Text));
  return Check;
}

ErrorCheck UpdateInterfaceFromData(void )
{
  UpdateCTWindow();
  UpdateLLClist();
  UpdateDLClist();
  return AllOk;
}

ErrorCheck CheckCircuitLibrary(GCD *CircuitData)
{
  Component *CompCursor;
  AttachPoint *AttachCursor;
  Boolean ErrorFound;
  ErrorCheck Check;
  int Counter;

  CompCursor = CircuitData->ListComponent;
  ErrorFound = False;
  while ((CompCursor != NULL) && (ErrorFound == False))
    {
      Check = CheckCompDesc(CircuitData->CompLib, CompCursor->CompDescName);
      if (Check == NotFound)
	ErrorFound = True;

      Check = CheckLayout(CircuitData->CompLib, CompCursor->LayoutName, CompCursor->LayoutClass);
      if (Check == NotFound)
	ErrorFound = True;

      CompCursor = CompCursor->pNext;
    }
  AttachCursor = CircuitData->ListAttachPoint;
  while ((AttachCursor != NULL) &&(ErrorFound == False))
    {
      Check = CheckSpot(CircuitData->CompLib, AttachCursor->SpotName);
      if (Check == NotFound)
	ErrorFound = True;
      AttachCursor = AttachCursor->pNext;
    }
  if (CircuitData->ComponentBufferEmpty == False)
    {
      Check  = CheckCompDesc(CircuitData->CompLib, CircuitData->ComponentBuffer.CompDescName);
      if (Check == NotFound)
	ErrorFound = True;
      Check  = CheckLayout(CircuitData->CompLib, CircuitData->ComponentBuffer.LayoutName, CircuitData->ComponentBuffer.LayoutClass);
      if (Check == NotFound)
	ErrorFound = True;
    }
  if (CircuitData->SubFunctionBufferEmpty == False)
    {
      for(Counter = 0; Counter < CircuitData->ListSubFunctionBuffer.NbComp ; Counter++)
	{
	  Check  = CheckCompDesc(CircuitData->CompLib,
				 CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].CompDescName);
	  if (Check == NotFound)
	    ErrorFound = True;
	  Check  = CheckLayout(CircuitData->CompLib,
				 CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].LayoutName,
				 CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].LayoutClass);
	  if (Check == NotFound)
	    ErrorFound = True;
	}
    }
  if (ErrorFound == True)
    return LibraryError;
  else
    return AllOk;
}
    
