
#include <stdio.h>
#include <stdlib.h>
#include "filltab.h"
#include "globalvariables.h"
#include "equipot.h"
#include "misc.h"
#include "roadncircuit.h"

ErrorCheck FillTab (int CurrentLineNb, Circuit *pPlate, RoadMap Map, Point *pMeetPoint)
/* remplit le tableau Plate en fonction des equipotentielles de depart et d arrivee
   le nb de points de chaque equipot est StartSize et EndSize
   Renvoie Found si on arrive a joindre l arrivee NotFound sinon 
   EndNb tableau des numeros des points equipotentiels possibles pour l arrivee*/

{
  /* SquareFree == True : le voisin n est utilise ni pour cette liaison ni pour une liaison precedente */
  /* SquareFriend == True : le voisin est utilise par une piste equipot au point d arrivee */
  ErrorCheck Match, MatchEnd, SameEquiPot;
  int CurrentLevel, Counter, DuplicateCounter, x, y, Currentx, Currenty, VortexCounter, Vx, Vy,
    StartPointNb, EndPointNb, Basex, Basey, DirectionCounter, DirectionTabSize;
  Boolean SquareFree, SquareFriend, Reachable;
  Direction CurrentDirection, DirectionTab[8];
  EquiPotential Next, Current, End, VortexEqui, TempEquiPot;
  Point TempEquiPoint;
  
  TempEquiPot.PointTab = &TempEquiPoint;
  TempEquiPot.Size = 1;
 
  CurrentLevel = 1;

  pPlate->VortexNb =  0;
  pPlate->VortexTab = (VortexCarac *) malloc( sizeof(VortexCarac) * pPlate->AttachNb);

  StartPointNb = pPlate->LinkTab[CurrentLineNb].Start;
  EndPointNb = pPlate->LinkTab[CurrentLineNb].End;
    
  MakeEquiPot(StartPointNb, *pPlate, Map, &Current, NULL); /* on utilise current pour le depart */
  MakeEquiPot(EndPointNb, *pPlate, Map, &End, NULL);

  Match = FindInterEquiPot(Current, End, pMeetPoint);
  if (Match ==Found)
    {
      free(Current.PointTab);
      free(End.PointTab);
      return Found;
    }
  
  Next.Size = 0;
  Next.PointTab = (Point *) malloc (sizeof (Point) * 8 * Current.Size);
  
  for (Counter = 0; Counter < Current.Size; Counter++)
    {
      x = Current.PointTab[Counter].x;
      y = Current.PointTab[Counter].y;

      pPlate->SquareTab[x][y].CalculatingLineNb = CurrentLineNb;
      pPlate->SquareTab[x][y].CalculatingLevelNb = 1;
    }

  do
    {
      for (Counter = 0; Counter < Current.Size; Counter++)
	 {
	   x = Current.PointTab[Counter].x;
	   y = Current.PointTab[Counter].y;
	   ComputeDirection(pPlate, NoDirection, DirectionTab, &DirectionTabSize);
	   for (DirectionCounter = 0 ; DirectionCounter < DirectionTabSize ; DirectionCounter++)
	     {
	       CurrentDirection = DirectionTab[DirectionCounter]; 
	       Currentx = x + ForwardDirection[CurrentDirection][0];
	       Currenty = y + ForwardDirection[CurrentDirection][1];
	       
	       SquareFree = False;
	       SquareFriend = False;

	       if (IsDiagonalDirection(CurrentDirection))
		 Reachable = False;
	       else 
		 Reachable = True;

	       
	       if ( Reachable == False)
		 { /* uniquement pour les diagonales */
		   Basex = x + BaseDirection[ Direction2DiagonalType(CurrentDirection) ][0];
		   Basey = y + BaseDirection[ Direction2DiagonalType(CurrentDirection) ][1];

		   if (IsFreeFourCorners(Basex, Basey, pPlate->SquareTab))
		     Reachable = True;
		   
		   if (IsInvalidFourCorners(Basex, Basey, pPlate->SquareTab))
		     {
		       printf("\bInvalid four corners at %d %d\n", Basex, Basey);
		       free(Current.PointTab);
		       free(Next.PointTab);
		       free(End.PointTab);
		       return Bug;
		     }	  
		 }
	       
	       if ((Reachable == True) 
		   && (pPlate->SquareTab[Currentx][Currenty].CalculatingLineNb != CurrentLineNb) 
		   && ( !IsSquareUsed(pPlate->SquareTab[Currentx][Currenty].SquareState))
		   && ( !IsFeatherUsed(pPlate->SquareTab[Currentx][Currenty].SquareState)))
		 SquareFree = True;
	       
	       if ( (Reachable == True)
		    &&  (IsSquareUsed(pPlate->SquareTab[Currentx][Currenty].SquareState)) 
		    && (pPlate->SquareTab[Currentx][Currenty].AttachLinkNb != NoAttachLink)
		    && (pPlate->SquareTab[Currentx][Currenty].CalculatingLineNb != CurrentLineNb ) )
		 {
		   SameEquiPot = IsSameEquiPot(pPlate, pPlate->SquareTab[Currentx][Currenty].AttachLinkNb,
					 pPlate->SquareTab[Currentx][Currenty].SquareState, CurrentLineNb);
		   
		   if (SameEquiPot == Found)
		     {
		       TempEquiPot.PointTab[0].x = Currentx;
		       TempEquiPot.PointTab[0].y = Currenty;
		       MatchEnd = FindInterEquiPot(TempEquiPot, End, NULL);

		       if (MatchEnd == Found)
			 SquareFree = True;  
		       else 
			 SquareFriend = True;
		     }
		 }
	       if ( SquareFree==True ) 
		 {
		   pPlate->SquareTab[Currentx][Currenty].CalculatingLineNb = CurrentLineNb;
		   pPlate->SquareTab[Currentx][Currenty].CalculatingLevelNb = CurrentLevel+1;
		   
		   Next.PointTab[Next.Size].x = Currentx;
		   Next.PointTab[Next.Size].y = Currenty;
		   Next.Size++;
		 }
	       
	       if (SquareFriend== True)
		 {
		   MakeVortex(pPlate, Map,  Currentx, Currenty, &VortexEqui);
		   AddEquiPot(&Next, Current.Size, VortexEqui);
		   for (VortexCounter = 0 ; VortexCounter < VortexEqui.Size ; VortexCounter++)
		     {
		       Vx = VortexEqui.PointTab[VortexCounter].x;
		       Vy = VortexEqui.PointTab[VortexCounter].y;
		       pPlate->SquareTab[Vx][Vy].CalculatingVortexNb  =  pPlate->VortexNb;
		       pPlate->SquareTab[Vx][Vy].CalculatingVortexLineNb  =  CurrentLineNb;
		       pPlate->SquareTab[Vx][Vy].CalculatingLineNb  =  CurrentLineNb;
		       pPlate->SquareTab[Vx][Vy].CalculatingLevelNb  =  CurrentLevel+1;
		     }     	      
		   free(VortexEqui.PointTab);
		   pPlate->VortexTab[pPlate->VortexNb].VortexExit.x = Currentx;
		   pPlate->VortexTab[pPlate->VortexNb].VortexExit.y = Currenty;   
		   
		   pPlate->VortexNb++;
		 }
	     } /* for qui parcourt les 8 directions */
	 } /* fin du for qui parcourt l equipot */
      
      Match = FindInterEquiPot (Next, End, pMeetPoint);
      if (Match == Found)
	{
	  free (Current.PointTab);
	  free (Next.PointTab);
	  free(End.PointTab);
	  return Found;
	}
      
      if (Match != NotFound)
	{
	  free (Current.PointTab);
	  free (Next.PointTab);
	  free(End.PointTab);
	  return Match;
	}

      if (Next.Size == 0)
	{
	  free(Current.PointTab);
	  free(Next.PointTab);
	  free(End.PointTab);
	  
	  return NotFound;
	}

      free (Current.PointTab);
      Current.PointTab = (Point *) malloc (Next.Size * sizeof (Point));
      Current.Size = Next.Size;

      for (DuplicateCounter = 0; DuplicateCounter < Next.Size; DuplicateCounter++)
	{
	  Current.PointTab[DuplicateCounter].x = Next.PointTab[DuplicateCounter].x;
	  Current.PointTab[DuplicateCounter].y = Next.PointTab[DuplicateCounter].y;
	}
      
      free (Next.PointTab);
      Next.PointTab = (Point *) malloc (Current.Size * 8 * sizeof (Point));
      Next.Size = 0;
      CurrentLevel++;
    }
  while (1);
  printf(" \n\n\bCaramba!!!!!\n\n");
  return Bug;
}

