#include "layout.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "spot.h"

ErrorCheck LoadLayoutLib(ComponentLibrary *CompLib, char *LayoutFileName)
{
  int Counter, LayoutNb, LayoutCounter; 
  FILE *LayoutFile;
  LayoutDef LayoutBuffer;
  Boolean ErrorFound;
  ErrorCheck Check;

  LayoutFile = fopen(LayoutFileName,"r");

  if (LayoutFile == NULL)
    {
      printf(" Error Opening LayoutFile %s\n", LayoutFileName);
      return FileAccessError;
    }
  ErrorFound = False;
  fscanf(LayoutFile, "%d\n", &LayoutNb);
  for (LayoutCounter = 0 ; LayoutCounter < LayoutNb ; LayoutCounter++)
    {
      fscanf(LayoutFile, "%[^\n]\n",  LayoutBuffer.LayoutName);
      fscanf(LayoutFile, "%[^\n]\n",  LayoutBuffer.LayoutClass);
      fscanf(LayoutFile,"%d %d %d %d\n", &(LayoutBuffer.PinNb), &(LayoutBuffer.BetweenPinsSize), 
	     &(LayoutBuffer.CompoInSize), &(LayoutBuffer.CompoOutSize));
      if ( LayoutBuffer.PinNb != 0) 
	{
	  LayoutBuffer.PinPosition = (Point *) malloc ( LayoutBuffer.PinNb * sizeof(Point));
	  LayoutBuffer.SpotNameTab = (char **) malloc ( LayoutBuffer.PinNb * sizeof(char *));
	}
      else 
	printf("WARNING !!! : A layout with no pin can make the program crash...\n");
      
      if ( LayoutBuffer.BetweenPinsSize != 0) 
	LayoutBuffer.BetweenPinsPosition = (Point *) malloc (LayoutBuffer.BetweenPinsSize * sizeof(Point));
      if ( LayoutBuffer.CompoInSize != 0) 
	LayoutBuffer.CompoInPosition = (Point *) malloc (LayoutBuffer.CompoInSize * sizeof(Point));
      if ( LayoutBuffer.CompoOutSize != 0) 
	LayoutBuffer.CompoOutPosition = (Point *) malloc (LayoutBuffer.CompoOutSize * sizeof(Point));

      for (Counter = 0 ; Counter < LayoutBuffer.PinNb ; Counter++)
	{
	  LayoutBuffer.SpotNameTab[Counter] = (char *) malloc(sizeof(char) * SpotNameLength);
	  fscanf(LayoutFile,"%d %d\n", &(LayoutBuffer.PinPosition[Counter].x),&(LayoutBuffer.PinPosition[Counter].y));
	  fscanf(LayoutFile,"%[^\n]\n", LayoutBuffer.SpotNameTab[Counter]); 
	  Check = CheckSpot(CompLib, LayoutBuffer.SpotNameTab[Counter]);
	  if (Check == NotFound)
	    ErrorFound = True;
	}
    
      for (Counter = 0 ; Counter < LayoutBuffer.BetweenPinsSize ; Counter++)
	fscanf(LayoutFile,"%d %d\n", &(LayoutBuffer.BetweenPinsPosition[Counter].x),
	       &(LayoutBuffer.BetweenPinsPosition[Counter].y));
      for (Counter = 0 ; Counter < LayoutBuffer.CompoInSize ; Counter++)
	fscanf(LayoutFile,"%d %d\n", &(LayoutBuffer.CompoInPosition[Counter].x),
	       &(LayoutBuffer.CompoInPosition[Counter].y));
      for (Counter = 0 ; Counter < LayoutBuffer.CompoOutSize ; Counter++)
	fscanf(LayoutFile,"%d %d\n", &(LayoutBuffer.CompoOutPosition[Counter].x),
	       &(LayoutBuffer.CompoOutPosition[Counter].y));
      
      if (ErrorFound == False)
	AddLayoutDef(CompLib, LayoutBuffer);
      
      if ( LayoutBuffer.PinNb != 0) 
	{
	  free(LayoutBuffer.PinPosition);
	  for (Counter = 0 ; Counter < LayoutBuffer.PinNb ; Counter++)
	    free(LayoutBuffer.SpotNameTab[Counter]);
	  free(LayoutBuffer.SpotNameTab);
	}
      if ( LayoutBuffer.BetweenPinsSize != 0) 
	free(LayoutBuffer.BetweenPinsPosition);
      if ( LayoutBuffer.CompoInSize != 0) 
	free(LayoutBuffer.CompoInPosition);
      if ( LayoutBuffer.CompoOutSize != 0) 
	free(LayoutBuffer.CompoOutPosition);
      
    }
  fclose(LayoutFile);
  return AllOk;
}

ErrorCheck AddLayoutDef( ComponentLibrary *CompLib, LayoutDef LayoutDefProp)
{
  LayoutDef *LayoutDefCursor, *LayoutDefCursorPrevious, *NewLayoutDef;
  Boolean LayoutFound;
  int Counter;
  
  LayoutFound = False;
  
  LayoutDefCursor = CompLib->LayoutDefList;
  LayoutDefCursorPrevious  = NULL;
  
  while ( (LayoutDefCursor != NULL ) && (LayoutFound == False))
    {
      if ( strcmp(LayoutDefCursor->LayoutName, LayoutDefProp.LayoutName) >= 0 )
	LayoutFound = True;
      else
	{
	  LayoutDefCursorPrevious = LayoutDefCursor;
	  LayoutDefCursor = LayoutDefCursor->pNext;
	}
    }
  
  NewLayoutDef = (LayoutDef *) malloc( sizeof(LayoutDef));
  
  NewLayoutDef->pNext = LayoutDefCursor;

  if (LayoutDefCursorPrevious != NULL)
    LayoutDefCursorPrevious->pNext = NewLayoutDef;
  else 
    CompLib->LayoutDefList =  NewLayoutDef;
  
  strncpy(NewLayoutDef->LayoutName, LayoutDefProp.LayoutName, LayoutNameLength);
  NewLayoutDef->LayoutName[LayoutNameLength-1] = '\0';

  strncpy(NewLayoutDef->LayoutClass, LayoutDefProp.LayoutClass, LayoutClassLength);
  NewLayoutDef->LayoutName[LayoutClassLength-1] = '\0';
  
  NewLayoutDef->PinNb =  LayoutDefProp.PinNb;
  if (NewLayoutDef->PinNb != 0)
    {
      NewLayoutDef->PinPosition = (Point *) malloc(sizeof(Point) *   NewLayoutDef->PinNb );
      NewLayoutDef->SpotNameTab = (char **) malloc (sizeof(char *) * NewLayoutDef->PinNb );
    }

  for (Counter =  0 ; Counter <  NewLayoutDef->PinNb ; Counter++)
    {
      NewLayoutDef->SpotNameTab[Counter] = (char *) malloc (sizeof(char) * SpotNameLength);
      NewLayoutDef->PinPosition[Counter].x  =  LayoutDefProp.PinPosition[Counter].x;
      NewLayoutDef->PinPosition[Counter].y  =  LayoutDefProp.PinPosition[Counter].y;
      
      strncpy(NewLayoutDef->SpotNameTab[Counter],LayoutDefProp.SpotNameTab[Counter], SpotNameLength);
      NewLayoutDef->SpotNameTab[Counter][SpotNameLength-1] = '\0';
    }
  NewLayoutDef->BetweenPinsSize = LayoutDefProp.BetweenPinsSize;
  NewLayoutDef->CompoInSize = LayoutDefProp.CompoInSize;
  NewLayoutDef->CompoOutSize = LayoutDefProp.CompoOutSize;

  if (NewLayoutDef->BetweenPinsSize != 0)
    NewLayoutDef->BetweenPinsPosition = (Point *) malloc(sizeof(Point) * NewLayoutDef->BetweenPinsSize);
  if (NewLayoutDef->CompoInSize != 0)
    NewLayoutDef->CompoInPosition = (Point *) malloc(sizeof(Point) * NewLayoutDef->CompoInSize);
  if (NewLayoutDef->CompoOutSize != 0)
    NewLayoutDef->CompoOutPosition = (Point *) malloc(sizeof(Point) * NewLayoutDef->CompoOutSize);
  
  for (Counter = 0 ; Counter < NewLayoutDef->BetweenPinsSize  ; Counter++)
    NewLayoutDef->BetweenPinsPosition[Counter] = LayoutDefProp.BetweenPinsPosition[Counter];
  for (Counter = 0 ; Counter < NewLayoutDef->CompoInSize  ; Counter++)
    NewLayoutDef->CompoInPosition[Counter] = LayoutDefProp.CompoInPosition[Counter];
  for (Counter = 0 ; Counter < NewLayoutDef->CompoOutSize  ; Counter++)
    NewLayoutDef->CompoOutPosition[Counter] = LayoutDefProp.CompoOutPosition[Counter];
  
  return AllOk;
}
ErrorCheck AddLayoutLibToLoad(ComponentLibrary *CompLib, LibToLoad LayoutLib)
{
  LibToLoad *NewLayout, *LayoutCursor;
  
  NewLayout = NULL;
  LayoutCursor = CompLib->LayoutLibList;
  while (LayoutCursor != NULL)
    {
      NewLayout = LayoutCursor;
      LayoutCursor = LayoutCursor->pNext;
    }
  
  if (NewLayout != NULL)
    {
      NewLayout->pNext = (LibToLoad *) malloc (sizeof(LibToLoad));
      LayoutCursor = NewLayout->pNext;
    }
  else 
    {
      CompLib->LayoutLibList = (LibToLoad *) malloc (sizeof(LibToLoad));
      LayoutCursor = CompLib->LayoutLibList;
    }

  strncpy(LayoutCursor->LibName, LayoutLib.LibName, LibNameLength);
  LayoutCursor->pNext = NULL;
  return AllOk;
}

ErrorCheck GetLayoutProperties(ComponentLibrary *CompLib, char *LayoutName, char *LayoutClass, LayoutDef *LayoutProp,
			       Boolean WithPinPosition, Boolean WithAreaTabs)
{
  LayoutDef  *LayoutCursor;
  Boolean LayoutFound;
  int Counter;

  LayoutCursor = CompLib->LayoutDefList;
  LayoutFound  = False;
  while ((LayoutCursor != NULL) && (LayoutFound == False))
    {
      if ((strcmp(LayoutCursor->LayoutName, LayoutName) == 0) &&
	  (strcmp(LayoutCursor->LayoutClass, LayoutClass) == 0))
	{
	  printf("LayoutFound\n");
	  LayoutFound = True;
	  strncpy(LayoutProp->LayoutName, LayoutCursor->LayoutName, LayoutNameLength);
	  LayoutProp->LayoutName[LayoutNameLength-1] = '\0';
	  strncpy(LayoutProp->LayoutClass, LayoutCursor->LayoutClass, LayoutClassLength);
	  LayoutProp->LayoutClass[LayoutClassLength-1] = '\0';
	  
	  LayoutProp->PinNb = LayoutCursor->PinNb;
	  LayoutProp->BetweenPinsSize = LayoutCursor->BetweenPinsSize;
	  LayoutProp->CompoInSize = LayoutCursor->CompoInSize;
	  LayoutProp->CompoOutSize = LayoutCursor->CompoOutSize;
	  if (WithPinPosition == True)
	    {
	      for (Counter = 0 ; Counter < LayoutProp->PinNb ; Counter++)	 
		{
		  LayoutProp->PinPosition[Counter] = LayoutCursor->PinPosition[Counter];
		  strncpy(LayoutProp->SpotNameTab[Counter], LayoutCursor->SpotNameTab[Counter], SpotNameLength);
		  LayoutProp->SpotNameTab[Counter][SpotNameLength-1] = '\0';
		}
	    }
	  if (WithAreaTabs == True)
	    {
	      for (Counter = 0 ; Counter < LayoutProp->BetweenPinsSize ; Counter++)	 
		LayoutProp->BetweenPinsPosition[Counter] = LayoutCursor->BetweenPinsPosition[Counter];
	      for (Counter = 0 ; Counter < LayoutProp->CompoInSize ; Counter++)	 
		LayoutProp->CompoInPosition[Counter] = LayoutCursor->CompoInPosition[Counter];
	      for (Counter = 0 ; Counter < LayoutProp->CompoOutSize ; Counter++)	 
		LayoutProp->CompoOutPosition[Counter] = LayoutCursor->CompoOutPosition[Counter];
	    }
	}
      LayoutCursor = LayoutCursor->pNext;
    }
  
  if (LayoutFound == True)
    return Found;
  else return NotFound;
}	    
ErrorCheck GetLayoutNumberForClass(ComponentLibrary *CompLib, char *ClassName, int *LayoutNumber)
{
  LayoutDef *LayoutCursor;
  int Counter;
  LayoutCursor = CompLib->LayoutDefList;
  Counter = 0;
  while (LayoutCursor != NULL)
    {
      if ( strcmp(ClassName, LayoutCursor->LayoutClass) == 0)
	Counter++;
      LayoutCursor = LayoutCursor->pNext;
    }
  *LayoutNumber = Counter;
 
  return AllOk;
}
ErrorCheck GetLayoutNameTabForClass(ComponentLibrary *CompLib, char *ClassName, char **LayoutTab)
{
  LayoutDef *LayoutCursor;
  int Counter;

  LayoutCursor = CompLib->LayoutDefList;
  Counter = 0;
  while (LayoutCursor != NULL)
    {
      if ( strcmp(ClassName, LayoutCursor->LayoutClass) == 0)
	{
	  strncpy(LayoutTab[Counter], LayoutCursor->LayoutName, LayoutNameLength);
	  LayoutTab[Counter][LayoutNameLength-1] = '\0';
	  Counter++;
	}
      LayoutCursor = LayoutCursor->pNext;
    }
 
  return AllOk;
}

ErrorCheck GetLayoutClassNumber(ComponentLibrary *CompLib, int *ClassNumber)
{
  LayoutDef *LayoutCursor;
  int Counter, LayoutNb, LayoutClassNb;
  char **LayoutClassTab;
  Boolean NewClass;

  LayoutCursor = CompLib->LayoutDefList;
  Counter = 0;
  while (LayoutCursor != NULL)
    {
      Counter++;
      LayoutCursor = LayoutCursor->pNext;
    }

  LayoutNb = Counter;
  LayoutClassTab = (char **) malloc(sizeof(char *) * LayoutNb);

  LayoutCursor = CompLib->LayoutDefList;
  LayoutClassNb = 0;
  
  while (LayoutCursor != NULL)
    {
      NewClass = True;
      for (Counter = 0 ; (Counter < LayoutClassNb) && (NewClass == True) ; Counter++)
	if (strcmp(LayoutCursor->LayoutClass, LayoutClassTab[Counter]) ==0) 
	  NewClass = False; /* la classe a deja ete comptee */
      if (NewClass == True) /* C est une nouvelle classe */
	{
	  LayoutClassTab[LayoutClassNb] = (char *)malloc(sizeof (char) * LayoutClassLength);

	  strncpy (LayoutClassTab[LayoutClassNb], LayoutCursor->LayoutClass , LayoutClassLength);
	  LayoutClassTab[LayoutClassNb][LayoutClassLength-1] = '\0';
	  LayoutClassNb++;
	}
      LayoutCursor = LayoutCursor->pNext;
    }
  *ClassNumber  = LayoutClassNb;

  for (Counter = 0 ; Counter < LayoutClassNb ; Counter++)
    free(LayoutClassTab[Counter]);
  free(LayoutClassTab);

  return AllOk;
}	    

ErrorCheck GetLayoutClassTab(ComponentLibrary *CompLib, char **LayoutClassTab)
{
  LayoutDef *LayoutCursor;
  int Counter, LayoutClassCounter;
  Boolean NewClass;

  LayoutCursor = CompLib->LayoutDefList;

  LayoutClassCounter = 0;
  while (LayoutCursor != NULL)
    {
      NewClass = True;
      for (Counter = 0 ; (Counter < LayoutClassCounter) && (NewClass == True) ; Counter++)
	if (strcmp(LayoutCursor->LayoutClass, LayoutClassTab[Counter]) ==0) 
	  NewClass = False; /* la classe a deja ete comptee */

      if (NewClass == True) /* C est une nouvelle classe */
	{
	  strncpy (LayoutClassTab[LayoutClassCounter], LayoutCursor->LayoutClass , LayoutClassLength);
	  LayoutClassTab[LayoutClassCounter][LayoutClassLength-1] = '\0';
	  LayoutClassCounter++;
	}
      LayoutCursor = LayoutCursor->pNext;
    }

  return AllOk;
}	      
      
ErrorCheck CheckLayout(ComponentLibrary *CompLib, char *LayoutName, char *LayoutClass)
{
  LayoutDef *LayoutCursor;
  Boolean LayoutFound;
  
  LayoutCursor = CompLib->LayoutDefList;
  LayoutFound = False;

  while ((LayoutCursor != NULL) && (LayoutFound == False))
    {
      if (strcmp(LayoutCursor->LayoutName, LayoutName) == 0)
	LayoutFound = True;
      LayoutCursor = LayoutCursor->pNext;
    }
  if (LayoutFound == True)
    return Found;
  else return NotFound;
}

ErrorCheck DisplayLayoutLib(ComponentLibrary *CompLib)
{
  LayoutDef *LayoutCursor;
  int Counter;

  LayoutCursor = CompLib->LayoutDefList;
  printf("Layout\n");
  while (LayoutCursor != NULL)
    {
      printf("---------------------------\n");
      printf("%s %s\n", LayoutCursor->LayoutName, LayoutCursor->LayoutClass);
      printf("%d %d %d\n", LayoutCursor->BetweenPinsSize, LayoutCursor->CompoInSize, LayoutCursor->CompoOutSize);
      for (Counter = 0 ; Counter < LayoutCursor->PinNb ; Counter++)
	{
	  printf("%d: %s  %d %d\n", Counter, LayoutCursor->SpotNameTab[Counter], 
		 LayoutCursor->PinPosition[Counter].x, LayoutCursor->PinPosition[Counter].y);
	}
      LayoutCursor = LayoutCursor->pNext;
    }
  return AllOk;
}
