#include "link.h"
#include <stdlib.h>
#include "mainwindow.h"
#include "component.h"
#include "subfunction.h"
#include "dlink.h"

int NewLinkID(GCD *CircuitData, int SetLinkID)
{
  CircuitData->Info.LinkID += 10;
  if (SetLinkID != NoID)
    CircuitData->Info.LinkID = SetLinkID-10;
  /* SetLinkID est le prochain ID utilisable, il doit etre renvoye lors du prochain appel */
  return CircuitData->Info.LinkID;
}

ErrorCheck AddLink(GCD *CircuitData, int LinkID)
{
  Link *NewLink, *LinkCursor, *PrevLink;
  
  NewLink=(Link*)malloc(sizeof(Link));
  if( NewLink==NULL) 
    {
      printf("***ERROR***\n");
      return MemoryAllocationError;
    }
  
  NewLink->IDNumber= LinkID;

  InitState(NewLink->State);
  UnSetLinkSelected(NewLink->State);
  UnSetLinkFull(NewLink->State);
  UnSetLinkDLSelected(NewLink->State);
  UnSetLinkRouted(NewLink->State);

  NewLink->StartAttachID  = ExceptionAttach;
  NewLink->EndAttachID  = ExceptionAttach;
  NewLink->LinkLength = LinkLengthException;
  NewLink->RoutedOrder = NoOrder;
  NewLink->RoutingLinkID = NoID;
  NewLink->LinkWidth = 0;
  NewLink->LinkColor.Red = CircuitData->Info.Opt.ColorPalette[LinkDefaultPalette].Red;
  NewLink->LinkColor.Green = CircuitData->Info.Opt.ColorPalette[LinkDefaultPalette].Green;
  NewLink->LinkColor.Blue = CircuitData->Info.Opt.ColorPalette[LinkDefaultPalette].Blue;
  NewLink->pNext = NULL;
  
  PrevLink = NULL;
  LinkCursor = CircuitData->ListLink;
  while (LinkCursor != NULL)
    {
      PrevLink = LinkCursor;
      LinkCursor = LinkCursor->pNext;
    }
  if (PrevLink == NULL)
    CircuitData->ListLink = NewLink;
  else
    PrevLink->pNext = NewLink;

  return AllOk;
}
  

ErrorCheck EditLink(GCD *CircuitData,Boolean DoSelected,int LinkID, int StartAttachID, int EndAttachID)
{
  Link *LinkCursor;

  LinkCursor=CircuitData->ListLink;
  while(LinkCursor!=NULL)
    {
      if( ((DoSelected == True) && (IsLinkSelected(LinkCursor->State))) || ((DoSelected == False) &&(LinkCursor->IDNumber== LinkID)))  
	{
	  if ((StartAttachID != ExceptionAttach) && (StartAttachID != LinkCursor->EndAttachID))
	    LinkCursor->StartAttachID=StartAttachID;
	  if ( (EndAttachID!= ExceptionAttach)  && (EndAttachID !=  LinkCursor->StartAttachID))
	    LinkCursor->EndAttachID=EndAttachID;
	  
	  if ( (LinkCursor->StartAttachID != ExceptionAttach) && 
	       (LinkCursor->EndAttachID !=  ExceptionAttach))
	    {
	      SetLinkFull(LinkCursor->State);
	      UpdateLink(CircuitData, LinkCursor->IDNumber);
	    }
	  UnDrawAllLinks(CircuitData);
	}
      LinkCursor=LinkCursor->pNext;
    }
  return AllOk;
}

ErrorCheck UpdateLink(GCD *CircuitData, int LinkID)
{
  Link *LinkCursor;
  Boolean IDFound, StartAttachIDFound, EndAttachIDFound;
  Component CompProp;
  SubFunction SubFProp;
  int StartAttachID, EndAttachID;
  AttachPoint *AttachPointCursor;
  
  IDFound = False;
  LinkCursor = CircuitData->ListLink;
  while (( LinkCursor != NULL) && (IDFound == False))
    {
      if (LinkCursor->IDNumber == LinkID)
	{
	  IDFound = False;

	  if (IsLinkFull(LinkCursor->State))
	    {
	      StartAttachID = LinkCursor->StartAttachID;
	      EndAttachID = LinkCursor->EndAttachID;
	      StartAttachIDFound = False;
	      EndAttachIDFound = False;
	      
	      /* recherche des coordonnees des points de depart et d'arrivee du lien */ 
	      AttachPointCursor=CircuitData->ListAttachPoint;
	      while ((AttachPointCursor!=NULL) && ((StartAttachIDFound ==  False) ||(EndAttachIDFound ==  False) ))
		{
		  if (AttachPointCursor->IDNumber == LinkCursor->StartAttachID)
		    {
		      StartAttachIDFound = True;
		      GetComponentProperties(CircuitData, AttachPointCursor->ComponentID, &CompProp, False);
		      GetSubFunctionProperties(CircuitData, CompProp.SubFunctionID, &SubFProp);
		      LinkCursor->Start.x = SubFProp.Reference.x + CompProp.Reference.x + AttachPointCursor->Reference.x;
		      LinkCursor->Start.y = SubFProp.Reference.y + CompProp.Reference.y + AttachPointCursor->Reference.y;
		    }
		  if(AttachPointCursor->IDNumber == LinkCursor->EndAttachID)
		    {
		      EndAttachIDFound = True;
		      GetComponentProperties(CircuitData, AttachPointCursor->ComponentID, &CompProp, False);
		      GetSubFunctionProperties(CircuitData, CompProp.SubFunctionID, &SubFProp);
		      LinkCursor->End.x = SubFProp.Reference.x + CompProp.Reference.x + AttachPointCursor->Reference.x;
		      LinkCursor->End.y = SubFProp.Reference.y + CompProp.Reference.y + AttachPointCursor->Reference.y;
		    }
		  AttachPointCursor = AttachPointCursor->pNext;
		}
	      LinkCursor->LinkLength =  (LinkCursor->End.x - LinkCursor->Start.x) * (LinkCursor->End.x - LinkCursor->Start.x) 
		+ (LinkCursor->End.y -  LinkCursor->Start.y) * (LinkCursor->End.y -  LinkCursor->Start.y);
	      
	      if (IsLinkDLSelected(LinkCursor->State))
		{
		  UnDrawLink(CircuitData, LinkCursor->IDNumber);
		  UpdateDLink(CircuitData, LinkCursor->IDNumber);
		}
	    }
	}
      LinkCursor = LinkCursor->pNext;
    }
  if (IDFound == True)
    return Found;
  else return NotFound;
}

ErrorCheck DisplayLink(GCD *CircuitData)
{
  Link *LinkCursor;

  printf("Liste des liens:\n****************\n"); 
  LinkCursor=CircuitData->ListLink;
  while(LinkCursor!=NULL)
    {
     printf("ID des comp: %d %d. Link ID: %d. CoordStart:%d %d. CoordEnd: %d %d \n",LinkCursor->StartAttachID,LinkCursor->EndAttachID,LinkCursor->IDNumber,LinkCursor->Start.x,LinkCursor->Start.y,LinkCursor->End.x,LinkCursor->End.y);
     LinkCursor=LinkCursor->pNext;
    }
  printf("\n");
  return AllOk;
}

ErrorCheck DeleteLink(GCD *CircuitData, int LinkID)
{
  Link *LinkCursor, **LinkNext;
  Boolean IDFound, InDrawList; 

  LinkNext = &( CircuitData->ListLink);
  LinkCursor= CircuitData->ListLink;
  IDFound = False;
  InDrawList = False;
  
  UnDrawAllLinks(CircuitData);
  while ((LinkCursor!=NULL) && (IDFound == False))
   {
     if (LinkCursor->IDNumber == LinkID)
       {
	 IDFound = True;
	 InDrawList = (IsLinkDLSelected(LinkCursor->State)) ? True : False;
	 *LinkNext = LinkCursor->pNext;
      	 free(LinkCursor);
       }
     else 
       LinkNext = &(LinkCursor->pNext);
     
     LinkCursor = *LinkNext;
   }
  if (InDrawList == True)
    DeleteDLink( CircuitData, LinkID);
  

  if (IDFound == True)
    return Found;
  else return NotFound;
}

ErrorCheck GetLinkProperties(GCD *CircuitData, int LinkID, Link *LinkProp)
{
  Link *LinkCursor;
  Boolean IDFound;

  IDFound = False;
/*   printf("link properties:\n");  */
  LinkCursor=CircuitData->ListLink;
  while ( (LinkCursor!=NULL) && (IDFound == False))
    {
      if(LinkCursor->IDNumber== LinkID)
	{
	  IDFound = True;
	  LinkProp->IDNumber=LinkCursor->IDNumber;
	  LinkProp->StartAttachID=LinkCursor->StartAttachID;
	  LinkProp->EndAttachID=LinkCursor->EndAttachID;
	  LinkProp->State = LinkCursor->State;
	  LinkProp->LinkLength=LinkCursor->LinkLength;
	  LinkProp->LinkWidth = LinkCursor->LinkWidth;
	  LinkProp->RoutingLinkID = LinkCursor->RoutingLinkID;
	  LinkProp->RoutedOrder=LinkCursor->RoutedOrder;
	  LinkProp->LinkColor.Red = LinkCursor->LinkColor.Red;
	  LinkProp->LinkColor.Green = LinkCursor->LinkColor.Green;
	  LinkProp->LinkColor.Blue = LinkCursor->LinkColor.Blue;
	  LinkProp->Road.SegSize = LinkCursor->Road.SegSize;
	  LinkProp->Road.DiagonalSize = LinkCursor->Road.DiagonalSize;
	  LinkProp->Road.DepSize = LinkCursor->Road.DepSize;
	}
      LinkCursor=LinkCursor->pNext;
    }
  if (IDFound == True)
    return Found;
  else return NotFound;
}


ErrorCheck GetSelectLink(GCD *CircuitData, int* TabSelectedLink, int *TabSize)
{
  int Counter;
  Link *LinkCursor;

  Counter=0; 
  LinkCursor=CircuitData->ListLink;
  while(LinkCursor!=NULL)
    {
      if (IsLinkSelected(LinkCursor->State))
	{
	  TabSelectedLink[Counter]=LinkCursor->IDNumber;
	  Counter++;
	}
     LinkCursor=LinkCursor->pNext;
    }

  *TabSize=Counter;
  return AllOk;
}

ErrorCheck GetUnSelectLink(GCD *CircuitData, int* TabUnSelectedLink, int *TabSize)
{
  int Counter;
  Link *LinkCursor;

  Counter=0; 
  LinkCursor=CircuitData->ListLink;

  while(LinkCursor!=NULL)
    {
      if( !IsLinkSelected(LinkCursor->State))
	{
	  TabUnSelectedLink[Counter]=LinkCursor->IDNumber;
	  Counter++;
	}
     LinkCursor=LinkCursor->pNext;
    }
  *TabSize =Counter;
  return AllOk;
}

ErrorCheck GetAllLink(GCD *CircuitData, int* TabLink)
{
  int Counter;
  Link *LinkCursor;

  Counter=0;
  LinkCursor=CircuitData->ListLink;
  while(LinkCursor!=NULL)
    {
      TabLink[Counter]=LinkCursor->IDNumber;
      
      Counter++;
      LinkCursor=LinkCursor->pNext;
    }
  return AllOk;
}

ErrorCheck GetLinkNumber(GCD *CircuitData, int *LinkNb)
{
  int Counter;
  Link *LinkCursor;
  
  Counter=0; 
  LinkCursor=CircuitData->ListLink;
  while(LinkCursor!=NULL)
    {
      Counter++;
      LinkCursor=LinkCursor->pNext;
    }
  *LinkNb = Counter;
  return AllOk;
}


ErrorCheck SetLinkToDraw(GCD *CircuitData, int LinkID)
{
  Link *LinkCursor;
  Boolean IDFound;
  
  IDFound = False;
  LinkCursor=CircuitData->ListLink;
  while ((LinkCursor!=NULL) && (IDFound ==False))
    {
      if(LinkCursor->IDNumber == LinkID)
	{
	  IDFound = True;
	  SetLinkDLSelected(LinkCursor->State);
	}
      LinkCursor  = LinkCursor->pNext;

    }
  if (IDFound == True)
    return Found;
  else return NotFound;
}

ErrorCheck SetLinkNotToDraw(GCD *CircuitData, int LinkID)
{
  Link *LinkCursor;
  Boolean IDFound;
  
  IDFound = False;
  LinkCursor=CircuitData->ListLink;
  while ((LinkCursor!=NULL) && (IDFound ==False))
    {
      if(LinkCursor->IDNumber == LinkID)
	{
	  IDFound = True;
	  UnSetLinkDLSelected(LinkCursor->State);
	}
      LinkCursor  = LinkCursor->pNext;

    }
  if (IDFound == True)
    return Found;
  else return NotFound;
}

ErrorCheck SelectLink(GCD *CircuitData, int LinkID)
{
  Link *LinkCursor;
  Boolean IDFound;

  IDFound = False;
  LinkCursor=CircuitData->ListLink;
  while ((LinkCursor!=NULL) && (IDFound ==False))
    {
      if(LinkCursor->IDNumber == LinkID)
	{
	  IDFound = True;
	  SetLinkSelected(LinkCursor->State);
	}
      LinkCursor  = LinkCursor->pNext;
    }

  if (IDFound == True)
    return Found;
  else return NotFound;
}
ErrorCheck UnSelectLink(GCD *CircuitData, int LinkID)
{
  Link *LinkCursor;
  Boolean IDFound;

  LinkCursor=CircuitData->ListLink;
  IDFound = False;
  while ((LinkCursor!=NULL) && (IDFound ==False))
    {
      if (LinkCursor->IDNumber == LinkID)
	{
	  UnSetLinkSelected(LinkCursor->State);
	  IDFound = True;
	}
      LinkCursor  = LinkCursor->pNext;
    }

  if (IDFound == True)
    return Found;
  else return NotFound;
}

ErrorCheck LinkRoutingIDToID(GCD *CircuitData, int RoutingID, int *ID)
{
  Link *LinkCursor;
  Boolean IDFound;

  LinkCursor = CircuitData->ListLink;
  IDFound = False;
  while ((LinkCursor != NULL) && (IDFound == False))
    {
      if (LinkCursor->RoutingLinkID == RoutingID)
	{
	  *ID = LinkCursor->IDNumber;
	  IDFound = True;
	}
      LinkCursor = LinkCursor->pNext;
    }
  if (IDFound == True)
    return Found;
  else return NotFound;
}

ErrorCheck LinkIDToRoutingID(GCD *CircuitData, int ID, int *RoutingID)
{
  Link *LinkCursor;
  Boolean IDFound;

  LinkCursor = CircuitData->ListLink;
  IDFound = False;
  while ((LinkCursor != NULL) && (IDFound == False))
    {
      if (LinkCursor->IDNumber == ID)
	{
	  *RoutingID = LinkCursor->RoutingLinkID;
	  IDFound = True;
	}
      LinkCursor = LinkCursor->pNext;
    }
  if (IDFound == True)
    return Found;
  else return NotFound;
}

ErrorCheck SetLinkRoutingID(GCD *CircuitData, int LinkID, int RoutingID)
{
  Link *LinkCursor;
  Boolean IDFound;
  
  IDFound = False;
  LinkCursor = CircuitData->ListLink;
  while ((LinkCursor!=NULL) && (IDFound == False))
    {
      if(LinkCursor->IDNumber == LinkID)
	{
	  IDFound = True;
	  LinkCursor->RoutingLinkID = RoutingID;
	}
      LinkCursor  = LinkCursor->pNext;
    }
  
  if (IDFound == True)
    return Found;
  else return NotFound;
}

ErrorCheck UnDrawLink(GCD *CircuitData, int LinkID)
{
  Link *LinkCursor;
  DrawLink *DLinkCursor;
  Boolean IDFound;

  IDFound = False;
  DLinkCursor = CircuitData->TopLink;
  while ((DLinkCursor != NULL) && (IDFound == False))
    {
      if (DLinkCursor->LinkID == LinkID)
	{
	  IDFound = True;
	  UnSetDLinkRouted(DLinkCursor->State);
	  DLinkCursor->RoutedOrder = NoOrder;
	}
      DLinkCursor = DLinkCursor->LinkDown;
    }
  
  IDFound = False;
  LinkCursor = CircuitData->ListLink;
  while ((LinkCursor != NULL) && (IDFound == False))
    {
      if (LinkCursor->IDNumber == LinkID)
	{
	  IDFound = True;
	  UnSetLinkRouted(LinkCursor->State);
	  LinkCursor->RoutedOrder = NoOrder;
	  if (LinkCursor->Road.JobDone == True)
	    {
	      LinkCursor->Road.JobDone = False;
	      if (LinkCursor->Road.SegSize != 0)
		free(LinkCursor->Road.SegTab);
	      if (LinkCursor->Road.PathSize != 0)
		free(LinkCursor->Road.PathTab);
	      if (LinkCursor->Road.DiagonalSize != 0)
		free(LinkCursor->Road.DiagonalTab);
	      if (LinkCursor->Road.DepSize != 0)
		free(LinkCursor->Road.DepTab);
	    }
	}
      LinkCursor = LinkCursor->pNext;
    } 

  CheckDrawListConsistency(CircuitData);
  if (IDFound == True)
    return Found;
  else return NotFound;
}


ErrorCheck GetLinkIDFromCoordinates(GCD *CircuitData, int x, int y, int *LinkID)
{
  Link *LinkCursor;
  Boolean FirstLinkFound;
  int Deltax, Deltay, MinLength=0, MinID, Length, Counter;


  FirstLinkFound = False;
  LinkCursor = CircuitData->ListLink;
  MinID = NoID;
  while (LinkCursor != NULL)
    {
      if (IsLinkRouted(LinkCursor->State))
	for (Counter = 0 ; Counter < LinkCursor->Road.PathSize ; Counter++)
	  {
	    Deltax = x - LinkCursor->Road.PathTab[Counter].x;
	    Deltay = y - LinkCursor->Road.PathTab[Counter].y;
	    Length = Deltax * Deltax + Deltay * Deltay;
	    if ((FirstLinkFound == False) || (Length < MinLength))
	      {
		FirstLinkFound = True;
		MinLength = Length;
		MinID = LinkCursor->IDNumber;
	      }
	  }
      
      LinkCursor = LinkCursor->pNext;
    }
  *LinkID = MinID;
  if (FirstLinkFound ==True)
    return Found;
  else 
    return NotFound;
}

ErrorCheck GetLinkSegTab(GCD *CircuitData, int LinkID, FigPath *FPData)
{
  Link *LinkCursor;
  int Counter, Size;
  Boolean IDFound, Allocated;
  
  LinkCursor = CircuitData->ListLink;
  IDFound = False;
  Allocated = False;
  while ((LinkCursor != NULL) && (IDFound == False))
    {
      if (LinkCursor->IDNumber == LinkID)
	{
	  IDFound = True;
	  if ( IsLinkRouted(LinkCursor->State))
	    {
	      FPData->LinkWidth = LinkCursor->LinkWidth;
	      
	      Size = LinkCursor->Road.SegSize;
	      FPData->SegSize = Size;
	      if (Size != 0)
		{
		  Allocated = True;
		  FPData->SegTab = (Segment *) malloc (sizeof(Segment) * Size);
		}
	      for (Counter = 0 ; Counter < Size ; Counter++)
		{
		  FPData->SegTab[Counter] = LinkCursor->Road.SegTab[Counter];
/* 		  FPData->SegTab[Counter].Start.y = LinkCursor->Road.SegTab[Counter].Start.y; */
/* 		  FPData->SegTab[Counter].End.x = LinkCursor->Road.SegTab[Counter].End.x; */
/* 		  FPData->SegTab[Counter].End.y = LinkCursor->Road.SegTab[Counter].End.y; */
		}
	    }
	}
      LinkCursor = LinkCursor->pNext;
    }

  if ((IDFound == True) && (Allocated == True))
    return Found;
  else
    return NotFound;
}

ErrorCheck SetLinkColor(GCD *CircuitData, int LinkID, int Red, int Green, int Blue)
{
  Link *LinkCursor;
  Boolean IDFound;

  LinkCursor = CircuitData->ListLink;
  IDFound = False;
  while ((LinkCursor != NULL) && (IDFound == False))
    {
      if (LinkCursor->IDNumber == LinkID)
	{
	  IDFound = True;
	  LinkCursor->LinkColor.Red = Red;
	  LinkCursor->LinkColor.Green = Green;
	  LinkCursor->LinkColor.Blue = Blue;
	}
      LinkCursor = LinkCursor->pNext;
    }
  if (IDFound == True)
    return Found;
  else
    return NotFound;
}

ErrorCheck ExtendByColor(GCD *CircuitData)
{
  Link *LinkCursor;
  int *RedTab, *GreenTab, *BlueTab, LinkNb, ColorCounter, Counter;
  Boolean NewColor, InColor;

  GetLinkNumber(CircuitData, &LinkNb);
  if (LinkNb < 2) /* si il n y a qu un lien on ne peut etendre la selection */
    return AllOk;

  RedTab = (int *) malloc(sizeof(int) * LinkNb);
  GreenTab = (int *) malloc(sizeof(int) * LinkNb);
  BlueTab = (int *) malloc(sizeof(int) * LinkNb);
 
  LinkCursor = CircuitData->ListLink;
  ColorCounter = 0;
  while (LinkCursor != NULL)
    {
      if (IsLinkSelected(LinkCursor->State))
	{
	  NewColor = True;
	  for (Counter = 0 ; (NewColor == True) && (Counter < ColorCounter); Counter++)
	    if ((RedTab[Counter] == LinkCursor->LinkColor.Red) 
		&& (GreenTab[Counter] == LinkCursor->LinkColor.Green) 
		&&(BlueTab[Counter] == LinkCursor->LinkColor.Blue)) 
	      NewColor = False;

	  if (NewColor == True)
	    {
	      RedTab[ColorCounter] = LinkCursor->LinkColor.Red;
	      GreenTab[ColorCounter] = LinkCursor->LinkColor.Green;
	      BlueTab[ColorCounter] = LinkCursor->LinkColor.Blue;
	      ColorCounter++;
	    }
	}
      LinkCursor = LinkCursor->pNext;
    }

  LinkCursor = CircuitData->ListLink;
  while (LinkCursor != NULL)
    {
      if (!IsLinkSelected(LinkCursor->State))
	{
	  InColor = False;
	  for (Counter = 0 ; (InColor == False) && (Counter < ColorCounter) ; Counter++)
	    if ((LinkCursor->LinkColor.Red == RedTab[Counter]) 
		&& ( LinkCursor->LinkColor.Green == GreenTab[Counter])
		&& (LinkCursor->LinkColor.Blue == BlueTab[Counter]))
	      InColor = True;
	  
	  if (InColor == True)
	    SetLinkSelected(LinkCursor->State);
	}
      LinkCursor = LinkCursor->pNext;
    }
  free(RedTab);
  free(GreenTab);
  free(BlueTab);

  return AllOk;
}

ErrorCheck UnDrawAllLinks(GCD *CircuitData)
{
  if (CircuitData->TopLink != NULL)
    UnDrawLink(CircuitData, CircuitData->TopLink->LinkID);
  return AllOk;
}

ErrorCheck SetLinkWidth(GCD *CircuitData, int LinkID, int LinkWidth)
{
  Link *LinkCursor;
  
  LinkCursor=CircuitData->ListLink;
  while (LinkCursor!=NULL)
    {
      if ( ((LinkID == NoID) && (IsLinkSelected(LinkCursor->State))) || (LinkCursor->IDNumber == LinkID) )
	{
	  if (IsLinkDLSelected(LinkCursor->State))
	    UnDrawLink(CircuitData, LinkCursor->IDNumber);
	  LinkCursor->LinkWidth = LinkWidth;
	  UpdateDLink(CircuitData, LinkCursor->IDNumber);
	}
      LinkCursor  = LinkCursor->pNext;
    }
  return Found;
}
ErrorCheck SelectLinkAsDLink(GCD *CircuitData)
{
  DrawLink *DLinkCursor;
  
  DLinkCursor = CircuitData->TopLink;
  while (DLinkCursor != NULL)
    {
      if (IsDLinkSelected(DLinkCursor->State))
	SelectLink(CircuitData, DLinkCursor->LinkID);
      DLinkCursor = DLinkCursor->LinkDown;
    }
  return AllOk;
}
  
