
#include "liste.h"
#include "componentlibrary.h"
#include "optionfile.h"
#include "circuitviewwindow.h"
#include "mainwindow.h"
#include "subfunction.h"
#include "component.h"
#include "attachpoint.h"
#include "interface.h"

void ListInit(GtkWidget *Text)
{
  ComponentLibrary *CompLib = NULL;
  GCD *CircuitData;
  
  LoadComponentLibrary(&CompLib, DefaultSetFile, Text);
  
  FirstExecution();
  CircuitData = GetCircuitData();
  CircuitData->CompLib = CompLib;
  LoadConfigFile(CircuitData);
  strncpy(CircuitData->DataHeader.RevisionDate, CircuitData->Info.Opt.DataHeader.RevisionDate, RevisionDateLength);
  CircuitData->DataHeader.Width = CircuitData->Info.Opt.DataHeader.Width;
  CircuitData->DataHeader.Height =  CircuitData->Info.Opt.DataHeader.Height;
  strncpy(CircuitData->DataHeader.Name, CircuitData->Info.Opt.DataHeader.Name, NameLength); 
  strncpy(CircuitData->DataHeader.Version, CircuitData->Info.Opt.DataHeader.Version, VersionLength); 
  strncpy(CircuitData->DataHeader.Designer, CircuitData->Info.Opt.DataHeader.Designer, DesignerLength); 
  strncpy(CircuitData->DataHeader.Society, CircuitData->Info.Opt.DataHeader.Society, SocietyLength); 
  strncpy(CircuitData->DataHeader.Description, CircuitData->Info.Opt.DataHeader.Description, DescriptionLength); 
}

ErrorCheck CreateCircuitData(GCD **CircuitData)
{
  *CircuitData = (GCD*) malloc(sizeof(GCD));
  InitCircuitData(*CircuitData, True, True);
  return AllOk;
}

ErrorCheck InitCircuitData(GCD *CircuitData, Boolean InitNew, Boolean InitWindow)
{
  CircuitData->ListComponent=NULL;
  CircuitData->ListAttachPoint=NULL;
  CircuitData->ListLink=NULL;
  CircuitData->ListSubFunction=NULL;
  CircuitData->ListSubFunctionBuffer.TabCompSF=NULL;
  CircuitData->ListSubFunctionBuffer.TabLinkSF=NULL;
  CircuitData->ListSubFunctionBuffer.NbComp = 0;
  CircuitData->ListSubFunctionBuffer.NbLink = 0;
  CircuitData->ComponentBufferEmpty = True;
  CircuitData->SubFunctionBufferEmpty = True;
  
  CircuitData->ListPermutation = NULL;

  CircuitData->TopLink=NULL;   
  CircuitData->BottomLink=NULL;
  
  CircuitData->Info.CVOrigin.x = 0;
  CircuitData->Info.CVOrigin.y = 0;
  if (InitNew == True)
    {
      CircuitData->Info.AttachPointID = AttachIDRef;
      CircuitData->Info.ComponentID = ComponentIDRef;
      CircuitData->Info.SubFunctionID = SubFunctionIDRef;
      CircuitData->Info.LinkID = LinkIDRef;
      CircuitData->Info.RoutedOrder = -1;  
      CircuitData->Info.Busy = False;
      CircuitData->Info.RouterCancelRequest = False;
      CircuitData->Info.NamedFile = False;
      CircuitData->Info.NewModification = False;
      CircuitData->Info.CircuitFileName[0] = '\0';
      CircuitData->Info.ProgressBarRouter = NULL;
      CircuitData->Info.ProgressBarTrack = NULL;
      CircuitData->Info.RouterLabel = NULL;
      CircuitData->Info.TrackLabel = NULL;
      CircuitData->Info.ButtonAction = NoAction;
     }
  if (InitWindow == True)
    {
      CircuitData->Info.BufferWidth = 0;
      CircuitData->Info.BufferHeight = 0;
      CircuitData->Info.AboutWindow = create_AboutWindow();
      CircuitData->Info.CVBuffer = NULL;
      CircuitData->Info.CVBackupBuffer = NULL;    
    }
  return AllOk;
}

GCD *GetCircuitData(void)
{
  static GCD *CircuitData = NULL;

  if (CircuitData == NULL)
    CreateCircuitData(&CircuitData);
  return CircuitData;
}

ErrorCheck DeleteCircuitData(GCD *CircuitData, Boolean TotalAnnihilation)
{
  AttachPoint *AttachPointer;
  Component *ComponentPointer;
  Link  *LinkPointer;
  SubFunction  *SubFunctionPointer;
  DrawLink *DLinkPointer;
  LinkPermutation *PermPointer;
  int Counter;

  if (CircuitData==NULL)
    {
      printf("La mega structure est vide !!!\n");
      return Bug;
    }

  while ( CircuitData->ListComponent!=NULL)
    {
      ComponentPointer =  CircuitData->ListComponent;
      CircuitData->ListComponent = CircuitData->ListComponent->pNext;
      
      if (ComponentPointer->AttachPointNb != 0)
	free(ComponentPointer->AttachPointIDTab);

     free(ComponentPointer);
    }
  
  while ( CircuitData->ListAttachPoint!=NULL)
    {
      AttachPointer =  CircuitData->ListAttachPoint;
      CircuitData->ListAttachPoint = CircuitData->ListAttachPoint->pNext;
      
      if (AttachPointer->SpotSize != 0)
	free(AttachPointer->SpotLocation);
      if (AttachPointer->CoreSpotSize != 0)
	free(AttachPointer->CoreSpotLocation);
      if (AttachPointer->InvisibleSpotSize != 0)
	free(AttachPointer->InvisibleSpotLocation);
      
      free(AttachPointer);
    }

  while ( CircuitData->ListSubFunction!=NULL)
    {
      SubFunctionPointer =  CircuitData->ListSubFunction;
      CircuitData->ListSubFunction =   CircuitData->ListSubFunction->pNext;
      free(SubFunctionPointer);
    }

  while ( CircuitData->ListLink!=NULL)
    {
      LinkPointer =  CircuitData->ListLink;
       CircuitData->ListLink =   CircuitData->ListLink->pNext;
      if (IsLinkRouted(LinkPointer->State))
	{
	  if (LinkPointer->Road.PathSize != 0)
	    free(LinkPointer->Road.PathTab);
	  if (LinkPointer->Road.DiagonalSize != 0)
	    free(LinkPointer->Road.DiagonalTab);
	  if (LinkPointer->Road.DepSize != 0)
	  free(LinkPointer->Road.DepTab);
	  if (LinkPointer->Road.SegSize != 0)
	  free(LinkPointer->Road.SegTab);
	}
      free(LinkPointer);
    }
  
  while ( CircuitData->TopLink!=NULL)
    {
      DLinkPointer =  CircuitData->TopLink;
      CircuitData->TopLink = CircuitData->TopLink->LinkDown;
      free(DLinkPointer);
    }
  CircuitData->BottomLink = NULL;
   
  while ( CircuitData->ListPermutation != NULL)
    {
      PermPointer =  CircuitData->ListPermutation;
      CircuitData->ListPermutation = CircuitData->ListPermutation->pNext;
      free(PermPointer);
    }
  
  if (CircuitData->SubFunctionBufferEmpty==False)
    {
      if ( CircuitData->ListSubFunctionBuffer.NbComp != 0)
	free( CircuitData->ListSubFunctionBuffer.TabCompSF);

      if (CircuitData->ListSubFunctionBuffer.NbLink != 0)
	{
	free(CircuitData->ListSubFunctionBuffer.TabLinkSF);
	
	for (Counter = 0 ; Counter < CircuitData->ListSubFunctionBuffer.NbLink  ; Counter++)
	  free(CircuitData->ListSubFunctionBuffer.LinkInfo[Counter]);
	free(CircuitData->ListSubFunctionBuffer.LinkInfo);
	
	CircuitData->ListSubFunctionBuffer.TabCompSF=NULL;
      }
    }

  if ( TotalAnnihilation == True)
    {
      SaveConfigFile(CircuitData);
      if (CircuitData->Info.CVBuffer != NULL)
	free(CircuitData->Info.CVBuffer);
      if (CircuitData->Info.CVBackupBuffer != NULL)
	free(CircuitData->Info.CVBackupBuffer);
      DestroyComponentLibrary(CircuitData->CompLib);
      free(CircuitData);
    }
  return AllOk;
}

ErrorCheck EditCircuitInfo(GCD *CircuitData, Header DataHeader)
{
/* crer dans GCD un champ: Header *DataHeader; */
  strncpy(CircuitData->DataHeader.Name, DataHeader.Name,NameLength );
  CircuitData->DataHeader.Name[NameLength-1] = '\0';

  strncpy(CircuitData->DataHeader.Version, DataHeader.Version, VersionLength);
  CircuitData->DataHeader.Version[VersionLength-1] = '\0';

  strncpy(CircuitData->DataHeader.Designer, DataHeader.Designer, DesignerLength);
  CircuitData->DataHeader.Designer[DesignerLength-1] = '\0';

  strncpy(CircuitData->DataHeader.Society, DataHeader.Society, SocietyLength);
  CircuitData->DataHeader.Society[SocietyLength] = '\0';

  strncpy(CircuitData->DataHeader.Description, DataHeader.Description, DescriptionLength);
  CircuitData->DataHeader.Description[DescriptionLength] = '\0';
  
  strncpy(CircuitData->DataHeader.RevisionDate, DataHeader.RevisionDate, RevisionDateLength);
  CircuitData->DataHeader.RevisionDate[RevisionDateLength] = '\0';

  CircuitData->DataHeader.Width=DataHeader.Width;
  CircuitData->DataHeader.Height=DataHeader.Height;
  return AllOk;
}



ErrorCheck GetCircuitInfo(GCD *CircuitData, Header *DataHeader)
{
  strncpy(DataHeader->Name,CircuitData->DataHeader.Name,NameLength);
  DataHeader->Name[NameLength-1] = '\0';

  strncpy(DataHeader->Version,CircuitData->DataHeader.Version,VersionLength);
  DataHeader->Version[VersionLength-1] = '\0';

  strncpy(DataHeader->Designer,CircuitData->DataHeader.Designer, DesignerLength);
  DataHeader->Designer[DesignerLength-1] = '\0';

  strncpy(DataHeader->Society,CircuitData->DataHeader.Society,SocietyLength);
  DataHeader->Society[SocietyLength] = '\0';

  strncpy(DataHeader->Description,CircuitData->DataHeader.Description, DescriptionLength);
  DataHeader->Description[DescriptionLength] = '\0';

  strncpy(DataHeader->RevisionDate, CircuitData->DataHeader.RevisionDate, RevisionDateLength);
  DataHeader->RevisionDate[RevisionDateLength] = '\0';

  DataHeader->Height=CircuitData->DataHeader.Height;
  DataHeader->Width=CircuitData->DataHeader.Width;
  return AllOk;
}

ErrorCheck CheckCircuitIntegrity( GCD *CircuitData, Integrity *IntegrityTab, int *IntegritySize)
{
  AttachPoint *AttachCursor, AttachProp;
  Boolean **Tab, OverlapError, OutOfCircuitError;
  Component CompProp;
  SubFunction SubFProp;
  Link *LinkCursor;
  int Height, Width, Counter, Counter2, Deltax, Deltay, x=0, y=0, TabSize, SpotLevelCounter;
  
  Height = CircuitData->DataHeader.Height;
  Width = CircuitData->DataHeader.Width;
  
  Tab = (Boolean **) malloc (sizeof(Boolean *) * Width);
  for (Counter = 0 ; Counter < Width ; Counter++)
    {
      Tab[Counter] = (Boolean *) malloc(sizeof(Boolean) * Height);
      for (Counter2 = 0 ; Counter2 < Height ; Counter2++)
	Tab[Counter][Counter2] = NoID;
    }

  TabSize = 0;
  AttachCursor = CircuitData->ListAttachPoint;
  while (AttachCursor!=NULL)
    {
      GetComponentProperties(CircuitData, AttachCursor->ComponentID, &CompProp, False);
      GetSubFunctionProperties(CircuitData, CompProp.SubFunctionID, &SubFProp);
      Deltax = SubFProp.Reference.x + CompProp.Reference.x + AttachCursor->Reference.x;
      Deltay = SubFProp.Reference.y + CompProp.Reference.y + AttachCursor->Reference.y;
      for (SpotLevelCounter = 0 ; SpotLevelCounter < SpotLevelNumber ; SpotLevelCounter++)
	{
	  OverlapError = False;
	  OutOfCircuitError = False;
	  if (SpotLevelCounter == CoreSpotLevel)
	    for (Counter = 0 ; Counter < AttachCursor->CoreSpotSize ; Counter++)
	      {
		x = AttachCursor->CoreSpotLocation[Counter].x + Deltax;
		y = AttachCursor->CoreSpotLocation[Counter].y + Deltay;
		if ( (x > 0) && (y > 0) && (x < (Width-2)) && (y < (Height-2)))
		  if (Tab[x][y] == NoID)
		    Tab[x][y] = AttachCursor->IDNumber;
		  else 
		    OverlapError = True;
		else
		  OutOfCircuitError = True;
	      }
	  if (SpotLevelCounter == StandardSpotLevel)
	    for (Counter = 0 ; Counter < AttachCursor->SpotSize ; Counter++)
	      {
		x = AttachCursor->SpotLocation[Counter].x + Deltax;
		y = AttachCursor->SpotLocation[Counter].y + Deltay;
		if ( (x > 0) && (y > 0) && (x < (Width-2)) && (y < (Height-2)))
		  if (Tab[x][y] == NoID)
		    Tab[x][y] = AttachCursor->IDNumber;
		  else 
		    OverlapError = True;
		else
		  OutOfCircuitError = True;
	      }
	  if (SpotLevelCounter == InvisibleSpotLevel)
	    for (Counter = 0 ; Counter < AttachCursor->InvisibleSpotSize ; Counter++)
	      {
		x = AttachCursor->InvisibleSpotLocation[Counter].x + Deltax;
		y = AttachCursor->InvisibleSpotLocation[Counter].y + Deltay;
		
		if ( (x > 0) && (y > 0) && (x < (Width-2)) && (y < (Height-2)))
		  if (Tab[x][y] == NoID)
		    Tab[x][y] = AttachCursor->IDNumber;
		  else 
		    OverlapError = True;
		else
		  OutOfCircuitError = True;
	      }
	}
      if (OverlapError == True)
	{
	  GetAttachPointProperties(CircuitData, Tab[x][y], &AttachProp);
	  IntegrityTab[TabSize].Priority = CriticalPriority;
	  IntegrityTab[TabSize].Type = OverlappingAttach;
	  IntegrityTab[TabSize].TypeDependent.Overlap.Location.x = Deltax;
	  IntegrityTab[TabSize].TypeDependent.Overlap.Location.y = Deltay;
	  IntegrityTab[TabSize].TypeDependent.Overlap.ComponentInvolved = AttachCursor->ComponentID;
	  IntegrityTab[TabSize].TypeDependent.Overlap.OtherComponentInvolved = AttachProp.ComponentID;
	  TabSize++;
	}
      else
	if (OutOfCircuitError == True)
	  {
	    IntegrityTab[TabSize].Priority = CriticalPriority;
	    IntegrityTab[TabSize].Type = OutOfCircuit;
	    IntegrityTab[TabSize].TypeDependent.Out.Location.x = Deltax;
	    IntegrityTab[TabSize].TypeDependent.Out.Location.y = Deltay;
	    IntegrityTab[TabSize].TypeDependent.Out.ComponentInvolved = AttachCursor->ComponentID;
	    TabSize++;
	  }
      AttachCursor = AttachCursor->pNext;
    }

  LinkCursor = CircuitData->ListLink;
  while (LinkCursor != NULL)
    {
      OutOfCircuitError = False;
      if (IsLinkRouted(LinkCursor->State))
	{
	  for (Counter = 0 ; Counter < LinkCursor->Road.PathSize ; Counter++)
	    {
	      x = LinkCursor->Road.PathTab[Counter].x;
	      y = LinkCursor->Road.PathTab[Counter].y;
	      if ( (x < 0) || (y < 0) || (x > (Width-2)) || (y > (Height-2) ))
		OutOfCircuitError = True;
	    }
	  if (OutOfCircuitError == True)
	    {
	      IntegrityTab[TabSize].Priority = CriticalPriority;
	      IntegrityTab[TabSize].Type = LinkOutOfCircuit;
	      IntegrityTab[TabSize].TypeDependent.LinkOut.Location.x = x;
	      IntegrityTab[TabSize].TypeDependent.LinkOut.Location.y = y;
	      IntegrityTab[TabSize].TypeDependent.LinkOut.LinkInvolved = LinkCursor->IDNumber;
	      TabSize++;
	    }
	}
      LinkCursor = LinkCursor->pNext;
    }
  *IntegritySize = TabSize;

  for (Counter = 0 ; Counter < Width ; Counter++)
    free(Tab[Counter]);
  
  free(Tab);

  return AllOk;
}
