#include <gnome.h>

#include "magicrouter.h"
#include "xrmtypes.h"
#include "mainwindow.h"
#include "miscwindows.h"
#include "attachpoint.h"
#include "drawtracks.h"
#include "liste.h"
#include "link.h"
#include "dlink.h"
#include "filemng.h"

ErrorCheck MagicRouting(GCD *CircuitData) 
{
  int  TabSize, SwapNumber, LinkID1, LinkID2, RoutingCounter, RoutedOrder, MaxRouted, DLinkNb;
  Boolean NotFinished, Cancel;
  ErrorCheck Check;
  FILE *LogFile = NULL;
  char BestFile[FileNameLength+5];

  GetDLinkNumber(CircuitData, &DLinkNb);
  
  if (CircuitData->Info.Opt.LogToFile == True)
    LogFile = fopen(CircuitData->Info.Opt.LogFileName, "w");
  
  RoutingCounter = 0;
  NotFinished = True;
  MaxRouted = 0;

  sprintf(BestFile,"%s.Best", CircuitData->Info.CircuitFileName);
  DestroyPermutationList(CircuitData);
  Cancel = False;
  while ((NotFinished == True) && (RoutingCounter < CircuitData->Info.Opt.RoutingMax) && (Cancel == False))
    {
      if (CircuitData->Info.RouterCancelRequest == True)
	Cancel == True;
      if (LogFile != NULL)
	fprintf(LogFile,"\t\t\t  ****** %d ****** \n", RoutingCounter);

      Check = MediumRouting(CircuitData, True);
      
      RoutingCounter++;
      RoutedOrder = GetRoutedOrder(CircuitData, NoOrder);
      if (LogFile != NULL)
	fprintf(LogFile, "%d Links Routed\n", RoutedOrder);
      if (RoutedOrder > MaxRouted)
	{
	  MaxRouted = RoutedOrder;
	  if (LogFile != NULL)
	    fprintf(LogFile, "Saving File\n");
	  SaveCircuitFile(CircuitData, BestFile, NULL);
	}
      GetRoutedOrder(CircuitData, RoutedOrder);
      
      if (Check == NotFound) 
	{
	  if ((!IsDLinkRouted(CircuitData->TopLink->State)) )
	    {
	      if (CircuitData->Info.Opt.UnRoutableDelete == False)
		return Exception;
	      else
		RemoveUnDrawableDLink(CircuitData,  CircuitData->TopLink->LinkID, NoID, LogFile);
	    }
	  else
	    {
	      SwapNumber  = SwapLastDLink(CircuitData, &LinkID1, &LinkID2, LogFile);
	      if (SwapNumber > CircuitData->Info.Opt.SwapMax)
		{
		  if ((CircuitData->Info.Opt.UnRoutableDelete == False) || 
		      ((CircuitData->Info.Opt.UnRoutableDelete == True) 
		       && (SwapNumber < CircuitData->Info.Opt.UnRoutableSwapMax)))
		    RestartMagic(CircuitData, LinkID1, LinkID2, LogFile);
		  else
		    RemoveUnDrawableDLink(CircuitData,  LinkID1, LinkID2, LogFile);
		}
	    }
	}
      else
	NotFinished = False;
      if (LogFile != NULL)
	fflush(LogFile);
      UpdateRouterStatusWindow(CircuitData, RoutingCounter, CircuitData->Info.Opt.RoutingMax, 
			       MaxRouted, DLinkNb);

    }
  if (LogFile != NULL)
    fclose(LogFile);

  if (NotFinished == False)
    return Found;
  else
    return NotFound;
}

int SwapLastDLink(GCD *CircuitData, int *LinkID1, int *LinkID2,  FILE *LogFile)
{
  DrawLink *DLinkCursor, *DLink1, *DLink2;
  int PermNumber = 0;

  DLinkCursor = CircuitData->TopLink;
  DLink1 = NULL;
  while ((DLinkCursor != NULL) && (IsDLinkRouted(DLinkCursor->State)))
    {
      DLink1 = DLinkCursor;
      DLinkCursor = DLinkCursor->LinkDown;
    }
  
  if (DLink1 == NULL)
    {
      printf("***DLink 1 : Error Last Link routed!!\n");
      return 0;
    }

  DLink2 = DLink1->LinkDown;
  if (DLink2 == NULL)
    {
      printf("***DLink2 : Error Last Link routed!!\n");
      return 0;
    }

  if (DLink1 != DLink2)
    {
      if (LogFile != NULL)
	fprintf(LogFile, "Swapping %d %d\n", DLink1->LinkID, DLink2->LinkID);
      UnDrawLink(CircuitData, DLink1->LinkID);
      UnDrawLink(CircuitData, DLink2->LinkID);
      *LinkID1 =  DLink1->LinkID;
      *LinkID2 =  DLink2->LinkID;
      PermNumber = SavePermutation(CircuitData, DLink1->LinkID, DLink2->LinkID);
      ExchangeDrawLink(CircuitData, DLink1, DLink2);
    }
  else
    printf("***Error the 2 links to permute are same\n");

  return PermNumber;
}

int SavePermutation(GCD *CircuitData, int LinkID1, int LinkID2)
{
  LinkPermutation *LinkPermCursor, *PrevPerm, *NewPerm;
  Boolean PermFound;
  int PermNumber = 0;

  PermFound = False;
  PrevPerm = NULL;
  LinkPermCursor = CircuitData->ListPermutation;
  while ((LinkPermCursor != NULL) && (PermFound == False))
    {
      if ( ((LinkPermCursor->LinkID1 == LinkID1) && (LinkPermCursor->LinkID2 == LinkID2))
	   || ((LinkPermCursor->LinkID1 == LinkID2) && (LinkPermCursor->LinkID2 == LinkID1)))
	{
	  PermFound = True;
	  (LinkPermCursor->Number)++;
	  PermNumber = LinkPermCursor->Number;
	}
      PrevPerm = LinkPermCursor;
      LinkPermCursor = LinkPermCursor->pNext;
    }
	
  if (PermFound == False) 
    { /* nouvelle permutation */
      NewPerm = (LinkPermutation *) malloc(sizeof(LinkPermutation));
      NewPerm->LinkID1 = LinkID1;
      NewPerm->LinkID2 = LinkID2;
      NewPerm->Number = 1;
      PermNumber = NewPerm->Number;
      NewPerm->pNext = NULL;
      
      if (PrevPerm == NULL)
	CircuitData->ListPermutation = NewPerm;
      else
	PrevPerm->pNext = NewPerm;
    }

  return PermNumber;
}
	  
void RestartMagic(GCD *CircuitData, int LinkID1, int LinkID2, FILE *LogFile)
{
  DrawLink *DLinkCursor, *DLink1 = NULL, *DLink2 = NULL; 
  Boolean ID1Found, ID2Found;

  CheckDrawListConsistency(CircuitData);
  DLinkCursor = CircuitData->TopLink;
  ID1Found = False;
  ID2Found = False;
  while ((DLinkCursor !=NULL) && ((ID1Found == False) || (ID2Found == False)))
    {
      if (DLinkCursor->LinkID == LinkID1)
	{
	  ID1Found = True;
	  DLink1 = DLinkCursor;
	}
      if (DLinkCursor->LinkID == LinkID2)
	{
	  ID2Found = True;
	  DLink2 = DLinkCursor;
	}
      DLinkCursor = DLinkCursor->LinkDown;
    }
  if ((DLink1 == NULL) || (DLink2 == NULL))
    printf("Bug les liens impossibles a tracer n existent pas\n");
  
  if (LogFile != NULL)
    {
      fprintf(LogFile,"To Top %d\n",  DLink1->LinkID);
      fprintf(LogFile,"To Top %d\n",  DLink2->LinkID);
    }

  InsertDrawLink(CircuitData, DLink1, CircuitData->TopLink, False);
  InsertDrawLink(CircuitData, DLink2, CircuitData->TopLink, False);
  
  CheckDrawListConsistency(CircuitData);
  DestroyPermutationList(CircuitData);
}

void DisplayPermutation(GCD *CircuitData)  
{
  LinkPermutation *PermCursor;

  printf("Permutation *********************\n");
  PermCursor = CircuitData->ListPermutation;
  while (PermCursor != NULL)
    {
      printf("%d %d : Number -> %d\n", PermCursor->LinkID1, PermCursor->LinkID2, PermCursor->Number); 
      PermCursor = PermCursor->pNext;
    }
}

void RemoveUnDrawableDLink(GCD *CircuitData, int LinkID1, int LinkID2, FILE *LogFile)
{
  LinkPermutation *PermCursor, **PermNext;
  
   if (LinkID1 != NoID)
     {
       UnDrawLink(CircuitData, LinkID1);
       DeleteDLink(CircuitData, LinkID1);
       if (LogFile != NULL)
	 fprintf(LogFile, "Removing the DLink %d\n", LinkID1);	
     }
   if (LinkID2 != NoID)
     {
       UnDrawLink(CircuitData, LinkID2);
       DeleteDLink(CircuitData, LinkID2);
       if (LogFile != NULL)
	 fprintf(LogFile, "Removing the DLink %d\n", LinkID2);
     }
   
   PermCursor = CircuitData->ListPermutation;
   PermNext = &(CircuitData->ListPermutation);
   while (PermCursor != NULL)
     {
       if ((PermCursor->LinkID1 == LinkID1)  || (PermCursor->LinkID1 == LinkID2) 
	   || (PermCursor->LinkID2 == LinkID1)  || (PermCursor->LinkID2 == LinkID2))
	 {
	   *PermNext = PermCursor->pNext;
	   free(PermCursor);
	 }
       else
	 PermNext = &PermCursor->pNext;
       PermCursor = *PermNext;
     }
}

ErrorCheck DestroyPermutationList(GCD *CircuitData)
{
  LinkPermutation *PermPointer;
   
  while ( CircuitData->ListPermutation != NULL)
    {
      PermPointer =  CircuitData->ListPermutation;
      CircuitData->ListPermutation = CircuitData->ListPermutation->pNext;
      free(PermPointer);
    }
  return AllOk;
}
