
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gnome.h>
#include "xrmtypes.h"
#include "mainwindow.h"
#include "interface.h"
#include "circuitviewwindow.h"
#include "componentwindow.h"
#include "liste.h"
#include "menu.h"

int main (int argc, char *argv[])
{
  bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
  textdomain (PACKAGE);
  gnome_init ("xroadmaker", VERSION, argc, argv);
  create_AllWindows();
  gtk_main ();

  return 0;
}

void create_AllWindows(void)
{
  GtkWidget *MainWindow, *LinkWindow, *ComponentWindow, *CircuitViewWindow, *CTComponentTree, *CTRootItem, 
    *LLClist, *DLClist, *StartupWindow, *Text, *PaintColor;
  GCD *CircuitData;
  int Counter;

  LinkWindow = create_LinkWindow ();
  gdk_rgb_init();
  
  MainWindow = create_MainWindow ();
  gtk_signal_connect( GTK_OBJECT(MainWindow), "delete_event", GTK_SIGNAL_FUNC(on_Exit_activate), NULL);
  StartupWindow =  create_StartupWindow();
  Text = gtk_object_get_data(GTK_OBJECT(StartupWindow), "LibInfoText");
  ListInit(Text); /* ****************** GCD Creation  **************************** */
  CircuitData = GetCircuitData();
  /* ************* CircuitView */
  gtk_widget_push_visual(gdk_rgb_get_visual()); /* RGB */
  gtk_widget_push_colormap(gdk_rgb_get_cmap());

  CircuitViewWindow  = create_FinalCircuitViewWindow(); /* need to know the scale factor  */
  gtk_widget_pop_visual(); /* RGB */
  gtk_widget_pop_colormap();
  gtk_signal_connect_object (GTK_OBJECT(CircuitViewWindow), "delete_event", GTK_SIGNAL_FUNC (gtk_widget_hide), 
			     GTK_OBJECT(CircuitViewWindow));
  
  gtk_signal_connect_object (GTK_OBJECT(LinkWindow), "delete_event",  GTK_SIGNAL_FUNC (gtk_widget_hide),
			     GTK_OBJECT(LinkWindow));

  LLClist =  gtk_object_get_data( GTK_OBJECT(LinkWindow) , "LLClist"); 
  for (Counter = 0 ; Counter < LLColumnNb ; Counter++)
    gtk_clist_set_column_auto_resize(GTK_CLIST(LLClist), Counter, TRUE);
  gtk_clist_set_row_height(GTK_CLIST(LLClist), 24);
  
  DLClist =  gtk_object_get_data( GTK_OBJECT(LinkWindow) , "DLClist"); 
  for (Counter = 0 ; Counter < DLColumnNb ; Counter++)
    gtk_clist_set_column_auto_resize(GTK_CLIST(DLClist), Counter, TRUE);
  gtk_clist_set_row_height(GTK_CLIST(DLClist), 24);
  
  PaintColor =  gtk_object_get_data( GTK_OBJECT(LinkWindow) , "ColorDLinkPainter"); 
  SetColor(PaintColor, CircuitData->Info.Opt.ColorPalette[LinkDefaultPalette]);
  PaintColor =  gtk_object_get_data( GTK_OBJECT(LinkWindow) , "ColorLinkPainter"); 
  SetColor(PaintColor, CircuitData->Info.Opt.ColorPalette[LinkDefaultPalette]);
  
  ComponentWindow = create_ComponentTreeWindow ();
  gtk_signal_connect_object (GTK_OBJECT(ComponentWindow), "delete_event",  GTK_SIGNAL_FUNC (gtk_widget_hide),
			     GTK_OBJECT(ComponentWindow));
  CTComponentTree = gtk_object_get_data( GTK_OBJECT(ComponentWindow) , "CTComponentTree"); 
  CTRootItem = CreateNewTreeItem(CircuitData->DataHeader.Name, 0, CTComponentTree, ComponentWindow);
  
  CircuitData->Info.MainWindow = (void *) MainWindow; 
  CircuitData->Info.LinkWindow = (void *)LinkWindow; 
  CircuitData->Info.ComponentWindow = (void *)ComponentWindow; 
  CircuitData->Info.CircuitViewWindow = (void *)CircuitViewWindow; 
  CircuitData->Info.LLClist = (void *) LLClist; 
  CircuitData->Info.DLClist = (void *)DLClist; 
  CircuitData->Info.CTComponentTree =  (void *)CTComponentTree;
  CircuitData->Info.CTRootItem =  (void *)CTRootItem; 
  InitCircuitViewMask(CircuitData, CircuitViewWindow);

  gtk_widget_show(StartupWindow);
} 

void  StartupFinish (GtkWidget *widget, gpointer user_data)
{
  GCD *CircuitData;
  CircuitData = GetCircuitData();
 
  gtk_widget_show(CircuitData->Info.ComponentWindow);
  UpdateCircuitView(UCVDemand);
  gtk_widget_show(CircuitData->Info.CircuitViewWindow);
  gtk_widget_show(CircuitData->Info.MainWindow);
}

Boolean IRQ(Boolean Request)
{
  GCD *CircuitData;
  CircuitData = GetCircuitData();
  
  if (Request == True)
    if (CircuitData->Info.Busy == False)
      {
	UpdateAppBarWorking(True);
	CircuitData->Info.Busy = True;
	return True; 
      }
    else 
      {
	printf("Error trying to hold ressources\n");
	return False;
      }
  else
    if (CircuitData->Info.Busy == False)
      {
	printf("Error : Trying to release ressources not held\n");
	return False;
      }
    else
      {
	CircuitData->Info.Busy = False;
	UpdateAppBarWorking(False);
	return True;
      }
  return False;
}

void UpdateAppBarWorking(Boolean Busy)
{
  GtkWidget *MainWindow, *CircuitViewWindow, *LinkWindow, *ComponentWindow, *AppBarLink, *AppBarComponent, 
    *AppBarCircuit, *AppBarMain;
  GCD *CircuitData;
  char *Working="Working...", *Waiting="Idle";
  
  CircuitData = GetCircuitData();
  CircuitViewWindow = CircuitData->Info.CircuitViewWindow;
  LinkWindow = CircuitData->Info.LinkWindow;
  ComponentWindow = CircuitData->Info.ComponentWindow;
  MainWindow = CircuitData->Info.MainWindow;

  AppBarCircuit = gtk_object_get_data(GTK_OBJECT(CircuitViewWindow), "AppBar");
  AppBarLink = gtk_object_get_data(GTK_OBJECT(LinkWindow), "AppBar");
  AppBarComponent = gtk_object_get_data(GTK_OBJECT(ComponentWindow), "AppBar");
  AppBarMain = gtk_object_get_data(GTK_OBJECT(MainWindow), "AppBar");
  
  if (Busy == True)
    {
      gnome_appbar_set_status(GNOME_APPBAR(AppBarMain), Working);
      gnome_appbar_set_status(GNOME_APPBAR(AppBarLink), Working);
      gnome_appbar_set_status(GNOME_APPBAR(AppBarComponent), Working);
      gnome_appbar_set_status(GNOME_APPBAR(AppBarCircuit), Working);
    }
  else
    {
      gnome_appbar_set_status(GNOME_APPBAR(AppBarMain), Waiting);
      gnome_appbar_set_status(GNOME_APPBAR(AppBarLink), Waiting);
      gnome_appbar_set_status(GNOME_APPBAR(AppBarComponent), Waiting);
      gnome_appbar_set_status(GNOME_APPBAR(AppBarCircuit), Waiting);
    }
}
