

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include "optionfile.h"
#include "xrmtypes.h"
#include "mainwindow.h"

ErrorCheck SaveConfigFile(GCD *CircuitData)
{
  FILE *ConfigFile;
  char Buffer[BufferLength], *HomeDir;
  int ColorCounter;

  HomeDir = getenv("HOME");
  if (HomeDir == NULL) 
    printf("Environment variable HOME not set!\ndon t know where to save config\n\n");
  
  sprintf(Buffer, "%s/%s/xroadmaker.conf", HomeDir, ConfigDir);
  ConfigFile = fopen(Buffer, "w");
  if (ConfigFile == NULL)
    return FileAccessError;
  
  fprintf(ConfigFile,"%s\n",CircuitData->Info.Opt.DataHeader.Name);
  fprintf(ConfigFile,"%s\n",CircuitData->Info.Opt.DataHeader.Version);
  fprintf(ConfigFile,"%s\n",CircuitData->Info.Opt.DataHeader.Designer);
  fprintf(ConfigFile,"%s\n",CircuitData->Info.Opt.DataHeader.Society);
  fprintf(ConfigFile,"%s\n",CircuitData->Info.Opt.DataHeader.Description);
  fprintf(ConfigFile,"%s\n",CircuitData->Info.Opt.DataHeader.RevisionDate);
  fprintf(ConfigFile,"%d\n",CircuitData->Info.Opt.DataHeader.Width);
  fprintf(ConfigFile,"%d\n",CircuitData->Info.Opt.DataHeader.Height);
  fprintf(ConfigFile,"\n");
  
  for (ColorCounter = 0 ; ColorCounter < ColorPaletteSize ; ColorCounter++)
    {
      fprintf(ConfigFile,"%d ",CircuitData->Info.Opt.ColorPalette[ColorCounter].Red);
      fprintf(ConfigFile,"%d ",CircuitData->Info.Opt.ColorPalette[ColorCounter].Green);
      fprintf(ConfigFile,"%d\n",CircuitData->Info.Opt.ColorPalette[ColorCounter].Blue);
    }

  fprintf(ConfigFile,"ScaleFactor: %f\n",CircuitData->Info.Opt.ScaleFactor);
  fprintf(ConfigFile,"CircuitMask: %d\n",CircuitData->Info.Opt.CircuitMask);
  fprintf(ConfigFile,"UnitMillimeter: %d\n",CircuitData->Info.Opt.Millimeter);
  fprintf(ConfigFile,"SelectionFollowMouse: %d\n",CircuitData->Info.Opt.SelectionFollowMouse);
  fprintf(ConfigFile,"UpdateContinually: %d\n",CircuitData->Info.Opt.UpdateContinually);
  fprintf(ConfigFile,"UpdateClicked: %d\n",CircuitData->Info.Opt.UpdateClicked);
  fprintf(ConfigFile,"UpdateDemand: %d\n",CircuitData->Info.Opt.UpdateDemand);
  fprintf(ConfigFile,"ShortLinkFirst: %d\n",CircuitData->Info.Opt.ShortFirst);
  fprintf(ConfigFile,"NoLinkIcon: %d\n",CircuitData->Info.Opt.NoLinkIcon);
  fprintf(ConfigFile,"NoLinkColor: %d\n",CircuitData->Info.Opt.NoLinkColor);
  fprintf(ConfigFile,"LogToFile: %d\n",CircuitData->Info.Opt.LogToFile);
  fprintf(ConfigFile,"LogFileName: %s\n",CircuitData->Info.Opt.LogFileName);
  fprintf(ConfigFile,"SwapMax: %d\n",CircuitData->Info.Opt.SwapMax);
  fprintf(ConfigFile,"RoutingMax: %d\n",CircuitData->Info.Opt.RoutingMax);
  fprintf(ConfigFile,"UnRoutableDelete: %d\n",CircuitData->Info.Opt.UnRoutableDelete);
  fprintf(ConfigFile,"UnRoutableSwapMax: %d\n",CircuitData->Info.Opt.UnRoutableSwapMax);
  fprintf(ConfigFile,"DirectionPref: %d\n",CircuitData->Info.Opt.DirectionPrefTab[0]);
  fprintf(ConfigFile,"DirectionPref: %d\n",CircuitData->Info.Opt.DirectionPrefTab[1]);
  fprintf(ConfigFile,"DirectionPref: %d\n",CircuitData->Info.Opt.DirectionPrefTab[2]);
  fprintf(ConfigFile,"LengthSort: %d\n",CircuitData->Info.Opt.LengthOrder);
  fprintf(ConfigFile,"WidthSort: %d\n",CircuitData->Info.Opt.WidthOrder);
  fprintf(ConfigFile,"SortWidthFirst: %d\n",CircuitData->Info.Opt.SortWidthFirst);
  fprintf(ConfigFile,"HoleForce: %d\n",CircuitData->Info.Opt.HoleForce);
  fprintf(ConfigFile,"AllowEditComponent: %d\n",CircuitData->Info.Opt.AllowEditComponent);
  fprintf(ConfigFile,"AllowDeleteComponent: %d\n",CircuitData->Info.Opt.AllowDeleteComponent);
  fprintf(ConfigFile,"AllowDeleteSubFunction: %d\n",CircuitData->Info.Opt.AllowDeleteSubFunction);
  fprintf(ConfigFile,"DefaultSubFunctionName: %s\n",CircuitData->Info.Opt.DefaultSubFunctionName);
  fprintf(ConfigFile,"DefaultComponentName: %s\n",CircuitData->Info.Opt.DefaultComponentName);
  fprintf(ConfigFile,"OpenInformation: %d\n",CircuitData->Info.Opt.OpenInformation);
  fprintf(ConfigFile,"RevertInformation: %d\n",CircuitData->Info.Opt.RevertInformation);
  fprintf(ConfigFile,"SaveInformation: %d\n",CircuitData->Info.Opt.SaveInformation);
  fprintf(ConfigFile,"UseMagic: %d\n",CircuitData->Info.Opt.UseMagic);
  fprintf(ConfigFile,"StopRouterOnError: %d\n",CircuitData->Info.Opt.StopRouterOnError);
  fprintf(ConfigFile,"CircuitDirectory: %s\n",CircuitData->Info.Opt.CircuitDirectory);
  fclose(ConfigFile);
  return AllOk;
}


ErrorCheck LoadConfigFile(GCD *CircuitData)
{
  char Buffer[BufferLength], *HomeDir;
  FILE *ConfigFile = NULL;
  int ColorCounter, ColorValue;
  


  HomeDir = getenv("HOME");
  if (HomeDir == NULL) 
    printf("Environment variable HOME not set!\ndon t know where to save config\n\n");
  else
    {  
      sprintf(Buffer, "%s/%s/xroadmaker.conf", HomeDir, ConfigDir);
      ConfigFile = fopen(Buffer, "r");
    }
  if (ConfigFile == NULL)
    {
      ConfigFile = fopen(DefaultConfigFile, "r");
      if (ConfigFile == NULL)
	{
	  printf("***** Error cannot open default config file \"%s\" \n\n", DefaultConfigFile);
	  return FileAccessError;
	}
    }
  fscanf(ConfigFile,"%[^\n]\n",CircuitData->Info.Opt.DataHeader.Name);
  fscanf(ConfigFile,"%[^\n]\n",CircuitData->Info.Opt.DataHeader.Version);
  fscanf(ConfigFile,"%[^\n]\n",CircuitData->Info.Opt.DataHeader.Designer);
  fscanf(ConfigFile,"%[^\n]\n",CircuitData->Info.Opt.DataHeader.Society);
  fscanf(ConfigFile,"%[^\n]\n",CircuitData->Info.Opt.DataHeader.Description);
  fscanf(ConfigFile,"%[^\n]\n",CircuitData->Info.Opt.DataHeader.RevisionDate);
  fscanf(ConfigFile,"%d\n", &(CircuitData->Info.Opt.DataHeader.Width));
  fscanf(ConfigFile,"%d\n", &(CircuitData->Info.Opt.DataHeader.Height));
  fscanf(ConfigFile,"\n");
  
  for (ColorCounter = 0 ; ColorCounter < ColorPaletteSize ; ColorCounter++)
    {
      fscanf(ConfigFile,"%d ", &(ColorValue));
      CircuitData->Info.Opt.ColorPalette[ColorCounter].Red = ColorValue;
      fscanf(ConfigFile,"%d ", &(ColorValue));
      CircuitData->Info.Opt.ColorPalette[ColorCounter].Green = ColorValue;
      fscanf(ConfigFile,"%d\n", &(ColorValue));
      CircuitData->Info.Opt.ColorPalette[ColorCounter].Blue = ColorValue;
    }
  fscanf(ConfigFile,"ScaleFactor: %f\n", &(CircuitData->Info.Opt.ScaleFactor));
  fscanf(ConfigFile,"CircuitMask: %d\n", &(CircuitData->Info.Opt.CircuitMask));
  fscanf(ConfigFile,"UnitMillimeter: %d\n", (int *) &(CircuitData->Info.Opt.Millimeter));
  fscanf(ConfigFile,"SelectionFollowMouse: %d\n", (int*) &(CircuitData->Info.Opt.SelectionFollowMouse));
  fscanf(ConfigFile,"UpdateContinually: %d\n", (int *) &(CircuitData->Info.Opt.UpdateContinually));
  fscanf(ConfigFile,"UpdateClicked: %d\n", (int *) &(CircuitData->Info.Opt.UpdateClicked));
  fscanf(ConfigFile,"UpdateDemand: %d\n", (int *) &(CircuitData->Info.Opt.UpdateDemand));
  fscanf(ConfigFile,"ShortLinkFirst: %d\n", (int *) &(CircuitData->Info.Opt.ShortFirst));
  fscanf(ConfigFile,"NoLinkIcon: %d\n", (int *) &(CircuitData->Info.Opt.NoLinkIcon));
  fscanf(ConfigFile,"NoLinkColor: %d\n", (int *) &(CircuitData->Info.Opt.NoLinkColor));
  fscanf(ConfigFile,"LogToFile: %d\n", (int *) &(CircuitData->Info.Opt.LogToFile));
  fscanf(ConfigFile,"LogFileName: %[^\n]\n", CircuitData->Info.Opt.LogFileName);
  fscanf(ConfigFile,"SwapMax: %d\n", &(CircuitData->Info.Opt.SwapMax));
  fscanf(ConfigFile,"RoutingMax: %d\n", &(CircuitData->Info.Opt.RoutingMax));
  fscanf(ConfigFile,"UnRoutableDelete: %d\n", (int *) &(CircuitData->Info.Opt.UnRoutableDelete));
  fscanf(ConfigFile,"UnRoutableSwapMax: %d\n", &(CircuitData->Info.Opt.UnRoutableSwapMax));
  fscanf(ConfigFile,"DirectionPref: %d\n", (int *) &(CircuitData->Info.Opt.DirectionPrefTab[0]));
  fscanf(ConfigFile,"DirectionPref: %d\n", (int *) &(CircuitData->Info.Opt.DirectionPrefTab[1]));
  fscanf(ConfigFile,"DirectionPref: %d\n", (int *) &(CircuitData->Info.Opt.DirectionPrefTab[2]));
  fscanf(ConfigFile,"LengthSort: %d\n", (int *) &(CircuitData->Info.Opt.LengthOrder));
  fscanf(ConfigFile,"WidthSort: %d\n", (int *) &(CircuitData->Info.Opt.WidthOrder));
  fscanf(ConfigFile,"SortWidthFirst: %d\n", (int *) &(CircuitData->Info.Opt.SortWidthFirst));
  fscanf(ConfigFile,"HoleForce: %d\n", (int *) &(CircuitData->Info.Opt.HoleForce));
  fscanf(ConfigFile,"AllowEditComponent: %d\n", (int *) &(CircuitData->Info.Opt.AllowEditComponent));
  fscanf(ConfigFile,"AllowDeleteComponent: %d\n", (int *) &(CircuitData->Info.Opt.AllowDeleteComponent));
  fscanf(ConfigFile,"AllowDeleteSubFunction: %d\n", (int *) &(CircuitData->Info.Opt.AllowDeleteSubFunction));
  fscanf(ConfigFile,"DefaultSubFunctionName: %[^\n]\n", CircuitData->Info.Opt.DefaultSubFunctionName);
  fscanf(ConfigFile,"DefaultComponentName: %[^\n]\n", CircuitData->Info.Opt.DefaultComponentName);
  fscanf(ConfigFile,"OpenInformation: %d\n", (int *) &(CircuitData->Info.Opt.OpenInformation));
  fscanf(ConfigFile,"RevertInformation: %d\n", (int *) &(CircuitData->Info.Opt.RevertInformation));
  fscanf(ConfigFile,"SaveInformation: %d\n", (int *) &(CircuitData->Info.Opt.SaveInformation));
  fscanf(ConfigFile,"UseMagic: %d\n", (int *) &(CircuitData->Info.Opt.UseMagic));
  fscanf(ConfigFile,"StopRouterOnError: %d\n", (int *) &(CircuitData->Info.Opt.StopRouterOnError));
  fscanf(ConfigFile,"CircuitDirectory: %[^\n]\n", CircuitData->Info.Opt.CircuitDirectory);
  fclose(ConfigFile);
  
  return AllOk;
}


void FirstExecution(void)
{
  struct stat DirectoryInfo;
  char Buffer[BufferLength], *HomeDir;
  
  
  HomeDir = getenv("HOME");
  if (HomeDir != NULL)
    sprintf(Buffer,"%s/%s",HomeDir, ConfigDir);
  else
    printf("The HOME variable is not set!!!\nErrors may occur...!\n");

  if (stat(Buffer, &DirectoryInfo) != 0)
    {
      printf("The directory %s does not exist.\n",Buffer);
      printf("creating %s\n", Buffer);
      if (mkdir(Buffer, (S_IRUSR | S_IWUSR | S_IXUSR)) != 0)
	{
	  printf("Error creating directory %s\nErrors may occur... \n\n", Buffer);
	}
    }
}
	
  
