
#include <stdlib.h>
#include <math.h>
#include "roadncircuit.h"
#include "equipot.h"
#include "misc.h"

ErrorCheck InitSquareTab (Circuit *pPlate, int CurrentLink)
{ /* Attention ca pointe dans tous les sens */
  int Counterx, Countery, Counter, CounterAttach, x, y, Width, Height;
  Square **SquareTab;
  ErrorCheck Check;

  Width = pPlate->Width;
  Height = pPlate->Height;

  SquareTab = (Square **) malloc (sizeof (Square *) * Width);
  pPlate->SquareTab = SquareTab;
  
  for (Counterx = 0; Counterx < Width; Counterx++)
    SquareTab[Counterx] = (Square *) malloc (sizeof (Square) * Height);

 /* inititialisation commune a toutes les cases  */
  for (Counterx = 0; Counterx < Width; Counterx++)
    for (Countery = 0 ; Countery < Height; Countery++)
      {
	SquareTab[Counterx][Countery].CalculatingLineNb = NoLine;
	SquareTab[Counterx][Countery].CalculatingLevelNb = NoLevel;
	SquareTab[Counterx][Countery].CalculatingVortexNb = NoVortex;
	SquareTab[Counterx][Countery].CalculatingVortexLineNb = NoVortex;

	SquareTab[Counterx][Countery].SquareState = 0;
	SquareTab[Counterx][Countery].AttachLinkNb = NoAttachLink;
      }

  for (Counterx = 0; Counterx < Width; Counterx++)
    { /* les bords */
      SetSquareUsed( SquareTab[Counterx][0].SquareState);
      SetSquareUsed( SquareTab[Counterx][Height-1].SquareState);
    }
  for (Countery = 0; Countery < Height; Countery++)
    { /* les bords */
      SetSquareUsed( SquareTab[0][Countery].SquareState);
      SetSquareUsed( SquareTab[Width-1][Countery].SquareState);
    }
  
  for (CounterAttach = 0; CounterAttach < pPlate->AttachNb ; CounterAttach++)
    {
      for (Counter = 0 ; Counter < pPlate->AttachTab[CounterAttach].CoreAttachLocationNb ; Counter++)
	{
	  x = pPlate->AttachTab[CounterAttach].CoreAttachLocation[Counter].x;
	  y = pPlate->AttachTab[CounterAttach].CoreAttachLocation[Counter].y;
	  SetSquareAttach( SquareTab[x][y].SquareState);
	  SetSquareUsed( SquareTab[x][y].SquareState);
	  SquareTab[x][y].AttachLinkNb = CounterAttach;
	}
      for (Counter = 0 ; Counter < pPlate->AttachTab[CounterAttach].AttachLocationNb ; Counter++)
	{
	  x = pPlate->AttachTab[CounterAttach].AttachLocation[Counter].x;
	  y = pPlate->AttachTab[CounterAttach].AttachLocation[Counter].y;
	  SetSquareUsed( SquareTab[x][y].SquareState);
	}
      Check = FindNbInTab(CurrentLink, pPlate->AttachTab[CounterAttach].Attach2Link, pPlate->AttachTab[CounterAttach].Size);
      if(Check == NotFound)
	{
	  for (Counter = 0 ; Counter < pPlate->AttachTab[CounterAttach].InvisibleAttachLocationNb ; Counter++)
	    {
	      x = pPlate->AttachTab[CounterAttach].InvisibleAttachLocation[Counter].x;
	      y = pPlate->AttachTab[CounterAttach].InvisibleAttachLocation[Counter].y;
	      SetSquareUsed( SquareTab[x][y].SquareState);
	    }
	}
    }
  
  return AllOk;
}

ErrorCheck DestroyCircuit (Circuit *pPlate)
{
  int Counter;
  
  for (Counter = 0; Counter < pPlate->Width; Counter++)
    free (pPlate->SquareTab[Counter]);
  free (pPlate->SquareTab);
  
  for (Counter = 0 ; Counter < pPlate->LinkNb; Counter++)
    {
      if (pPlate->LinkTab[Counter].LinkSize != 0)
	free(pPlate->LinkTab[Counter].Link2Link);
    }
  if (pPlate->LinkNb != 0)
    free(pPlate->LinkTab);
  
  for (Counter = 0 ; Counter < pPlate->AttachNb; Counter++)
    {
      if ( pPlate->AttachTab[Counter].Size != 0)
	free(pPlate->AttachTab[Counter].Attach2Link);
      free(pPlate->AttachTab[Counter].AttachLocation);
    }

  if (pPlate->AttachNb != 0)
    free(pPlate->AttachTab);
    
  return AllOk;
}

ErrorCheck InitRoad(RoadMap *pMap, int LinkNb)
{
  int Counter;
  
  pMap->Size = LinkNb; 
  pMap->Line  = (RoadCarac *) malloc (sizeof(RoadCarac) * LinkNb);
  for (Counter = 0; Counter < LinkNb; Counter++)
    {
      pMap->Line[Counter].SegSize = 0;
      pMap->Line[Counter].PathSize = 0;
      pMap->Line[Counter].CorePathSize = 0;
      pMap->Line[Counter].DiagonalSize = 0;
      pMap->Line[Counter].DepSize = 0;

      pMap->Line[Counter].DiagonalTab = NULL;
      pMap->Line[Counter].PathTab = NULL;
      pMap->Line[Counter].SegTab = NULL;
      pMap->Line[Counter].DepTab = NULL;

      pMap->Line[Counter].JobDone = False;
    }
  return AllOk;
}

ErrorCheck DestroyRoad (RoadMap *pMap)
{
  int Counter;
  for (Counter = 0; Counter < pMap->Size; Counter++)
    {     
      if (pMap->Line[Counter].PathSize != 0)
	free (pMap->Line[Counter].PathTab);
       if (pMap->Line[Counter].DiagonalSize != 0)
	free (pMap->Line[Counter].DiagonalTab);
      if (pMap->Line[Counter].SegSize != 0)
	free (pMap->Line[Counter].SegTab);
      if (pMap->Line[Counter].DepSize != 0)
	free (pMap->Line[Counter].DepTab);
    }

  if (pMap->Size != 0)
    free(pMap->Line);
  
  return AllOk;
}

ErrorCheck TracePreviousRoad(Circuit *pBoard, RoadCarac *CaracTab, int RoadNbToTrace, int *RoadTab, int CurrentLink)
{
  int  x, y, Counter, RoadCounter, CurrentRoad, MaxPath, SquareState;

  for (RoadCounter = 0 ; RoadCounter < RoadNbToTrace; RoadCounter++) 
    { 
      CurrentRoad = RoadTab[RoadCounter];
      
      SquareState = SquareUnused;
      if (IsSameEquiPot(pBoard, CurrentLink, SquareState, CurrentRoad) == Found)
	MaxPath =  CaracTab[CurrentRoad].CorePathSize; /* meme equipot */
      else
	MaxPath =  CaracTab[CurrentRoad].PathSize;

      for (Counter = 0 ; Counter < MaxPath ; Counter++)
	{ 
	  x = CaracTab[CurrentRoad].PathTab[Counter].x;
	  y = CaracTab[CurrentRoad].PathTab[Counter].y;
	  if (! IsSquareUsed( pBoard->SquareTab[x][y].SquareState ))
	    {
	      SetSquareUsed( pBoard->SquareTab[x][y].SquareState );
	      pBoard->SquareTab[x][y].AttachLinkNb = CurrentRoad;
	    }
	}

      for (Counter = 0 ; Counter < CaracTab[CurrentRoad].DiagonalSize ; Counter++)
	{ /* on remet les coins utilises */
	  x = CaracTab[CurrentRoad].DiagonalTab[Counter].x;
	  y = CaracTab[CurrentRoad].DiagonalTab[Counter].y;
	  SetFullFourCorners(x, y, pBoard->SquareTab);
	}
    }
  return AllOk;
}

ErrorCheck GetSegmentPoint(RoadCarac *CaracTab, int RoadNb,  EquiPotential *TrackPoint)
{
  int Deltax, Deltay, Startx, Starty, Endx, Endy, SegmentNb, SegmentCounter, Stepx, Stepy, SquareNb, SquareCounter, TrackCounter;
  EquiPotential *SegmentTab;
  
  TrackPoint->Size = 0;
  SegmentNb = CaracTab[RoadNb].SegSize;
  if (SegmentNb ==0)
    return AllOk;

  SegmentTab = (EquiPotential *) malloc (sizeof(EquiPotential) * SegmentNb);
  
  for (SegmentCounter = 0 ; SegmentCounter < SegmentNb ; SegmentCounter++)
    {
      Startx = CaracTab[RoadNb].SegTab[SegmentCounter].Start.x;
      Starty = CaracTab[RoadNb].SegTab[SegmentCounter].Start.y;
      Endx = CaracTab[RoadNb].SegTab[SegmentCounter].End.x;
      Endy = CaracTab[RoadNb].SegTab[SegmentCounter].End.y;
      
      Deltax = Endx - Startx;
      Deltay = Endy - Starty;

      if ( abs(Deltax) > abs(Deltay))
	SquareNb = abs(Deltax) +1;
      else
	SquareNb = abs(Deltay) +1;
	
      SegmentTab[SegmentCounter].Size = SquareNb;
      SegmentTab[SegmentCounter].PointTab = (Point *) malloc(sizeof(Point) * SquareNb);
            
      if (Deltax == 0)
	Stepx = 0;
      else
	Stepx = (Deltax > 0) ? 1 : -1;
      
      if (Deltay == 0)
	Stepy = 0;
      else
	Stepy = (Deltay > 0) ? 1 : -1;

      for (SquareCounter = 0 ; SquareCounter < SquareNb ; SquareCounter++)
	{
	  SegmentTab[SegmentCounter].PointTab[SquareCounter].x = Startx + Stepx * SquareCounter;
	  SegmentTab[SegmentCounter].PointTab[SquareCounter].y = Starty + Stepy * SquareCounter;
	}
      TrackPoint->Size += SegmentTab[SegmentCounter].Size;
    }

  TrackPoint->PointTab = (Point *) malloc(sizeof(Point) * TrackPoint->Size);
  TrackCounter = 0;
  for (SegmentCounter = 0 ; SegmentCounter < SegmentNb ; SegmentCounter++)
    {
      for (SquareCounter = 0 ; SquareCounter <  SegmentTab[SegmentCounter].Size ; SquareCounter++)
	{
	  TrackPoint->PointTab[TrackCounter].x = SegmentTab[SegmentCounter].PointTab[SquareCounter].x;
	  TrackPoint->PointTab[TrackCounter].y = SegmentTab[SegmentCounter].PointTab[SquareCounter].y;
	  TrackCounter++;
	}
      if (SegmentTab[SegmentCounter].Size != 0)
	free(SegmentTab[SegmentCounter].PointTab);
    }
  free(SegmentTab);
  return AllOk;
}

ErrorCheck ComputeDirection(Circuit *pPlate, Direction LastDir, Direction *DirectionTab, int *DirectionTabSize)
{
  int Counter, DirectionNb, DirectionCounter;
  Boolean DirectionUsed[8];
  
  for (Counter = 0 ; Counter < 8 ; Counter++)
    DirectionUsed[Counter] = False;

  DirectionNb = 0;
  for (Counter = 0 ; Counter < 3 ; Counter++)
    {
      if ((pPlate->DirectionPrefTab[Counter] == SameDirectionPref) && (LastDir != NoDirection)
	  && (DirectionUsed[LastDir] == False))
	{
	  DirectionTab[DirectionNb] = LastDir;
	  DirectionNb++;
	  DirectionUsed[LastDir] = True;
	}
      
      if (pPlate->DirectionPrefTab[Counter] == OrthogonalPref)
	{
	  for (DirectionCounter = 0 ; DirectionCounter < 8 ; DirectionCounter+= 2)
	    if (DirectionUsed[DirectionCounter] == False)
	      {
		DirectionTab[DirectionNb] = DirectionCounter;
		DirectionNb++;
		DirectionUsed[DirectionCounter] = True;
	      }
	}

      if (pPlate->DirectionPrefTab[Counter] == DiagonalPref)
	{
	  for (DirectionCounter = 1 ; DirectionCounter < 8 ; DirectionCounter+= 2)
	    if (DirectionUsed[DirectionCounter] == False)
	      {
		DirectionTab[DirectionNb] = DirectionCounter;
		DirectionNb++;
		DirectionUsed[DirectionCounter] = True;
	      }
	}
    }
  *DirectionTabSize = DirectionNb;
  return AllOk;
}

ErrorCheck FeatherPath(Circuit Board, RoadMap *pMap, int CurrentLink)
{
  Point *LargePath;
  int LargePathSize, LinkWidth, PathCounter, Currentx, Currenty, xcenter, ycenter, x, y, 
    CurrentDistance, RadiusSquare, CheckPathCounter;
  Boolean FoundInPath;
  
  if (pMap->Line[CurrentLink].JobDone == False)
    {
      printf("Trying to feather a link ( %d ) not traced!!!\n", CurrentLink);
      return ArgumentError;
    }
  
  pMap->Line[CurrentLink].CorePathSize = pMap->Line[CurrentLink].PathSize;
  
  LinkWidth =  Board.LinkTab[CurrentLink].LinkWidth;
  if ((LinkWidth == 0) || (pMap->Line[CurrentLink].PathSize == 0))
    return AllOk;

  LargePathSize = (int) pMap->Line[CurrentLink].PathSize * (M_PI * LinkWidth * LinkWidth);
  LargePath = (Point *) malloc(sizeof(Point) * LargePathSize);
  
  for (PathCounter = 0 ; PathCounter < pMap->Line[CurrentLink].PathSize ; PathCounter++)
    {
      LargePath[PathCounter].x = pMap->Line[CurrentLink].PathTab[PathCounter].x;
      LargePath[PathCounter].y = pMap->Line[CurrentLink].PathTab[PathCounter].y;
    }
  LargePathSize = pMap->Line[CurrentLink].PathSize;
  RadiusSquare = LinkWidth * LinkWidth;
  for (PathCounter = 0 ; PathCounter < pMap->Line[CurrentLink].PathSize ; PathCounter++)
    {
      xcenter = pMap->Line[CurrentLink].PathTab[PathCounter].x;
      ycenter = pMap->Line[CurrentLink].PathTab[PathCounter].y;
      for (x = -LinkWidth ; x <= LinkWidth ; x++)
	for (y = -LinkWidth ; y <= LinkWidth ; y++)
	  {
	    CurrentDistance = x * x + y * y;
	    if ( (CurrentDistance > RadiusSquare) || ((x == 0) && (y == 0)) )
	      continue;
	    
	    Currentx = xcenter + x;
	    Currenty = ycenter + y;
	    
	    FoundInPath = False;
	    for (CheckPathCounter = 0 ; (CheckPathCounter < LargePathSize) && (FoundInPath ==False) ;
		 CheckPathCounter++)
	      if ( (LargePath[CheckPathCounter].x == Currentx) && (LargePath[CheckPathCounter].y == Currenty))
		FoundInPath = True;
	    
	    if (FoundInPath == False)
	      {
		LargePath[LargePathSize].x = Currentx;
		LargePath[LargePathSize].y = Currenty;
		LargePathSize++;
	      }
	  }
    }
  free(pMap->Line[CurrentLink].PathTab);
  
  pMap->Line[CurrentLink].PathSize = LargePathSize;
  pMap->Line[CurrentLink].PathTab = (Point *) malloc(sizeof(Point) * LargePathSize);
  for (PathCounter = 0 ; PathCounter < LargePathSize ; PathCounter++)
    {
      pMap->Line[CurrentLink].PathTab[PathCounter].x = LargePath[PathCounter].x;
      pMap->Line[CurrentLink].PathTab[PathCounter].y = LargePath[PathCounter].y;
    }
  free (LargePath);
  return AllOk;
}

ErrorCheck FeatherCircuit(Circuit *pBoard, int CurrentLink)
{
  int x,y, Width, Height, LinkWidth, CurrentDistance, RadiusSquare, xcenter, ycenter, Currentx, Currenty;
  ErrorCheck SameEquiPot;
  Boolean SquareToFeather;

  Width = pBoard->Width;
  Height = pBoard->Height;
  LinkWidth = pBoard->LinkTab[CurrentLink].LinkWidth;
  RadiusSquare = LinkWidth * LinkWidth;

  for (xcenter = 0 ; xcenter < Width ; xcenter++)
    for (ycenter = 0 ; ycenter < Height ; ycenter++)
      {
	SquareToFeather = False;
	
	if ( (IsSquareUsed(pBoard->SquareTab[xcenter][ycenter].SquareState)) 
	     && (pBoard->SquareTab[xcenter][ycenter].AttachLinkNb == NoAttachLink)) /* les bords du circuit */
	  SquareToFeather = True;
	
	if ( (SquareToFeather == False) 
	     && (IsSquareUsed(pBoard->SquareTab[xcenter][ycenter].SquareState)) )
	  {
	    SameEquiPot = IsSameEquiPot(pBoard, pBoard->SquareTab[xcenter][ycenter].AttachLinkNb,
					pBoard->SquareTab[xcenter][ycenter].SquareState, CurrentLink);
	    if (SameEquiPot == NotFound) /* une piste non equipotentielle */
	      SquareToFeather = True;
	  }	
	if (SquareToFeather == False)
	  continue;
	
	for (x = -LinkWidth ; x <= LinkWidth ; x++)
	  for (y = -LinkWidth ; y <= LinkWidth ; y++)
	    {
	      CurrentDistance = x * x + y * y;
	      if ( (CurrentDistance > RadiusSquare) || ((x == 0) && (y == 0)) )
		continue;
	      
	      Currentx = xcenter + x;
	      Currenty = ycenter + y;
	      
	      if ( (Currentx > 0) && (Currentx < (Width-1)) && (Currenty > 0) && (Currenty < (Height - 1))
		   && (!IsSquareUsed(pBoard->SquareTab[Currentx][Currenty].SquareState)) )
		SetFeatherUsed(pBoard->SquareTab[Currentx][Currenty].SquareState);
	    }
      }
  return AllOk;
}	
  

		  
	  
      
      
  
	
	      
	    
