
#include "subfunction.h"
#include "mainwindow.h"
#include "link.h"
#include "component.h"
#include "attachpoint.h"
#include "misc.h"

int NewSubFunctionID(GCD *CircuitData, int SetSubFunctionID)
{
  CircuitData->Info.SubFunctionID += 10;
  if (SetSubFunctionID != NoID)
    CircuitData->Info.SubFunctionID = SetSubFunctionID-10;
  /* SetSubFunctionID est le prochain ID utilisable, il doit etre renvoye lors du prochain appel */

  return CircuitData->Info.SubFunctionID;
}
 
ErrorCheck AddSubFunction(GCD *CircuitData, int SubFID)
{
  SubFunction *NewSubF, *SubFCursor, *PrevSubF;

  NewSubF=(SubFunction*)malloc(sizeof(SubFunction));
  if (NewSubF == NULL)
    {
      printf ("Error \n");
      return MemoryAllocationError;
    }
  
  NewSubF->IDNumber = SubFID;
  strncpy(NewSubF->Name, CircuitData->Info.Opt.DefaultSubFunctionName, SubFunctionNameLength);
  NewSubF->Name[SubFunctionNameLength-1] = '\0';
  NewSubF->Reference.x = 2;
  NewSubF->Reference.y = 2;
  InitState(NewSubF->State);
  UnSetSubFSelected(NewSubF->State);
  NewSubF->pNext = NULL;

  PrevSubF = NULL;
  SubFCursor = CircuitData->ListSubFunction;
  while (SubFCursor != NULL)
    {
      PrevSubF = SubFCursor;
      SubFCursor = SubFCursor->pNext;
    }
  if (PrevSubF == NULL)
    CircuitData->ListSubFunction = NewSubF;
  else 
    PrevSubF->pNext = NewSubF;
  
  return AllOk;
}


ErrorCheck EditSubFunction(GCD *CircuitData, int SubFID, SubFunction SubFProp)
{
  SubFunction *SubFCursor;
  Boolean  IDFound;
  int LinkNb, *LinkIDTab, Counter;
  
  IDFound = False;
  SubFCursor=CircuitData->ListSubFunction;
  while( (SubFCursor!=NULL) && (IDFound == False))
    {
      if(SubFCursor->IDNumber == SubFID)
	{
	  IDFound = True;
	  strncpy(SubFCursor->Name, SubFProp.Name, SubFunctionNameLength);
	  SubFCursor->Name[SubFunctionNameLength-1]='\0';
	  if ((SubFCursor->Reference.x != SubFProp.Reference.x)||(SubFCursor->Reference.y != SubFProp.Reference.y))
	    {
	      UnDrawAllLinks(CircuitData);
	      SubFCursor->Reference.x = SubFProp.Reference.x;
	      SubFCursor->Reference.y = SubFProp.Reference.y;
	      
	      GetLinkNumber(CircuitData, &LinkNb);
	      if (LinkNb != 0)
		{
		  LinkIDTab = (int *) malloc (sizeof(int) * LinkNb);
		  GetLinkUsingSubFunction(CircuitData, SubFID, LinkIDTab, &LinkNb);
		  
		  for (Counter = 0 ; Counter < LinkNb ; Counter++)
		    UpdateLink(CircuitData, LinkIDTab[Counter]);
		  free(LinkIDTab);
		}
  	    }
	}
      SubFCursor=SubFCursor->pNext;
    }
  if (IDFound == True)
    return Found;
  else return NotFound;
}

ErrorCheck DisplaySubFunction(GCD *CircuitData)
{
  SubFunction *SubFCursor;

  printf("Liste des sous fonctions:\n"); 
  SubFCursor=CircuitData->ListSubFunction;
  while(SubFCursor!=NULL)
    {
      printf("sous fonction: %s, ID %d  %d %d\n",SubFCursor->Name,SubFCursor->IDNumber, SubFCursor->Reference.x, 
	     SubFCursor->Reference.y );
      SubFCursor=SubFCursor->pNext;
    }
  printf("\n");
 return AllOk;
}


ErrorCheck DeleteSubFunction(GCD *CircuitData, int SubFID)
{
  SubFunction *SubFCursor, **SubFNext ;
  Boolean IDFound;
  int Counter, LinkNb, *LinkIDTab;
/*   printf("Suppression de la sous fonction %d\n",SubFID);  */

  SubFNext  = &(CircuitData->ListSubFunction);
  SubFCursor = CircuitData->ListSubFunction;
  IDFound = False;
  while ( (SubFCursor!=NULL) && (IDFound == False))
    {
      if (SubFCursor->IDNumber == SubFID)
	{
	  IDFound = True;
	  	      
	  GetLinkNumber(CircuitData, &LinkNb);
	  if (LinkNb != 0)
	    {
	      LinkIDTab = (int *) malloc (sizeof(int) * LinkNb);
	      GetLinkUsingSubFunction(CircuitData, SubFID, LinkIDTab, &LinkNb);
	  
	      for (Counter = 0 ; Counter < LinkNb ; Counter++)
		DeleteLink(CircuitData, LinkIDTab[Counter]);
	      
	      free(LinkIDTab);
	    }
	  *SubFNext = SubFCursor->pNext;
	  free(SubFCursor);
	}
      else SubFNext = &(SubFCursor->pNext);
      
      SubFCursor = *SubFNext;
    }

  if (IDFound == True)
    return Found;
  else return NotFound;

}

extern ErrorCheck GetSubFunctionProperties(GCD *CircuitData, int SubFID, SubFunction *SubFProp)
{
  SubFunction *SubFCursor;
  Boolean IDFound;
  
  SubFCursor =CircuitData->ListSubFunction;
  IDFound = False;
  while ((SubFCursor!=NULL) && (IDFound == False))
    {
      if(SubFCursor->IDNumber==SubFID)
	{
	  IDFound = True;
	  strncpy(SubFProp->Name, SubFCursor->Name, SubFunctionNameLength);
	  SubFProp->Name[SubFunctionNameLength-1]='\0';
	  SubFProp->IDNumber= SubFCursor->IDNumber;
	  SubFProp->State= SubFCursor->State;
	  SubFProp->Reference.x = SubFCursor->Reference.x;
	  SubFProp->Reference.y = SubFCursor->Reference.y;
	}
      SubFCursor= SubFCursor->pNext;
    }
  if (IDFound == True)
    return Found;
  else return NotFound;

}


ErrorCheck GetSubFunctionCompNumber(GCD *CircuitData, int SubFID, int *CompNb)
{
  /* le nb de comp par subfunction*/
  int Counter;
  Component *CompCursor;
  Counter=0; 

  CompCursor=CircuitData->ListComponent;
  while (CompCursor!=NULL)
    {
      if(CompCursor->SubFunctionID == SubFID)
	Counter++;
      CompCursor = CompCursor->pNext;
    }
  *CompNb = Counter;

  return AllOk;
}

ErrorCheck GetSubFunctionCompIDTab(GCD *CircuitData, int SubFID, int *CompIDTab)
{
  int Counter;
  Component *CompCursor;
  
  Counter=0; 
  CompCursor=CircuitData->ListComponent;
  while(CompCursor!=NULL)
    {
      if(CompCursor->SubFunctionID== SubFID)
	{
	  CompIDTab[Counter] = CompCursor->IDNumber;
	  Counter++;
	}
      CompCursor = CompCursor->pNext;
    }
return AllOk;
}

ErrorCheck GetLinkUsingSubFunction(GCD *CircuitData, int SubFID, int *LinkIDTab, int *LinkNb)
{
  Link *LinkCursor;
  int LinkCounter, Counter, AttachNb, *AttachIDTab, Start, End;
  Boolean Used;

  GetSubFunctionAttachNumber(CircuitData, SubFID, &AttachNb);
  if (AttachNb == 0)
    {
      *LinkNb = 0;
      return AllOk;
    }
  AttachIDTab = (int *) malloc(sizeof(int) * AttachNb);
  GetSubFunctionAttachID(CircuitData, SubFID, AttachIDTab);
  
  LinkCursor = CircuitData->ListLink;
  LinkCounter = 0;
  while (LinkCursor != NULL)
    {
      Start = LinkCursor->StartAttachID;
      End = LinkCursor->EndAttachID;

      Used = False;
      for (Counter = 0 ; (Counter < AttachNb) && (Used == False) ; Counter++)
	if ((AttachIDTab[Counter] == Start) || (AttachIDTab[Counter] == End))
	  Used = True;
      
      if (Used == True)
	{
	  LinkIDTab[LinkCounter] = LinkCursor->IDNumber;
	  LinkCounter++;
	}
      LinkCursor = LinkCursor->pNext;
    }

  *LinkNb = LinkCounter;
  return AllOk;
}
ErrorCheck GetSubFunctionAttachNumber(GCD *CircuitData, int SubFID, int *AttachNb)
{
  Component *CompCursor;
  int AttachPointNb;
  
  AttachPointNb = 0;
  CompCursor = CircuitData->ListComponent;
  while (CompCursor != NULL)
    {
      if (CompCursor->SubFunctionID == SubFID)
	AttachPointNb += CompCursor->AttachPointNb;
      CompCursor = CompCursor->pNext;
    }
  *AttachNb = AttachPointNb;
  return AllOk;
}

ErrorCheck GetSubFunctionAttachID(GCD *CircuitData, int SubFID, int *AttachIDTab)
{
  Component *CompCursor;
  int AttachCounter, Counter;

  AttachCounter = 0;
  CompCursor = CircuitData->ListComponent;
  while (CompCursor != NULL)
    {
      if (CompCursor->SubFunctionID == SubFID)
	for (Counter = 0 ; Counter < CompCursor->AttachPointNb ; Counter++)
	  {
	    AttachIDTab[AttachCounter] = CompCursor->AttachPointIDTab[Counter];
	    AttachCounter++;
	  }
      CompCursor = CompCursor->pNext;
    }

  return AllOk;
}

ErrorCheck DeleteSubFunctionBuffer(GCD *CircuitData)
{
  int Counter;
  
  if (CircuitData->ListSubFunctionBuffer.NbComp != 0)
    {
      free(CircuitData->ListSubFunctionBuffer.TabCompSF);
      CircuitData->ListSubFunctionBuffer.TabCompSF=NULL;
    }
  if (CircuitData->ListSubFunctionBuffer.NbLink != 0)
    {
      free(CircuitData->ListSubFunctionBuffer.TabLinkSF);

      for (Counter = 0 ; Counter < CircuitData->ListSubFunctionBuffer.NbLink  ; Counter++)
	free(CircuitData->ListSubFunctionBuffer.LinkInfo[Counter]);
      free(CircuitData->ListSubFunctionBuffer.LinkInfo);

      CircuitData->ListSubFunctionBuffer.TabCompSF=NULL;
    }

  CircuitData->SubFunctionBufferEmpty = True;
  return AllOk;
}

ErrorCheck CutSubFunction(GCD *CircuitData, int SubFID)
{
  int Counter, NbLink;

  AddSubFunctionToBuffer(CircuitData, SubFID);
  
  GetSubFunctionLinkNumber(CircuitData, SubFID, &NbLink, False);
  if (NbLink != 0)
    {
      if (CircuitData->Info.Opt.AllowDeleteSubFunction == True)/* effacement de tous les composants trouvs */
	{
	  for(Counter = 0 ; Counter < CircuitData->ListSubFunctionBuffer.NbComp ; Counter++)
	    DeleteComponent(CircuitData, CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].IDNumber);
	  DeleteSubFunction(CircuitData, SubFID);
	}
      else
	return Forbidden;
    }
  else
    {
      for(Counter=0;Counter < CircuitData->ListSubFunctionBuffer.NbComp; Counter++)
	DeleteComponent(CircuitData, CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].IDNumber);
      DeleteSubFunction(CircuitData, SubFID);
    }

  return AllOk;
}
ErrorCheck CopySubFunction(GCD *CircuitData, int SubFID)
{
  AddSubFunctionToBuffer(CircuitData, SubFID);
  return AllOk;
}

ErrorCheck AddSubFunctionToBuffer(GCD *CircuitData, int SubFID)
{
  Component *CompCursor, CompProp;
  SubFunction *SubFCursor;
  int Counter, NbLink, NbComp, *CompIDTab= NULL, *LinkIDTab, StartAttachPos=0, EndAttachPos=0, CompCounter, 
    StartCompIDPos=0, EndCompIDPos=0, AttachCounter;
  Link *LinkCursor;
  AttachPoint AttachStartProp, AttachEndProp;
  Boolean IDFound, StartFound, EndFound, AttachFound;
  ErrorCheck LinkInSubF;
  
  DeleteSubFunctionBuffer(CircuitData);

  /* recherche du nom de la sous fonction */ 
  SubFCursor=CircuitData->ListSubFunction;
  IDFound = False;
  while ((SubFCursor!=NULL) && (IDFound == False))
    {
      if( SubFCursor->IDNumber==SubFID)
	{
	  IDFound = True;

	  strncpy(CircuitData->ListSubFunctionBuffer.Name,SubFCursor->Name, SubFunctionNameLength);
	  CircuitData->ListSubFunctionBuffer.Name[SubFunctionNameLength-1]='\0';
	  UnSetSubFSelected(SubFCursor->State);
	  CircuitData->ListSubFunctionBuffer.State = SubFCursor->State;
	  CircuitData->ListSubFunctionBuffer.Reference.x = SubFCursor->Reference.x;
	  CircuitData->ListSubFunctionBuffer.Reference.y = SubFCursor->Reference.y;
	}
      SubFCursor = SubFCursor->pNext;
    }

  /* recherche de tous les composants de la sous fonction SubFID */ 
  GetSubFunctionCompNumber(CircuitData,SubFID, &(NbComp));
  CircuitData->ListSubFunctionBuffer.NbComp = NbComp;
  CircuitData->SubFunctionBufferEmpty = False;
  
  if (CircuitData->ListSubFunctionBuffer.NbComp != 0)
    {
      CircuitData->ListSubFunctionBuffer.TabCompSF = (Component*)malloc(CircuitData->ListSubFunctionBuffer.NbComp*sizeof(Component));
      
      CompIDTab = (int *) malloc(sizeof(int) * NbComp);
      CompCursor=CircuitData->ListComponent;
      Counter=0;
      while(CompCursor!=NULL)
	{
	  if (CompCursor->SubFunctionID == SubFID)
	    {
	      CompIDTab[Counter] = CompCursor->IDNumber;
	      strncpy(CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].ComponentName,
		      CompCursor->ComponentName, ComponentNameLength);
	      CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].ComponentName[ComponentNameLength-1]='\0';
	      
	      strncpy(CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].CompDescName,
		      CompCursor->CompDescName,CompDescNameLength);
	      CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].CompDescName[CompDescNameLength-1]='\0';
		
	      strncpy(CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].LayoutName,
		      CompCursor->LayoutName,LayoutNameLength);
	      CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].LayoutName[LayoutNameLength-1]='\0';
	      strncpy(CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].LayoutClass,
		      CompCursor->LayoutClass,LayoutClassLength);
	      CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].LayoutClass[LayoutClassLength-1]='\0';

	      CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].IDNumber=CompCursor->IDNumber;
	      CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].State = CompCursor->State;
	      UnSetComponentSelected(CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].State);
	      CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].Reference.x=CompCursor->Reference.x;
	      CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].Reference.y=CompCursor->Reference.y;
	      Counter++;
	    }
	  CompCursor = CompCursor->pNext;
	}
    }

  GetSubFunctionLinkNumber(CircuitData,SubFID, &(NbLink), True);
  CircuitData->ListSubFunctionBuffer.NbLink = NbLink;

  if (NbLink != 0)
    {
      LinkIDTab = (int *) malloc(sizeof(int) * NbLink);
      GetSubFunctionLinkTab(CircuitData,SubFID, LinkIDTab);
      
      CircuitData->ListSubFunctionBuffer.TabLinkSF = (Link *)malloc(NbLink * sizeof(Link));
      CircuitData->ListSubFunctionBuffer.LinkInfo = (int **) malloc(NbLink * sizeof(int *));
      LinkCursor=CircuitData->ListLink;
      Counter=0;
      while(LinkCursor!=NULL)
	{
	  LinkInSubF = FindNbInTab(LinkCursor->IDNumber, LinkIDTab, NbLink);
	  if (LinkInSubF == Found)
	    {
	      CircuitData->ListSubFunctionBuffer.LinkInfo[Counter] = (int *) malloc(sizeof(int) * 4);
	      GetAttachPointProperties(CircuitData, LinkCursor->StartAttachID, &AttachStartProp);
	      GetAttachPointProperties(CircuitData, LinkCursor->EndAttachID, &AttachEndProp);
	      StartFound = False;
	      EndFound = False;
	      for (CompCounter = 0 ; ((StartFound == False) || (EndFound == False)) && (CompCounter < NbComp) ; 
		   CompCounter++)
		{
		  if (CompIDTab[CompCounter] == AttachStartProp.ComponentID)
		    {
		      StartCompIDPos = CompCounter;
		      StartFound = True;
		    }
		  if (CompIDTab[CompCounter] == AttachEndProp.ComponentID)
		    {
		      EndCompIDPos = CompCounter;
		      EndFound = True;
		    }
		}
	      CircuitData->ListSubFunctionBuffer.LinkInfo[Counter][0] = StartCompIDPos;
	      CircuitData->ListSubFunctionBuffer.LinkInfo[Counter][2] = EndCompIDPos;
	      GetComponentProperties(CircuitData, AttachStartProp.ComponentID, &CompProp, True);
	      AttachFound = False;
	      for (AttachCounter = 0 ; (AttachFound == False) && (AttachCounter < CompProp.AttachPointNb) ;
		   AttachCounter++)
		{
		  if (CompProp.AttachPointIDTab[AttachCounter] ==  AttachStartProp.IDNumber)
		    {
		      StartAttachPos = AttachCounter;
		      AttachFound = True;
		    }
		}
	      free(CompProp.AttachPointIDTab);

	      GetComponentProperties(CircuitData, AttachEndProp.ComponentID, &CompProp, True);
	      AttachFound = False;
	      for (AttachCounter = 0 ; (AttachFound == False) && (AttachCounter < CompProp.AttachPointNb) ;
		   AttachCounter++)
		{
		  if (CompProp.AttachPointIDTab[AttachCounter] ==  AttachEndProp.IDNumber)
		    {
		      EndAttachPos = AttachCounter;
		      AttachFound = True;
		    }
		}
	      free(CompProp.AttachPointIDTab);
	      CircuitData->ListSubFunctionBuffer.LinkInfo[Counter][1] = StartAttachPos;
	      CircuitData->ListSubFunctionBuffer.LinkInfo[Counter][3] = EndAttachPos;
	      
	      CircuitData->ListSubFunctionBuffer.TabLinkSF[Counter].StartAttachID = LinkCursor->StartAttachID;
	      CircuitData->ListSubFunctionBuffer.TabLinkSF[Counter].EndAttachID = LinkCursor->EndAttachID;
	      CircuitData->ListSubFunctionBuffer.TabLinkSF[Counter].LinkWidth = LinkCursor->LinkWidth;
	      CircuitData->ListSubFunctionBuffer.TabLinkSF[Counter].LinkColor.Red = LinkCursor->LinkColor.Red;
	      CircuitData->ListSubFunctionBuffer.TabLinkSF[Counter].LinkColor.Green = LinkCursor->LinkColor.Green;
	      CircuitData->ListSubFunctionBuffer.TabLinkSF[Counter].LinkColor.Blue = LinkCursor->LinkColor.Blue;
	      Counter++;
	    }
	  LinkCursor = LinkCursor->pNext;
	}
      
      free(LinkIDTab);
    }

  if (NbComp != 0)
    free(CompIDTab);

  return AllOk;
}

ErrorCheck PasteSubFunction(GCD *CircuitData, int *SubFID, int x, int y)
{
  int NewSubFID, NewCompID, Counter, *CompIDTab=NULL, AttachStartPos, AttachEndPos, CompStartPos, CompEndPos,
    NewID, AttachStart, AttachEnd, Red, Green, Blue;
  SubFunction SubF;
  Component Comp, CompProp;
  
  if ( CircuitData->SubFunctionBufferEmpty == True)
    return BufferEmpty;
  
  NewSubFID=NewSubFunctionID(CircuitData, NoID);
  *SubFID = NewSubFID;
  AddSubFunction(CircuitData,NewSubFID);
  
  strncpy(SubF.Name,CircuitData->ListSubFunctionBuffer.Name, SubFunctionNameLength);
  SubF.Name[SubFunctionNameLength-1] = '\0';
  SubF.State = CircuitData->ListSubFunctionBuffer.State;
  if ((x == NoX) || (y==NoY))
    {
      SubF.Reference.x = CircuitData->ListSubFunctionBuffer.Reference.x;
      SubF.Reference.y = CircuitData->ListSubFunctionBuffer.Reference.y;
    }
  else
    {
      SubF.Reference.x = x;
      SubF.Reference.y = y;
    }
  
  EditSubFunction(CircuitData, NewSubFID, SubF);
  if (CircuitData->ListSubFunctionBuffer.NbComp != 0)
    CompIDTab = (int *) malloc (sizeof(int) * CircuitData->ListSubFunctionBuffer.NbComp);
  
  for(Counter = 0; Counter < CircuitData->ListSubFunctionBuffer.NbComp ; Counter++)
    {
      NewCompID=NewComponentID(CircuitData, NoID);
      CompIDTab[Counter] = NewCompID;
      
      AddComponent(CircuitData, NewCompID, NewSubFID); 
      
      strncpy(Comp.ComponentName,CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].ComponentName, ComponentNameLength);
      Comp.ComponentName[ComponentNameLength-1]='\0';
      strncpy(Comp.CompDescName, CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].CompDescName, CompDescNameLength);
      Comp.CompDescName[CompDescNameLength-1]='\0';
      strncpy(Comp.LayoutName, CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].LayoutName, LayoutNameLength);
      Comp.LayoutName[LayoutNameLength-1]='\0';  
      strncpy(Comp.LayoutClass, CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].LayoutClass, LayoutClassLength);
      Comp.LayoutClass[LayoutClassLength-1]='\0';  
     
      Comp.State = CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].State;
      Comp.Reference.x=CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].Reference.x;
      Comp.Reference.y=CircuitData->ListSubFunctionBuffer.TabCompSF[Counter].Reference.y;
      EditComponent(CircuitData, NewCompID, Comp, True);
    }
  
  for (Counter = 0 ; Counter <  CircuitData->ListSubFunctionBuffer.NbLink ; Counter++)
    {
      NewID = NewLinkID(CircuitData, NoID);
      AddLink(CircuitData, NewID);
      CompStartPos = CircuitData->ListSubFunctionBuffer.LinkInfo[Counter][0];
      CompEndPos =  CircuitData->ListSubFunctionBuffer.LinkInfo[Counter][2];
      AttachStartPos = CircuitData->ListSubFunctionBuffer.LinkInfo[Counter][1];
      AttachEndPos =  CircuitData->ListSubFunctionBuffer.LinkInfo[Counter][3];

      GetComponentProperties(CircuitData, CompIDTab[CompStartPos], &CompProp, True);
      AttachStart = CompProp.AttachPointIDTab[AttachStartPos];
      free(CompProp.AttachPointIDTab);
      
      GetComponentProperties(CircuitData, CompIDTab[CompEndPos], &CompProp, True);
      AttachEnd = CompProp.AttachPointIDTab[AttachEndPos];
      free(CompProp.AttachPointIDTab);
      
      EditLink(CircuitData, False, NewID, AttachStart, ExceptionAttach);
      EditLink(CircuitData, False, NewID, ExceptionAttach, AttachEnd);

      Red = CircuitData->ListSubFunctionBuffer.TabLinkSF[Counter].LinkColor.Red;
      Green = CircuitData->ListSubFunctionBuffer.TabLinkSF[Counter].LinkColor.Green;
      Blue = CircuitData->ListSubFunctionBuffer.TabLinkSF[Counter].LinkColor.Blue;
      SetLinkColor(CircuitData, NewID, Red, Green, Blue);

      SetLinkWidth(CircuitData, NewID, CircuitData->ListSubFunctionBuffer.TabLinkSF[Counter].LinkWidth);
    }
  if ( CircuitData->ListSubFunctionBuffer.NbComp != 0)
    free (CompIDTab);
  return AllOk;
}

ErrorCheck SetIIDForSubFunction(GCD *CircuitData, int SubFID, void *InterfaceID)
{
  SubFunction *SubFCursor;
  Boolean IDFound;

  SubFCursor = CircuitData->ListSubFunction;
  IDFound = False;
  while ((SubFCursor != NULL)  && (IDFound == False))
    {
      if ( SubFCursor->IDNumber == SubFID)
	{
	  IDFound = True;
	  SubFCursor->InterfaceID = InterfaceID;
	}
      SubFCursor = SubFCursor->pNext;
    }
  if (IDFound ==True)
    return Found;
  else return NotFound;
}


ErrorCheck GetIIDForSubFunction(GCD *CircuitData, int ID, void **InterfaceID)
{
  SubFunction *SubFCursor;
  Boolean IDFound;

  SubFCursor = CircuitData->ListSubFunction;
  IDFound = False;
  while ((SubFCursor != NULL)  && (IDFound == False))
    {
      if ( SubFCursor->IDNumber == ID)
	{
	  IDFound = True;
	  *InterfaceID = SubFCursor->InterfaceID;
	}
      SubFCursor = SubFCursor->pNext;
    }
  if (IDFound ==True)
    return Found;
  else return NotFound;
}


ErrorCheck GetSubFunctionNumber(GCD *CircuitData, int *SubFunctionNb)
{
  SubFunction *SubFCursor;
  int Counter;

  Counter = 0;
  SubFCursor = CircuitData->ListSubFunction;
  while (SubFCursor != NULL)
    {
     Counter++;
     SubFCursor = SubFCursor->pNext;
    }
  *SubFunctionNb = Counter;
  return AllOk;
}

ErrorCheck GetSubFunctionIDFromCoordinates(GCD *CircuitData, int x, int y, int *SubFID)
{
  SubFunction *SubFCursor;
  Boolean FirstSubFFound;
  int Deltax, Deltay, MinLength=0, MinID, Length;


  SubFCursor = CircuitData->ListSubFunction;
  FirstSubFFound = False;
  MinID = NoID;
  while (SubFCursor != NULL)
    {
      Deltax = SubFCursor->Reference.x - x;
      Deltay = SubFCursor->Reference.y - y;
      Length = Deltax * Deltax + Deltay * Deltay;
      if ((FirstSubFFound == False) || (Length < MinLength))
	{
	  FirstSubFFound = True;
	  MinLength = Length;
	  MinID = SubFCursor->IDNumber;
	}  
      SubFCursor = SubFCursor->pNext;
    }

  *SubFID = MinID;

  if (FirstSubFFound == True)
    return Found;
  else 
    return NotFound;
}

ErrorCheck SelectSubFunctionFromID(GCD *CircuitData, int SubFID)
{
  Boolean IDFound;
  SubFunction *SubFCursor;
  
  IDFound = False;
  SubFCursor = CircuitData->ListSubFunction;
  while (SubFCursor != NULL)
    {
      if (SubFCursor->IDNumber == SubFID)
	{
	  IDFound = True;
	  SetSubFSelected(SubFCursor->State);
	}
      else
	UnSetSubFSelected(SubFCursor->State);
      SubFCursor = SubFCursor->pNext;
    }
  if (IDFound == True)
    return Found;
  else return NotFound;
}

ErrorCheck GetSubFunctionLinkNumber(GCD *CircuitData, int SubFID, int *LinkNb, Boolean Internal)
{
  Link *LinkCursor;
  int Counter, StartAttachID, EndAttachID;
  AttachPoint AttachStartProp, AttachEndProp;
  Component CompStartProp, CompEndProp;

  Counter = 0; 
  LinkCursor = CircuitData->ListLink;
  while (LinkCursor != NULL)
    {
      StartAttachID = LinkCursor->StartAttachID;
      EndAttachID = LinkCursor->EndAttachID;

      GetAttachPointProperties(CircuitData, StartAttachID, &AttachStartProp);
      GetAttachPointProperties(CircuitData, EndAttachID, &AttachEndProp);
      GetComponentProperties(CircuitData, AttachStartProp.ComponentID, &CompStartProp, False);
      GetComponentProperties(CircuitData, AttachEndProp.ComponentID, &CompEndProp, False);
	 
      if ((Internal == True) && ((CompStartProp.SubFunctionID == SubFID) && (CompEndProp.SubFunctionID == SubFID)))
	Counter++;

      if ((Internal == False) && ((CompStartProp.SubFunctionID == SubFID) || (CompEndProp.SubFunctionID == SubFID)))
	Counter++;
      
      LinkCursor = LinkCursor->pNext;
    }
  *LinkNb = Counter;
  
  return AllOk;
}

ErrorCheck GetSubFunctionLinkTab(GCD *CircuitData, int SubFID, int *LinkTab)
{
  Link *LinkCursor;
  int Counter, StartAttachID, EndAttachID;
  AttachPoint AttachStartProp, AttachEndProp;
  Component CompStartProp, CompEndProp;

  Counter = 0; 
  LinkCursor = CircuitData->ListLink;
  while (LinkCursor != NULL)
    {
      StartAttachID = LinkCursor->StartAttachID;
      EndAttachID = LinkCursor->EndAttachID;

      GetAttachPointProperties(CircuitData, StartAttachID, &AttachStartProp);
      GetAttachPointProperties(CircuitData, EndAttachID, &AttachEndProp);
      GetComponentProperties(CircuitData, AttachStartProp.ComponentID, &CompStartProp, False);
      GetComponentProperties(CircuitData, AttachEndProp.ComponentID, &CompEndProp, False);
	 
      if ((CompStartProp.SubFunctionID == SubFID) && (CompEndProp.SubFunctionID == SubFID))
	{
	  LinkTab[Counter] = LinkCursor->IDNumber;
	  Counter++;
	}
      LinkCursor = LinkCursor->pNext;
    }
  
  return AllOk;
}
