#ifndef XRMMACROS_H
#define XRMMACROS_H

#define CircuitIDRef       0 /* Type identifiers */
#define ComponentIDRef     1
#define AttachIDRef        2
#define LinkIDRef          3
#define SubFunctionIDRef   4

#if 0
#define free(Pointer) printf(#Pointer " : %p\n", ##Pointer ); free( ##Pointer )
#endif


/* Type Test */
#define IsCircuitID(Value)        (  (int) (Value) == 0)
#define IsComponentID(Value)      (( (int)(Value) % 10 ) == ComponentIDRef)
#define IsAttachID(Value)         (( (int)(Value) % 10 ) == AttachIDRef) 
#define IsLinkID(Value)           (( (int)(Value) % 10 ) == LinkIDRef)
#define IsSubFunctionID(Value)    (( (int)(Value) % 10 ) == SubFunctionIDRef)

#define Top(a)    ((a)+1) /* deplacement in SquareTab */
#define Bottom(a) ((a)-1)
#define Left(a)   ((a)-1)
#define Right(a)  ((a)+1)
#define Same(a)   (a)

/* Preferences and Paint (Linkwindow) */
#define SetColor(ColorPicker, ColorValue) gnome_color_picker_set_i8( GNOME_COLOR_PICKER(ColorPicker), \
                                          (ColorValue).Red, (ColorValue).Green, (ColorValue).Blue, 0)
#define GetColor(ColorPicker, ColorValue, Dummy) gnome_color_picker_get_i8( GNOME_COLOR_PICKER(ColorPicker), \
                                          &((ColorValue).Red), &((ColorValue).Green), &((ColorValue).Blue), &(Dummy))
/* CircuitView */
#define BufferTabR( Buffer, x, y, Width)   (##Buffer [y * Width * 3 + x * 3])
#define BufferTabG( Buffer, x, y, Width)   (##Buffer [y * Width * 3 + x * 3 + 1])
#define BufferTabB( Buffer, x, y, Width)   (##Buffer [y * Width * 3 + x * 3 + 2])

#define CircuitMaskBetweenPins    (1 << 0)
#define CircuitMaskCompoIn        (1 << 1)
#define CircuitMaskCompoOut       (1 << 2)
#define CircuitMaskSubFunction    (1 << 3)
#define CircuitMaskComponent      (1 << 4)
#define CircuitMaskAttachPoint    (1 << 5)
#define CircuitMaskInvisible      (1 << 6)
#define CircuitMaskLink           (1 << 7)

#define IsOnBetweenPins(Value)    IsState(Value, CircuitMaskBetweenPins)
#define IsOnCompoIn(Value)        IsState(Value, CircuitMaskCompoIn)
#define IsOnCompoOut(Value)       IsState(Value, CircuitMaskCompoOut)
#define IsOnSubFunction(Value)    IsState(Value, CircuitMaskSubFunction)
#define IsOnComponent(Value)      IsState(Value, CircuitMaskComponent)
#define IsOnAttachPoint(Value)    IsState(Value, CircuitMaskAttachPoint)
#define IsOnInvisible(Value)      IsState(Value, CircuitMaskInvisible)
#define IsOnLink(Value)           IsState(Value, CircuitMaskLink)

#define SetOnBetweenPins(Value)   SetState(Value, CircuitMaskBetweenPins)
#define SetOnCompoIn(Value)       SetState(Value, CircuitMaskCompoIn)
#define SetOnCompoOut(Value)      SetState(Value, CircuitMaskCompoOut)
#define SetOnSubFunction(Value)   SetState(Value, CircuitMaskSubFunction)
#define SetOnComponent(Value)     SetState(Value, CircuitMaskComponent)
#define SetOnAttachPoint(Value)   SetState(Value, CircuitMaskAttachPoint)
#define SetOnInvisible(Value)     SetState(Value, CircuitMaskInvisible)
#define SetOnLink(Value)          SetState(Value, CircuitMaskLink)

#define SetOffBetweenPins(Value)  UnSetState(Value, CircuitMaskBetweenPins)
#define SetOffCompoIn(Value)      UnSetState(Value, CircuitMaskCompoIn)
#define SetOffCompoOut(Value)     UnSetState(Value, CircuitMaskCompoOut)
#define SetOffSubFunction(Value)  UnSetState(Value, CircuitMaskSubFunction)
#define SetOffComponent(Value)    UnSetState(Value, CircuitMaskComponent)
#define SetOffAttachPoint(Value)  UnSetState(Value, CircuitMaskAttachPoint)
#define SetOffInvisible(Value)    UnSetState(Value, CircuitMaskInvisible)
#define SetOffLink(Value)         UnSetState(Value, CircuitMaskLink)


#define EVENT_METHOD(i, x) GTK_WIDGET_CLASS(GTK_OBJECT(i)->klass)->x

/* SquareState */
#define SquareUnused            (0)   
#define UpLeftUsed              (1 << 0)
#define UpRightUsed             (1 << 1)
#define DownLeftUsed            (1 << 2)
#define DownRightUsed           (1 << 3)
#define SquareUsed              (1 << 4)
#define SquareAttach            (1 << 5) 
#define FeatherUsed             (1 << 6)

#define CheckSquareState(Value, Type)  (((Value) & (Type)) == Type)   
#define SetSquareState(Value, Type)    ((Value) = (Value) | (Type))   

#define IsUpLeftCornerUsed(Value)           CheckSquareState(Value, UpLeftUsed)
#define IsUpRightCornerUsed(Value)          CheckSquareState(Value, UpRightUsed)
#define IsDownLeftCornerUsed(Value)         CheckSquareState(Value, DownLeftUsed)
#define IsDownRightCornerUsed(Value)        CheckSquareState(Value, DownRightUsed)
#define IsSquareUsed(Value)                 CheckSquareState(Value, SquareUsed)
#define IsSquareAttach(Value)               CheckSquareState(Value, SquareAttach)
#define IsFeatherUsed(Value)                CheckSquareState(Value, FeatherUsed)

#define SetUpLeftCornerUsed(Value)          SetSquareState(Value, UpLeftUsed)
#define SetUpRightCornerUsed(Value)         SetSquareState(Value, UpRightUsed)
#define SetDownLeftCornerUsed(Value)        SetSquareState(Value, DownLeftUsed)
#define SetDownRightCornerUsed(Value)       SetSquareState(Value, DownRightUsed)
#define SetSquareUsed(Value)                SetSquareState(Value, SquareUsed)
#define SetSquareAttach(Value)              SetSquareState(Value, SquareAttach)
#define SetFeatherUsed(Value)               SetSquareState(Value, FeatherUsed)

#define IsDiagonalDirection(Value)          ((Value %2) != 0)
#define Direction2DiagonalType(Value)       (Value / 2)

#define IsFreeFourCorners(x, y, Tab)         (  (!IsUpRightCornerUsed(Tab[x][y].SquareState))		\
				      	   && (!IsDownLeftCornerUsed(Tab[x+1][y+1].SquareState))	\
				       	   && (!IsUpLeftCornerUsed(Tab[x+1][y].SquareState))		\
				       	   && (!IsDownRightCornerUsed(Tab[x][y+1].SquareState)))

/* Invalide si ((tous libres) ou (tous utilises)) est faux */
#define IsInvalidFourCorners(x, y, Tab)     ( !( (  (!IsUpRightCornerUsed(Tab[x][y].SquareState))		\
					       && (!IsDownLeftCornerUsed(Tab[x+1][y+1].SquareState))	\
					       && (!IsUpLeftCornerUsed(Tab[x+1][y].SquareState))	\
					       && (!IsDownRightCornerUsed(Tab[x][y+1].SquareState)))	\
                                             ||								\
                                               (  (IsUpRightCornerUsed(Tab[x][y].SquareState))		\
					       && (IsDownLeftCornerUsed(Tab[x+1][y+1].SquareState))	\
					       && (IsUpLeftCornerUsed(Tab[x+1][y].SquareState))		\
					       && (IsDownRightCornerUsed(Tab[x][y+1].SquareState)))))

#define SetFullFourCorners(x, y, Tab)    SetUpRightCornerUsed(Tab[x][y].SquareState);		\
				         SetDownLeftCornerUsed(Tab[x+1][y+1].SquareState);	\
				         SetUpLeftCornerUsed(Tab[x+1][y].SquareState);		\
				         SetDownRightCornerUsed(Tab[x][y+1].SquareState)
#define SaveCorners(Tab, Direction)     

/*  ***********  */
#define SetState(Value, Type)     ((Value) = (Value) | (Type))
#define UnSetState(Value, Type)   ((Value) = (Value) & ~(Type))
#define IsState(Value, Type)      (((Value) & (Type)) == (Type))
#define InitState(Value)          (Value = 0)

/* Link State */
#define S_LinkFull                (1 << 0)
#define S_LinkSelected            (1 << 1)
#define S_LinkDLSelected          (1 << 2)
#define S_LinkRouted              (1 << 3)

#define IsLinkFull(Value)             IsState(Value, S_LinkFull)
#define IsLinkSelected(Value)         IsState(Value, S_LinkSelected)
#define IsLinkDLSelected(Value)       IsState(Value, S_LinkDLSelected)
#define IsLinkRouted(Value)           IsState(Value, S_LinkRouted)

#define SetLinkFull(Value)            SetState(Value, S_LinkFull)
#define SetLinkSelected(Value)        SetState(Value, S_LinkSelected)
#define SetLinkDLSelected(Value)      SetState(Value, S_LinkDLSelected)
#define SetLinkRouted(Value)          SetState(Value, S_LinkRouted)

#define UnSetLinkFull(Value)            UnSetState(Value, S_LinkFull)
#define UnSetLinkSelected(Value)        UnSetState(Value, S_LinkSelected)
#define UnSetLinkDLSelected(Value)      UnSetState(Value, S_LinkDLSelected)
#define UnSetLinkRouted(Value)          UnSetState(Value, S_LinkRouted)

/* DrawLink State */
#define S_DLinkSortedOk           (1 << 0)
#define S_DLinkSelected           (1 << 1)
#define S_DLinkRouted             (1 << 2) 

#define IsDLinkSortedOk(Value)           IsState(Value, S_DLinkSortedOk)
#define IsDLinkSelected(Value)           IsState(Value, S_DLinkSelected)
#define IsDLinkRouted(Value)             IsState(Value, S_DLinkRouted)

#define SetDLinkSortedOk(Value)          SetState(Value, S_DLinkSortedOk)
#define SetDLinkSelected(Value)          SetState(Value, S_DLinkSelected)
#define SetDLinkRouted(Value)            SetState(Value, S_DLinkRouted)

#define UnSetDLinkSortedOk(Value)        UnSetState(Value, S_DLinkSortedOk)
#define UnSetDLinkSelected(Value)        UnSetState(Value, S_DLinkSelected)
#define UnSetDLinkRouted(Value)          UnSetState(Value, S_DLinkRouted)

/* SubFunction State */
#define S_SubFSelected            (1 << 0)
#define IsSubFSelected(Value)           IsState(Value, S_SubFSelected)
#define SetSubFSelected(Value)          SetState(Value, S_SubFSelected)
#define UnSetSubFSelected(Value)        UnSetState(Value, S_SubFSelected)

/* ComponentState */
#define S_ComponentSelected       (1 << 0)
#define S_ComponentToLeft         (1 << 1)
#define S_ComponentToRight        (1 << 2)
#define S_ComponentToTop          (1 << 3)
#define S_ComponentToBottom       (1 << 4)
#define S_ComponentBetweenPins    (1 << 5)
#define S_ComponentCompoIn        (1 << 6)
#define S_ComponentCompoOut       (1 << 7)

#define IsComponentSelected(Value)        IsState(Value, S_ComponentSelected)
#define SetComponentSelected(Value)       SetState(Value, S_ComponentSelected)
#define UnSetComponentSelected(Value)     UnSetState(Value, S_ComponentSelected)

#define IsComponentBetweenPins(Value)     IsState(Value, S_ComponentBetweenPins)
#define SetComponentBetweenPins(Value)    SetState(Value, S_ComponentBetweenPins)
#define UnSetComponentBetweenPins(Value)  UnSetState(Value, S_ComponentBetweenPins)

#define IsComponentCompoIn(Value)         IsState(Value, S_ComponentCompoIn)
#define SetComponentCompoIn(Value)        SetState(Value, S_ComponentCompoIn)
#define UnSetComponentCompoIn(Value)      UnSetState(Value, S_ComponentCompoIn)

#define IsComponentCompoOut(Value)        IsState(Value, S_ComponentCompoOut)
#define SetComponentCompoOut(Value)       SetState(Value, S_ComponentCompoOut)
#define UnSetComponentCompoOut(Value)     UnSetState(Value, S_ComponentCompoOut)

#define IsComponentToLeft(Value)          IsState(Value, S_ComponentToLeft)
#define SetComponentToLeft(Value)         SetState(Value, S_ComponentToLeft)
#define UnSetComponentToLeft(Value)       UnSetState(Value, S_ComponentToLeft)

#define IsComponentToRight(Value)         IsState(Value, S_ComponentToRight)
#define SetComponentToRight(Value)        SetState(Value, S_ComponentToRight)
#define UnSetComponentToRight(Value)      UnSetState(Value, S_ComponentToRight)

#define IsComponentToTop(Value)           IsState(Value, S_ComponentToTop)
#define SetComponentToTop(Value)          SetState(Value, S_ComponentToTop)
#define UnSetComponentToTop(Value)        UnSetState(Value, S_ComponentToTop)

#define IsComponentToBottom(Value)        IsState(Value, S_ComponentToBottom)
#define SetComponentToBottom(Value)       SetState(Value, S_ComponentToBottom)
#define UnSetComponentToBottom(Value)     UnSetState(Value, S_ComponentToBottom)

/* Attach State */
#define S_AttachSelected          (1 << 0)
#define S_AttachToLeft            (1 << 1)
#define S_AttachToRight           (1 << 2)
#define S_AttachToTop             (1 << 3)
#define S_AttachToBottom          (1 << 4)

#define IsAttachSelected(Value)           IsState(Value, S_AttachSelected)
#define SetAttachSelected(Value)          SetState(Value, S_AttachSelected)
#define UnSetAttachSelected(Value)        UnSetState(Value, S_AttachSelected)


#define IsAttachToLeft(Value)             IsState(Value, S_AttachToLeft)
#define SetAttachToLeft(Value)            SetState(Value, S_AttachToLeft)
#define UnSetAttachToLeft(Value)          UnSetState(Value, S_AttachToLeft)

#define IsAttachToRight(Value)            IsState(Value, S_AttachToRight)
#define SetAttachToRight(Value)           SetState(Value, S_AttachToRight)
#define UnSetAttachToRight(Value)         UnSetState(Value, S_AttachToRight)

#define IsAttachToTop(Value)              IsState(Value, S_AttachToTop)
#define SetAttachToTop(Value)             SetState(Value, S_AttachToTop)
#define UnSetAttachToTop(Value)           UnSetState(Value, S_AttachToTop)

#define IsAttachToBottom(Value)           IsState(Value, S_AttachToBottom)
#define SetAttachToBottom(Value)          SetState(Value, S_AttachToBottom)
#define UnSetAttachToBottom(Value)        UnSetState(Value, S_AttachToBottom)

/**************/
/* Fig Export */
/**************/
#define FigDeltax 10 /* delta from left in the fig file (value in square unit) */
#define FigDeltay 10 /* delta from right in the fig file (value in square unit) */
#define FrameFigWidth SquareToFigWidth(1) / 2
#define FigMagnification 1
#define RadiusCornerPercentage (0.01)
#define PercentageOut (0.05)    /* proportion de la largeur du carreau laisse libre */
#define PercentageIn  (0.9)    /* proportion de la largeur du carreau occupe */
#define RadiusRatio (6)        /* le rayon du cercle interieur est 6 * plus petit que la plus petite dimension */
#define SquarePerInch (200.0)   /* nombre de points par pouce dans la representation du circuit */
#define FigPerInch (1200.0)    /* la resolution pour la position des points */
#define FigWidthPerInch (80) /* pour la largeur des polylines */

#define SquareToFig(SquareDim)      ( (int) ((SquareDim)  * FigMagnification * FigPerInch / SquarePerInch))
#define SquareToFigWidth(SquareDim) ( (int) ((SquareDim)  * FigMagnification * FigWidthPerInch / SquarePerInch))

#define MillimeterPerInch (25.4) 
#define SquareToInch(SquareDim)          ( (SquareDim) / SquarePerInch)
#define SquareToMillimeter(SquareDim)    ( (SquareDim) / SquarePerInch * MillimeterPerInch)

#define echo(String) printf(String "\n")


#endif /* XRMMACROS_H */
