#ifndef  XRMTYPES_H
#define  XRMTYPES_H

#include <gnome.h>
#include "path.h"
#include "commontypes.h"
#include "xrmmacros.h"
#include "libtypes.h"

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define NoRoad -5 /* initialisation and test value for some variables */
#define NoLink -6 /* values are almost random and always negative */
#define NoLevel -6
#define NoLine -6
#define NoAttach -7
#define NoAttachLink -33
#define NoVortex -1
#define NoX -8
#define NoY -9
#define ExceptionAttach -11
#define NoOrder -211
#define LinkLengthException -27
#define ExceptionRoutingLinkID -14
#define NoID -54
#define XRMVersion VERSION 
#define FileVersion VERSION
/* for file loading compatibility test */
/* (saved but not tested since file structure is moving rapidly */

#define NameLength 25      /* String Length in HeaderFile */
#define VersionLength 8
#define DesignerLength 25
#define SocietyLength 30
#define DescriptionLength 50
#define RevisionDateLength 20

#define ComponentNameLength 12
#define SubFunctionNameLength 12

#define FileNameLength 80

/* Used by interface... */
#define WhiteRGB 0xffffff
#define BlackRGB 0x000000

#define TranslateCursor 52
#define ZoomInCursor 34
#define ZoomOutCursor 34
#define StandardCursor 68

#define ObjID "ID-DataKey" /* for storing data pointer */
#define DirectoryEntry "DirEntry"
#define RulerWidth (20)
#define ErrorListColumnNb 3
#define IDTypeNumber 4 
#define LLColumnNb 4
#define DLColumnNb 5
#define AttachColumnStart 1
#define AttachColumnEnd 2
#define DefaultConfigFile     XROADMAKER_PATH "/config/xroadmaker.conf"
#define ConfigDir ".xroadmaker"
#define DefaultSetFile        XROADMAKER_PATH "/libs/stdcomponent.set"
#define DefaultCircuitName "Circuit"
#define DefaultAttachName "Attach"

/* Arbitrary Number for identification of selected items in the Choose Desc windows */
#define DescSelected           21
#define DescUnSelected         22
#define DescClassSelected      23
#define DescClassUnSelected    24

#define SubFunctionSelected -58
#define SubFunctionUnSelected -60
#define ColorPaletteSize 20
#define SpotLevelNumber 3
#define LayoutLevelNumber 3

typedef enum IntegrityType { OutOfCircuit, LinkOutOfCircuit, OverlappingAttach, SpotMissing, LayoutMissing, CompDescMissing} IntegrityType;
typedef enum IntegrityPriority { CriticalPriority, WarningPriority} IntegrityPriority;

typedef enum { SpotPalette, SpotUnSelectedPalette, CoreSpotPalette, CoreSpotUnSelectedPalette, 
	       InvisibleSpotPalette, InvisibleSpotUnSelectedPalette, 
	       BetweenPinsPalette, BetweenPinsUnSelectedPalette, CompoInPalette, CompoInUnSelectedPalette,
	       CompoOutPalette, CompoOutUnSelectedPalette, AlternatePalette, ComponentPalette, 
	       ComponentUnSelectedPalette, SubFunctionPalette, SubFunctionUnSelectedPalette,
	       LinkPalette, LinkUnSelectedPalette, LinkDefaultPalette } PaletteIndex;

typedef enum { RightDirection, TopRightDirection, TopDirection, TopLeftDirection, LeftDirection, 
	       BottomLeftDirection, BottomDirection, BottomRightDirection, NoDirection }  Direction;

typedef enum { CompoOutLevel, CompoInLevel, BetweenPinsLevel} LayoutShapeLevel;
typedef enum { StandardSpotLevel, CoreSpotLevel, InvisibleSpotLevel} SpotShapeLevel;

typedef enum {AttachDep, LinkDep, NoDep} DepType;
/* defini la nature de l evenement qui requiert la mise a jour et les politiques a mettre en oeuvre */
/* UCV pour UpdateCircuitView */
typedef enum  {UCVContinually, UCVModification, UCVSelection, UCVClick, UCVDemand } UCVEvent; 
typedef enum { TranslateAction, TranslateStart,  ZoomInAction, ZoomOutAction,  NoAction} ButtonActionType;

typedef enum {AttachCoord, AttachSelected, ComponentCoord, SubFunctionCoord, LinkCoord } CoordType;
typedef enum {OrthogonalPref, DiagonalPref, SameDirectionPref} DirectionPref;

typedef enum {AscendantOrder, DescendantOrder, IndifferentOrder} SortOrder;

/*  Structures used in routing process ******************************** */
extern int ForwardDirection [8][2];
extern int BaseDirection [4][2];

typedef struct {
  DepType Type;
  int  Nb;
} DepCarac;

typedef struct {
  DepCarac *Tab;
  int Size;
} Dep;

typedef struct {
  int CalculatingLineNb;
  int CalculatingLevelNb;
  int CalculatingVortexNb;
  int CalculatingVortexLineNb;
  
  int SquareState;
  int AttachLinkNb;
} Square;

/* structure contenant des donnees sur une equipotentielle */
typedef struct { 
  int Size;         /* le nb de points equipotentiels */
  Point *PointTab ;  /* listes des points equipotentiels */
} EquiPotential;


typedef struct{
  Point Start;
  Point End;
} Segment; 


/* Structure contenant la taille du chemin reliant 2 points et la liste des points correspondants */
typedef struct {  
  int SegSize;
  int PathSize;
  int CorePathSize;
  int DiagonalSize;
  int DepSize;
  Boolean JobDone;

  Segment *SegTab;  
  Point *PathTab;
  Point *DiagonalTab;
  DepCarac *DepTab;
} RoadCarac; 

/* structure contenant le nb de pistes et les caracs de chaques pistes */

typedef struct {
  int Size;
  RoadCarac *Line;
} RoadMap;

/* structure contenant pour chaque attache sa position et la liste des liens qui lui sont relies */

typedef struct {
  int Size;
  int *Attach2Link;
  int AttachLocationNb;
  int CoreAttachLocationNb;
  int InvisibleAttachLocationNb;
  Point *AttachLocation;
  Point *CoreAttachLocation;
  Point *InvisibleAttachLocation;
} AttachCarac;

/* structure contenant pour chaque vortex le point d arrivee du vortex */

typedef struct {
  Point VortexExit;
} VortexCarac;

/* structure contenant pour chaque ligne le point de depart et d arrivee et les equivalent ( equipot) 
   pour chacun des deux */

typedef struct {
  int Start; /* numero du point de depart */
  int End; /* numero du point d arrivee */
  int LinkWidth; /* largeur de la piste / 0 -> 1 carreau / 1 -> 3 carreau ... */
  int LinkSize;  /* nb de liens equivalent au point de depart */
  int *Link2Link;    /* liste des liens equivalents au point d arrivee */
} LinkCarac; 

/* structure de reperage d'une zone interdite */
typedef struct 
{
    int Level;
    int LeftLimit;
    int RightLimit;
    int TopLimit;
    int BottomLimit;
} AreaCarac;

/* structure contenant toutes les donnees envoyees par l interface et deduites de celui ci */
typedef struct {
  DirectionPref DirectionPrefTab[3];  
  int Width; 
  int Height;
  int AttachNb;  /* nb de fixations */
  int LinkNb;    /* nb de liens */ 
  int AreaNb;
  int VortexNb;
  
  AttachCarac *AttachTab; /* caracs de chaque fixations */
  LinkCarac *LinkTab;   /* caracs de chaques liens */
  AreaCarac *AreaTab;
  VortexCarac *VortexTab;

  Square **SquareTab;   /* tableau pour le calcul des chemins */
} Circuit;

/* Structures used in GCD  ********************************* */

typedef struct 
{
  char Name[NameLength];
  char Version[VersionLength];
  char Designer[DesignerLength];
  char Society[SocietyLength];
  char RevisionDate[RevisionDateLength];
  int Width;
  int Height;
  char Description[DescriptionLength];
} Header;

typedef struct
{
  guint8 Red;
  guint8 Green;
  guint8 Blue;
} Color;

typedef struct AttachPoint 
{
  char AttachPointName[PinNameLength];
  char SpotName[SpotNameLength];
  int IDNumber;
  void *InterfaceID;  /* Not Saved in File */
  int ComponentID;
  Point Reference;
  int RoutingAttachID;
  int SpotSize;
  int CoreSpotSize;
  int InvisibleSpotSize;
  int State;
  Point *SpotLocation; /* tableau de tous les points recouverts par le point d'attache */
  Point *CoreSpotLocation; /* tableau de tous les points recouverts par le point d'attache */
  Point *InvisibleSpotLocation; /* tableau de tous les points recouverts par le point d'attache */
  struct AttachPoint *pNext;
} AttachPoint; /* structure de type liste */ 

typedef struct Component 
{
  char ComponentName[ComponentNameLength];
  char CompDescName[CompDescNameLength];
  char LayoutName[LayoutNameLength];
  char LayoutClass[LayoutNameLength];
  int IDNumber; 
  void *InterfaceID; /* Not Saved in File */
  int SubFunctionID;
  Point Reference;
  int AttachPointNb;
  int *AttachPointIDTab;
  int State;
  struct Component *pNext;
} Component; /* structure de type liste */  

typedef struct SubFunction 
{
  char Name[SubFunctionNameLength];
  int IDNumber;  
  Point Reference;
  void *InterfaceID; /* Not Saved in File */
  int State;
  struct SubFunction *pNext;
} SubFunction; /* structure de type liste */ 

typedef struct Link 
{
  int IDNumber;
  int StartAttachID;
  int EndAttachID;

  int LinkWidth;
  int LinkLength;
  void *InterfaceID; /* Not Saved in File */
  Point Start;
  Point End;
  int State;
  int RoutedOrder;
  int RoutingLinkID;
  Color LinkColor;
  RoadCarac Road;
  struct Link *pNext;
} Link; /* structure de type liste */

typedef struct DrawLink 
{
  int LinkID;
  int LinkLength;
  int LinkWidth;
  int RoutedOrder;
  int State; 
  struct DrawLink *LinkUp; 
  struct DrawLink *LinkDown;
} DrawLink;

typedef struct 
{
  char Name[SubFunctionNameLength];
  int State;
  int NbComp;
  int NbLink;
  Point Reference;
  Component *TabCompSF;
  Link *TabLinkSF;
  int **LinkInfo;
} SubFunctionBuffer ;

typedef struct LinkPermutation
{
  int LinkID1;
  int LinkID2;
  int Number;
  struct LinkPermutation *pNext;
} LinkPermutation;

typedef struct 
{
  Header DataHeader;
  Color ColorPalette[ColorPaletteSize];
  Boolean UpdateContinually;
  Boolean UpdateClicked;
  Boolean UpdateDemand;
  float ScaleFactor;
  int CircuitMask;
  Boolean SelectionFollowMouse;
  Boolean ShortFirst;
  Boolean NoLinkIcon;
  Boolean NoLinkColor;
  Boolean Millimeter;
  Boolean LogToFile;
  char LogFileName[FileNameLength];
  int SwapMax;
  int RoutingMax;
  Boolean UnRoutableDelete;
  int UnRoutableSwapMax;
  Boolean DirectionPrefTab[3];
  SortOrder WidthOrder;
  SortOrder LengthOrder;
  Boolean SortWidthFirst;
  Boolean HoleForce;
  Boolean AllowEditComponent;
  Boolean AllowDeleteComponent;
  Boolean AllowDeleteSubFunction;
  char DefaultSubFunctionName[SubFunctionNameLength];
  char DefaultComponentName[ComponentNameLength];
  Boolean OpenInformation;
  Boolean RevertInformation;
  Boolean SaveInformation;
  Boolean UseMagic;
  Boolean StopRouterOnError;
  char CircuitDirectory[FileNameLength];
} OptionSet;

typedef struct 
{
  GdkColor PaintColor;
} PaletteElement;


typedef struct {
  OptionSet Opt;
  int AttachPointID;
  int ComponentID;
  int LinkID;
  int SubFunctionID;
  int RoutedOrder;
  void *MainWindow;
  void *LinkWindow;
  void *ComponentWindow;
  void *CircuitViewWindow;
  void *AboutWindow;
  void *CTRootItem;
  void *CTComponentTree;
  void *LLClist;
  void *DLClist;
  void *CVPopupMenu;
  void *ProgressBarRouter;
  void *ProgressBarTrack;
  void *RouterLabel;
  void *TrackLabel;
  PaletteElement ColorPalette[ColorPaletteSize];
  Boolean NamedFile;
  Boolean NewModification;
  char CircuitFileName[FileNameLength];
  PaletteElement Palette[ColorPaletteSize];
  guchar *CVBuffer;
  guchar *CVBackupBuffer;
  int BufferWidth;
  int BufferHeight;
/*   GdkPixmap *CVPixmap; */
/*   GdkPixmap *CVBackupPixmap; */
  int Order;
  Boolean Busy;
  Boolean RouterCancelRequest;
  ButtonActionType ButtonAction;
  Point CVOrigin;
  Point CVOriginStart;
  Point TranslateStart;
  Point TraceStart;
  Point PointerLocation;
} InterfaceInformation ;

typedef struct 
{
  AttachPoint *ListAttachPoint;
  Component *ListComponent;
  Link *ListLink;
  Component ComponentBuffer;
  SubFunction *ListSubFunction;
  SubFunctionBuffer ListSubFunctionBuffer;
  Header DataHeader;
  Boolean ComponentBufferEmpty;
  Boolean SubFunctionBufferEmpty;
  
  DrawLink *TopLink;
  DrawLink *BottomLink;
  LinkPermutation *ListPermutation;
  ComponentLibrary *CompLib;

  InterfaceInformation Info;
} GCD;

/* Miscellaneous Structures  ************************************** */

typedef struct 
{
  int **DataTab;
  int Width;
  int Height;
} CircuitViewData;

typedef union 
{
  struct
  {
    int ComponentInvolved;
    Point Location;
  } Out;
  struct
  {
    int LinkInvolved;
    Point Location;
  } LinkOut;
  struct 
  {
    int  ComponentInvolved;
    int OtherComponentInvolved;
    Point Location;
  } Overlap;
  struct 
  {
    char SpotName[SpotNameLength];
    char ComponentName[ComponentNameLength];
    char SubFunctionName[SubFunctionNameLength];
  } SpotMissing;
  struct  
  {
    char LayoutName[LayoutNameLength];
    char ComponentName[ComponentNameLength];
    char SubFunctionName[SubFunctionNameLength];
  } LayoutMissing;
  struct  
  {
    char CompDescName[CompDescNameLength];
    char ComponentName[ComponentNameLength];
    char SubFunctionName[SubFunctionNameLength];
  } CompDescMissing;
} TypeDependentUnion;

typedef struct 
{
  IntegrityPriority Priority;
  IntegrityType Type;
  TypeDependentUnion TypeDependent;
} Integrity;
/* Fig File **********/
typedef struct 
{
  Point Reference;
  SpotDef SpotProp;
} FigSpot;

typedef struct 
{
  int SegSize;
  int LinkWidth;
  Segment *SegTab;
} FigPath;


#endif  /* XRMTYPES_H */


